//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit BurroNAT;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, BurroSocket, BurroUDP,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  extctrls, QBTimer;

type
   TTaskSuccessEvent = procedure (Sender: TObject) of object;              
   TTaskFailEvent = procedure (Sender: TObject) of object;                 

type
  TBurroNAT = class(TComponent)
  private
    fTimeout: integer;           
    fTaskType: integer;          
    fLocalIp: Ansistring;            
    fRouterIp: Ansistring;           
    fRouterPort: integer;        
    fRouterLocation: Ansistring;     
    fRouterName: Ansistring;         
    fRouterUSN: Ansistring;          
    fRouterURL: Ansistring;          
    fExternalIp: Ansistring;         
    fControlURL: Ansistring;         
    fServiceType: Ansistring;        
    fURLbase: Ansistring;            
    FOnTaskSuccess: TNotifyEvent;    
    FOnTaskFail: TNotifyEvent;       
    request: Ansistring;              
    requested: boolean;           
    response: Ansistring;             
    UDP: TBurroUDP;                        
    Sock: TBClientSocket;                  
    TaskTimer: TQBTimer;                     
    TaskExecuting: boolean;                
    Taskfinished: boolean;                 
    Tasksuccess: boolean;                  
    procedure SetTimeout(Timeout: integer);
    procedure SuccessEvent;
    procedure FailEvent;
    procedure TaskTimerTimer(Sender: TObject);        
    procedure UDPInvalidHost(var handled: Boolean);   
    procedure UDPBufferInvalid(var handled: Boolean; var Buff: array of AnsiChar; var length: Integer);
    procedure UDPStreamInvalid(var handled: Boolean; Stream: TStream);
    procedure UDPDataReceived(Sender: TComponent; NumberBytes: Integer; FromIP: AnsiString; Port: Integer);
    procedure ClientSocketWrite(Sender: TObject; Socket: TBCustomWinSocket);
    procedure ClientSocketRead(Sender: TObject; Socket: TBCustomWinSocket);
    procedure ResolveResult;
    function ResponseFinished(ResponseData: Ansistring): boolean;
    procedure ClientSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure ClientSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
  protected
    

  public
    Constructor Create(AOwner: TComponent); override;           
    Destructor Destroy(); override;                             
    function SearchRouter(): boolean;
    function GetControlURL(): boolean;
    function AddNatMapping(NatPortName: Ansistring; ExternalPort: integer; LocalIp: Ansistring; LocalPort: integer; Protocol: Ansistring): boolean;
    function GetExternalIp: boolean;
    function DeleteNatMapping(ExternalPort: integer; Protocol: Ansistring): boolean;
  published
    property Timeout: integer read fTimeout write settimeout; 
    property TaskType: integer read fTaskType;                
    property LocalIp: Ansistring read fLocalIp;                   
    property RouterIp: Ansistring read fRouterIp;                 
    property RouterPort: integer read fRouterPort;            
    property RouterLocation: Ansistring read fRouterLocation;     
    property RouterName: Ansistring read fRouterName;             
    property RouterUSN: Ansistring read fRouterUSN;               
    property RouterURL: Ansistring read fRouterURL;               
    property ExternalIp: Ansistring read fExternalIp;             
    property ControlURL: Ansistring read fControlURL;             
    property ServiceType: Ansistring read fServiceType;           
    property URLBase: Ansistring read fURLBase;                   
    property OnTaskSuccess: TNotifyEvent read FOnTaskSuccess write FOnTaskSuccess;  
    property OnTaskFail: TNotifyEvent read FOnTaskFail write FOnTaskFail;           
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Basic', [TBurroNAT]);
end;

constructor TBurroNAT.Create(AOwner: TComponent);
begin
   inherited;
   UDP:=TBurroUDP.Create(self);                
   udp.RemoteHost:='239.255.255.250';          
   udp.RemotePort:=1900;                         
   udp.ServerPort:=1900;
   UDP.OnInvalidHost:=UDPInvalidHost;          
   UDP.OnBufferInvalid:=UDPBufferInvalid;      
   UDP.OnStreamInvalid:=UDPStreamInvalid;      
   UDP.OnDataReceived:=UDPDataReceived;        
   sock:=TBClientSocket.create(self);          
   sock.ClientType:=ctNonBlocking;             
   sock.OnWrite:=clientsocketwrite;            
   sock.OnRead:=clientsocketread;              
   sock.OnError:=clientsocketerror;            
   sock.OnDisconnect:=clientsocketdisconnect;  
   TaskTimer:=TQBTimer.Create(self);             
   TaskTimer.Interval:=1000*5;                 
   TaskTimer.enabled:=false;                   
   TaskTimer.OnTimer:=TaskTimerTimer;          
   taskexecuting:=false;                       
   taskfinished:=false;                        
   tasksuccess:=false;                         
   fexternalip:='0.0.0.0';                     
   ftimeout:=5000;                             
end;

destructor TBurroNAT.Destroy;
begin
  UDP.free;
  tasktimer.free;
  sock.free;
  inherited;
end;

procedure TBurroNAT.SetTimeout(Timeout: integer);
begin
   if (timeout<1000) or (timeout>30*1000) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1000-30000)','Invalid',mb_ok+mb_iconinformation);
         exit;
      end;
   ftimeout:=timeout;
end;

procedure TBurroNAT.SuccessEvent;
begin
   tasktimer.enabled:=false;
   taskexecuting:=false;           
   taskfinished:=true;             
   tasksuccess:=true;              
   if assigned(fontasksuccess) then
      fontasksuccess(self);
end;

procedure TBurroNAT.FailEvent;
begin
   tasktimer.enabled:=false;
   taskexecuting:=false;           
   taskfinished:=true;             
   tasksuccess:=false;             
   if assigned(fontaskfail) then
      fontaskfail(self);
end;

procedure TBurroNAT.UDPInvalidHost(var handled: Boolean);
begin
   failevent;                      
   handled:=true;
end;

procedure TBurroNAT.UDPBufferInvalid(var handled: Boolean; var Buff: array of AnsiChar; var length: Integer);
begin
   failevent;                      
   handled:=true;
end;

procedure TBurroNAT.UDPStreamInvalid(var handled: Boolean; Stream: TStream);
begin
   failevent;                      
   handled:=true;
end;

procedure TBurroNAT.UDPDataReceived(Sender: TComponent; NumberBytes: Integer; FromIP: AnsiString; Port: Integer);
var
   tmpstr: Ansistring;
   j: integer;
begin
   if (numberbytes<=0) or (numberbytes>4096) then
      exit;
   setlength(tmpstr,numberbytes);
   udp.ReadBuffer(tmpstr);
   if uppercase(copy(tmpstr,1,5))<>'HTTP/' then
      begin
         if ftasktype=1 then
            failevent;                      
         exit;
      end;
   fRouterLocation:=tmpstr;
   j:=Pos(AnsiString('LOCATION:'),uppercase(fRouterLocation));
   if j<0 then
      fRouterLocation:=''
   else
      begin
         delete(fRouterLocation,1,j+8);
         j:=Pos(AnsiString(#13#10),fRouterLocation);
         fRouterLocation:=trim(copy(fRouterLocation,1,j-1));
      end;
   fRouterName:=tmpstr;
   j:=Pos(AnsiString('SERVER:'),uppercase(fRouterName));
   if j<0 then
      fRouterName:=''
   else
      begin
         delete(fRouterName,1,j+6);
         j:=Pos(AnsiString(#13#10),fRouterName);
         fRouterName:=trim(copy(fRouterName,1,j-1));
      end;
   fRouterUSN:=tmpstr;
   j:=Pos(AnsiString('USN:'),uppercase(fRouterUSN));
   if j<0 then
      fRouterUSN:=''
   else
      begin
         delete(fRouterUSN,1,j+3);
         j:=Pos(AnsiString(#13#10),fRouterUSN);
         fRouterUSN:=trim(copy(fRouterUSN,1,j-1));
      end;
   tmpstr:=fRouterLocation;
   if copy(uppercase(tmpstr),1,7)='HTTP://' then
      delete(tmpstr,1,7);
   j:=Pos(AnsiString(':'),tmpstr);
   if j<=0 then
      begin
         if ftasktype=1 then
            failevent;                      
         exit;
      end;
   fRouterIp:=copy(tmpstr,1,j-1);
   delete(tmpstr,1,j);
   j:=Pos(AnsiString('/'),tmpstr);
   if j>1 then
      begin
         try
            fRouterPort:=strtoint(copy(string(tmpstr),1,j-1));
         except
            fRouterPort:=-1;
         end;
         delete(tmpstr,1,j-1);
         fRouterURL:=tmpstr;
      end
   else
      begin
         j:=Pos(AnsiString(#13#10),tmpstr);
         if j<=1 then
            begin
               if ftasktype=1 then
                  failevent;                      
               exit;
            end;
         try
            fRouterPort:=strtoint(copy(string(tmpstr),1,j-1));
         except
            fRouterPort:=-1;
         end;
         fRouterURL:='/';
      end;
   tmpstr:=AnsiString('http://')+frouterip+AnsiString(':'+inttostr(frouterport))+frouterurl;
   furlbase:='';
   j:=Pos(AnsiString('/'),tmpstr);
   while j>0 do
      begin
         furlbase:=furlbase+copy(tmpstr,1,j);
         delete(tmpstr,1,j);
         j:=Pos(AnsiString('/'),tmpstr);
      end;
   delete(furlbase,length(furlbase),1);
   if (trim(fRouterIp)='') or (fRouterPort<=0) then
      begin
         if ftasktype=1 then
            failevent;                      
      end
   else
      begin
         if ftasktype=1 then
            successevent;               
      end;
end;

procedure TBurroNAT.TaskTimerTimer(Sender: TObject);
begin
   failevent;
   ftasktype:=-1;                  
end;

function TBurroNAT.ResponseFinished(ResponseData: Ansistring): boolean;
var
   head: Ansistring;
   contentlength,j,headlength: integer;
begin
   result:=false;
   j:=Pos(AnsiString(#13#10#13#10),responsedata);
   if j<=0 then
      exit;
   head:=copy(responsedata,1,j-1);
   headlength:=j+3;
   j:=Pos(AnsiString('CONTENT-LENGTH:'),uppercase(head));
   if j<=0 then
      begin
         if (Pos(AnsiString('</HTML>'),uppercase(responsedata))>0) or (Pos(AnsiString('</ROOT>'),uppercase(responsedata))>0) then
            result:=true
         else
            result:=false;
         exit;
      end;
   delete(head,1,j+14);
   j:=Pos(AnsiString(#13#10),head);
   try
      contentlength:=strtoint(copy(string(head),1,j-1));
   except
      contentlength:=999999;
   end;
   if (length(responsedata)-headlength)>=contentlength then
      result:=true;
end;

procedure TBurroNAT.ClientSocketWrite(Sender: TObject; Socket: TBCustomWinSocket);
var
   j: integer;
begin
   if requested then
      exit;
   tasktimer.enabled:=false;
   response:='';              
   j:=-1;
   while j=-1 do
      j:=socket.SendBuf(request[1],length(request));
   requested:=true;           
   TaskTimer.Enabled:=true;
end;

procedure TBurroNAT.ClientSocketRead(Sender: TObject; Socket: TBCustomWinSocket);
var
   tmpstr: Ansistring;
   j: integer;
begin
   j:=socket.ReceiveLength;
   setlength(tmpstr,j);
   socket.ReceiveBuf(tmpstr[1],j);
   response:=response+tmpstr;
   if not responsefinished(response) then
      exit;
   sock.close;
end;

procedure TBurroNAT.ResolveResult;
var
   tmpstr: Ansistring;
   j: integer;
begin
   case ftasktype of
      2: begin
            tmpstr:=response;
            j:=Pos(uppercase(AnsiString('<URLBASE>')),uppercase(tmpstr));
            if j>0 then
               begin
                  delete(tmpstr,1,j+length('<URLBASE>')-1);
                  j:=Pos(uppercase(AnsiString('</URLBASE>')),uppercase(tmpstr));
                  if j>1 then
                     furlbase:=copy(tmpstr,1,j-1);
               end;
            j:=Pos(uppercase(AnsiString('<deviceType>urn:schemas-upnp-org:device:InternetGatewayDevice:1</deviceType>')),uppercase(tmpstr));
            if j<=0 then
               begin
                  failevent;
                  sock.Close;      
                  exit;
               end;
            delete(tmpstr,1,j+length('<deviceType>urn:schemas-upnp-org:device:InternetGatewayDevice:1</deviceType>')-1);
            j:=Pos(uppercase(AnsiString('<deviceType>urn:schemas-upnp-org:device:WANDevice:1</deviceType>')),uppercase(tmpstr));
            if j<=0 then
               begin
                  failevent;
                  sock.Close;      
                  exit;
               end;
            delete(tmpstr,1,j+length('<deviceType>urn:schemas-upnp-org:device:WANDevice:1</deviceType>')-1);
            j:=Pos(uppercase(AnsiString('<deviceType>urn:schemas-upnp-org:device:WANConnectionDevice:1</deviceType>')),uppercase(tmpstr));
            if j<=0 then
               begin
                  failevent;
                  sock.Close;      
                  exit;
               end;
            delete(tmpstr,1,j+length('<deviceType>urn:schemas-upnp-org:device:WANConnectionDevice:1</deviceType>')-1);
            j:=Pos(uppercase(AnsiString('<serviceType>urn:schemas-upnp-org:service:WANIPConnection:1</serviceType>')),uppercase(tmpstr));
            if j<=0 then
               begin
                  j:=Pos(uppercase(AnsiString('<serviceType>urn:schemas-upnp-org:service:WANPPPConnection:1</serviceType>')),uppercase(tmpstr));
                  if j<=0 then
                     begin
                        failevent;
                        sock.Close;      
                        exit;
                     end
                  else
                     begin
                        delete(tmpstr,1,j+length('<serviceType>urn:schemas-upnp-org:service:WANPPPConnection:1</serviceType>')-1);
                        fservicetype:='urn:schemas-upnp-org:service:WANPPPConnection:1';
                     end;
               end
            else
               begin
                  delete(tmpstr,1,j+length('<serviceType>urn:schemas-upnp-org:service:WANIPConnection:1</serviceType>')-1);
                  fservicetype:='urn:schemas-upnp-org:service:WANIPConnection:1';
               end;
            j:=Pos(uppercase(AnsiString('<controlURL>')),uppercase(tmpstr));
            if j<=0 then
               begin
                  failevent;
                  sock.Close;      
                  exit;
               end;
            delete(tmpstr,1,j+length('<controlURL>')-1);
            j:=Pos(uppercase(AnsiString('</controlURL>')),uppercase(tmpstr));
            if j<=0 then
               begin
                  failevent;
                  sock.Close;      
                  exit;
               end;
            fcontrolurl:=copy(tmpstr,1,j-1);
            if (copy(urlbase,length(urlbase),1)='/') and (copy(fcontrolurl,1,1)='/') then
               delete(fcontrolurl,1,1);
            if (copy(urlbase,length(urlbase),1)<>'/') and (copy(fcontrolurl,1,1)<>'/') then
               fcontrolurl:='/'+fcontrolurl;
            fcontrolurl:=urlbase+fcontrolurl;
            successevent;
         end;
      3: begin
            tmpstr:=response;
            j:=Pos(AnsiString(#13#10),tmpstr);
            tmpstr:=uppercase(copy(tmpstr,1,j-1));
            if Pos(AnsiString('200 OK'),tmpstr)>0 then
               successevent
            else
               failevent;
         end;
      4: begin
            tmpstr:=response;
            j:=Pos(AnsiString(#13#10),tmpstr);
            tmpstr:=uppercase(copy(tmpstr,1,j-1));
            if Pos(AnsiString('200 OK'),tmpstr)>0 then
               begin
                  j:=Pos(uppercase(AnsiString('<NewExternalIPAddress>')),uppercase(response));
                  if j<=0 then
                     fexternalip:='0.0.0.0'
                  else
                     begin
                        fexternalip:=response;
                        delete(fexternalip,1,j+length('<NewExternalIPAddress>')-1);
                        j:=Pos(uppercase(AnsiString('</NewExternalIPAddress>')),uppercase(fexternalip));
                        if j<=0 then
                           fexternalip:='0.0.0.0'
                        else
                           fexternalip:=copy(fexternalip,1,j-1);
                     end;
                  successevent;
               end
            else
               failevent;
         end;
      5: begin
            tmpstr:=response;
            j:=Pos(AnsiString(#13#10),tmpstr);
            tmpstr:=uppercase(copy(tmpstr,1,j-1));
            if Pos(AnsiString('200 OK'),tmpstr)>0 then
               successevent
            else
               failevent;
         end;
   end;
   sock.Close;      
end;

procedure TBurroNAT.ClientSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
   errorcode:=0;
   if Pos(AnsiString(#13#10#13#10),response)>0 then
      ResolveResult
   else
      begin
         if taskexecuting then
            failevent;
      end;
end;

procedure TBurroNAT.ClientSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
begin
   if Pos(AnsiString(#13#10#13#10),response)>0 then
      ResolveResult
   else
      begin
         if taskexecuting then
            failevent;
      end;
end;

function TBurroNAT.SearchRouter(): boolean;
var
   tmpstr: Ansistring;
   buffer: array [0..4096] of Ansichar;
   j: integer;
begin
   if tasktimer.enabled then
      begin
         result:=false;
         exit;
      end;
   tmpstr:='M-SEARCH * HTTP/1.1'#13#10
          +'HOST: 239.255.255.250:1900'#13#10
          +'MAN: "ssdp:discover"'#13#10
          +'MX: 3'#13#10
          +'ST: upnp:rootdevice'#13#10#13#10;
   j:=length(tmpstr);
   strplcopy(buffer,tmpstr,j);
   ftasktype:=1;                   
   taskexecuting:=true;            
   taskfinished:=false;            
   tasksuccess:=false;             
   requested:=false;               
   tasktimer.interval:=ftimeout;   
   tasktimer.enabled:=true;
   udp.SendBuffer(buffer,j);       
   result:=true;
end;

function TBurroNAT.GetControlURL(): boolean;
begin
   if tasktimer.enabled then
      begin
         result:=false;
         exit;
      end;
   request:=AnsiString('GET ')+frouterurl+AnsiString(' HTTP/1.1'#13#10
           +'Host: ')+frouterip+AnsiString(':'+inttostr(frouterport)+#13#10#13#10);
   ftasktype:=2;                   
   taskexecuting:=true;            
   taskfinished:=false;            
   tasksuccess:=false;             
   requested:=false;               
   sock.host:=frouterip;           
   sock.port:=frouterport;         
   sock.active:=true;              
   tasktimer.interval:=ftimeout;   
   tasktimer.enabled:=true;
   result:=true;
end;

function TBurroNAT.AddNatMapping(NatPortName: Ansistring; ExternalPort: integer; LocalIp: Ansistring; LocalPort: integer; Protocol: Ansistring): boolean;
var
   url,body: Ansistring;
   j: integer;
begin
   if tasktimer.enabled then
      begin
         result:=false;
         exit;
      end;
   body:=AnsiString('<?xml version="1.0"?>'#13#10
        +'<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/"'#13#10
        +'s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">'#13#10
        +'<s:Body>'#13#10
        +'<u:AddPortMapping xmlns:u="')+fservicetype+AnsiString('">'#13#10
        +'<NewRemoteHost></NewRemoteHost>'#13#10
        +'<NewExternalPort>'+inttostr(externalport)+'</NewExternalPort>'#13#10
        +'<NewProtocol>')+protocol+AnsiString('</NewProtocol>'#13#10
        +'<NewInternalPort>'+inttostr(localport)+'</NewInternalPort>'#13#10
        +'<NewInternalClient>')+localip+AnsiString('</NewInternalClient>'#13#10
        +'<NewEnabled>1</NewEnabled>'#13#10
        +'<NewPortMappingDescription> </NewPortMappingDescription>'#13#10
        +'<NewLeaseDuration>0</NewLeaseDuration>'#13#10
        +'</u:AddPortMapping>'#13#10
        +'</s:Body>'#13#10
        +'</s:Envelope>'#13#10);
   url:=fcontrolurl;
   delete(url,1,7);
   j:=Pos(AnsiString('/'),url);
   delete(url,1,j-1);
   request:=AnsiString('POST '+url+' HTTP/1.1'#13#10
           +'Host: ')+frouterip+AnsiString(':'+inttostr(routerport)+#13#10
           +'SoapAction: "')+fservicetype+AnsiString('#AddPortMapping"'#13#10
           +'Content-Type: text/xml; charset="utf-8"'#13#10
           +'Content-Length: '+inttostr(length(body))+#13#10#13#10)+body;
   ftasktype:=3;                   
   taskexecuting:=true;            
   taskfinished:=false;            
   tasksuccess:=false;             
   requested:=false;               
   tasktimer.interval:=ftimeout;   
   tasktimer.enabled:=true;
   sock.host:=frouterip;           
   sock.port:=frouterport;         
   sock.active:=true;
   result:=true;
end;

function TBurroNAT.GetExternalIp(): boolean;
var
   url,body: Ansistring;
   j: integer;
begin
   if tasktimer.enabled then
      begin
         result:=false;
         exit;
      end;
   body:='<?xml version="1.0"?>'#13#10
        +'<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">'#13#10
        +'<s:Body>'#13#10
        +'<u:GetExternalIPAddress xmlns:u="'+fservicetype+'">'#13#10
        +'</u:GetExternalIPAddress>'#13#10
        +'</s:Body>'#13#10
        +'</s:Envelope>'#13#10;
   url:=fcontrolurl;
   delete(url,1,7);
   j:=Pos(AnsiString('/'),url);
   delete(url,1,j-1);
   request:=AnsiString('POST '+url+' HTTP/1.0'#13#10
           +'Host: ')+frouterip+AnsiString(':'+inttostr(frouterport)+#13#10
           +'SoapAction: "')+fservicetype+AnsiString('#GetExternalIPAddress"'#13#10
           +'Content-Type: text/xml; charset="utf-8"'#13#10
           +'Content-Length: '+inttostr(length(body))+#13#10#13#10)+body;
   ftasktype:=4;                   
   taskexecuting:=true;            
   taskfinished:=false;            
   tasksuccess:=false;             
   requested:=false;               
   tasktimer.interval:=ftimeout;   
   tasktimer.enabled:=true;
   sock.host:=frouterip;
   sock.port:=frouterport;
   sock.active:=true;
   result:=true;
end;

function TBurroNAT.DeleteNatMapping(ExternalPort: integer; Protocol: Ansistring): boolean;
var
   url,body: Ansistring;
   j: integer;
begin
   if tasktimer.enabled then
      begin
         result:=false;
         exit;
      end;
   body:=AnsiString('<?xml version="1.0"?>'#13#10
        +'<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/" s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">'#13#10
        +'<s:Body>'#13#10
        +'<u:DeletePortMapping xmlns:u="')+fservicetype+AnsiString('">'#13#10
        +'<NewRemoteHost></NewRemoteHost>'#13#10
        +'<NewExternalPort>'+inttostr(externalport)+'</NewExternalPort>'#13#10
        +'<NewProtocol>')+protocol+AnsiString('</NewProtocol>'#13#10
        +'</u:DeletePortMapping>'#13#10
        +'</s:Body>'#13#10
        +'</s:Envelope>'#13#10);
   url:=fcontrolurl;
   delete(url,1,7);
   j:=Pos(AnsiString('/'),url);
   delete(url,1,j-1);
   request:=AnsiString('POST '+url+' HTTP/1.0'#13#10
           +'Host: ')+routerip+AnsiString(':'+inttostr(routerport)+#13#10
           +'SoapAction: "')+fservicetype+AnsiString('#DeletePortMapping"'#13#10
           +'Content-Type: text/xml; charset="utf-8"'#13#10
           +'Content-Length: '+inttostr(length(body))+#13#10#13#10)+body;
   ftasktype:=5;                   
   taskexecuting:=true;            
   taskfinished:=false;            
   tasksuccess:=false;             
   requested:=false;               
   tasktimer.interval:=ftimeout;    
   tasktimer.enabled:=true;
   sock.host:=frouterip;
   sock.port:=frouterport;
   sock.active:=true;
   result:=true;
end;

end.
