//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit BurroSocket;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
   SysUtils,Windows, Messages, Classes,
{$IFDEF UNICODE}
   AnsiStrings,
{$ENDIF}
  WinSock, SyncObjs;

const
  CM_SOCKETMESSAGE = WM_USER + $0001;
  CM_DEFERFREE = WM_USER + $0002;
  CM_LOOKUPCOMPLETE = WM_USER + $0003;

type
  ESocketError = class(Exception);

  TCMSocketMessage = record
    Msg: Cardinal;
    Socket: TSocket;
    SelectEvent: Word;
    SelectError: Word;
    Result: Longint;
  end;

  TCMLookupComplete = record
    Msg: Cardinal;
    LookupHandle: THandle;
    AsyncBufLen: Word;
    AsyncError: Word;
    Result: Longint;
  end;

  TBCustomWinSocket = class;
  TBCustomSocket = class;
  TBServerAcceptThread = class;
  TBServerClientThread = class;
  TBServerWinSocket = class;
  TBServerClientWinSocket = class;

  TServerType = (stNonBlocking, stThreadBlocking);
  TClientType = (ctNonBlocking, ctBlocking);
  TAsyncStyle = (asRead, asWrite, asOOB, asAccept, asConnect, asClose);
  TAsyncStyles = set of TAsyncStyle;
  TSocketEvent = (seLookup, seConnecting, seConnect, seDisconnect, seListen, seAccept, seWrite, seRead);
  TLookupState = (lsIdle, lsLookupAddress, lsLookupService);
  TErrorEvent = (eeGeneral, eeSend, eeReceive, eeConnect, eeDisconnect, eeAccept, eeLookup);

  TSocketEventEvent = procedure (Sender: TObject; Socket: TBCustomWinSocket; SocketEvent: TSocketEvent) of object;
  TSocketErrorEvent = procedure (Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer) of object;
  TGetSocketEvent = procedure (Sender: TObject; Socket: TSocket; var ClientSocket: TBServerClientWinSocket) of object;
  TGetThreadEvent = procedure (Sender: TObject; ClientSocket: TBServerClientWinSocket; var SocketThread: TBServerClientThread) of object;
  TSocketNotifyEvent = procedure (Sender: TObject; Socket: TBCustomWinSocket) of object;

  TBCustomWinSocket = class
  private
    FSocket: TSocket;
    FConnected: Boolean;
    FSendStream: TStream;
    FDropAfterSend: Boolean;
    FHandle: HWnd;
    FAddr: TSockAddrIn;
    FAsyncStyles: TASyncStyles;
    FLookupState: TLookupState;
    FLookupHandle: THandle;
    FOnSocketEvent: TSocketEventEvent;
    FOnErrorEvent: TSocketErrorEvent;
    FSocketLock: TCriticalSection;
    FGetHostData: Pointer;
    FData: Pointer;
    FService: Ansistring;
    FPort: Word;
    FClient: Boolean;
    FQueueSize: Integer;
    function SendStreamPiece: Boolean;
    procedure WndProc(var Message: TMessage);
    procedure CMLookupComplete(var Message: TCMLookupComplete); message CM_LOOKUPCOMPLETE;
    procedure CMSocketMessage(var Message: TCMSocketMessage); message CM_SOCKETMESSAGE;
    procedure CMDeferFree(var Message); message CM_DEFERFREE;
    procedure DeferFree;
    procedure DoSetAsyncStyles;
    function GetHandle: HWnd;
    function GetLocalHost: Ansistring;
    function GetLocalAddress: Ansistring;
    function GetLocalPort: Integer;
    function GetRemoteHost: Ansistring;
    function GetRemoteAddress: Ansistring;
    function GetRemotePort: Integer;
    function GetRemoteAddr: TSockAddrIn;

  protected
    procedure AsyncInitSocket(const Name, Address, Service: AnsiString; Port: Word; QueueSize: Integer; Client: Boolean);
    procedure DoOpen;
    procedure DoListen(QueueSize: Integer);
    function InitSocket(const Name, Address, Service: AnsiString; Port: Word; Client: Boolean): TSockAddrIn;
    procedure Event(Socket: TBCustomWinSocket; SocketEvent: TSocketEvent); dynamic;
    procedure Error(Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer); dynamic;
    procedure SetAsyncStyles(Value: TASyncStyles);

  public
    constructor Create(ASocket: TSocket);
    destructor Destroy; override;
    procedure Close;
    procedure DefaultHandler(var Message); override;
    procedure Lock;
    procedure Unlock;
    procedure Listen(const Name, Address, Service: AnsiString; Port: Word; QueueSize: Integer; Block: Boolean = True);
    procedure Open(const Name, Address, Service: AnsiString; Port: Word; Block: Boolean = True);
    procedure Accept(Socket: TSocket); virtual;
    procedure Connect(Socket: TSocket); virtual;
    procedure Disconnect(Socket: TSocket); virtual;

    procedure Read(Socket: TSocket); virtual;
    procedure Write(Socket: TSocket); virtual;

    function LookupName(const name: AnsiString): TInAddr;
    function LookupService(const service: Ansistring): Integer;

    function ReceiveLength: Integer;
    function ReceiveBuf(var Buf; Count: Integer): Integer;
    function ReceiveText: Ansistring;

    function SendBuf(var Buf; Count: Integer): Integer;
    function SendStream(AStream: TStream): Boolean;
    function SendStreamThenDrop(AStream: TStream): Boolean;
    function SendText(const S: Ansistring): Integer;

    property LocalHost: Ansistring read GetLocalHost;
    property LocalAddress: Ansistring read GetLocalAddress;
    property LocalPort: Integer read GetLocalPort;

    property RemoteHost: Ansistring read GetRemoteHost;
    property RemoteAddress: Ansistring read GetRemoteAddress;
    property RemotePort: Integer read GetRemotePort;
    property RemoteAddr: TSockAddrIn read GetRemoteAddr;

    property Connected: Boolean read FConnected;
    property Addr: TSockAddrIn read FAddr;
    property ASyncStyles: TAsyncStyles read FAsyncStyles write SetAsyncStyles;
    property Handle: HWnd read GetHandle;
    property SocketHandle: TSocket read FSocket;
    property LookupState: TLookupState read FLookupState;

    property OnSocketEvent: TSocketEventEvent read FOnSocketEvent write FOnSocketEvent;
    property OnErrorEvent: TSocketErrorEvent read FOnErrorEvent write FOnErrorEvent;

    property Data: Pointer read FData write FData;

  end;

  TBClientWinSocket = class(TBCustomWinSocket)
  private
    FClientType: TClientType;
  protected
    procedure SetClientType(Value: TClientType);
  public
    procedure Connect(Socket: TSocket); override;
    property ClientType: TClientType read FClientType write SetClientType;
  end;

  TBServerClientWinSocket = class(TBCustomWinSocket)
  private
    FServerWinSocket: TBServerWinSocket;
  public
    constructor Create(Socket: TSocket; ServerWinSocket: TBServerWinSocket);
    destructor Destroy; override;

    property ServerWinSocket: TBServerWinSocket read FServerWinSocket;
  end;

  TThreadNotifyEvent = procedure (Sender: TObject; Thread: TBServerClientThread) of object;

  TBServerWinSocket = class(TBCustomWinSocket)
  private
    FServerType: TServerType;
    FThreadCacheSize: Integer;
    FConnections: TList;
    FActiveThreads: TList;
    FListLock: TCriticalSection;
    FServerAcceptThread: TBServerAcceptThread;
    FOnGetSocket: TGetSocketEvent;
    FOnGetThread: TGetThreadEvent;
    FOnThreadStart: TThreadNotifyEvent;
    FOnThreadEnd: TThreadNotifyEvent;
    FOnClientConnect: TSocketNotifyEvent;
    FOnClientDisconnect: TSocketNotifyEvent;
    FOnClientRead: TSocketNotifyEvent;
    FOnClientWrite: TSocketNotifyEvent;
    FOnClientError: TSocketErrorEvent;

    procedure AddClient(AClient: TBServerClientWinSocket);
    procedure RemoveClient(AClient: TBServerClientWinSocket);
    procedure AddThread(AThread: TBServerClientThread);
    procedure RemoveThread(AThread: TBServerClientThread);
    procedure ClientEvent(Sender: TObject; Socket: TBCustomWinSocket; SocketEvent: TSocketEvent);
    procedure ClientError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    function GetActiveConnections: Integer;
    function GetActiveThreads: Integer;
    function GetConnections(Index: Integer): TBCustomWinSocket;
    function GetIdleThreads: Integer;

  protected
    function DoCreateThread(ClientSocket: TBServerClientWinSocket): TBServerClientThread; virtual;
    procedure Listen(var Name, Address, Service: Ansistring; Port: Word; QueueSize: Integer);
    procedure SetServerType(Value: TServerType);
    procedure SetThreadCacheSize(Value: Integer);
    procedure ThreadEnd(AThread: TBServerClientThread); dynamic;
    procedure ThreadStart(AThread: TBServerClientThread); dynamic;
    function GeTBClientSocket(Socket: TSocket): TBServerClientWinSocket; dynamic;
    function GetServerThread(ClientSocket: TBServerClientWinSocket): TBServerClientThread; dynamic;
    procedure ClientRead(Socket: TBCustomWinSocket); dynamic;
    procedure ClientWrite(Socket: TBCustomWinSocket); dynamic;
    procedure ClientConnect(Socket: TBCustomWinSocket); dynamic;
    procedure ClientDisconnect(Socket: TBCustomWinSocket); dynamic;
    procedure ClientErrorEvent(Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer); dynamic;

  public
    constructor Create(ASocket: TSocket);
    destructor Destroy; override;
    procedure Accept(Socket: TSocket); override;
    procedure Disconnect(Socket: TSocket); override;
    function GetClientThread(ClientSocket: TBServerClientWinSocket): TBServerClientThread;
    property ActiveConnections: Integer read GetActiveConnections;
    property ActiveThreads: Integer read GetActiveThreads;
    property Connections[Index: Integer]: TBCustomWinSocket read GetConnections;
    property IdleThreads: Integer read GetIdleThreads;
    property ServerType: TServerType read FServerType write SetServerType;
    property ThreadCacheSize: Integer read FThreadCacheSize write SetThreadCacheSize;
    property OnGetSocket: TGetSocketEvent read FOnGetSocket write FOnGetSocket;
    property OnGetThread: TGetThreadEvent read FOnGetThread write FOnGetThread;
    property OnThreadStart: TThreadNotifyEvent read FOnThreadStart write FOnThreadStart;
    property OnThreadEnd: TThreadNotifyEvent read FOnThreadEnd write FOnThreadEnd;
    property OnClientConnect: TSocketNotifyEvent read FOnClientConnect write FOnClientConnect;
    property OnClientDisconnect: TSocketNotifyEvent read FOnClientDisconnect write FOnClientDisconnect;
    property OnClientRead: TSocketNotifyEvent read FOnClientRead write FOnClientRead;
    property OnClientWrite: TSocketNotifyEvent read FOnClientWrite write FOnClientWrite;
    property OnClientError: TSocketErrorEvent read FOnClientError write FOnClientError;

  end;

  TBServerAcceptThread = class(TThread)
  private
    FServerSocket: TBServerWinSocket;
  public
    constructor Create(CreateSuspended: Boolean; ASocket: TBServerWinSocket);
    procedure Execute; override;

    property ServerSocket: TBServerWinSocket read FServerSocket;
  end;

  TBServerClientThread = class(TThread)
  private
    FClientSocket: TBServerClientWinSocket;
    FServerSocket: TBServerWinSocket;
    FException: Exception;
    FEvent: TSimpleEvent;
    FKeepInCache: Boolean;
    FData: Pointer;
    procedure HandleEvent(Sender: TObject; Socket: TBCustomWinSocket;
      SocketEvent: TSocketEvent);
    procedure HandleError(Sender: TObject; Socket: TBCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure DoHandleException;
    procedure DoRead;
    procedure DoWrite;

  protected
    procedure DoTerminate; override;
    procedure Execute; override;
    procedure ClientExecute; virtual;
    procedure Event(SocketEvent: TSocketEvent); virtual;
    procedure Error(ErrorEvent: TErrorEvent; var ErrorCode: Integer); virtual;
    procedure HandleException; virtual;
    procedure ReActivate(ASocket: TBServerClientWinSocket);
    function StartConnect: Boolean;
    function EndConnect: Boolean;

  public
    constructor Create(CreateSuspended: Boolean; ASocket: TBServerClientWinSocket);
    destructor Destroy; override;

    property ClientSocket: TBServerClientWinSocket read FClientSocket;
    property ServerSocket: TBServerWinSocket read FServerSocket;
    property KeepInCache: Boolean read FKeepInCache write FKeepInCache;
    property Data: Pointer read FData write FData;

  end;

  TBAbstractSocket = class(TComponent)
  private
    FActive: Boolean;
    FPort: Integer;
    FAddress: Ansistring;
    FHost: Ansistring;
    FService: Ansistring;
    procedure DoEvent(Sender: TObject; Socket: TBCustomWinSocket; SocketEvent: TSocketEvent);
    procedure DoError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);

  protected
    procedure Event(Socket: TBCustomWinSocket; SocketEvent: TSocketEvent); virtual; abstract;
    procedure Error(Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer); virtual; abstract;
    procedure DoActivate(Value: Boolean); virtual; abstract;
    procedure InitSocket(Socket: TBCustomWinSocket);
    procedure Loaded; override;
    procedure SetActive(Value: Boolean);
    procedure SetAddress(Value: Ansistring);
    procedure SetHost(Value: Ansistring);
    procedure SetPort(Value: Integer);
    procedure SetService(Value: Ansistring);
    property Active: Boolean read FActive write SetActive;
    property Address: Ansistring read FAddress write SetAddress;
    property Host: Ansistring read FHost write SetHost;
    property Port: Integer read FPort write SetPort;
    property Service: Ansistring read FService write SetService;

  public
    procedure Open;
    procedure Close;

  end;

  TBCustomSocket = class(TBAbstractSocket)
  private
    FOnLookup: TSocketNotifyEvent;
    FOnConnect: TSocketNotifyEvent;
    FOnConnecting: TSocketNotifyEvent;
    FOnDisconnect: TSocketNotifyEvent;
    FOnListen: TSocketNotifyEvent;
    FOnAccept: TSocketNotifyEvent;
    FOnRead: TSocketNotifyEvent;
    FOnWrite: TSocketNotifyEvent;
    FOnError: TSocketErrorEvent;

  protected
    procedure Event(Socket: TBCustomWinSocket; SocketEvent: TSocketEvent); override;
    procedure Error(Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer); override;
    property OnLookup: TSocketNotifyEvent read FOnLookup write FOnLookup;
    property OnConnecting: TSocketNotifyEvent read FOnConnecting write FOnConnecting;
    property OnConnect: TSocketNotifyEvent read FOnConnect write FOnConnect;
    property OnDisconnect: TSocketNotifyEvent read FOnDisconnect write FOnDisconnect;
    property OnListen: TSocketNotifyEvent read FOnListen write FOnListen;
    property OnAccept: TSocketNotifyEvent read FOnAccept write FOnAccept;
    property OnRead: TSocketNotifyEvent read FOnRead write FOnRead;
    property OnWrite: TSocketNotifyEvent read FOnWrite write FOnWrite;
    property OnError: TSocketErrorEvent read FOnError write FOnError;

  end;

  TBWinSocketStream = class(TStream)
  private
    FSocket: TBCustomWinSocket;
    FTimeout: Longint;
    FEvent: TSimpleEvent;

  public
    constructor Create(ASocket: TBCustomWinSocket; TimeOut: Longint);
    destructor Destroy; override;
    function WaitForData(Timeout: Longint): Boolean;
    function Read(var Buffer; Count: Longint): Longint; override;
    function Write(const Buffer; Count: Longint): Longint; override;
    function Seek(Offset: Longint; Origin: Word): Longint; override;
    property TimeOut: Longint read FTimeout write FTimeout;

  end;

  TBClientSocket = class(TBCustomSocket)
  private
    FClientSocket: TBClientWinSocket;
  protected
    procedure DoActivate(Value: Boolean); override;
    function GetClientType: TClientType;
    procedure SetClientType(Value: TClientType);

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Socket: TBClientWinSocket read FClientSocket;

  published
    property Active;
    property Address;
    property ClientType: TClientType read GetClientType write SetClientType;
    property Host;
    property Port;
    property Service;
    property OnLookup;
    property OnConnecting;
    property OnConnect;
    property OnDisconnect;
    property OnRead;
    property OnWrite;
    property OnError;

  end;

  TBCustomServerSocket = class(TBCustomSocket)
  protected
    FServerSocket: TBServerWinSocket;
    procedure DoActivate(Value: Boolean); override;
    function GetServerType: TServerType;
    function GetGetThreadEvent: TGetThreadEvent;
    function GetGetSocketEvent: TGetSocketEvent;
    function GetThreadCacheSize: Integer;
    function GetOnThreadStart: TThreadNotifyEvent;
    function GetOnThreadEnd: TThreadNotifyEvent;
    function GetOnClientEvent(Index: Integer): TSocketNotifyEvent;
    function GetOnClientError: TSocketErrorEvent;
    procedure SetServerType(Value: TServerType);
    procedure SetGetThreadEvent(Value: TGetThreadEvent);
    procedure SetGetSocketEvent(Value: TGetSocketEvent);
    procedure SetThreadCacheSize(Value: Integer);
    procedure SetOnThreadStart(Value: TThreadNotifyEvent);
    procedure SetOnThreadEnd(Value: TThreadNotifyEvent);
    procedure SetOnClientEvent(Index: Integer; Value: TSocketNotifyEvent);
    procedure SetOnClientError(Value: TSocketErrorEvent);
    property ServerType: TServerType read GetServerType write SetServerType;
    property ThreadCacheSize: Integer read GetThreadCacheSize write SetThreadCacheSize;
    property OnGetThread: TGetThreadEvent read GetGetThreadEvent write SetGetThreadEvent;
    property OnGetSocket: TGetSocketEvent read GetGetSocketEvent write SetGetSocketEvent;
    property OnThreadStart: TThreadNotifyEvent read GetOnThreadStart write SetOnThreadStart;
    property OnThreadEnd: TThreadNotifyEvent read GetOnThreadEnd write SetOnThreadEnd;
    property OnClientConnect: TSocketNotifyEvent index 2 read GetOnClientEvent write SetOnClientEvent;
    property OnClientDisconnect: TSocketNotifyEvent index 3 read GetOnClientEvent write SetOnClientEvent;
    property OnClientRead: TSocketNotifyEvent index 0 read GetOnClientEvent write SetOnClientEvent;
    property OnClientWrite: TSocketNotifyEvent index 1 read GetOnClientEvent write SetOnClientEvent;
    property OnClientError: TSocketErrorEvent read GetOnClientError write SetOnClientError;

  public
    destructor Destroy; override;

  end;

  TBServerSocket = class(TBCustomServerSocket)
  public
    constructor Create(AOwner: TComponent); override;
    property Socket: TBServerWinSocket read FServerSocket;

  published
    property Active;
    property Port;
    property Service;
    property ServerType;
    property ThreadCacheSize default 10;
    property OnListen;
    property OnAccept;
    property OnGetThread;
    property OnGetSocket;
    property OnThreadStart;
    property OnThreadEnd;
    property OnClientConnect;
    property OnClientDisconnect;
    property OnClientRead;
    property OnClientWrite;
    property OnClientError;

  end;

  TSocketErrorProc = procedure (ErrorCode: Integer);

function SetErrorProc(ErrorProc: TSocketErrorProc): TSocketErrorProc;

procedure Register;

implementation

uses forms, BSConsts;

threadvar
  SocketErrorProc: TSocketErrorProc;

var
  WSAData: TWSAData;

procedure Register;
begin
  RegisterComponents('QuickBurro Basic', [TBClientSocket,TBServerSocket]);
end;

function SetErrorProc(ErrorProc: TSocketErrorProc): TSocketErrorProc;
begin
  Result := SocketErrorProc;
  SocketErrorProc := ErrorProc;
end;

function CheckSocketResult(ResultCode: Integer; const Op: Ansistring): Integer;
begin
  if ResultCode <> 0 then
     begin
       Result := WSAGetLastError;
       if Result <> WSAEWOULDBLOCK then
         if Assigned(SocketErrorProc) then
            SocketErrorProc(Result)
         else
            raise ESocketError.CreateResFmt(@sWindowsSocketError, [SysErrorMessage(Result), Result, string(Op)]);
     end
  else
     Result := 0;
end;

procedure Startup;
var
  ErrorCode: Integer;
begin
   {$IFDEF VER130}
   ErrorCode := WSAStartup($0101, WSAData);
   {$ELSE}
   ErrorCode := WSAStartup(MAKEWORD(2,0), WSAData);
   {$ENDIF}
  if ErrorCode <> 0 then
    raise ESocketError.CreateResFmt(@sWindowsSocketError,
      [SysErrorMessage(ErrorCode), ErrorCode, 'WSAStartup']);
end;

procedure Cleanup;
var
  ErrorCode: Integer;
begin
  ErrorCode := WSACleanup;
  if ErrorCode <> 0 then
    raise ESocketError.CreateResFmt(@sWindowsSocketError,
      [SysErrorMessage(ErrorCode), ErrorCode, 'WSACleanup']);
end;


constructor TBCustomWinSocket.Create(ASocket: TSocket);
begin
  inherited Create;
  Startup;
  FSocketLock := TCriticalSection.Create;
  FASyncStyles := [asRead, asWrite, asConnect, asClose];
  FSocket := ASocket;
  FAddr.sin_family := PF_INET;
  FAddr.sin_addr.s_addr := INADDR_ANY;
  FAddr.sin_port := 0;
  FConnected := FSocket <> INVALID_SOCKET;
end;

destructor TBCustomWinSocket.Destroy;
begin
  FOnSocketEvent := nil;  
  if FConnected and (FSocket <> INVALID_SOCKET) then
    Disconnect(FSocket);
  if FHandle <> 0 then
     DeallocateHWnd(FHandle);
  FSocketLock.Free;
  Cleanup;
  FreeMem(FGetHostData);
  FGetHostData := nil;
  inherited Destroy;
end;

procedure TBCustomWinSocket.Accept(Socket: TSocket);
begin
end;

procedure TBCustomWinSocket.AsyncInitSocket(const Name, Address, Service: Ansistring; Port: Word; QueueSize: Integer; Client: Boolean);
var
  ErrorCode: Integer;
begin
  try
    case FLookupState of
      lsIdle:
        begin
          if not Client then
             begin
               FLookupState := lsLookupAddress;
               FAddr.sin_addr.S_addr := INADDR_ANY;
             end
          else
             if Name <> '' then
                begin
                  if FGetHostData = nil then
                    FGetHostData := AllocMem(MAXGETHOSTSTRUCT);
                  FLookupHandle := WSAAsyncGetHostByName(Handle, CM_LOOKUPCOMPLETE, PAnsiChar(Name), FGetHostData, MAXGETHOSTSTRUCT);
                  CheckSocketResult(Ord(FLookupHandle = 0), 'WSAASyncGetHostByName');
                  FService := Service;
                  FPort := Port;
                  FQueueSize := QueueSize;
                  FClient := Client;
                  FLookupState := lsLookupAddress;
                  Exit;
                end
             else
                if Address <> '' then
                   begin
                     FLookupState := lsLookupAddress;
                     FAddr.sin_addr.S_addr := inet_addr(PAnsiChar(Address));
                   end
                else
                   begin
                     ErrorCode := 1110;
                     Error(Self, eeLookup, ErrorCode);
                     Disconnect(FSocket);
                     if ErrorCode <> 0 then
                       raise ESocketError.CreateRes(@sNoAddress);
                     Exit;
                   end;
        end;
      lsLookupAddress:
        begin
          if Service <> '' then
             begin
               if FGetHostData = nil then
                 FGetHostData := AllocMem(MAXGETHOSTSTRUCT);
               FLookupHandle := WSAASyncGetServByName(Handle, CM_LOOKUPCOMPLETE, PAnsiChar(Service), 'tcp' , FGetHostData, MAXGETHOSTSTRUCT);
               CheckSocketResult(Ord(FLookupHandle = 0), 'WSAASyncGetServByName');
               FLookupState := lsLookupService;
               Exit;
             end
          else
             begin
               FLookupState := lsLookupService;
               FAddr.sin_port := htons(Port);
             end;
        end;
      lsLookupService:
        begin
          FLookupState := lsIdle;
          if Client then
            DoOpen
          else
            DoListen(QueueSize);
        end;
    end;
    if FLookupState <> lsIdle then
      ASyncInitSocket(Name, Address, Service, Port, QueueSize, Client);
  except
    Disconnect(FSocket);
    raise;
  end;
end;

procedure TBCustomWinSocket.Close;
begin
  Disconnect(FSocket);
end;

procedure TBCustomWinSocket.Connect(Socket: TSocket);
begin
end;

procedure TBCustomWinSocket.Lock;
begin
  FSocketLock.Enter;
end;

procedure TBCustomWinSocket.Unlock;
begin
  FSocketLock.Leave;
end;

procedure TBCustomWinSocket.CMSocketMessage(var Message: TCMSocketMessage);

  function CheckError: Boolean;
  var
    ErrorEvent: TErrorEvent;
    ErrorCode: Integer;
  begin
    if Message.SelectError <> 0 then
       begin
         Result := False;
         ErrorCode := Message.SelectError;
         case Message.SelectEvent of
           FD_CONNECT: ErrorEvent := eeConnect;
           FD_CLOSE: ErrorEvent := eeDisconnect;
           FD_READ: ErrorEvent := eeReceive;
           FD_WRITE: ErrorEvent := eeSend;
           FD_ACCEPT: ErrorEvent := eeAccept;
         else
           ErrorEvent := eeGeneral;
         end;
         Error(Self, ErrorEvent, ErrorCode);
         if ErrorCode <> 0 then
           raise ESocketError.CreateResFmt(@sASyncSocketError, [ErrorCode]);
       end
    else
       Result := True;
  end;

begin
  with Message do
    if CheckError then
      case SelectEvent of
        FD_CONNECT: Connect(Socket);
        FD_CLOSE: Disconnect(Socket);
        FD_READ: Read(Socket);
        FD_WRITE: Write(Socket);
        FD_ACCEPT: Accept(Socket);
      end;
end;

procedure TBCustomWinSocket.CMDeferFree(var Message);
begin
  Free;
end;

procedure TBCustomWinSocket.DeferFree;
begin
  if FHandle <> 0 then
     PostMessage(FHandle, CM_DEFERFREE, 0, 0);
end;

procedure TBCustomWinSocket.DoSetAsyncStyles;
var
  Msg: Integer;
  Wnd: HWnd;
  Blocking: Longint;
begin
  Msg := 0;
  Wnd := 0;
  if FAsyncStyles <> [] then
  begin
    Msg := CM_SOCKETMESSAGE;
    Wnd := Handle;
  end;
  WSAAsyncSelect(FSocket, Wnd, Msg, Longint(Byte(FAsyncStyles)));

  if FASyncStyles = [] then
     begin
        Blocking := 0;
        ioctlsocket(FSocket, FIONBIO, Blocking);
     end;
end;

procedure TBCustomWinSocket.DoListen(QueueSize: Integer);
begin
  CheckSocketResult(bind(FSocket, FAddr, SizeOf(FAddr)), 'bind');
  DoSetASyncStyles;
  if QueueSize > SOMAXCONN then
     QueueSize := SOMAXCONN;
  Event(Self, seListen);
  CheckSocketResult(Winsock.listen(FSocket, QueueSize), 'listen');
  FLookupState := lsIdle;
  FConnected := True;
end;

procedure TBCustomWinSocket.DoOpen;
begin
  DoSetASyncStyles;
  Event(Self, seConnecting);
  CheckSocketResult(WinSock.connect(FSocket, FAddr, SizeOf(FAddr)), 'connect');
  FLookupState := lsIdle;
  if not (asConnect in FAsyncStyles) then
     begin
       FConnected := FSocket <> INVALID_SOCKET;
       Event(Self, seConnect);
     end;
end;

function TBCustomWinSocket.GetHandle: HWnd;
begin
  if FHandle = 0 then
    FHandle := AllocateHwnd(WndProc);
  Result := FHandle;
end;

function TBCustomWinSocket.GetLocalAddress: Ansistring;
var
  SockAddrIn: TSockAddrIn;
  Size: Integer;
begin
  Lock;
  try
    Result := '';
    if FSocket = INVALID_SOCKET then
       Exit;
    Size := SizeOf(SockAddrIn);
    if getsockname(FSocket, SockAddrIn, Size) = 0 then
      Result := inet_ntoa(SockAddrIn.sin_addr);
  finally
    Unlock;
  end;
end;

function TBCustomWinSocket.GetLocalHost: Ansistring;
var
  LocalName: array[0..255] of AnsiChar;
begin
  Lock;
  try
    Result := '';
    if FSocket = INVALID_SOCKET then
       Exit;
    if gethostname(LocalName, SizeOf(LocalName)) = 0 then
      Result := LocalName;
  finally
    Unlock;
  end;
end;

function TBCustomWinSocket.GetLocalPort: Integer;
var
  SockAddrIn: TSockAddrIn;
  Size: Integer;
begin
  Lock;
  try
    Result := -1;
    if FSocket = INVALID_SOCKET then
       Exit;
    Size := SizeOf(SockAddrIn);
    if getsockname(FSocket, SockAddrIn, Size) = 0 then
      Result := ntohs(SockAddrIn.sin_port);
  finally
    Unlock;
  end;
end;

function TBCustomWinSocket.GetRemoteHost: Ansistring;
var
  SockAddrIn: TSockAddrIn;
  Size: Integer;
  HostEnt: PHostEnt;
begin
  Lock;
  try
    Result := '';
    if not FConnected then
       Exit;
    Size := SizeOf(SockAddrIn);
    CheckSocketResult(getpeername(FSocket, SockAddrIn, Size), 'getpeername');
    HostEnt := gethostbyaddr(@SockAddrIn.sin_addr.s_addr, 4, PF_INET);
    if HostEnt <> nil then
       Result := HostEnt.h_name;
  finally
    Unlock;
  end;
end;

function TBCustomWinSocket.GetRemoteAddress: Ansistring;
var
  SockAddrIn: TSockAddrIn;
  Size: Integer;
begin
  Lock;
  try
    Result := '';
    if not FConnected then
       Exit;
    Size := SizeOf(SockAddrIn);
    CheckSocketResult(getpeername(FSocket, SockAddrIn, Size), 'getpeername');
    Result := inet_ntoa(SockAddrIn.sin_addr);
  finally
    Unlock;
  end;
end;

function TBCustomWinSocket.GetRemotePort: Integer;
var
  SockAddrIn: TSockAddrIn;
  Size: Integer;
begin
  Lock;
  try
    Result := 0;
    if not FConnected then
       Exit;
    Size := SizeOf(SockAddrIn);
    CheckSocketResult(getpeername(FSocket, SockAddrIn, Size), 'getpeername');
    Result := ntohs(SockAddrIn.sin_port);
  finally
    Unlock;
  end;
end;

function TBCustomWinSocket.GetRemoteAddr: TSockAddrIn;
var
  Size: Integer;
begin
  Lock;
  try
    FillChar(Result, SizeOf(Result), 0);
    if not FConnected then
       Exit;
    Size := SizeOf(Result);
    if getpeername(FSocket, Result, Size) <> 0 then
      FillChar(Result, SizeOf(Result), 0);
  finally
    Unlock;
  end;
end;

function TBCustomWinSocket.LookupName(const Name: Ansistring): TInAddr;
var
  HostEnt: PHostEnt;
  InAddr: TInAddr;
begin
  HostEnt := gethostbyname(PAnsiChar(Name));
  FillChar(InAddr, SizeOf(InAddr), 0);
  if HostEnt <> nil then
     begin
       with InAddr, HostEnt^ do
          begin
            S_un_b.s_b1 := h_addr^[0];
            S_un_b.s_b2 := h_addr^[1];
            S_un_b.s_b3 := h_addr^[2];
            S_un_b.s_b4 := h_addr^[3];
          end;
     end;
  Result := InAddr;
end;

function TBCustomWinSocket.LookupService(const Service: Ansistring): Integer;
var
  ServEnt: PServEnt;
begin
  ServEnt := getservbyname(PAnsiChar(Service), 'tcp');
  if ServEnt <> nil then
    Result := ntohs(ServEnt.s_port)
  else
     Result := 0;
end;

function TBCustomWinSocket.InitSocket(const Name, Address, Service: Ansistring; Port: Word;
  Client: Boolean): TSockAddrIn;
begin
  Result.sin_family := PF_INET;
  if Name <> '' then
    Result.sin_addr := LookupName(name)
  else
     if Address <> '' then
        Result.sin_addr.s_addr := inet_addr(PAnsiChar(Address))
     else
        if not Client then
           Result.sin_addr.s_addr := INADDR_ANY
        else
           raise ESocketError.CreateRes(@sNoAddress);
  if Service <> '' then
    Result.sin_port := htons(LookupService(Service))
  else
    Result.sin_port := htons(Port);
end;

procedure TBCustomWinSocket.Listen(const Name, Address, Service: Ansistring; Port: Word;
  QueueSize: Integer; Block: Boolean);
begin
  if FConnected then
     raise ESocketError.CreateRes(@sCannotListenOnOpen);
  FSocket := socket(PF_INET, SOCK_STREAM, IPPROTO_IP);
  if FSocket = INVALID_SOCKET then
     raise ESocketError.CreateRes(@sCannotCreateSocket);
  try
    Event(Self, seLookUp);
    if Block then
       begin
         FAddr := InitSocket(Name, Address, Service, Port, False);
         DoListen(QueueSize);
       end
    else
       AsyncInitSocket(Name, Address, Service, Port, QueueSize, False);
  except
    Disconnect(FSocket);
    raise;
  end;
end;

procedure TBCustomWinSocket.Open(const Name, Address, Service: Ansistring; Port: Word; Block: Boolean);
begin
  if FConnected then
     raise ESocketError.CreateRes(@sSocketAlreadyOpen);
  FSocket := socket(PF_INET, SOCK_STREAM, IPPROTO_IP);
  if FSocket = INVALID_SOCKET then
     raise ESocketError.CreateRes(@sCannotCreateSocket);
  try
    Event(Self, seLookUp);
    if Block then
       begin
         FAddr := InitSocket(Name, Address, Service, Port, True);
         DoOpen;
       end
    else
       AsyncInitSocket(Name, Address, Service, Port, 0, True);
  except
    Disconnect(FSocket);
    raise;
  end;
end;

procedure TBCustomWinSocket.Disconnect(Socket: TSocket);
begin
  Lock;
  try
    if FLookupHandle <> 0 then
      CheckSocketResult(WSACancelASyncRequest(FLookupHandle), 'WSACancelASyncRequest');
    FLookupHandle := 0;
    if (Socket = INVALID_SOCKET) or (Socket <> FSocket) then
       exit;
    Event(Self, seDisconnect);
    CheckSocketResult(closesocket(FSocket), 'closesocket');
    FSocket := INVALID_SOCKET;
    FAddr.sin_family := PF_INET;
    FAddr.sin_addr.s_addr := INADDR_ANY;
    FAddr.sin_port := 0;
    FConnected := False;
    FreeAndNil(FSendStream);
  finally
    Unlock;
  end;
end;

procedure TBCustomWinSocket.DefaultHandler(var Message);
begin
  with TMessage(Message) do
    if FHandle <> 0 then
      Result := CallWindowProc(@DefWindowProc, FHandle, Msg, wParam, lParam);
end;

procedure TBCustomWinSocket.Event(Socket: TBCustomWinSocket; SocketEvent: TSocketEvent);
begin
  if Assigned(FOnSocketEvent) then
     FOnSocketEvent(Self, Socket, SocketEvent);
end;

procedure TBCustomWinSocket.Error(Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
  if Assigned(FOnErrorEvent) then
     FOnErrorEvent(Self, Socket, ErrorEvent, ErrorCode);
end;

function TBCustomWinSocket.SendText(const s: Ansistring): Integer;
begin
  Result := SendBuf(Pointer(S)^, Length(S));
end;

function TBCustomWinSocket.SendStreamPiece: Boolean;
var
  Buffer: array[0..4095] of Byte;
  StartPos: Integer;
  AmountInBuf: Integer;
  AmountSent: Integer;
  ErrorCode: Integer;

  procedure DropStream;
  begin
    if FDropAfterSend then
       Disconnect(FSocket);
    FDropAfterSend := False;
    FSendStream.Free;
    FSendStream := nil;
  end;

begin
  Lock;
  try
    Result := False;
    if FSendStream <> nil then
       begin
          if (FSocket = INVALID_SOCKET) or (not FConnected) then
             exit;
          while True do
             begin
                StartPos := FSendStream.Position;
                AmountInBuf := FSendStream.Read(Buffer, SizeOf(Buffer));
                if AmountInBuf > 0 then
                   begin
                      AmountSent := send(FSocket, Buffer, AmountInBuf, 0);
                      if AmountSent = SOCKET_ERROR then
                         begin
                            ErrorCode := WSAGetLastError;
                            if ErrorCode <> WSAEWOULDBLOCK then
                               begin
                                  Error(Self, eeSend, ErrorCode);
                                  Disconnect(FSocket);
                                  DropStream;
                                  if FAsyncStyles <> [] then
                                     Abort;
                                  Break;
                               end
                            else
                               begin
                                  FSendStream.Position := StartPos;
                                  Break;
                               end;
                         end
                      else
                         begin
                            if AmountInBuf > AmountSent then
                               FSendStream.Position := StartPos + AmountSent
                            else
                               begin
                                  if FSendStream.Position = FSendStream.Size then
                                     begin
                                        DropStream;
                                        Break;
                                     end;
                               end;
                         end;
                   end
                else
                   begin
                      DropStream;
                      Break;
                   end;
             end;
          Result := True;
       end;
  finally
    Unlock;
  end;
end;

function TBCustomWinSocket.SendStream(AStream: TStream): Boolean;
begin
  Result := False;
  if FSendStream = nil then
     begin
       FSendStream := AStream;
       Result := SendStreamPiece;
     end;
end;

function TBCustomWinSocket.SendStreamThenDrop(AStream: TStream): Boolean;
begin
  FDropAfterSend := True;
  Result := SendStream(AStream);
  if not Result then
     FDropAfterSend := False;
end;

function TBCustomWinSocket.SendBuf(var Buf; Count: Integer): Integer;
var
  ErrorCode: Integer;
begin
  Lock;
  try
    Result := 0;
    if not FConnected then
       Exit;
    Result := send(FSocket, Buf, Count, 0);
    if Result = SOCKET_ERROR then
       begin
         ErrorCode := WSAGetLastError;
         if (ErrorCode <> WSAEWOULDBLOCK) then
            begin
              Error(Self, eeSend, ErrorCode);
              Disconnect(FSocket);
              if ErrorCode <> 0 then
                raise ESocketError.CreateResFmt(@sWindowsSocketError, [SysErrorMessage(ErrorCode), ErrorCode, 'send']);
            end;
       end;
  finally
    Unlock;
  end;
end;

procedure TBCustomWinSocket.SetAsyncStyles(Value: TASyncStyles);
begin
  if Value <> FASyncStyles then
     begin
       FASyncStyles := Value;
       if FSocket <> INVALID_SOCKET then
         DoSetAsyncStyles;
     end;
end;

procedure TBCustomWinSocket.Read(Socket: TSocket);
begin
  if (FSocket = INVALID_SOCKET) or (Socket <> FSocket) then
     Exit;
  Event(Self, seRead);
end;

function TBCustomWinSocket.ReceiveBuf(var Buf; Count: Integer): Integer;
var
  ErrorCode: Integer;
begin
  Lock;
  try
    Result := 0;
    if (Count = -1) and FConnected then
      ioctlsocket(FSocket, FIONREAD, Longint(Result))
    else
       begin
         if not FConnected then
            Exit;
         Result := recv(FSocket, Buf, Count, 0);
         if Result = SOCKET_ERROR then
            begin
              ErrorCode := WSAGetLastError;
              if ErrorCode <> WSAEWOULDBLOCK then
                 begin
                   Error(Self, eeReceive, ErrorCode);
                   Disconnect(FSocket);
                   if ErrorCode <> 0 then
                     raise ESocketError.CreateResFmt(@sWindowsSocketError, [SysErrorMessage(ErrorCode), ErrorCode, 'recv']);
                 end;
            end;
       end;
  finally
    Unlock;
  end;
end;

function TBCustomWinSocket.ReceiveLength: Integer;
begin
  Result := ReceiveBuf(Pointer(nil)^, -1);
end;

function TBCustomWinSocket.ReceiveText: Ansistring;
begin
  SetLength(Result, ReceiveBuf(Pointer(nil)^, -1));
  SetLength(Result, ReceiveBuf(Pointer(Result)^, Length(Result)));
end;

procedure TBCustomWinSocket.WndProc(var Message: TMessage);
begin
  try
    Dispatch(Message);
  except
    Application.HandleException(Self);
  end;
end;

procedure TBCustomWinSocket.Write(Socket: TSocket);
begin
  if (FSocket = INVALID_SOCKET) or (Socket <> FSocket) then
     Exit;
  if not SendStreamPiece then
     Event(Self, seWrite);
end;

procedure TBCustomWinSocket.CMLookupComplete(var Message: TCMLookupComplete);
var
  ErrorCode: Integer;
begin
  if Message.LookupHandle = FLookupHandle then
     begin
       FLookupHandle := 0;
       if Message.AsyncError <> 0 then
          begin
            ErrorCode := Message.AsyncError;
            Error(Self, eeLookup, ErrorCode);
            Disconnect(FSocket);
            if ErrorCode <> 0 then
              raise ESocketError.CreateResFmt(@sWindowsSocketError, [SysErrorMessage(Message.AsyncError), Message.ASyncError, 'ASync Lookup']);
            Exit;
          end;
       if FLookupState = lsLookupAddress then
          begin
            FAddr.sin_addr.S_addr := Integer(Pointer(PHostEnt(FGetHostData).h_addr^)^);
            ASyncInitSocket('', '', FService, FPort, FQueueSize, FClient);
          end
       else
          if FLookupState = lsLookupService then
             begin
               FAddr.sin_port := PServEnt(FGetHostData).s_port;
               FPort := 0;
               FService := '';
               ASyncInitSocket('', '', '', 0, FQueueSize, FClient);
             end;
     end;
end;



procedure TBClientWinSocket.Connect(Socket: TSocket);
begin
  FConnected := True;
  Event(Self, seConnect);
end;

procedure TBClientWinSocket.SetClientType(Value: TClientType);
begin
  if Value <> FClientType then
    if not FConnected then
       begin
         FClientType := Value;
         if FClientType = ctBlocking then
           ASyncStyles := []
         else
           ASyncStyles := [asRead, asWrite, asConnect, asClose];
       end
    else
       raise ESocketError.CreateRes(@sCantChangeWhileActive);
end;



constructor TBServerClientWinSocket.Create(Socket: TSocket; ServerWinSocket: TBServerWinSocket);
begin
  FServerWinSocket := ServerWinSocket;
  if Assigned(FServerWinSocket) then
     begin
       FServerWinSocket.AddClient(Self);
       if FServerWinSocket.AsyncStyles <> [] then
          begin
            OnSocketEvent := FServerWinSocket.ClientEvent;
            OnErrorEvent := FServerWinSocket.ClientError;
          end;
     end;
  inherited Create(Socket);
  if FServerWinSocket.ASyncStyles <> [] then
     DoSetAsyncStyles;
  if FConnected then
     Event(Self, seConnect);
end;

destructor TBServerClientWinSocket.Destroy;
begin
  if Assigned(FServerWinSocket) then
    FServerWinSocket.RemoveClient(Self);
  inherited Destroy;
end;



constructor TBServerWinSocket.Create(ASocket: TSocket);
begin
  FConnections := TList.Create;
  FActiveThreads := TList.Create;
  FListLock := TCriticalSection.Create;
  inherited Create(ASocket);
  FAsyncStyles := [asAccept];
end;

destructor TBServerWinSocket.Destroy;
begin
  inherited Destroy;
  FConnections.Free;
  FActiveThreads.Free;
  FListLock.Free;
end;

procedure TBServerWinSocket.AddClient(AClient: TBServerClientWinSocket);
begin
  FListLock.Enter;
  try
    if FConnections.IndexOf(AClient) < 0 then
      FConnections.Add(AClient);
  finally
    FListLock.Leave;
  end;
end;

procedure TBServerWinSocket.RemoveClient(AClient: TBServerClientWinSocket);
begin
  FListLock.Enter;
  try
    FConnections.Remove(AClient);
  finally
    FListLock.Leave;
  end;
end;

procedure TBServerWinSocket.AddThread(AThread: TBServerClientThread);
begin
  FListLock.Enter;
  try
    if FActiveThreads.IndexOf(AThread) < 0 then
    begin
      FActiveThreads.Add(AThread);
      if FActiveThreads.Count <= FThreadCacheSize then
        AThread.KeepInCache := True;
    end;
  finally
    FListLock.Leave;
  end;
end;

procedure TBServerWinSocket.RemoveThread(AThread: TBServerClientThread);
begin
  FListLock.Enter;
  try
    FActiveThreads.Remove(AThread);
  finally
    FListLock.Leave;
  end;
end;

procedure TBServerWinSocket.ClientEvent(Sender: TObject; Socket: TBCustomWinSocket;
  SocketEvent: TSocketEvent);
begin
  case SocketEvent of
    seAccept,
    seLookup,
    seConnecting,
    seListen:
      begin
      end;
    seConnect: ClientConnect(Socket);
    seDisconnect: ClientDisconnect(Socket);
    seRead: ClientRead(Socket);
    seWrite: ClientWrite(Socket);
  end;
end;

procedure TBServerWinSocket.ClientError(Sender: TObject; Socket: TBCustomWinSocket;
  ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
  ClientErrorEvent(Socket, ErrorEvent, ErrorCode);
end;

function TBServerWinSocket.GetActiveConnections: Integer;
begin
  Result := FConnections.Count;
end;

function TBServerWinSocket.GetConnections(Index: Integer): TBCustomWinSocket;
begin
  Result := FConnections[Index];
end;

function TBServerWinSocket.GetActiveThreads: Integer;
var
  I: Integer;
begin
  FListLock.Enter;
  try
    Result := 0;
    for I := 0 to FActiveThreads.Count - 1 do
      if TBServerClientThread(FActiveThreads[I]).ClientSocket <> nil then
        Inc(Result);
  finally
    FListLock.Leave;
  end;
end;

function TBServerWinSocket.GetIdleThreads: Integer;
var
  I: Integer;
begin
  FListLock.Enter;
  try
    Result := 0;
    for I := 0 to FActiveThreads.Count - 1 do
      if TBServerClientThread(FActiveThreads[I]).ClientSocket = nil then
        Inc(Result);
  finally
    FListLock.Leave;
  end;
end;

procedure TBServerWinSocket.Accept(Socket: TSocket);
var
  ClientSocket: TBServerClientWinSocket;
  ClientWinSocket: TSocket;
  Addr: TSockAddrIn;
  Len: Integer;
  OldOpenType, NewOpenType: Integer;
begin
  Len := SizeOf(OldOpenType);
  if getsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, PAnsiChar(@OldOpenType),
    Len) = 0 then
  try
    if FServerType = stThreadBlocking then
       begin
         NewOpenType := SO_SYNCHRONOUS_NONALERT;
         setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, PAnsiChar(@NewOpenType), Len);
       end;
    Len := SizeOf(Addr);
    ClientWinSocket := WinSock.accept(Socket, @Addr, @Len);
    if ClientWinSocket <> INVALID_SOCKET then
       begin
         ClientSocket := GeTBClientSocket(ClientWinSocket);
         if Assigned(FOnSocketEvent) then
           FOnSocketEvent(Self, ClientSocket, seAccept);
         if FServerType = stThreadBlocking then
            begin
              ClientSocket.ASyncStyles := [];
              GetServerThread(ClientSocket);
            end;
       end;
  finally
    Len := SizeOf(OldOpenType);
    setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, PAnsiChar(@OldOpenType), Len);
  end;
end;

procedure TBServerWinSocket.Disconnect(Socket: TSocket);
var
  SaveCacheSize: Integer;
begin
  Lock;
  try
    SaveCacheSize := ThreadCacheSize;
    try
      ThreadCacheSize := 0;
      while FActiveThreads.Count > 0 do
        with TBServerClientThread(FActiveThreads.Last) do
           begin
             FreeOnTerminate := False;
             Terminate;
             FEvent.SetEvent;
             if (ClientSocket <> nil) and ClientSocket.Connected then
               ClientSocket.Close;
             WaitFor;
             Free;
           end;
      while FConnections.Count > 0 do
        TBCustomWinSocket(FConnections.Last).Free;
      if FServerAcceptThread <> nil then
        FServerAcceptThread.Terminate;
      inherited Disconnect(Socket);
      FServerAcceptThread.Free;
      FServerAcceptThread := nil;
    finally
      ThreadCacheSize := SaveCacheSize;
    end;
  finally
    Unlock;
  end;
end;

function TBServerWinSocket.DoCreateThread(ClientSocket: TBServerClientWinSocket): TBServerClientThread;
begin
  Result := TBServerClientThread.Create(False, ClientSocket);
end;

procedure TBServerWinSocket.Listen(var Name, Address, Service: Ansistring; Port: Word;
  QueueSize: Integer);
begin
  inherited Listen(Name, Address, Service, Port, QueueSize, ServerType = stThreadBlocking);
  if FConnected and (ServerType = stThreadBlocking) then
    FServerAcceptThread := TBServerAcceptThread.Create(False, Self);
end;

procedure TBServerWinSocket.SetServerType(Value: TServerType);
begin
  if Value <> FServerType then
    if not FConnected then
       begin
         FServerType := Value;
         if FServerType = stThreadBlocking then
           ASyncStyles := []
         else
           ASyncStyles := [asAccept];
       end
    else
       raise ESocketError.CreateRes(@sCantChangeWhileActive);
end;

procedure TBServerWinSocket.SetThreadCacheSize(Value: Integer);
var
  xStart, I: Integer;
begin
  if Value <> FThreadCacheSize then
     begin
       if Value < FThreadCacheSize then
         xStart := Value
       else
         xStart := FThreadCacheSize;
       FThreadCacheSize := Value;
       FListLock.Enter;
       try
         for I := 0 to FActiveThreads.Count - 1 do
           with TBServerClientThread(FActiveThreads[I]) do
             KeepInCache := (I < xStart);
       finally
         FListLock.Leave;
       end;
     end;
end;

function TBServerWinSocket.GeTBClientSocket(Socket: TSocket): TBServerClientWinSocket;
begin
  Result := nil;
  if Assigned(FOnGetSocket) then
     FOnGetSocket(Self, Socket, Result);
  if Result = nil then
    Result := TBServerClientWinSocket.Create(Socket, Self);
end;

procedure TBServerWinSocket.ThreadEnd(AThread: TBServerClientThread);
begin
  if Assigned(FOnThreadEnd) then
     FOnThreadEnd(Self, AThread);
end;

procedure TBServerWinSocket.ThreadStart(AThread: TBServerClientThread);
begin
  if Assigned(FOnThreadStart) then
     FOnThreadStart(Self, AThread);
end;

function TBServerWinSocket.GetServerThread(ClientSocket: TBServerClientWinSocket): TBServerClientThread;
var
  I: Integer;
begin
  Result := nil;
  FListLock.Enter;
  try
    for I := 0 to FActiveThreads.Count - 1 do
      if TBServerClientThread(FActiveThreads[I]).ClientSocket = nil then
         begin
           Result := FActiveThreads[I];
           Result.ReActivate(ClientSocket);
           Break;
         end;
  finally
    FListLock.Leave;
  end;
  if Result = nil then
     begin
       if Assigned(FOnGetThread) then
          FOnGetThread(Self, ClientSocket, Result);
       if Result = nil then
          Result := DoCreateThread(ClientSocket);
     end;
end;

function TBServerWinSocket.GetClientThread(ClientSocket: TBServerClientWinSocket): TBServerClientThread;
var
  I: Integer;
begin
  Result := nil;
  FListLock.Enter;
  try
    for I := 0 to FActiveThreads.Count - 1 do
      if TBServerClientThread(FActiveThreads[I]).ClientSocket = ClientSocket then
         begin
           Result := FActiveThreads[I];
           Break;
         end;
  finally
    FListLock.Leave;
  end;
end;

procedure TBServerWinSocket.ClientConnect(Socket: TBCustomWinSocket);
begin
  if Assigned(FOnClientConnect) then
     FOnClientConnect(Self, Socket);
end;

procedure TBServerWinSocket.ClientDisconnect(Socket: TBCustomWinSocket);
begin
  if Assigned(FOnClientDisconnect) then
     FOnClientDisconnect(Self, Socket);
  if ServerType = stNonBlocking then
     Socket.DeferFree;
end;

procedure TBServerWinSocket.ClientRead(Socket: TBCustomWinSocket);
begin
  if Assigned(FOnClientRead) then
     FOnClientRead(Self, Socket);
end;

procedure TBServerWinSocket.ClientWrite(Socket: TBCustomWinSocket);
begin
  if Assigned(FOnClientWrite) then
     FOnClientWrite(Self, Socket);
end;

procedure TBServerWinSocket.ClientErrorEvent(Socket: TBCustomWinSocket;
  ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
  if Assigned(FOnClientError) then
     FOnClientError(Self, Socket, ErrorEvent, ErrorCode);
end;



constructor TBServerAcceptThread.Create(CreateSuspended: Boolean;
  ASocket: TBServerWinSocket);
begin
  FServerSocket := ASocket;
  inherited Create(CreateSuspended);
end;

procedure TBServerAcceptThread.Execute;
begin
  while not Terminated do
    FServerSocket.Accept(FServerSocket.SocketHandle);
end;



constructor TBServerClientThread.Create(CreateSuspended: Boolean;
  ASocket: TBServerClientWinSocket);
begin
  FreeOnTerminate := True;
  FEvent := TSimpleEvent.Create;
  inherited Create(True);
  Priority := tpHigher;
  ReActivate(ASocket);
  if not CreateSuspended then
     Resume;
end;

destructor TBServerClientThread.Destroy;
begin
  FClientSocket.Free;
  FEvent.Free;
  inherited Destroy;
end;

procedure TBServerClientThread.ReActivate(ASocket: TBServerClientWinSocket);
begin
  FClientSocket := ASocket;
  if Assigned(FClientSocket) then
     begin
       FServerSocket := FClientSocket.ServerWinSocket;
       FServerSocket.AddThread(Self);
       FClientSocket.OnSocketEvent := HandleEvent;
       FClientSocket.OnErrorEvent := HandleError;
       FEvent.SetEvent;
     end;
end;

procedure TBServerClientThread.DoHandleException;
begin
  if GetCapture <> 0 then
     SendMessage(GetCapture, WM_CANCELMODE, 0, 0);
  if FException is Exception then
     Application.ShowException(FException)
  else
     SysUtils.ShowException(FException, nil);
end;

procedure TBServerClientThread.DoRead;
begin
  ClientSocket.ServerWinSocket.Event(ClientSocket, seRead);
end;

procedure TBServerClientThread.DoTerminate;
begin
  inherited DoTerminate;
  if Assigned(FServerSocket) then
    FServerSocket.RemoveThread(Self);
end;

procedure TBServerClientThread.DoWrite;
begin
  FServerSocket.Event(ClientSocket, seWrite);
end;

procedure TBServerClientThread.HandleEvent(Sender: TObject; Socket: TBCustomWinSocket;
  SocketEvent: TSocketEvent);
begin
  Event(SocketEvent);
end;

procedure TBServerClientThread.HandleError(Sender: TObject; Socket: TBCustomWinSocket;
  ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
  Error(ErrorEvent, ErrorCode);
end;

procedure TBServerClientThread.Event(SocketEvent: TSocketEvent);
begin
  FServerSocket.ClientEvent(Self, ClientSocket, SocketEvent);
end;

procedure TBServerClientThread.Error(ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
  FServerSocket.ClientError(Self, ClientSocket, ErrorEvent, ErrorCode);
end;

procedure TBServerClientThread.HandleException;
begin
  FException := Exception(ExceptObject);
  try
    if not (FException is EAbort) then
      Synchronize(DoHandleException);
  finally
    FException := nil;
  end;
end;

procedure TBServerClientThread.Execute;
begin
  FServerSocket.ThreadStart(Self);
  try
    try
      while True do
         begin
           if StartConnect then ClientExecute;
           if EndConnect then Break;
         end;
    except
      HandleException;
      KeepInCache := False;
    end;
  finally
    FServerSocket.ThreadEnd(Self);
  end;
end;

procedure TBServerClientThread.ClientExecute;
var
  FDSet: TFDSet;
  TimeVal: TTimeVal;
begin
  while not Terminated and ClientSocket.Connected do
     begin
       FD_ZERO(FDSet);
       FD_SET(ClientSocket.SocketHandle, FDSet);
       TimeVal.tv_sec := 0;
       TimeVal.tv_usec := 500;
       if (select(0, @FDSet, nil, nil, @TimeVal) > 0) and not Terminated then
         if ClientSocket.ReceiveBuf(FDSet, -1) = 0 then
            Break
         else
            Synchronize(DoRead);
       if (select(0, nil, @FDSet, nil, @TimeVal) > 0) and not Terminated then
         Synchronize(DoWrite);
     end;
end;

function TBServerClientThread.StartConnect: Boolean;
begin
  if FEvent.WaitFor(INFINITE) = wrSignaled then
    FEvent.ResetEvent;
  Result := not Terminated;
end;

function TBServerClientThread.EndConnect: Boolean;
begin
  FClientSocket.Free;
  FClientSocket := nil;
  Result := Terminated or not KeepInCache;
end;



procedure TBAbstractSocket.DoEvent(Sender: TObject; Socket: TBCustomWinSocket;
  SocketEvent: TSocketEvent);
begin
  Event(Socket, SocketEvent);
end;

procedure TBAbstractSocket.DoError(Sender: TObject; Socket: TBCustomWinSocket;
  ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
  Error(Socket, ErrorEvent, ErrorCode);
end;

procedure TBAbstractSocket.SetActive(Value: Boolean);
begin
  if Value <> FActive then
  begin
    if (csDesigning in ComponentState) or (csLoading in ComponentState) then
      FActive := Value;
    if not (csLoading in ComponentState) then
      DoActivate(Value);
  end;
end;

procedure TBAbstractSocket.InitSocket(Socket: TBCustomWinSocket);
begin
  Socket.OnSocketEvent := DoEvent;
  Socket.OnErrorEvent := DoError;
end;

procedure TBAbstractSocket.Loaded;
begin
  inherited Loaded;
  DoActivate(FActive);
end;

procedure TBAbstractSocket.SetAddress(Value: Ansistring);
begin
  if CompareText(Value, FAddress) <> 0 then
     begin
       if not (csLoading in ComponentState) and FActive then
         raise ESocketError.CreateRes(@sCantChangeWhileActive);
       FAddress := Value;
     end;
end;

procedure TBAbstractSocket.SetHost(Value: Ansistring);
begin
  if CompareText(Value, FHost) <> 0 then
     begin
       if not (csLoading in ComponentState) and FActive then
         raise ESocketError.CreateRes(@sCantChangeWhileActive);
       FHost := Value;
     end;
end;

procedure TBAbstractSocket.SetPort(Value: Integer);
begin
  if FPort <> Value then
     begin
       if not (csLoading in ComponentState) and FActive then
         raise ESocketError.CreateRes(@sCantChangeWhileActive);
       FPort := Value;
     end;
end;

procedure TBAbstractSocket.SetService(Value: Ansistring);
begin
  if CompareText(Value, FService) <> 0 then
     begin
       if not (csLoading in ComponentState) and FActive then
         raise ESocketError.CreateRes(@sCantChangeWhileActive);
       FService := Value;
     end;
end;

procedure TBAbstractSocket.Open;
begin
  Active := True;
end;

procedure TBAbstractSocket.Close;
begin
  Active := False;
end;



procedure TBCustomSocket.Event(Socket: TBCustomWinSocket; SocketEvent: TSocketEvent);
begin
  case SocketEvent of
    seLookup: if Assigned(FOnLookup) then FOnLookup(Self, Socket);
    seConnecting: if Assigned(FOnConnecting) then FOnConnecting(Self, Socket);
    seConnect:
      begin
        FActive := True;
        if Assigned(FOnConnect) then FOnConnect(Self, Socket);
      end;
    seListen:
      begin
        FActive := True;
        if Assigned(FOnListen) then FOnListen(Self, Socket);
      end;
    seDisconnect:
      begin
        FActive := False;
        if Assigned(FOnDisconnect) then FOnDisconnect(Self, Socket);
      end;
    seAccept: if Assigned(FOnAccept) then FOnAccept(Self, Socket);
    seRead: if Assigned(FOnRead) then FOnRead(Self, Socket);
    seWrite: if Assigned(FOnWrite) then FOnWrite(Self, Socket);
  end;
end;

procedure TBCustomSocket.Error(Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
  if Assigned(FOnError) then FOnError(Self, Socket, ErrorEvent, ErrorCode);
end;



constructor TBWinSocketStream.Create(ASocket: TBCustomWinSocket; TimeOut: Longint);
begin
  if ASocket.ASyncStyles <> [] then
    raise ESocketError.CreateRes(@sSocketMustBeBlocking);
  FSocket := ASocket;
  FTimeOut := TimeOut;
  FEvent := TSimpleEvent.Create;
  inherited Create;
end;

destructor TBWinSocketStream.Destroy;
begin
  FEvent.Free;
  inherited Destroy;
end;

function TBWinSocketStream.WaitForData(Timeout: Longint): Boolean;
var
  FDSet: TFDSet;
  TimeVal: TTimeVal;
begin
  TimeVal.tv_sec := Timeout div 1000;
  TimeVal.tv_usec := (Timeout mod 1000) * 1000;
  FD_ZERO(FDSet);
  FD_SET(FSocket.SocketHandle, FDSet);
  Result := select(0, @FDSet, nil, nil, @TimeVal) > 0;
end;

function TBWinSocketStream.Read(var Buffer; Count: Longint): Longint;
var
  Overlapped: TOverlapped;
  ErrorCode: Integer;
begin
  FSocket.Lock;
  try
    FillChar(OVerlapped, SizeOf(Overlapped), 0);
    Overlapped.hEvent := FEvent.Handle;
    if not ReadFile(FSocket.SocketHandle, Buffer, Count, DWORD(Result),@Overlapped) and (GetLastError <> ERROR_IO_PENDING) then
       begin
         ErrorCode := GetLastError;
         raise ESocketError.CreateResFmt(@sSocketIOError, [sSocketRead, ErrorCode, SysErrorMessage(ErrorCode)]);
       end;
    if FEvent.WaitFor(FTimeOut) <> wrSignaled then
       Result := 0
    else
       begin
         GetOverlappedResult(FSocket.SocketHandle, Overlapped, DWORD(Result), False);
         FEvent.ResetEvent;
       end;
  finally
    FSocket.Unlock;
  end;
end;

function TBWinSocketStream.Write(const Buffer; Count: Longint): Longint;
var
  Overlapped: TOverlapped;
  ErrorCode: Integer;
begin
  FSocket.Lock;
  try
    FillChar(OVerlapped, SizeOf(Overlapped), 0);
    Overlapped.hEvent := FEvent.Handle;
    if not WriteFile(FSocket.SocketHandle, Buffer, Count, DWORD(Result), @Overlapped) and (GetLastError <> ERROR_IO_PENDING) then
       begin
         ErrorCode := GetLastError;
         raise ESocketError.CreateResFmt(@sSocketIOError, [sSocketWrite, ErrorCode, SysErrorMessage(ErrorCode)]);
       end;
    if FEvent.WaitFor(FTimeOut) <> wrSignaled then
      Result := 0
    else
      GetOverlappedResult(FSocket.SocketHandle, Overlapped, DWORD(Result), False);
  finally
    FSocket.Unlock;
  end;
end;

function TBWinSocketStream.Seek(Offset: Longint; Origin: Word): Longint;
begin
  Result := 0;
end;



constructor TBClientSocket.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FClientSocket := TBClientWinSocket.Create(INVALID_SOCKET);
  InitSocket(FClientSocket);
end;

destructor TBClientSocket.Destroy;
begin
  FClientSocket.Free;
  inherited Destroy;
end;

procedure TBClientSocket.DoActivate(Value: Boolean);
begin
  if (Value <> FClientSocket.Connected) and not (csDesigning in ComponentState) then
     begin
        if FClientSocket.Connected then
           FClientSocket.Disconnect(FClientSocket.FSocket)
        else
           FClientSocket.Open(FHost, FAddress, FService, FPort, ClientType = ctBlocking);
     end;
end;

function TBClientSocket.GetClientType: TClientType;
begin
  Result := FClientSocket.ClientType;
end;

procedure TBClientSocket.SetClientType(Value: TClientType);
begin
  FClientSocket.ClientType := Value;
end;



destructor TBCustomServerSocket.Destroy;
begin
  FServerSocket.Free;
  inherited Destroy;
end;

procedure TBCustomServerSocket.DoActivate(Value: Boolean);
begin
  if (Value <> FServerSocket.Connected) and not (csDesigning in ComponentState) then
     begin
        if FServerSocket.Connected then
           FServerSocket.Disconnect(FServerSocket.SocketHandle)
        else
           FServerSocket.Listen(FHost, FAddress, FService, FPort, SOMAXCONN);
     end;
end;

function TBCustomServerSocket.GetServerType: TServerType;
begin
  Result := FServerSocket.ServerType;
end;

procedure TBCustomServerSocket.SetServerType(Value: TServerType);
begin
  FServerSocket.ServerType := Value;
end;

function TBCustomServerSocket.GetGetThreadEvent: TGetThreadEvent;
begin
  Result := FServerSocket.OnGetThread;
end;

procedure TBCustomServerSocket.SetGetThreadEvent(Value: TGetThreadEvent);
begin
  FServerSocket.OnGetThread := Value;
end;

function TBCustomServerSocket.GetGetSocketEvent: TGetSocketEvent;
begin
  Result := FServerSocket.OnGetSocket;
end;

procedure TBCustomServerSocket.SetGetSocketEvent(Value: TGetSocketEvent);
begin
  FServerSocket.OnGetSocket := Value;
end;

function TBCustomServerSocket.GetThreadCacheSize: Integer;
begin
  Result := FServerSocket.ThreadCacheSize;
end;

procedure TBCustomServerSocket.SetThreadCacheSize(Value: Integer);
begin
  FServerSocket.ThreadCacheSize := Value;
end;

function TBCustomServerSocket.GetOnThreadStart: TThreadNotifyEvent;
begin
  Result := FServerSocket.OnThreadStart;
end;

function TBCustomServerSocket.GetOnThreadEnd: TThreadNotifyEvent;
begin
  Result := FServerSocket.OnThreadEnd;
end;

procedure TBCustomServerSocket.SetOnThreadStart(Value: TThreadNotifyEvent);
begin
  FServerSocket.OnThreadStart := Value;
end;

procedure TBCustomServerSocket.SetOnThreadEnd(Value: TThreadNotifyEvent);
begin
  FServerSocket.OnThreadEnd := Value;
end;

function TBCustomServerSocket.GetOnClientEvent(Index: Integer): TSocketNotifyEvent;
begin
  case Index of
    0: Result := FServerSocket.OnClientRead;
    1: Result := FServerSocket.OnClientWrite;
    2: Result := FServerSocket.OnClientConnect;
    3: Result := FServerSocket.OnClientDisconnect;
  end;
end;

procedure TBCustomServerSocket.SetOnClientEvent(Index: Integer;
  Value: TSocketNotifyEvent);
begin
  case Index of
    0: FServerSocket.OnClientRead := Value;
    1: FServerSocket.OnClientWrite := Value;
    2: FServerSocket.OnClientConnect := Value;
    3: FServerSocket.OnClientDisconnect := Value;
  end;
end;

function TBCustomServerSocket.GetOnClientError: TSocketErrorEvent;
begin
  Result := FServerSocket.OnClientError;
end;

procedure TBCustomServerSocket.SetOnClientError(Value: TSocketErrorEvent);
begin
  FServerSocket.OnClientError := Value;
end;



constructor TBServerSocket.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FServerSocket := TBServerWinSocket.Create(INVALID_SOCKET);
  InitSocket(FServerSocket);
  FServerSocket.ThreadCacheSize := 10;
end;

end.

