//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit BurroUDP;

interface

uses
  Winsock, Classes, Sysutils, WinTypes, Messages,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  Forms;

const

  Status_None = 0;
  Status_Informational = 1;
  Status_Basic = 2;
  Status_Routines = 4;
  Status_Debug = 8;
  Status_Trace = 16;

  WM_ASYNCHRONOUSPROCESS = WM_USER + 101; 

const 
  Const_cmd_true = AnsiString('TRUE');

{$IFDEF NMF3}
resourcestring
{$ELSE}
const
{$ENDIF}
  Cons_Palette_Inet = 'QuickBurro Basic';
  Cons_Msg_Wsk = 'Initializing Winsock';
  Cons_Msg_Lkp = 'Host Lookup Canceled';
  Cons_Msg_Data = 'Sending Data';
  Cons_Msg_InvStrm = 'Invalid stream';
  Cons_Msg_Echk = 'Checking Error In Error Manager';
  Cons_Msg_Eno = 'Unknown Error No. ';
  Cons_Msg_ELkp = 'Looking Up Error Message';
  Cons_Err_Addr = 'Null Remote Address';
  Cons_Err_Buffer = 'Invalid buffer';

type

  UDPSockError = class(Exception);
  TOnErrorEvent = procedure(Sender: TComponent; errno: word; Errmsg: Ansistring) of object;
  TOnStatus = procedure(Sender: TComponent; status: Ansistring) of object;
  TOnReceive = procedure(Sender: TComponent; NumberBytes: Integer; FromIP: Ansistring; Port: integer) of object;
  THandlerEvent = procedure(var handled: boolean) of object;
  TBuffInvalid = procedure(var handled: boolean; var Buff: array of Ansichar; var length: integer) of object;
  TStreamInvalid = procedure(var handled: boolean; Stream: TStream) of object;

  TBurroUDP = class(TComponent)
  private
    IBuff: array[0..8192] of Ansichar;        
    IBuffSize: integer;
    FRemoteHost: Ansistring;
    FRemotePort: integer;
    FLocalPort: integer;                      
    RemoteAddress, RemoteAddress2: TSockAddr; 
    FSocketWindow: hwnd;
    Wait_Flag: boolean;                       
    RemoteHostS: PHostEnt;                    
    Canceled: boolean;                        
    Succeed: boolean;                         
    MyWSAData: TWSADATA;                      
    FOnStatus: TOnStatus;                     
    FReportLevel: integer;                    
    _status: Ansistring;                      
    _ProcMsg: boolean;                        
    FLastErrorno: integer;                    
    FOnErrorEvent: TOnErrorEvent;             
    FOnDataReceived: TOnReceive;
    FOnDataSend: TNotifyEvent;
    FOnInvalidHost: THandlerEvent;
    FOnStreamInvalid: TStreamInvalid;
    FOnBufferInvalid: TBuffInvalid;
    procedure WndProc(var message: TMessage);
    procedure ResolveRemoteHost;
    procedure SetLocalPort(NewLocalPort: integer);
    procedure ProcessIncomingdata;

  protected
    procedure StatusMessage(Level: byte; value: Ansistring);
    function ErrorManager(ignore: word): Ansistring;
    function SocketErrorStr(Errno: word): Ansistring;
    procedure Wait;

  public
    EventHandle: THandle;
    ThisSocket: TSocket; 
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure Cancel;
    procedure SendStream(DataStream: TStream);
    procedure SendBuffer(Buff: array of Ansichar; length: integer); overload;
    procedure SendBuffer(aString: Ansistring); overload;
    procedure ReadStream(DataStream: TStream);
    procedure ReadBuffer(var Buff: array of Ansichar; var length: integer); overload;
    procedure ReadBuffer(var aString: Ansistring); overload;

  published
    property RemoteHost: Ansistring read FRemoteHost write FRemoteHost; 
    property RemotePort: integer read FRemotePort write FRemotePort; 
    property ServerPort: integer read FLocalPort write SetLocalPort; 
    property ReportLevel: integer read FReportLevel write FReportLevel;
    property OnDataReceived: TOnReceive read FOnDataReceived write FOnDataReceived;
    property OnDataSend: TNotifyEvent read FOnDataSend write FOnDataSend;
    property OnStatus: TOnStatus read FOnStatus write FOnStatus;
    property OnInvalidHost: THandlerEvent read FOnInvalidHost write FOnInvalidHost;
    property OnStreamInvalid: TStreamInvalid read FOnStreamInvalid write FOnStreamInvalid;
    property OnBufferInvalid: TBuffInvalid read FOnBufferInvalid write FOnBufferInvalid;

  end; 


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents(Cons_Palette_Inet, [TBurroUDP]);
end;

procedure WaitforSync(Handle: THandle);
begin
  repeat
    if MsgWaitForMultipleObjects(1, Handle, False, INFINITE, QS_ALLINPUT)= WAIT_OBJECT_0+1 then
       Application.ProcessMessages
    else 
       BREAK;
  until True = False;
end;

procedure TBurroUDP.Cancel;
begin
  StatusMessage(Status_Debug, 'Canceling');
  Canceled := True;
  SetEvent(EventHandle);
end;

constructor TBurroUDP.Create(AOwner: TComponent);
var
   ErrorCode: integer;
begin
   inherited create(AOwner);
   _ProcMsg := FALSE; 
   getmem(RemoteHostS, MAXGETHOSTSTRUCT);      
   FSocketWindow := AllocateHWnd(WndProc);     
   FreportLevel := Status_Informational;       
   Canceled := FALSE;                          
   EventHandle := CreateEvent(nil, True, False, '');
   StatusMessage(Status_debug, Cons_Msg_Wsk);  
   {$IFDEF VER130}
   ErrorCode := WSAStartup($0101, MyWSADATA);
   {$ELSE}
   ErrorCode := WSAStartup(MAKEWORD(2,0), MyWSADATA);
   {$ENDIF}
   if ErrorCode = 0 then
      try
         ThisSocket := Socket(AF_INET, SOCK_DGRAM, 0);
         if ThisSocket = TSocket(INVALID_SOCKET) then
            ErrorManager(WSAEWOULDBLOCK); 
         setsockopt(ThisSocket, SOL_SOCKET, SO_DONTLINGER, PansiChar(Const_cmd_true), 4);
      except
         WSACleanup;
         raise; 
      end
   else
      ErrorManager(WSAEWOULDBLOCK);
   _ProcMsg := true;
end;

destructor TBurroUDP.Destroy;
begin
   closesocket(ThisSocket);
   freemem(RemoteHostS, MAXGETHOSTSTRUCT); 
   DeAllocateHWnd(FSocketWindow);
   CloseHandle(EventHandle);
   WSACleanUp; 
   inherited destroy;
end;

procedure TBurroUDP.SetLocalPort(NewLocalPort: integer);
var
   ErrorCode: integer;
begin
   if ThisSocket <> 0 then 
      closesocket(ThisSocket);
    WSAcleanup;
   {$IFDEF VER130}
   ErrorCode := WSAStartup($0101, MyWSADATA);
   {$ELSE}
   ErrorCode := WSAStartup(MAKEWORD(2,0), MyWSADATA);
   {$ENDIF}
   if ErrorCode = 0 then
      try
         ThisSocket := Socket(AF_INET, SOCK_DGRAM, 0);
         if ThisSocket = TSocket(INVALID_SOCKET) then
            ErrorManager(WSAEWOULDBLOCK);
         setsockopt(ThisSocket, SOL_SOCKET, SO_DONTLINGER, PansiChar(Const_cmd_true), 4);
      except
         WSACleanup;
         raise;
      end
   else
      ErrorManager(WSAEWOULDBLOCK);
   FLocalPort := NewLocalPort;
   Loaded;
end;

procedure TBurroUDP.Loaded;
var
   buf: array[0..17] of Ansichar;
begin
   if not (csDesigning in ComponentState) then 
      begin
         RemoteAddress2.sin_addr.S_addr := Inet_Addr(strpcopy(buf, '0.0.0.0'));
         RemoteAddress2.sin_family := AF_INET; 
         RemoteAddress2.sin_port := htons(FLocalPort); 
         wait_flag := FALSE; 
         WinSock.bind(ThisSocket, RemoteAddress2, SizeOf(RemoteAddress2));
         WSAAsyncselect(ThisSocket, FSocketWindow, WM_ASYNCHRONOUSPROCESS, FD_READ);
      end;
end;

procedure TBurroUDP.ResolveRemoteHost;
var
   BUF: array[0..127] of Ansichar;
   CTry: integer;
   Handled: boolean;
begin
   remoteaddress.sin_addr.S_addr := Inet_Addr(strpcopy(buf, FRemoteHost));
   if remoteaddress.sin_addr.S_addr = SOCKET_ERROR then
      begin
         CTry := 0;
         repeat
            Wait_Flag := FALSE; 
            wsaasyncgethostbyname(FSocketWindow, WM_ASYNCHRONOUSPROCESS, Buf, PAnsiChar(remotehostS), MAXGETHOSTSTRUCT);
            repeat
               Wait;
            until Wait_Flag or Canceled;
            if Canceled then
               raise UDPSockError.create(Cons_Msg_Lkp);
            if Succeed = FALSE then 
               begin
                  if CTry < 1 then 
                     begin
                        CTry := CTry + 1;
                        Handled := FALSE;
                        if assigned(FOnInvalidHost) then 
                           FOnInvalidHost(Handled);
                        if not handled then 
                           UDPSockError.create(Cons_Msg_Lkp);
                     end
                  else 
                     raise UDPSockError.create(Cons_Msg_Lkp);
               end
            else
               with RemoteAddress.sin_addr.S_un_b do 
                  begin
                     s_b1 := remotehostS.h_addr_list^[0];
                     s_b2 := remotehostS.h_addr_list^[1];
                     s_b3 := remotehostS.h_addr_list^[2];
                     s_b4 := remotehostS.h_addr_list^[3];
                  end;
         until Succeed = true;
      end;
end;

procedure TBurroUDP.SendStream(DataStream: TStream);
var 
   Ctry, i: integer;
   BUf: array[0..8192] of Ansichar;            
   Handled: boolean;
begin
   CTry := 0;
   while DataStream.size = 0 do
      begin
         if CTry > 0 then
            raise Exception.create(Cons_Msg_InvStrm)
         else
            begin
               if not assigned(FOnStreamInvalid) then
                  raise Exception.create(Cons_Msg_InvStrm)
               else
                  begin
                     Handled := FALSE;
                     FOnStreamInvalid(Handled, DataStream);
                     if not Handled then
                        raise Exception.create(Cons_Msg_InvStrm)
                     else
                        CTry := CTry + 1;
                  end;
            end;
      end;
   Canceled := FALSE;
   ResolveRemoteHost;
   if RemoteAddress.sin_addr.S_addr <> 0 then
      begin
         StatusMessage(Status_Basic, Cons_Msg_Data);
         RemoteAddress.sin_family := AF_INET;
         {$R-}
         RemoteAddress.sin_port := htons(FRemotePort);
         {$R+}
         i := SizeOf(RemoteAddress);
         DataStream.position := 0;
         DataStream.ReadBuffer(Buf, DataStream.size);
         WinSock.SendTo(ThisSocket, Buf, DataStream.size, 0, RemoteAddress, i);
         if assigned(FOnDataSend) then
            FOnDataSend(self);
      end;
end;

procedure TBurroUDP.SendBuffer(Buff: array of Ansichar; length: integer);
var
   Ctry, i: integer;
   Handled: boolean;
begin
   CTry := 0;
   while length = 0 do
      begin
         if CTry > 0 then
            raise Exception.create(Cons_Err_Buffer)
         else
            begin
               if not assigned(FOnBufferInvalid) then
                  raise Exception.create(Cons_Err_Buffer)
               else
                  begin
                     Handled := FALSE;
                     FOnBufferInvalid(Handled, Buff, length);
                     if not Handled then
                        raise Exception.create(Cons_Err_Buffer)
                     else
                        CTry := CTry + 1;
                  end;
            end;
      end;
   Canceled := FALSE;
   ResolveRemoteHost;
   if RemoteAddress.sin_addr.S_addr <> 0 then
      begin
         StatusMessage(Status_Basic, Cons_Msg_Data);
         RemoteAddress.sin_family := AF_INET;
         {$R-}
         RemoteAddress.sin_port := htons(FRemotePort);
         {$R+}
         i := SizeOf(RemoteAddress);
         WinSock.SendTo(ThisSocket, Buff, length, 0, RemoteAddress, i);
         if assigned(FOnDataSend) then
            FOnDataSend(self);
      end;
end;

function TBurroUDP.ErrorManager(ignore: word): Ansistring;
var
   slasterror: Ansistring;
begin
   StatusMessage(STATUS_TRACE, Cons_Msg_Echk);
   FLastErrorno := wsagetlasterror;
   if (FLastErrorno and ignore) <> ignore then
      begin
         slasterror := SocketErrorStr(FLastErrorno);
         if assigned(fonerrorevent) then
            FOnerrorEvent(Self, FLastErrorno, slasterror);
         raise UDPSockError.create(string(slasterror));
      end;
  result := slasterror; 
end;

function TBurroUDP.SocketErrorStr(ErrNo: word): Ansistring;
begin
   if ErrNo <> 0 then
      Result := AnsiString(Cons_Msg_Eno + IntToStr(ErrNo));
   StatusMessage(Status_DEBUG, Cons_Msg_ELkp + result);
end;

procedure TBurroUDP.StatusMessage(Level: byte; value: Ansistring);
begin
   if level <= FReportLevel then
      begin
         _status := value;
         if assigned(FOnStatus) then
            FOnStatus(self, _status);
      end;
end;

procedure TBurroUDP.WndProc(var message: TMessage);
begin
   if _ProcMsg then
      begin
         with message do
            if msg = WM_ASYNCHRONOUSPROCESS then
               begin
                  if lparamLo = FD_Read then
                     ProcessIncomingdata
                  else
                     begin
                        wait_flag := TRUE;
                        if lparamhi > 0 then
                           Succeed := FALSE
                        else
                           Succeed := TRUE;
                     end;
                  SetEvent(EventHandle);
               end
            else
               Result := DefWindowProc(FSocketWindow, Msg, wParam, lParam);
      end;
end;

procedure TBurroUDP.ProcessIncomingdata;
var
   From: TSockAddr;
   i: integer;
   s1: Ansistring;
   p1: u_short;
begin
   i := sizeof(From);
   IBuffSize := Winsock.RecvFrom(ThisSocket, IBuff, 8192, 0, From, i);            
   if IBuffSize=SOCKET_ERROR then
      exit;
   if assigned(FOnDataReceived) then
      begin
         try
            s1 := AnsiString(format('%d.%d.%d.%d', [ord(From.sin_addr.S_un_b.S_b1), ord(From.sin_addr.S_un_b.S_b2), ord(From.sin_addr.S_un_b.S_b3), ord(From.sin_addr.S_un_b.S_b4)]));
            p1 := ntohs(From.sin_port);
            FOnDataReceived(self, IBuffSize, s1, p1);
         except
            exit;
         end;
      end;
end;

procedure TBurroUDP.ReadStream(DataStream: TStream);
begin
   DataStream.WriteBuffer(IBuff, IBuffSize);
   DataStream.position := 0;
end;

procedure TBurroUDP.Wait;
begin
   WaitforSync(EventHandle);
   ResetEvent(EventHandle);
end;

procedure TBurroUDP.ReadBuffer(var Buff: array of Ansichar; var length: integer);
begin
   Move(IBuff, Buff, IBuffSize);
   length := IBuffSize;
end;

procedure TBurroUDP.SendBuffer(aString: Ansistring);
var
   i: integer;
begin
   Canceled := FALSE;
   ResolveRemoteHost;
   if RemoteAddress.sin_addr.S_addr <> 0 then
      begin
         StatusMessage(Status_Basic, AnsiString(Cons_Msg_Data));
         RemoteAddress.sin_family := AF_INET;
         {$R-}
         RemoteAddress.sin_port := htons(FRemotePort);
         {$R+}
         i := SizeOf(RemoteAddress);
         WinSock.SendTo(ThisSocket, aString[1], length(aString), 0, RemoteAddress, i);
         if assigned(FOnDataSend) then
            FOnDataSend(self);
      end;
end;

procedure TBurroUDP.ReadBuffer(var aString: Ansistring);
begin
  Move(IBuff, aString[1], IBuffSize);
end;

end.

