//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit DbAccessor;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
  Windows, Messages, SysUtils, Classes, Forms,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  UserConnection, QBParcel, QBCommon, QBZlibExGz,
  dbclient, TransactionHeartbeat, PageQueryHeartbeat, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TCreateTransactionEvent = procedure (Sender: TObject) of object;
   TFreeTransactionEvent = procedure (Sender: TObject) of Object;
   TCommandSuccessEvent = procedure (Sender: TObject) of object;
   TCommandFailEvent = procedure (Sender: TObject) of Object;
   TDatasetFetchProgressEvent = procedure (Sender: TObject; AllPages: integer; CurrentPage: integer) of object;

type
   TCdsRecord=record
       TableName: string;
       FilterCondition: string;
       ClientDataset: TClientDataset;
   end;

type
   TComplexCDSRecord=record
       CdsCount: integer;
       CdsList: array of TCdsRecord;
   end;

type
  TDBAccessor = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fTargetNodeId: AnsiString;
     fTargetDatabaseId: AnsiString;
     fAutoIncFieldValue: integer;
     fInTransaction: boolean;
     fDbTransSessionId: int64;
     fPageQueryId: integer;
     fPageQuerySessionId: integer;
     fOnCreateTransaction: TCreateTransactionEvent;
     fOnFreeTransaction: TFreeTransactionEvent;
     fOnCommandSuccess: TCommandSuccessEvent;
     fOnCommandFail: TCommandFailEvent;
     fOnDatasetFetchProgress: TDatasetFetchProgressEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     TransHeartbeatThread: TTransactionHeartbeatThread;
     QueryHeartbeatThread: TPageQueryHeartbeatThread;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetTargetNodeId(aNodeId: AnsiString);
     procedure SetDatabaseId(aDatabaseId: AnsiString);
     function CdsZipToStream(aCds: TClientDataset; aStream: TMemoryStream): boolean;

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     property AutoIncFieldValue: integer read fAutoIncFieldValue;
     property InTransaction: boolean read fInTransaction;
     property DbTransSessionId: int64 read fDbTransSessionId;
     property PageQueryId: integer read fPageQueryId;
     property PageQuerySessionId: integer read fPageQuerySessionId;
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     function BeginTrans: boolean;
     function CommitTrans: boolean;
     function IfTransAlive: boolean;
     function RollbackTrans: boolean;
     function ExecuteSql(SqlCommand: string; HasResult: boolean; OutOfTrans: boolean; var RetValue: string): boolean;
     function ExecuteBatchSql(BatchSqlCommand: TStringList; OutOfTrans: boolean): boolean;
     function ExecuteStoredProc(StoredProcName: string; Parameters: TQbParcel; OutOfTrans: boolean): boolean;
     function FileToBlobField(aFilename: string; TableName: string; BlobFieldName: string; LocateCondition: string): boolean;
     function BlobFieldToFile(TableName: string; BlobFieldName: string; LocateCondition: string; aFilename: string): boolean;
     function ClearBlobField(TableName: string; BlobFieldName: string; LocateCondition: string): boolean;
     function SaveToFile(SqlCommand: string; RemoteFilename: string; DataFormat: integer): boolean;
     function LoadFromFile(RemoteFilename: string; TableName: string): boolean;
     function ReadDataset(SqlCommand: string; var Cds: TClientDataset): boolean;
     function ReadLargeDataset(aSQLStr: String; RecordsPerPage: integer; var RetCDS: TClientDataset): boolean;
     function WriteDataset(Cds: TClientDataset; TableName: string): boolean;
     function AppendRecord(TableName: string; RecordParcel: TQBParcel): boolean; overload;
     function AppendRecord(TableName: string; RecordCds: TClientDataset): boolean; overload;
     function UpdateRecord(TableName: string; FilterCondition: string; RecordParcel: TQBParcel): boolean; overload;
     function UpdateRecord(TableName: string; FilterCondition: string; RecordCds: TClientDataset): boolean; overload;
     function ReadTableHead(TableName: string; var Cds: TClientDataset): boolean;
     function GenerateId(TableName: string; KeyFieldName: string; FilterCondition: string;
              FieldMask: ansistring; ConstList: ansistring; var NewId: ansistring): boolean;
     function FreeUnusedId(TableName: string; KeyFieldName: string; FilterCondition: string; IdToFree: ansistring): boolean;
     function CommitCdsDelta(TableName: String; KeyFieldList: String; Delta: OleVariant): boolean;
     function CreateQuerySession(SqlCommand: String; var QueryId: integer; var QuerySessionId: integer; var AllRecords: integer): boolean;
     function FreeQuerySession(QueryId: integer; QuerySessionId: integer): boolean;
     function QueryPageData(QueryId: integer; QuerySessionId: integer; PageSize: integer; PageNumber: integer; ToCds: TClientDataset): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property TargetDatabaseId: AnsiString read fTargetDatabaseId write SetDatabaseId;
     property OnCreateTransaction: TCreateTransactionEvent read fOnCreateTransaction write fOnCreateTransaction;
     property OnFreeTransaction: TFreeTransactionEvent read fOnFreeTransaction write fOnFreeTransaction;
     property OnCommandSuccess: TCommandSuccessEvent read fOnCommandSuccess write fOnCommandSuccess;
     property OnCommandFail: TCommandFailEvent read fOnCommandFail write fOnCommandFail;
     property OnDatasetFetchProgress: TDatasetFetchProgressEvent read fOnDatasetFetchProgress write fOnDatasetFetchProgress;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TDBAccessor]);
end;

procedure TDBAccessor.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TDBAccessor.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
     fUserConn := nil;
end;

procedure TDBAccessor.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TDBAccessor.SetTargetNodeId(aNodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(aNodeId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aNodeId;
end;

procedure TDBAccessor.SetDatabaseId(aDatabaseId: AnsiString);
begin
   if StrIComp(PAnsiChar(aDatabaseId),PAnsiChar(fTargetDatabaseId))=0 then
      exit;
   fTargetDatabaseId:=aDatabaseId;
end;

function TDBAccessor.CdsZipToStream(aCds: TClientDataset; aStream: TMemoryStream): boolean;
var
   tmpStream: TMemoryStream;
begin
   tmpStream:=TMemoryStream.Create;
   try
      aCds.SaveToStream(tmpStream);
      tmpStream.Position:=0;
      aStream.Clear;
      GZCompressStream(tmpStream,aStream);
      Result:=true;
   except
      Result:=false;
   end;
   tmpStream.Free;
end;

constructor TDBAccessor.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   fTargetNodeId:='';
   fTargetDatabaseId:='';
   fInTransaction:=false;
   fDbTransSessionId:=0;
   LastErrorCode:='';
   LastErrorText:='';
   if not (csDesigning in ComponentState) then
      begin
         TransHeartbeatThread:=TTransactionHeartbeatThread.Create(self,true);
         QueryHeartbeatThread:=TPageQueryHeartbeatThread.Create(self,true);
      end;
end;

destructor TDBAccessor.Destroy;
begin
   fOnCreateTransaction:=nil;
   fOnFreeTransaction:=nil;
   if not (csDesigning in ComponentState) then
      begin
         if TransHeartbeatThread.Suspended then
            begin
               TransHeartbeatThread.Terminate;
               TransHeartbeatThread.Resume;
            end
         else
            TransHeartbeatThread.Terminate;
         if QueryHeartbeatThread.Suspended then
            begin
               QueryHeartbeatThread.Terminate;
               QueryHeartbeatThread.Resume;
            end
         else
            QueryHeartbeatThread.Terminate;
      end;
   inherited;
end;

function TDBAccessor.BeginTrans: boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (trim(fTargetDatabaseId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0207403';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',74);
   Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     fInTransaction:=true;
                     fDbTransSessionId:=Parcel.GetInt64Goods('TransSessionId');
                     if TransHeartbeatThread.Suspended then
                        TransHeartbeatThread.Resume;
                     if Assigned(fOnCreateTransaction) then
                        fOnCreateTransaction(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0207404';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0207405';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.CommitTrans: boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (trim(fTargetDatabaseId)='') or (not fInTransaction) or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0207503';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',75);
   Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('AutoIncFieldValue') then
                        fAutoIncFieldValue:=Parcel.GetIntegerGoods('AutoIncFieldValue')
                     else
                        fAutoIncFieldValue:=0;
                     fInTransaction:=false;
                     fDbTransSessionId:=0;
                     if not TransHeartbeatThread.Suspended then
                        TransHeartbeatThread.Suspend;
                     if Assigned(fOnFreeTransaction) then
                        fOnFreeTransaction(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0207504';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0207505';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.RollbackTrans: boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (not fInTransaction) or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0207603';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',76);
   Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   fInTransaction:=false;
   fDbTransSessionId:=0;
   if not TransHeartbeatThread.Suspended then
      TransHeartbeatThread.Suspend;
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnFreeTransaction) then
                        fOnFreeTransaction(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0207604';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0207605';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.IfTransAlive: boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (trim(fTargetDatabaseId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0207803';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if not fInTransaction then
      begin
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',78);
   Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     fInTransaction:=false;
                     fDbTransSessionId:=0;
                     if not TransHeartbeatThread.Suspended then
                        TransHeartbeatThread.Suspend;
                     if Assigned(fOnFreeTransaction) then
                        fOnFreeTransaction(self);
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               fInTransaction:=false;
               fDbTransSessionId:=0;
               if not TransHeartbeatThread.Suspended then
                  TransHeartbeatThread.Suspend;
               if Assigned(fOnFreeTransaction) then
                  fOnFreeTransaction(self);
               LastErrorCode:='0207604';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         fInTransaction:=false;
         fDbTransSessionId:=0;
         if not TransHeartbeatThread.Suspended then
            TransHeartbeatThread.Suspend;
         if Assigned(fOnFreeTransaction) then
            fOnFreeTransaction(self);
         LastErrorCode:='0207605';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.ExecuteSql(SqlCommand: string; HasResult: boolean; OutOfTrans: boolean; var RetValue: string): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205405';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0205406';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',54);
   if fInTransaction and (not OutOfTrans) then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('SQLCommand',sqlcommand);
   Parcel.PutBooleanGoods('HasResult',HasResult);
   Parcel.PutIntegerGoods('CommandTimeout',fTaskTimeout);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     RetValue:=Parcel.GetStringGoods('ReturnValue');
                     if Assigned(fOnCommandSuccess) then
                        fOnCommandSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnCommandFail) then
                        fOnCommandFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0205407';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnCommandFail) then
                  fOnCommandFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0205408';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnCommandFail) then
            fOnCommandFail(self);
      end;
end;

function TDBAccessor.ExecuteBatchSql(BatchSqlCommand: TStringList; OutOfTrans: boolean): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205505';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0205506';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',55);
   if fInTransaction and (not OutOfTrans) then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('BatchSQLCommand',BatchSqlcommand.Text);
   Parcel.PutIntegerGoods('CommandTimeout',fTaskTimeout);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnCommandSuccess) then
                        fOnCommandSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnCommandFail) then
                        fOnCommandFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0205507';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnCommandFail) then
                  fOnCommandFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0205508';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnCommandFail) then
            fOnCommandFail(self);
      end;
end;

function TDBAccessor.ExecuteStoredProc(StoredProcName: string; Parameters: TQbParcel; OutOfTrans: boolean): boolean;
var
   Parcel: TQBParcel;
   Stream,tmpStream: TMemoryStream;
begin
   if (StoredProcName='') or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205605';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0205606';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   if Assigned(Parameters) then
      begin
         Stream:=TMemoryStream.Create;
         tmpStream:=TMemoryStream.Create;
         try
            Parameters.SaveToStream(tmpStream);
            tmpStream.Position:=0;
            GZCompressStream(tmpStream,Stream);
            result:=true;
         except
            result:=false;
         end;
         if not Result then
            begin
               tmpStream.Free;
               Stream.free;
               LastErrorCode:='0205609';
               LastErrorText:='Zip parameters failed.';
               exit;
            end;
         tmpStream.Free;
         Stream.Position:=0;
      end
   else
      Stream:=nil;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',56);
   if fInTransaction and (not OutOfTrans) then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('StoredProcName',StoredProcName);
   if Stream<>nil then
      Parcel.PutStreamGoods('Parameters',Stream);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   if Stream<>nil then
      Stream.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if Result and Parcel.GoodsExists('Parameters') then
                  begin
                     tmpStream:=TMemoryStream.Create;
                     Stream:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('Parameters',tmpStream);
                        tmpStream.Position:=0;
                        GZDecompressStream(tmpStream,Stream);
                        Stream.Position:=0;
                        Parameters.Clear;
                        result:=Parameters.LoadFromStream(Stream);
                     except
                        result:=false;
                     end;
                     tmpStream.Free;
                     Stream.Free;
                     if not result then
                        begin
                           LastErrorCode:='0205611';
                           LastErrorText:='Decompress the return parameters fail.';
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206207';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206208';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.FileToBlobField(aFilename: string; TableName: string; BlobFieldName: string; LocateCondition: string): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205705';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0205706';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',57);
   if fInTransaction then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      begin
         if trim(fTargetDatabaseId)<>'' then
            Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
      end;
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStringGoods('BlobFieldName',BlobFieldName);
   Parcel.PutStringGoods('LocateCondition',LocateCondition);
   Parcel.PutStringGoods('DataFilename',aFilename);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0205707';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0205708';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.BlobFieldToFile(TableName: string; BlobFieldName: string; LocateCondition: string; aFilename: string): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetDatabaseId)='') or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205803';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',58);
   Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStringGoods('BlobFieldName',BlobFieldName);
   Parcel.PutStringGoods('LocateCondition',LocateCondition);
   Parcel.PutStringGoods('DataFilename',aFilename);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0205805';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0205806';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.ClearBlobField(TableName: string; BlobFieldName: string; LocateCondition: string): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0208105';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0208106';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',81);
   if fInTransaction then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      begin
         if trim(fTargetDatabaseId)<>'' then
            Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
      end;
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStringGoods('BlobFieldName',BlobFieldName);
   Parcel.PutStringGoods('LocateCondition',LocateCondition);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0208107';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0208108';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.SaveToFile(SqlCommand: string; RemoteFilename: string; DataFormat: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetDatabaseId)='') or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205903';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',59);
   Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('SqlCommand',SqlCommand);
   Parcel.PutStringGoods('DataFilename',RemoteFilename);
   Parcel.PutIntegerGoods('DataFormat',DataFormat);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0205905';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0205906';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.LoadFromFile(RemoteFilename: string; TableName: string): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206005';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0206006';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',60);
   if fInTransaction then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      begin
         if trim(fTargetDatabaseId)<>'' then
            Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
      end;
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStringGoods('DataFilename',RemoteFilename);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206007';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206008';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.ReadDataset(SqlCommand: string; var Cds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetDatabaseId)='') or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206104';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',61);
   Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('SqlCommand',SqlCommand);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     Cds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('Dataset',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        Cds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if not result then
                        begin
                           LastErrorCode:='0206107';
                           LastErrorText:='Decompress the return dataset fail.';
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206105';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206106';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.ReadLargeDataset(aSQLStr: String; RecordsPerPage: integer; var RetCDS: TClientDataset): boolean;
var
   tmpcds: tclientdataset;
   i,QueryId,QSessionId,AllRecords,allpages: integer;
begin
   RetCds.Close;
   tmpcds:=tclientdataset.Create(nil);
   if not CreateQuerySession(aSqlStr,QueryId,QSessionId,allrecords) then
      begin
         tmpcds.Free;
         result:=false;
         exit;
      end;
   if AllRecords=0 then
      begin
         tmpcds.free;
         FreeQuerySession(queryid,qsessionid);
         result:=ReadDataset(aSqlStr,RetCds);
         exit;
      end;
   allpages:=round(allrecords div RecordsPerPage);
   if allpages*RecordsPerPage<allrecords then
      inc(allpages);
   result:=true;
   for i:=0 to allpages-1 do
      begin
         tmpcds.Close;
         if QueryPageData(queryid,qsessionid,RecordsPerPage,i,tmpcds) then
            begin
               if not RetCds.Active then
                  RetCds.Data:=tmpcds.Data
               else
                  begin
                     RetCds.AppendData(tmpcds.Data,false);
                     RetCds.MergeChangeLog;
                  end;
               if assigned(fOnDatasetFetchProgress) then
                  fOnDatasetFetchProgress(self,AllPages,i+1);
            end
         else
            begin
               result:=false;
               break;
            end;
         application.ProcessMessages;
      end;
   tmpcds.Close;
   tmpcds.free;
   FreeQuerySession(queryid,qsessionid);
end;

function TDBAccessor.WriteDataset(Cds: TClientDataset; TableName: string): boolean;
var
   Parcel: TQBParcel;
   Stream: TMemoryStream;
begin
   if (not Cds.Active) or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206205';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0206206';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Result:=CdsZipToStream(Cds,Stream);
   if not Result then
      begin
         Stream.free;
         LastErrorCode:='0206209';
         LastErrorText:='Zip source dataset failed.';
         exit;
      end;
   Stream.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',62);
   if fInTransaction then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      begin
         if trim(fTargetDatabaseId)<>'' then
            Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
      end;
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStreamGoods('Dataset',Stream);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   Stream.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206207';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206208';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.AppendRecord(TableName: string; RecordParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream0,Stream: TMemoryStream;
begin
   if (RecordParcel.GoodsCount<=0) or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206305';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0206306';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Stream0:=TMemoryStream.Create;
   Stream:=TMemoryStream.Create;
   try
      result:=RecordParcel.SaveToStream(Stream0);
      Stream0.Position:=0;
      GZCompressStream(Stream0,Stream);
   except
      result:=false;
   end;
   Stream0.Free;
   if not Result then
      begin
         Stream.free;
         LastErrorCode:='0206309';
         LastErrorText:='Zip record data failed.';
         exit;
      end;
   Stream.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',63);
   if fInTransaction then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      begin
         if trim(fTargetDatabaseId)<>'' then
            Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
      end;
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStreamGoods('RecordData',Stream);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   Stream.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end
               else
                  begin
                     if (not fInTransaction) and Parcel.GoodsExists('AutoIncFieldValue') then
                        fAutoIncFieldValue:=Parcel.GetIntegerGoods('AutoIncFieldValue')
                     else
                        fAutoIncFieldValue:=0;
                  end;
            end
         else
            begin
               LastErrorCode:='0206307';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206308';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.AppendRecord(TableName: string; RecordCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream: TMemoryStream;
begin
   if (RecordCds.RecordCount<>1) or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206305';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0206306';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   result:=CdsZipToStream(RecordCds,Stream);
   if not Result then
      begin
         Stream.free;
         LastErrorCode:='0206309';
         LastErrorText:='Zip record data failed.';
         exit;
      end;
   Stream.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',63);
   if fInTransaction then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      begin
         if trim(fTargetDatabaseId)<>'' then
            Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
      end;
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStreamGoods('RecordData',Stream);
   Parcel.PutBooleanGoods('OldMethodFlag',true);   // Ϸ
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   Stream.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end
               else
                  begin
                     if (not fInTransaction) and Parcel.GoodsExists('AutoIncFieldValue') then
                        fAutoIncFieldValue:=Parcel.GetIntegerGoods('AutoIncFieldValue')
                     else
                        fAutoIncFieldValue:=0;
                  end;
            end
         else
            begin
               LastErrorCode:='0206307';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206308';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.UpdateRecord(TableName: string; FilterCondition: string; RecordParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream0,Stream: TMemoryStream;
begin
   if (RecordParcel.GoodsCount<=0) or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206405';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0206406';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Stream0:=TMemoryStream.Create;
   Stream:=TMemoryStream.Create;
   try
      result:=RecordParcel.SaveToStream(Stream0);
      Stream0.Position:=0;
      GZCompressStream(Stream0,Stream);
   except
      result:=false;
   end;
   Stream0.Free;
   if not Result then
      begin
         Stream.free;
         LastErrorCode:='0206409';
         LastErrorText:='Zip record data failed.';
         exit;
      end;
   Stream.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',64);
   if fInTransaction then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      begin
         if trim(fTargetDatabaseId)<>'' then
            Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
      end;
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStringGoods('FilterCondition',FilterCondition);
   Parcel.PutStreamGoods('RecordData',Stream);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   Stream.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206407';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206408';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.UpdateRecord(TableName: string; FilterCondition: string; RecordCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream: TMemoryStream;
begin
   if (RecordCds.RecordCount<>1) or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206405';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0206406';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   result:=CdsZipToStream(RecordCds,Stream);
   if not Result then
      begin
         Stream.free;
         LastErrorCode:='0206409';
         LastErrorText:='Zip record data failed.';
         exit;
      end;
   Stream.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',64);
   if fInTransaction then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      begin
         if trim(fTargetDatabaseId)<>'' then
            Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
      end;
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStringGoods('FilterCondition',FilterCondition);
   Parcel.PutStreamGoods('RecordData',Stream);
   Parcel.PutBooleanGoods('OldMethodFlag',true);   // Ϸ
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   Stream.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206407';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206408';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.ReadTableHead(TableName: string; var Cds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(TableName)='') or (trim(fTargetDatabaseId)='') or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206504';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',65);
   Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     Cds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('TableHead',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        Cds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if not result then
                        begin
                           LastErrorCode:='0206507';
                           LastErrorText:='Decompress the return data fail.';
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206505';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206506';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.GenerateId(TableName: string; KeyFieldName: string; FilterCondition: string;
         FieldMask: ansistring; ConstList: ansistring; var NewId: ansistring): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetDatabaseId)='') or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206604';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(TableName)='') or (trim(KeyFieldName)='') or (trim(FieldMask)='') then
      begin
         LastErrorCode:='0206601';
         LastErrorText:='Invalid parameters.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',66);
   Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStringGoods('KeyFieldName',KeyFieldName);
   Parcel.PutStringGoods('FilterCondition',FilterCondition);
   Parcel.PutEncryptStringGoods('FieldMask',FieldMask);
   Parcel.PutEncryptStringGoods('ConstList',ConstList);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  NewId:=Parcel.GetEncryptStringGoods('NewKeyFieldId')
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206605';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206606';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.FreeUnusedId(TableName: string; KeyFieldName: string; FilterCondition: string; IdToFree: ansistring): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetDatabaseId)='') or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206704';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(TableName)='') or (trim(KeyFieldName)='') or (trim(IdToFree)='') then
      begin
         LastErrorCode:='0206701';
         LastErrorText:='Invalid parameters.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',67);
   Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStringGoods('KeyFieldName',KeyFieldName);
   Parcel.PutStringGoods('FilterCondition',FilterCondition);
   Parcel.PutEncryptStringGoods('IdToFree',IdToFree);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206705';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206706';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.CommitCdsDelta(TableName: String; KeyFieldList: String; Delta: OleVariant): boolean;
var
   Parcel: TQBParcel;
   Stream,TargetStream: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0206806';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   if (trim(fTargetDatabaseId)='') and (not fInTransaction) then
      begin
         LastErrorCode:='0206807';
         LastErrorText:='Target database invalid.';
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   TargetStream:=TMemoryStream.Create;
   try
      VariantToStream(Delta,Stream);
      Stream.Position:=0;
      GZCompressStream(Stream,TargetStream);
      result:=true;
   except
      result:=false;
   end;
   if not Result then
      begin
         TargetStream.Free;
         Stream.free;
         LastErrorCode:='0206810';
         LastErrorText:='Zip delta data failed.';
         exit;
      end;
   TargetStream.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',68);
   if fInTransaction then
      Parcel.PutInt64Goods('TransSessionId',fDbTransSessionId)
   else
      begin
         if trim(fTargetDatabaseId)<>'' then
            Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
      end;
   Parcel.PutStringGoods('TableName',TableName);
   Parcel.PutStringGoods('KeyFieldList',KeyFieldList);
   Parcel.PutStreamGoods('DeltaData',TargetStream);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   Stream.Free;
   TargetStream.free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0206808';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0206809';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.CreateQuerySession(SqlCommand: String; var QueryId: integer; var QuerySessionId: integer; var AllRecords: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetDatabaseId)='') or (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0207004';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',70);
   Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStringGoods('SQLCommand',SqlCommand);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     QueryId:=Parcel.GetIntegerGoods('QueryId');
                     QuerySessionId:=Parcel.GetIntegerGoods('QuerySessionId');
                     fPageQueryId:=QueryId;
                     fPageQuerySessionId:=QuerySessionId;
                     AllRecords:=Parcel.GetIntegerGoods('AllRecords');
                     if QueryHeartbeatThread.Suspended then
                        QueryHeartbeatThread.Resume;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0207005';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0207006';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.QueryPageData(QueryId: integer; QuerySessionId: integer; PageSize: integer; PageNumber: integer; ToCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0207104';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',71);
   Parcel.PutIntegerGoods('QueryId',QueryId);
   Parcel.PutIntegerGoods('QuerySessionId',QuerySessionId);
   Parcel.PutIntegerGoods('PageSize',PageSize);
   Parcel.PutIntegerGoods('PageNumber',PageNumber);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     ToCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('PageData',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        ToCds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if not result then
                        begin
                           LastErrorCode:='0207107';
                           LastErrorText:='Decompress the return data fail.';
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0207105';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0207106';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDBAccessor.FreeQuerySession(QueryId: integer; QuerySessionId: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0207304';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',73);
   Parcel.PutIntegerGoods('QueryId',QueryId);
   Parcel.PutIntegerGoods('QuerySessionId',QuerySessionId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if not QueryHeartbeatThread.Suspended then
                        QueryHeartbeatThread.Suspend;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0207305';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0207306';
         LastErrorText:='Communication error on commit task.';
      end;
end;

procedure TDBAccessor.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=String(LastErrorCode);
   ErrorText:=String(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
