//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit DbConfig;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  DbClient, UserConnection, QBParcel, QBCommon, QBZlibExGz, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TDatabaseRecord=record
      DatabaseId: AnsiString;
      DatabaseDesc: AnsiString;
      DatabaseType: integer;
      DatabaseProvider: ansistring;
      DatabaseConnectString: AnsiString;
   end;

type
   TFetchDatabaseListSuccessEvent = procedure (Sender: TObject) of object;
   TFetchDatabaseListFailEvent = procedure (Sender: TObject) of Object;
   TAddDatabaseSuccessEvent = procedure (Sender: TObject) of object;
   TAddDatabaseFailEvent = procedure (Sender: TObject) of object;
   TUpdateDatabaseSuccessEvent = procedure (Sender: TObject) of object;
   TUpdateDatabaseFailEvent = procedure (Sender: TObject) of object;
   TRemoveDatabaseSuccessEvent = procedure (Sender: TObject) of object;
   TRemoveDatabaseFailEvent = procedure (Sender: TObject) of object;

type
  TDbConfig = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fTargetNodeId: AnsiString;
     fOnFetchDatabaseListSuccess: TFetchDatabaseListSuccessEvent;
     fOnFetchDatabaseListFail: TFetchDatabaseListFailEvent;
     fOnAddDatabaseSuccess: TAddDatabaseSuccessEvent;
     fOnAddDatabaseFail: TAddDatabaseFailEvent;
     fOnUpdateDatabaseSuccess: TUpdateDatabaseSuccessEvent;
     fOnUpdateDatabaseFail: TUpdateDatabaseFailEvent;
     fOnRemoveDatabaseSuccess: TRemoveDatabaseSuccessEvent;
     fOnRemoveDatabaseFail: TRemoveDatabaseFailEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetTargetNodeId(aDatabaseId: AnsiString);

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     function FetchDatabaseList(aCds: TClientDataset): boolean;
     function AddDatabase(aDatabase: TDatabaseRecord): boolean;
     function UpdateDatabase(OldDatabaseId: AnsiString; aDatabase: TDatabaseRecord): boolean;
     function RemoveDatabase(aDatabaseId: AnsiString): boolean;
     function TestDatabase(aDatabaseId: AnsiString): boolean;
     function TestExternalDatabase(Provider: String; ConnectString: string): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property OnFetchDatabaseListSuccess: TFetchDatabaseListSuccessEvent read fOnFetchDatabaseListSuccess write fOnFetchDatabaseListSuccess;
     property OnFetchDatabaseListFail: TFetchDatabaseListFailEvent read fOnFetchDatabaseListFail write fOnFetchDatabaseListFail;
     property OnAddDatabaseSuccess: TAddDatabaseSuccessEvent read fOnAddDatabaseSuccess write fOnAddDatabaseSuccess;
     property OnAddDatabaseFail: TAddDatabaseFailEvent read fOnAddDatabaseFail write fOnAddDatabaseFail;
     property OnUpdateDatabaseSuccess: TUpdateDatabaseSuccessEvent read fOnUpdateDatabaseSuccess write fOnUpdateDatabaseSuccess;
     property OnUpdateDatabaseFail: TUpdateDatabaseFailEvent read fOnUpdateDatabaseFail write fOnUpdateDatabaseFail;
     property OnRemoveDatabaseSuccess: TRemoveDatabaseSuccessEvent read fOnRemoveDatabaseSuccess write fOnRemoveDatabaseSuccess;
     property OnRemoveDatabaseFail: TRemoveDatabaseFailEvent read fOnRemoveDatabaseFail write fOnRemoveDatabaseFail;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TDbConfig]);
end;

procedure TDbConfig.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TDbConfig.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TDbConfig.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TDbConfig.SetTargetNodeId(aDatabaseId: AnsiString);
begin
   if StrIComp(PAnsiChar(aDatabaseId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aDatabaseId;
end;

constructor TDbConfig.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   LastErrorCode:='';
   LastErrorText:='';
end;

destructor TDbConfig.Destroy;
begin
   fOnFetchDatabaseListSuccess:=nil;
   fOnFetchDatabaseListFail:=nil;
   fOnAddDatabaseSuccess:=nil;
   fOnAddDatabaseFail:=nil;
   fOnUpdateDatabaseSuccess:=nil;
   fOnUpdateDatabaseFail:=nil;
   fOnRemoveDatabaseSuccess:=nil;
   fOnRemoveDatabaseFail:=nil;
  inherited;
end;

function TDbConfig.FetchDatabaseList(aCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204302';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnFetchDatabaseListFail) then
            fOnFetchDatabaseListFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',43);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     aCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('DatabaseList',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        aCds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if result then
                        begin
                           if Assigned(fOnFetchDatabaseListSuccess) then
                              fOnFetchDatabaseListSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0204305';
                           LastErrorText:='Decompress the return dataset fail.';
                           if Assigned(fOnFetchDatabaseListFail) then
                              fOnFetchDatabaseListFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnFetchDatabaseListFail) then
                        fOnFetchDatabaseListFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0204303';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnFetchDatabaseListFail) then
                  fOnFetchDatabaseListFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204304';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnFetchDatabaseListFail) then
            fOnFetchDatabaseListFail(self);
      end;
end;

function TDbConfig.AddDatabase(aDatabase: TDatabaseRecord): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204404';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnAddDatabaseFail) then
            fOnAddDatabaseFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',44);
   Parcel.PutEncryptStringGoods('DatabaseId',aDatabase.DatabaseId);
   Parcel.PutEncryptStringGoods('DatabaseDescription',aDatabase.DatabaseDesc);
   Parcel.PutIntegerGoods('DatabaseType',aDatabase.DatabaseType);
   Parcel.PutEncryptStringGoods('DatabaseProvider',aDatabase.DatabaseProvider);
   Parcel.PutEncryptStringGoods('DatabaseConnectString',aDatabase.DatabaseConnectString);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnAddDatabaseSuccess) then
                        fOnAddDatabaseSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnAddDatabaseFail) then
                        fOnAddDatabaseFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0204405';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnAddDatabaseFail) then
                  fOnAddDatabaseFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204406';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnAddDatabaseFail) then
            fOnAddDatabaseFail(self);
      end;
end;

function TDbConfig.UpdateDatabase(OldDatabaseId: AnsiString; aDatabase: TDatabaseRecord): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204504';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnUpdateDatabaseFail) then
            fOnUpdateDatabaseFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',45);
   Parcel.PutEncryptStringGoods('OldDatabaseId',OldDatabaseId);
   Parcel.PutEncryptStringGoods('DatabaseId',aDatabase.DatabaseId);
   Parcel.PutEncryptStringGoods('DatabaseDescription',aDatabase.DatabaseDesc);
   Parcel.PutIntegerGoods('DatabaseType',aDatabase.DatabaseType);
   Parcel.PutEncryptStringGoods('DatabaseProvider',aDatabase.DatabaseProvider);
   Parcel.PutEncryptStringGoods('DatabaseConnectString',aDatabase.DatabaseConnectString);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnUpdateDatabaseSuccess) then
                        fOnUpdateDatabaseSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnUpdateDatabaseFail) then
                        fOnUpdateDatabaseFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0204505';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnUpdateDatabaseFail) then
                  fOnUpdateDatabaseFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204506';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnUpdateDatabaseFail) then
            fOnUpdateDatabaseFail(self);
      end;
end;

function TDbConfig.RemoveDatabase(aDatabaseId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204604';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnRemoveDatabaseFail) then
            fOnRemoveDatabaseFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',46);
   Parcel.PutEncryptStringGoods('DatabaseId',aDatabaseId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnRemoveDatabaseSuccess) then
                        fOnRemoveDatabaseSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnRemoveDatabaseFail) then
                        fOnRemoveDatabaseFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0204605';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnRemoveDatabaseFail) then
                  fOnRemoveDatabaseFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204606';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnRemoveDatabaseFail) then
            fOnRemoveDatabaseFail(self);
      end;
end;

function TDbConfig.TestDatabase(aDatabaseId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204704';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',47);
   Parcel.PutEncryptStringGoods('DatabaseId',aDatabaseId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0204705';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204706';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TDbConfig.TestExternalDatabase(Provider: String; ConnectString: string): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204804';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',48);
   Parcel.PutStringGoods('Provider',Provider);
   Parcel.PutStringGoods('ConnectString',ConnectString);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0204805';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204806';
         LastErrorText:='Communication error on commit task.';
      end;
end;

procedure TDbConfig.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=String(LastErrorCode);
   ErrorText:=String(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
