//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit DllPlugin;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  DbClient, UserConnection, QBParcel, QBCommon, QBZlibExGz, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TPluginRecord=record
      PluginId: AnsiString;
      PluginDesc: AnsiString;
      PluginFilename: string;
      PluginPassword: AnsiString;
      PluginThreadMode: integer;
      PluginExecMode: integer;
      PluginDbSupport: integer;
   end;
   
type
   TFetchPluginListSuccessEvent = procedure (Sender: TObject) of object;
   TFetchPluginListFailEvent = procedure (Sender: TObject) of Object;
   TAddPluginSuccessEvent = procedure (Sender: TObject) of object;
   TAddPluginFailEvent = procedure (Sender: TObject) of object;
   TUpdatePluginSuccessEvent = procedure (Sender: TObject) of object;
   TUpdatePluginFailEvent = procedure (Sender: TObject) of object;
   TRemovePluginSuccessEvent = procedure (Sender: TObject) of object;
   TRemovePluginFailEvent = procedure (Sender: TObject) of object;
   TBinaryRPCSuccessEvent = procedure (Sender: TObject) of object;
   TBinaryRPCFailEvent = procedure (Sender: TObject) of object;

type
  TDllPlugin = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fTargetNodeId: AnsiString;
     fTargetDatabaseId: AnsiString;
     fOnFetchPluginListSuccess: TFetchPluginListSuccessEvent;
     fOnFetchPluginListFail: TFetchPluginListFailEvent;
     fOnAddPluginSuccess: TAddPluginSuccessEvent;
     fOnAddPluginFail: TAddPluginFailEvent;
     fOnUpdatePluginSuccess: TUpdatePluginSuccessEvent;
     fOnUpdatePluginFail: TUpdatePluginFailEvent;
     fOnRemovePluginSuccess: TRemovePluginSuccessEvent;
     fOnRemovePluginFail: TRemovePluginFailEvent;
     fOnBinaryRPCSuccess: TBinaryRPCSuccessEvent;
     fOnBinaryRPCFail: TBinaryRPCFailEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetTargetNodeId(aNodeId: AnsiString);
     procedure SetTargetDatabaseId(aDatabaseId: AnsiString);

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     function FetchPluginList(aCds: TClientDataset): boolean;
     function AddPlugin(aPlugin: TPluginRecord): boolean;
     function UpdatePlugin(OldPluginId: AnsiString; OldPluginPassword: AnsiString; aPlugin: TPluginRecord): boolean;
     function RemovePlugin(aPluginId: AnsiString; aPluginPassword: AnsiString): boolean;
     function BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property TargetDatabaseId: AnsiString read fTargetDatabaseId write SetTargetDatabaseId;
     property OnFetchPluginListSuccess: TFetchPluginListSuccessEvent read fOnFetchPluginListSuccess write fOnFetchPluginListSuccess;
     property OnFetchPluginListFail: TFetchPluginListFailEvent read fOnFetchPluginListFail write fOnFetchPluginListFail;
     property OnAddPluginSuccess: TAddPluginSuccessEvent read fOnAddPluginSuccess write fOnAddPluginSuccess;
     property OnAddPluginFail: TAddPluginFailEvent read fOnAddPluginFail write fOnAddPluginFail;
     property OnUpdatePluginSuccess: TUpdatePluginSuccessEvent read fOnUpdatePluginSuccess write fOnUpdatePluginSuccess;
     property OnUpdatePluginFail: TUpdatePluginFailEvent read fOnUpdatePluginFail write fOnUpdatePluginFail;
     property OnRemovePluginSuccess: TRemovePluginSuccessEvent read fOnRemovePluginSuccess write fOnRemovePluginSuccess;
     property OnRemovePluginFail: TRemovePluginFailEvent read fOnRemovePluginFail write fOnRemovePluginFail;
     property OnBinaryRPCSuccess: TBinaryRPCSuccessEvent read fOnBinaryRPCSuccess write fOnBinaryRPCSuccess;
     property OnBinaryRPCFail: TBinaryRPCFailEvent read fOnBinaryRPCFail write fOnBinaryRPCFail;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TDllPlugin]);
end;

procedure TDllPlugin.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TDllPlugin.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) then
     begin
        if (AComponent = fUserConn) then
           fUserConn := nil;
     end;
end;

procedure TDllPlugin.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TDllPlugin.SetTargetNodeId(aNodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(aNodeId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aNodeId;
end;

procedure TDllPlugin.SetTargetDatabaseId(aDatabaseId: AnsiString);
begin
   if StrIComp(PAnsiChar(aDatabaseId),PAnsiChar(fTargetDatabaseId))=0 then
      exit;
   fTargetDatabaseId:=aDatabaseId;
end;

constructor TDllPlugin.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   fTargetNodeId:='';
   fTargetDatabaseId:='';
   LastErrorCode:='';
   LastErrorText:='';
end;

destructor TDllPlugin.Destroy;
begin
   fOnFetchPluginListSuccess:=nil;
   fOnFetchPluginListFail:=nil;
   fOnAddPluginSuccess:=nil;
   fOnAddPluginFail:=nil;
   fOnUpdatePluginSuccess:=nil;
   fOnUpdatePluginFail:=nil;
   fOnRemovePluginSuccess:=nil;
   fOnRemovePluginFail:=nil;
   fOnBinaryRPCSuccess:=nil;
   fOnBinaryRPCFail:=nil;
  inherited;
end;

function TDllPlugin.FetchPluginList(aCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204902';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnFetchPluginListFail) then
            fOnFetchPluginListFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',49);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     aCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('PluginList',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        aCds.LoadFromStream(Stream2);
                        result:=aCds.Active;
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if result then
                        begin
                           if Assigned(fOnFetchPluginListSuccess) then
                              fOnFetchPluginListSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0204905';
                           LastErrorText:='Decompress the return dataset fail.';
                           if Assigned(fOnFetchPluginListFail) then
                              fOnFetchPluginListFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnFetchPluginListFail) then
                        fOnFetchPluginListFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0204903';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnFetchPluginListFail) then
                  fOnFetchPluginListFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204904';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnFetchPluginListFail) then
            fOnFetchPluginListFail(self);
      end;
end;

function TDllPlugin.AddPlugin(aPlugin: TPluginRecord): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205004';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnAddPluginFail) then
            fOnAddPluginFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',50);
   Parcel.PutEncryptStringGoods('PluginId',aPlugin.PluginId);
   Parcel.PutEncryptStringGoods('PluginDescription',aPlugin.PluginDesc);
   Parcel.PutStringGoods('PluginFilename',aPlugin.PluginFilename);
   Parcel.PutEncryptStringGoods('PluginPassword',aPlugin.PluginPassword);
   Parcel.PutIntegerGoods('PluginThreadMode',aPlugin.PluginThreadMode);
   Parcel.PutIntegerGoods('PluginExecMode',aPlugin.PluginExecMode);
   Parcel.PutIntegerGoods('PluginDbSupport',aPlugin.PluginDbSupport);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnAddPluginSuccess) then
                        fOnAddPluginSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnAddPluginFail) then
                        fOnAddPluginFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0205005';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnAddPluginFail) then
                  fOnAddPluginFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0205006';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnAddPluginFail) then
            fOnAddPluginFail(self);
      end;
end;

function TDllPlugin.UpdatePlugin(OldPluginId: AnsiString; OldPluginPassword: AnsiString; aPlugin: TPluginRecord): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205104';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnUpdatePluginFail) then
            fOnUpdatePluginFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',51);
   Parcel.PutEncryptStringGoods('OldPluginId',OldPluginId);
   Parcel.PutEncryptStringGoods('OldPluginPassword',OldPluginPassword);
   Parcel.PutEncryptStringGoods('PluginId',aPlugin.PluginId);
   Parcel.PutEncryptStringGoods('PluginDescription',aPlugin.PluginDesc);
   Parcel.PutStringGoods('PluginFilename',aPlugin.PluginFilename);
   Parcel.PutEncryptStringGoods('PluginPassword',aPlugin.PluginPassword);
   Parcel.PutIntegerGoods('PluginThreadMode',aPlugin.PluginThreadMode);
   Parcel.PutIntegerGoods('PluginExecMode',aPlugin.PluginExecMode);
   Parcel.PutIntegerGoods('PluginDbSupport',aPlugin.PluginDbSupport);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnUpdatePluginSuccess) then
                        fOnUpdatePluginSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnUpdatePluginFail) then
                        fOnUpdatePluginFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0205105';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnUpdatePluginFail) then
                  fOnUpdatePluginFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0205106';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnUpdatePluginFail) then
            fOnUpdatePluginFail(self);
      end;
end;

function TDllPlugin.RemovePlugin(aPluginId: AnsiString; aPluginPassword: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205204';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnRemovePluginFail) then
            fOnRemovePluginFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',52);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnRemovePluginSuccess) then
                        fOnRemovePluginSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnRemovePluginFail) then
                        fOnRemovePluginFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0205205';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnRemovePluginFail) then
                  fOnRemovePluginFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0205206';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnRemovePluginFail) then
            fOnRemovePluginFail(self);
      end;
end;

function TDllPlugin.BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream,Stream1: TMemoryStream;
   ok: boolean;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0205305';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnBinaryRPCFail) then
            fOnBinaryRPCFail(self);
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Stream1:=TMemoryStream.Create;
   try
      ok:=InParcel.SaveToStream(Stream);
      if ok then
         begin
            Stream.Position:=0;
            GZCompressStream(Stream,Stream1);
            ok:=(Stream1.size>0);
         end;
   except
      ok:=false;
   end;
   if not ok then
      begin
         Stream.Free;
         Stream1.Free;
         LastErrorCode:='0205308';
         LastErrorText:='Local zip InParcel failed.';
         if Assigned(fOnBinaryRPCFail) then
            fOnBinaryRPCFail(self);
         result:=false;
         exit;
      end;
   Stream1.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',53);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   if trim(fTargetDatabaseId)<>'' then
      Parcel.PutEncryptStringGoods('DatabaseId',fTargetDatabaseId);
   Parcel.PutStreamGoods('InParcelStream',Stream1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('OutParcelStream') then
                        begin
                           Stream.Clear;
                           Stream1.Clear;
                           try
                              Parcel.GetStreamGoods('OutParcelStream',Stream);
                              GZDecompressStream(Stream,Stream1);
                              Stream1.Position:=0;
                              OutParcel.Clear;
                              result:=OutParcel.LoadFromStream(Stream1);
                           except
                              result:=false;
                           end;
                        end
                     else
                        Result:=true;
                     if result then
                        begin
                           if Assigned(fOnBinaryRPCSuccess) then
                              fOnBinaryRPCSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0205309';
                           LastErrorText:='Local unzip OutParcel failed.';
                           if Assigned(fOnBinaryRPCFail) then
                              fOnBinaryRPCFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnBinaryRPCFail) then
                        fOnBinaryRPCFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0205306';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnBinaryRPCFail) then
                  fOnBinaryRPCFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0205307';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnBinaryRPCFail) then
            fOnBinaryRPCFail(self);
      end;
   Stream.Free;
   Stream1.Free;
end;

procedure TDllPlugin.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=String(LastErrorCode);
   ErrorText:=String(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
