//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit FileBlockThread;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
   windows, Messages, SysUtils, Classes, QBParcel, TaskCommon;

type
   TBlockThread=class(TThread)
   private
      MainObj: TComponent;
      TransferType: integer;
      fHandle: integer;
      BlockId: integer;
      BlockSize: integer;
      TransferFileName: string;
      RequestBody: AnsiString;
      ResponseBody: AnsiString;

   protected
      procedure Execute; override;

   public
      Constructor Create(ParentObj: TComponent; SuspendedOnCreate: boolean); Overload;
      Destructor Destroy; Override;

   end;

implementation

uses FileTransfer;

Constructor TBlockThread.Create(ParentObj: TComponent; SuspendedOnCreate: boolean);
begin
   Inherited Create(SuspendedOnCreate);
   freeOnTerminate:=true;
   MainObj:=ParentObj;
end;

Destructor TBlockThread.Destroy;
begin
   Inherited Destroy;
end;

procedure TBlockThread.Execute;
var
   MainTransfer: TFileTransfer;
   parcel: TQBParcel;
   ok: boolean;
   errortext: ansistring;
begin
   MainTransfer:=(MainObj as TFileTransfer);
   while not terminated do
      begin
         ok:=MainTransfer.AllocTask(TransferType,fHandle,BlockId,BlockSize,TransferFileName);
         if not ok then
            begin
               Synchronize(Self.Suspend);
               sleep(1);
               continue;
            end;
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=MainTransfer.UserConnection.TransferKey;
         Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
         Parcel.PutEncryptStringGoods('RequestUserId',MainTransfer.UserConnection.UserId+'@'+MainTransfer.UserConnection.UserNodeId);
         if StrIComp(PAnsiChar(MainTransfer.UserConnection.UserNodeId),PAnsiChar(MainTransfer.TargetNodeId))=0 then
            Parcel.PutInt64Goods('SessionId',MainTransfer.UserConnection.SessionId)
         else
            Parcel.PutInt64Goods('SessionId',MainTransfer.UserConnection.PublicSessionId);
         Parcel.PutStringGoods('FileName',TransferFileName);
         Parcel.PutIntegerGoods('BlockSize',BlockSize);
         Parcel.PutIntegerGoods('BlockOffset',BlockId*MainTransfer.FileBlockSize);
         if TransferType=1 then
            begin
               Parcel.PutIntegerGoods('TaskId',36);           
               EnterCriticalSection(MainTransfer.ReadWriteCs);
               FileSeek(fHandle, BlockId*MainTransfer.FileBlockSize, 0);
               ok:=Parcel.PutFileBlockGoods('BlockData',fHandle,BlockSize);     
               LeaveCriticalSection(MainTransfer.ReadWriteCs);
            end
         else
            Parcel.PutIntegerGoods('TaskId',35);        
         Parcel.SaveToStringWithLength(RequestBody);
         Parcel.clear;
         if ok then
            begin
               ok:=RemoteExecute(MainTransfer.UserConnection,MainTransfer.TargetNodeId,RequestBody,60,ResponseBody);
               if ok then
                  begin
                     Parcel.LoadFromString(ResponseBody);
                     ok:=(Parcel.GoodsCount>0);
                     if ok then
                        begin
                           ok:=Parcel.GetBooleanGoods('ProcessResult');
                           if ok then
                              begin
                                 if TransferType=1 then
                                    MainTransfer.TaskCallBack(BlockId,true,'')
                                 else
                                    begin
                                       EnterCriticalSection(MainTransfer.ReadWriteCs);
                                       FileSeek(fHandle,BlockId*MainTransfer.FileBlockSize,0);
                                       ok:=Parcel.GetFileBlockGoods('BlockData',fHandle);
                                       LeaveCriticalSection(MainTransfer.ReadWriteCs);
                                       if ok then
                                          MainTransfer.TaskCallBack(BlockId,true,'')
                                       else
                                          MainTransfer.TaskCallBack(BlockId,false,'Write data block to local file failed.');
                                    end;
                              end
                           else
                              begin
                                 ErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                                 MainTransfer.TaskCallBack(BlockId,false,ErrorText);
                              end;
                        end
                     else
                        MainTransfer.TaskCallBack(BlockId,false,'The response message is invalid.');
                  end
               else
                  MainTransfer.TaskCallBack(BlockId,false,'Communication error.');
            end
         else
            MainTransfer.TaskCallBack(BlockId,false,'Read local file error.');
         Parcel.free;
         sleep(1);
      end;
end;

end.
