//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit FileTransfer;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
  Windows, Messages, SysUtils, Classes, Forms,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  MidasLib, QBZlibExGz, DbClient, UserConnection, QBParcel, QBCommon, FileBlockThread, TaskCommon;

type
   TUploadSuccess = procedure (Sender: TObject) of object;
   TUploadFail = procedure (Sender: TObject) of Object;
   TDownloadSuccess = procedure (Sender: TObject) of Object;
   TDownloadFail = procedure (Sender: TObject) of Object;
   TTransferProgress = procedure (MaxBlocks: integer; OkBlocks: integer) of object;
   TTransferFolderProgress = procedure (Sender: TObject; AllObjects: integer; OkObjects: integer) of Object;

type
   TObjRecord=Record
      ObjName: string;
      ObjType: integer;
      ObjLevel: integer;
      Expanded: boolean;
   end;

type
  TFileTransfer = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTargetNodeId: AnsiString;
     fTaskTimeout: integer;
     fThreadCount: integer;
     fAssociatedPluginPassword: AnsiString;
     fOnUploadSuccess: TUploadSuccess;
     fOnUploadFail: TUploadFail;
     fOnDownloadSuccess: TDownloadSuccess;
     fOnDownloadFail: TDownloadFail;
     fOnFileTransferProgress: TTransferProgress;
     fOnFolderTransferProgress: TTransferFolderProgress;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     TransferFileName: string;
     Working: boolean;            
     TransferType: integer;       
     fHandle: integer;            
     AllBytes: integer;
     AllBlocks: integer;
     BlockOks: array of boolean;
     TaskIndex: integer;
     OkIndex: integer;
     TransferAbort: boolean;
     FolderAbort: boolean;
     TaskCs: TRTLCriticalSection;
     CallBackCs: TRTLCriticalSection;
     fUploadOkIndex: integer;                   
     fDownloadOkIndex: integer;                 
     Threads: array of TBlockThread;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetTargetNodeId(aNodeId: AnsiString);
     procedure SetThreadCount(aThreads: integer);
     procedure SetDllPassword(Password: AnsiString);
     function BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
     function CreateDirrectory(TargetDirectory: String): boolean;
     function ExpandFolder(TargetPath: String; ObjListCDS: TClientDataset): boolean;

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     FileBlockSize: integer;
     ReadWriteCs: TRTLCriticalSection;
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     function AllocTask(var aTransferType: integer; var aHandle: integer; var aBlockId: integer; var aBlockSize: integer; var aFileName: string): boolean;
     procedure TaskCallBack(aBlockId: integer; Successful: boolean; ErrText: AnsiString);
     function GetRemoteFileSize(aRemoteFileName: string; var aFileSize: integer): boolean;
     function CreateRemoteFile(aRemoteFileName: string; aFileSize: integer): boolean;
     function WriteFileBlock(aRemoteFileName: string; aBlockOffset: integer; aBlockSize: integer; aBlockData: AnsiString): boolean;
     function ReadFileBlock(aRemoteFileName: string; aBlockOffset: integer; aBlockSize: integer; var aBlockData: AnsiString): boolean;
     function UploadFile(aLocalFileName: string; aRemoteFileName: string; FirstBlockId: integer): boolean;
     function DownloadFile(aRemoteFileName: string; aLocalFileName: string; FirstBlockId: integer): boolean;
     function AbortTransfer: boolean;
     function LastTransferType: integer;
     function LastFileSize: integer;
     function LastFileBlocks: integer;
     function LastOkBlockId: integer;
     function UploadFolder(LocalFolder: string; RemoteFolder: string; BeginFileIndex: integer): boolean;
     function DownloadFolder(RemoteFolder: string; LocalFolder: string; BeginFileIndex: integer): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);
     property UploadOkIndex: integer read fUploadOkIndex;
     property DownloadOkIndex: integer read fDownloadOkIndex;

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property ThreadCount: integer read fThreadCount write SetThreadCount;
     property AssociatedPluginPassword: AnsiString read fAssociatedPluginPassword write SetDllPassword;
     property OnUploadSuccess: TUploadSuccess read fOnUploadSuccess write fOnUploadSuccess;
     property OnUploadFail: TUploadFail read fOnUploadFail write fOnUploadFail;
     property OnDownloadSuccess: TDownloadSuccess read fOnDownloadSuccess write fOnDownloadSuccess;
     property OnDownloadFail: TDownloadFail read fOnDownloadFail write fOnDownloadFail;
     property OnFileTransferProgress: TTransferProgress read fOnFileTransferProgress write fOnFileTransferProgress;
     property OnFolderTransferProgress: TTransferFolderProgress read fOnFolderTransferProgress write fOnFolderTransferProgress;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TFileTransfer]);
end;

procedure TFileTransfer.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TFileTransfer.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TFileTransfer.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TFileTransfer.SetTargetNodeId(aNodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(aNodeId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aNodeId;
end;

procedure TFileTransfer.SetThreadCount(aThreads: integer);
var
   i: integer;
begin
   if fThreadCount=aThreads then
      exit;
   if (aThreads<1) or (aThreads>16) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (1-16 threads allowed)','Invalid',0);
         exit;
      end;
   if not (csDesigning in ComponentState) then
      begin
         for i:=0 to fThreadCount - 1 do
            begin
               if Threads[i].Suspended then
                  begin
                     Threads[i].Terminate;
                     Threads[i].Resume;
                  end
               else
                  Threads[i].Terminate;
               WaitForSingleObject(Threads[i].Handle,INFINITE);
            end;
      end;
   fThreadCount:=athreads;
   if not (csDesigning in ComponentState) then
      begin
         SetLength(Threads,fThreadCount);
         for I := 0 to fThreadCount - 1 do
            Threads[i]:=TBlockThread.Create(self,true);
      end;
end;

procedure TFileTransfer.SetDllPassword(Password: AnsiString);
begin
   if fAssociatedPluginPassword=trim(password) then
      exit;
   fAssociatedPluginPassword:=password;
end;

constructor TFileTransfer.Create(AOwner: TComponent);
var
   i: integer;
begin
   inherited;
   fUserConn:=nil;
   fTargetNodeId:='';
   fTaskTimeout:=30;
   fThreadCount:=4;
   LastErrorCode:='';
   LastErrorText:='';
   FileBlockSize:=128*1024;
   Working:=false;
   okindex:=-1;
   fAssociatedPluginPassword:='';
   InitializeCriticalSection(TaskCs);
   InitializeCriticalSection(CallBackCs);
   InitializeCriticalSection(ReadWriteCs);
   if not (csDesigning in ComponentState) then
      begin
         SetLength(Threads,fThreadCount);
         for I := 0 to fThreadCount - 1 do
            Threads[i]:=TBlockThread.Create(self,true);
      end;
end;

destructor TFileTransfer.Destroy;
var
   i: integer;
begin
  fOnUploadSuccess:=nil;
  fOnUploadFail:=nil;
  fOnDownloadSuccess:=nil;
  fOnDownloadFail:=nil;
  fOnFileTransferProgress:=nil;
   if not (csDesigning in ComponentState) then
      begin
         for i := 0 to fThreadCount - 1 do
            begin
               if assigned(Threads[i]) then
                  begin
                     if Threads[i].Suspended then
                        begin
                           Threads[i].Terminate;
                           Threads[i].Resume;
                        end
                     else
                        Threads[i].Terminate;
                     WaitForSingleObject(Threads[i].Handle,INFINITE);
                  end;
            end;
      end;
  DeleteCriticalSection(TaskCs);
  DeleteCriticalSection(CallBackCs);
  DeleteCriticalSection(ReadWriteCs);
  inherited;
end;

function TFileTransfer.BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream,Stream1: TMemoryStream;
   ok: boolean;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Stream1:=TMemoryStream.Create;
   try
      ok:=InParcel.SaveToStream(Stream);
      if ok then
         begin
            Stream.Position:=0;
            GZCompressStream(Stream,Stream1);
            ok:=(Stream1.size>0);
         end;
   except
      ok:=false;
   end;
   if not ok then
      begin
         Stream.Free;
         Stream1.Free;
         result:=false;
         exit;
      end;
   Stream1.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',53);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   Parcel.PutStreamGoods('InParcelStream',Stream1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('OutParcelStream') then
                        begin
                           Stream.Clear;
                           Stream1.Clear;
                           try
                              Parcel.GetStreamGoods('OutParcelStream',Stream);
                              GZDecompressStream(Stream,Stream1);
                              Stream1.Position:=0;
                              OutParcel.Clear;
                              result:=OutParcel.LoadFromStream(Stream1);
                           except
                              result:=false;
                           end;
                        end
                     else
                        Result:=true;
                  end;
            end;
         Parcel.Free;
      end;
   Stream.Free;
   Stream1.Free;
end;

function TFileTransfer.CreateDirrectory(TargetDirectory: String): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',4);
   InParcel.PutStringGoods('DirName',TargetDirectory);
   result:=BinaryRPC('remotefiles',fAssociatedPluginPassword,InParcel,nil);
   InParcel.Free;
end;

function TFileTransfer.ExpandFolder(TargetPath: String; ObjListCDS: TClientDataset): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',16);
   InParcel.PutStringGoods('FolderName',TargetPath);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fAssociatedPluginPassword,InParcel,OutParcel);
   if result then
      OutParcel.GetCDSGoods('FolderObjects',ObjListCds);
   InParcel.Free;
   OutParcel.Free;
end;

function TFileTransfer.AllocTask(var aTransferType: integer; var aHandle: integer; var aBlockId: integer; var aBlockSize: integer; var aFileName: string): boolean;
begin
   EnterCriticalSection(TaskCs);
   if TransferAbort or (TaskIndex>=AllBlocks) then
      begin
         LeaveCriticalSection(TaskCs);
         result:=false;
         exit;
      end;
   aTransferType:=TransferType;
   aHandle:=fHandle;
   aBlockId:=TaskIndex;
   if TaskIndex<(AllBlocks-1) then
      aBlockSize:=FileBlockSize
   else
      aBlockSize:=AllBytes-(AllBlocks-1)*FileBlockSize;
   aFileName:=TransferFileName;
   inc(TaskIndex);
   LeaveCriticalSection(TaskCs);
   result:=true;
end;

procedure TFileTransfer.TaskCallBack(aBlockId: integer; Successful: boolean; ErrText: AnsiString);
var
   i,j,k: integer;
begin
   if successful then
      begin
         EnterCriticalSection(CallBackCs);
         BlockOks[aBlockId]:=true;
         if okindex=-1 then
            k:=0
         else
            k:=okindex;
         j:=okindex;
         for i := k to AllBlocks - 1 do
            begin
               if BlockOks[i] then
                  j:=i
               else
                  break;
            end;
         if j<>okindex then
            begin
               okIndex:=j;
               if Assigned(fOnFileTransferProgress) then
                  fOnFileTransferProgress(AllBlocks,OkIndex+1);
            end;
         LeaveCriticalSection(CallBackCs);
      end
   else
      begin
         EnterCriticalSection(TaskCs);
         TransferAbort:=true;
         LastErrorText:=ErrText;
         LeaveCriticalSection(TaskCs);
      end;
end;

function TFileTransfer.GetRemoteFileSize(aRemoteFileName: string; var aFileSize: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0203303';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',33);
   Parcel.PutStringGoods('FileName',aRemoteFileName);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  aFileSize:=Parcel.GetIntegerGoods('FileSize')
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0203305';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0203304';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TFileTransfer.CreateRemoteFile(aRemoteFileName: string; aFileSize: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0203403';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',34);
   Parcel.PutStringGoods('FileName',aRemoteFileName);
   Parcel.PutIntegerGoods('FileSize',aFileSize);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0203405';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0203404';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TFileTransfer.WriteFileBlock(aRemoteFileName: string; aBlockOffset: integer; aBlockSize: integer; aBlockData: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0203603';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',36);
   Parcel.PutStringGoods('FileName',aRemoteFileName);
   Parcel.PutIntegerGoods('BlockOffset',aBlockOffset);
   Parcel.PutIntegerGoods('BlockSize',aBlockSize);
   Parcel.PutBinaryGoods('BlockData',aBlockData);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0203605';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0203604';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TFileTransfer.ReadFileBlock(aRemoteFileName: string; aBlockOffset: integer; aBlockSize: integer; var aBlockData: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0203503';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',35);
   Parcel.PutStringGoods('FileName',aRemoteFileName);
   Parcel.PutIntegerGoods('BlockOffset',aBlockOffset);
   Parcel.PutIntegerGoods('BlockSize',aBlockSize);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  aBlockData:=Parcel.GetBinaryGoods('BlockData')
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0203505';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0203504';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TFileTransfer.UploadFile(aLocalFileName: string; aRemoteFileName: string; FirstBlockId: integer): boolean;
var
   i,j,k: integer;
begin
   if Working then
      begin
         LastErrorCode:='0250101';
         LastErrorText:='System busy.';
         if Assigned(fOnUploadFail) then
            fOnUploadFail(self);
         result:=false;
         exit;
      end;

   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0250102';
         LastErrorText:='System not ready to transfer.';
         if Assigned(fOnUploadFail) then
            fOnUploadFail(self);
         result:=false;
         exit;
      end;
   if not Fileexists(aLocalFileName) then
      begin
         LastErrorCode:='0250103';
         LastErrorText:='Local file not found.';
         if Assigned(fOnUploadFail) then
            fOnUploadFail(self);
         result:=false;
         exit;
      end;
   fHandle:=FileOpen(aLocalFileName,fmOpenRead);
   if fHandle=-1 then
      begin
         LastErrorCode:='0250104';
         LastErrorText:='Open local file failed.';
         if Assigned(fOnUploadFail) then
            fOnUploadFail(self);
         result:=false;
         exit;
      end;
   AllBytes:=FileSeek(fhandle,0,2);
   if AllBytes=0 then
      AllBlocks:=0
   else
      begin
         AllBlocks:=(AllBytes div FileBlockSize);
         if AllBlocks*FileBlockSize<AllBytes then
            inc(AllBlocks);
      end;
   if FirstBlockId>AllBlocks then
      begin
         FileClose(fHandle);
         LastErrorCode:='0250105';
         LastErrorText:='Invalid first block Id.';
         if Assigned(fOnUploadFail) then
            fOnUploadFail(self);
         result:=false;
         exit;
      end;
   if FirstBlockId=0 then
      begin
         fTaskTimeout:=180;          
         if not CreateRemoteFile(aRemoteFileName,AllBytes) then
            begin
               FileClose(fHandle);
               if Assigned(fOnUploadFail) then
                  fOnUploadFail(self);
               result:=false;
               exit;
            end;
      end;
   fTaskTimeout:=60;          
   Working:=true;
   TransferType:=1;                       
   setlength(BlockOks,AllBlocks);
   for i := 0 to AllBlocks - 1 do
      BlockOks[i]:=(i<FirstBlockId);
   TaskIndex:=FirstBlockId;
   OkIndex:=FirstBlockId-1;
   TransferAbort:=false;
   TransferFileName:=aRemoteFileName;
   if AllBlocks>0 then
      begin
         if fThreadCount>AllBlocks then
            j:=AllBlocks
         else
            j:=fThreadCount;
         k:=0;
         for i:=0 to fThreadCount-1 do
            begin
               if Threads[i].Suspended then
                  begin
                     threads[i].Resume;
                     inc(k);
                  end;
               if k>=j then
                  break;
            end;
         while (OkIndex<(AllBlocks-1)) and (not TransferAbort) do
            Application.ProcessMessages;
         result:=((OkIndex>=(AllBlocks-1)) and (not TransferAbort));
      end
   else
      result:=true;
   FileClose(fHandle);
   if result then
      begin
         if Assigned(fOnUploadSuccess) then
            fOnUploadSuccess(self);
      end
   else
      begin
         LastErrorCode:='0250106';
         if Assigned(fOnUploadFail) then
            fOnUploadFail(self);
      end;
   Working:=false;
end;

function TFileTransfer.DownloadFile(aRemoteFileName: string; aLocalFileName: string; FirstBlockId: integer): boolean;
var
   i,j,k: integer;
begin
   if Working then
      begin
         LastErrorCode:='0250201';
         LastErrorText:='System busy.';
         if Assigned(fOnDownloadFail) then
            fOnDownloadFail(self);
         result:=false;
         exit;
      end;
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0250202';
         LastErrorText:='System not ready to transfer.';
         if Assigned(fOnDownloadFail) then
            fOnDownloadFail(self);
         result:=false;
         exit;
      end;
   fTaskTimeout:=60;          
   if not GetRemoteFileSize(aRemoteFileName,AllBytes) then
      begin
         LastErrorCode:='0250203';
         LastErrorText:='Get remote file size failed.';
         if Assigned(fOnDownloadFail) then
            fOnDownloadFail(self);
         result:=false;
         exit;
      end;
   if AllBytes=0 then
      AllBlocks:=0
   else
      begin
         AllBlocks:=(AllBytes div FileBlockSize);
         if AllBlocks*FileBlockSize<AllBytes then
            inc(AllBlocks);
      end;
   if FirstBlockId>AllBlocks then
      begin
         LastErrorCode:='0250204';
         LastErrorText:='Invalid first block Id.';
         if Assigned(fOnDownloadFail) then
            fOnDownloadFail(self);
         result:=false;
         exit;
      end;

   if FirstBlockId=0 then
      begin
         fHandle:=FileCreate(aLocalFileName);
         if fHandle=-1 then
            begin
               LastErrorCode:='0250205';
               LastErrorText:='Create local file failed.';
               if Assigned(fOnDownloadFail) then
                  fOnDownloadFail(self);
               result:=false;
               exit;
            end;
      end
   else
      begin
         fHandle:=FileOpen(aLocalFileName,fmOpenWrite);
         if fHandle=-1 then
            begin
               LastErrorCode:='0250206';
               LastErrorText:='Open old local file failed.';
               if Assigned(fOnDownloadFail) then
                  fOnDownloadFail(self);
               result:=false;
               exit;
            end;
      end;
   fTaskTimeout:=60;          
   Working:=true;
   TransferType:=2;                       
   setlength(BlockOks,AllBlocks);
   for i := 0 to AllBlocks - 1 do
      BlockOks[i]:=(i<FirstBlockId);
   TaskIndex:=FirstBlockId;
   OkIndex:=FirstBlockId-1;
   TransferAbort:=false;
   TransferFileName:=aRemoteFileName;
   if AllBlocks>0 then
      begin
         if fThreadCount>AllBlocks then
            j:=AllBlocks
         else
            j:=fThreadCount;
         k:=0;
         for i:=0 to fThreadCount-1 do
            begin
               if Threads[i].Suspended then
                  begin
                     threads[i].Resume;
                     inc(k);
                  end;
               if k>=j then
                  break;
            end;
         while (OkIndex<(AllBlocks-1)) and (not TransferAbort) do
            Application.ProcessMessages;
         result:=((OkIndex>=(AllBlocks-1)) and (not TransferAbort));
      end
   else
      result:=true;
   FileClose(fHandle);
   if result then
      begin
         if Assigned(fOnDownloadSuccess) then
            fOnDownloadSuccess(self);
      end
   else
      begin
         LastErrorCode:='0250207';
         LastErrorText:='Download file block failed.';
         if Assigned(fOnDownloadFail) then
            fOnDownloadFail(self);
      end;
   Working:=false;
end;

function TFileTransfer.AbortTransfer: boolean;
begin
   if working then
      begin
         EnterCriticalSection(TaskCs);
         TransferAbort:=true;
         LeaveCriticalSection(TaskCs);
      end;
   FolderAbort:=true;
   result:=true;
end;

function TFileTransfer.LastTransferType: integer;
begin
   result:=TransferType;
end;

function TFileTransfer.LastFileSize: integer;
begin
   result:=AllBytes;
end;

function TFileTransfer.LastFileBlocks: integer;
begin
   result:=AllBlocks;
end;

function TFileTransfer.LastOkBlockId: integer;
begin
   result:=OkIndex;
end;

function TFileTransfer.UploadFolder(LocalFolder: string; RemoteFolder: string; BeginFileIndex: integer): boolean;
var
   sr: TSearchRec;
   ObjCount,i,j: integer;
   Objs: array of TObjRecord;
   tmp: TObjRecord;
   fPath,rpath: string;
   ok: boolean;
begin
    {$IFDEF VER130}
    ok:=FileCtrl.directoryexists(LocalFolder);
    {$ELSE}
    ok:=SysUtils.directoryexists(LocalFolder);
    {$ENDIF}
    if not ok then
       begin
          result:=false;
          exit;
       end;
    fPath:=LocalFolder;
    if fPath[length(fPath)]<>'\' then
       fPath:=fPath+'\';
    rPath:=RemoteFolder;
    if rPath[length(rPath)]<>'\' then
       rPath:=rPath+'\';
    ObjCount:=1;
    SetLength(Objs,ObjCount);
    Objs[0].ObjName:=fPath;
    Objs[0].ObjType:=0;
    Objs[0].ObjLevel:=0;
    Objs[0].Expanded:=false;
    i:=0;
    while i<ObjCount do
       begin
         if Objs[i].ObjType<>0 then
            begin
               inc(i);
               continue;
            end;
         if SysUtils.FindFirst(Objs[i].ObjName+'*.*', faAnyFile, sr) = 0 then
            begin
              repeat
                 if (sr.Name<>'.') and (sr.name<>'..') and ((sr.Attr and faVolumeID)=0) then
                    begin
                       j:=ObjCount;
                       ObjCount := ObjCount + 1;
                       SetLength(Objs,ObjCount);
                       if (sr.Attr and faDirectory)<>0 then
                          Objs[j].ObjType:=0
                       else
                          Objs[j].ObjType:=1;
                       Objs[j].ObjName:=Objs[i].ObjName+sr.Name;
                       if Objs[j].ObjType=0 then
                          begin
                             Objs[j].ObjName:=Objs[j].ObjName+'\';
                             Objs[j].Expanded:=false;
                          end
                       else
                          Objs[j].Expanded:=true;
                       Objs[j].ObjLevel:=Objs[i].ObjLevel+1;
                    end;
              until SysUtils.FindNext(sr) <> 0;
              SysUtils.FindClose(sr);
            end;
         inc(i);
       end;
    for i:=0 to ObjCount-2 do
       for j:=i to ObjCount-1 do
          begin
             if Objs[j].ObjName<Objs[i].ObjName then
                begin
                   tmp:=Objs[i];
                   Objs[i]:=Objs[j];
                   Objs[j]:=tmp;
                end;
          end;
    for I := 0 to ObjCount - 1 do
       delete(Objs[i].ObjName,1,length(fPath));
   FolderAbort:=false;
   for I := BeginFileIndex to ObjCount - 1 do
      begin
         if FolderAbort then
            begin
               fUploadOkIndex:=i-1;
               result:=false;
               exit;
            end;
         if Objs[i].ObjType=0 then
            begin
               if not self.CreateDirrectory(rPath+Objs[i].ObjName) then
                  begin
                     fUploadOkIndex:=i-1;
                     result:=false;
                     exit;
                  end;
            end
         else
            begin
               if not UploadFile(fPath+Objs[i].ObjName,rPath+Objs[i].ObjName,0) then
                  begin
                     fUploadOkIndex:=i-1;
                     result:=false;
                     exit;
                  end;
            end;
         if assigned(fOnFolderTransferProgress) then
            fOnFolderTransferProgress(self,ObjCount,i+1);
         SafeDelay(10);
      end;
   fUploadOkIndex:=ObjCount-1;
   result:=true;
end;

function TFileTransfer.DownloadFolder(RemoteFolder: string; LocalFolder: string; BeginFileIndex: integer): boolean;
var
   Cds: TClientDataset;
   i: integer;
   rPath,lPath,lName,rName: string;
   ok: boolean;
begin
    lPath:=LocalFolder;
    if lPath[length(lPath)]<>'\' then
       lPath:=lPath+'\';
    rPath:=RemoteFolder;
    if rPath[length(rPath)]<>'\' then
       rPath:=rPath+'\';
   Cds:=TClientDataset.Create(nil);
   if not ExpandFolder(RemoteFolder,Cds) then
      begin
         Cds.free;
         result:=false;
         exit;
      end;
   i:=0;
   FolderAbort:=false;
   while not Cds.Eof do
      begin
         if FolderAbort then
            begin
               fDownloadOkIndex:=i-1;
               Cds.free;
               result:=false;
               exit;
            end;
         if i<BeginFileIndex then
            begin
               Cds.next;
               inc(i);
               continue;
            end;
         if Cds.FieldByname('ObjectType').asinteger=0 then
            begin
               lName:=lPath+trim(Cds.FieldByName('ObjectName').asstring);
               CreateDir(lName);
               {$IFDEF VER130}
               ok:=FileCtrl.directoryexists(lName);
               {$ELSE}
               ok:=SysUtils.directoryexists(lName);
               {$ENDIF}
               if not ok then
                  begin
                     fDownloadOkIndex:=i-1;
                     Cds.free;
                     result:=false;
                     exit;
                  end;
            end
         else
            begin
               lName:=lPath+trim(Cds.FieldByName('ObjectName').asstring);
               rName:=rPath+trim(Cds.FieldByName('ObjectName').asstring);
               if not DownloadFile(rName,lName,0) then
                  begin
                     fDownloadOkIndex:=i-1;
                     Cds.free;
                     result:=false;
                     exit;
                  end;
            end;
         if assigned(fOnFolderTransferProgress) then
            fOnFolderTransferProgress(self,Cds.RecordCount,i+1);
         SafeDelay(100);
         Cds.next;
         inc(i);
      end;
   fDownloadOkIndex:=Cds.RecordCount-1;
   Cds.close;
   Cds.Free;
   Result:=true;
end;

procedure TFileTransfer.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=string(LastErrorCode);
   ErrorText:=string(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
