//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit GroupMemberHeartbeat;

interface

uses
   windows, Messages, SysUtils, Classes, Forms, QBParcel, TaskCommon;

type
   TGroupMemberHeartbeatThread=class(TThread)
   private
      MainObj: TComponent;
      HeartbeatInterval: integer;
      LastDateTime: TDateTime;
      RequestBody,ResponseBody: AnsiString;

   protected
      procedure Execute; override;

   public
      Constructor Create(ParentObj: TComponent; SuspendedOnCreate: boolean); Overload;
      Destructor Destroy; Override;

   end;

implementation

uses UserGroup;

Constructor TGroupMemberHeartbeatThread.Create(ParentObj: TComponent; SuspendedOnCreate: boolean);
begin
   Inherited Create(SuspendedOnCreate);
   freeOnTerminate:=true;
   MainObj:=ParentObj;
   HeartbeatInterval:=120;
   LastDateTime:=now;
end;

Destructor TGroupMemberHeartbeatThread.Destroy;
begin
   Inherited Destroy;
end;

procedure TGroupMemberHeartbeatThread.Execute;
var
   MainGroup: TUserGroup;
   Parcel: TQBParcel;
   ok: boolean;
   t0: TDateTime;
   i: integer;
begin
   MainGroup:=(MainObj as TUserGroup);
   while not terminated do
      begin
         t0:=now;
         while (not terminated) and (round((now-t0)*24*60*60)<5) do
            sleep(50);
         if not terminated then
            begin
               EnterCriticalSection(MainGroup.ActiveGroupCs);
               i:=0;
               while i<MainGroup.ActiveGroupCount do
                  begin
                     if terminated then
                        break;
                     if round((now-MainGroup.ActiveGroups[i].LastActiveTime)*24*60*60)<HeartbeatInterval then
                        begin
                           inc(i);
                           continue;
                        end;
                     Parcel:=TQBParcel.Create;
                     Parcel.EncryptKey:=MainGroup.UserConnection.TransferKey;
                     Parcel.PutEncryptStringGoods('SystemFlag','Message_Request');
                     Parcel.PutInt64Goods('SessionId',MainGroup.UserConnection.PublicSessionId);
                     Parcel.PutIntegerGoods('RequestId',8);
                     Parcel.PutEncryptStringGoods('GroupId',MainGroup.ActiveGroups[i].GroupId);
                     Parcel.PutEncryptStringGoods('UserId',MainGroup.UserConnection.UserId+'@'+MainGroup.UserConnection.UserNodeId);
                     Parcel.SaveToStringWithLength(RequestBody);
                     Parcel.Free;
                     ok:=RemoteMsgExecute(MainGroup.UserConnection,MainGroup.TargetNodeId,RequestBody,MainGroup.TaskTimeout,ResponseBody);
                     if ok then
                        begin
                           Parcel:=TQBParcel.Create;
                           Parcel.EncryptKey:=MainGroup.UserConnection.TransferKey;
                           Parcel.LoadFromString(ResponseBody);
                           if Parcel.GoodsCount>0 then
                              begin
                                 if not Parcel.GetBooleanGoods('ProcessResult') then
                                    begin
                                       Parcel.Free;
                                       MainGroup._FireAutoQuitEvent(MainGroup.ActiveGroups[i].GroupId);
                                    end
                                 else
                                    begin
                                       Parcel.Free;
                                       inc(i);
                                    end;
                              end
                           else
                              begin
                                 Parcel.Free;
                                 MainGroup._FireAutoQuitEvent(MainGroup.ActiveGroups[i].GroupId);
                              end;
                        end
                     else
                        MainGroup._FireAutoQuitEvent(MainGroup.ActiveGroups[i].GroupId);
                  end;
               LeaveCriticalSection(MainGroup.ActiveGroupCs);
            end;
      end;
end;

end.
