//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit MsgPoster;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  UserConnection, QBParcel, QBCommon, TaskCommon, DBClient
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TPostSuccessEvent = procedure (Sender: TObject) of object;
   TPostFailEvent = procedure (Sender: TObject) of Object;
   TPostProgressEvent = procedure (Sender: TObject; AllNodeCount: integer; OkNodeCount: integer) of Object;

type
  TMsgPoster = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fOnPostSuccess: TPostSuccessEvent;
     fOnPostFail: TPostFailEvent;
     fOnPostProgress: TPostProgressEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     function GetOnlineNodeList(var NodeIdList: TStringList): boolean;

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     function PostToOneUser(aTargetUserId: AnsiString; aMsgParcel: TQBParcel): boolean;
     function PostToNodeUsers(aTargetNodeId: AnsiString; aUserType: integer; aMsgParcel: TQBParcel): boolean;
     function PostToAllUsers(aUserType: integer; aMsgParcel: TQBParcel): boolean;
     function OneNodeBroadcast(aTargetNodeId: AnsiString; aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
     function AllNodeBroadcast(aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
     function OneNodeMulticast(aTargetNodeId: AnsiString; aMulticastIp: AnsiString; aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
     function AllNodeMulticast(aMulticastIp: AnsiString; aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
     function OneNodeUnicast(aTargetNodeId: AnsiString; aUnicastAddr: AnsiString; aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property OnPostSuccess: TPostSuccessEvent read fOnPostSuccess write fOnPostSuccess;
     property OnPostFail: TPostFailEvent read fOnPostFail write fOnPostFail;
     property OnPostProgress: TPostProgressEvent read fOnPostProgress write fOnPostProgress;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TMsgPoster]);
end;

procedure TMsgPoster.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TMsgPoster.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TMsgPoster.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

constructor TMsgPoster.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   LastErrorCode:='';
   LastErrorText:='';
end;

destructor TMsgPoster.Destroy;
begin
   fOnPostSuccess:=nil;
   fOnPostFail:=nil;
   fOnPostProgress:=nil;
  inherited;
end;

function TMsgPoster.GetOnlineNodeList(var NodeIdList: TStringList): boolean;
var
   Parcel: TQBParcel;
begin
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fUserConn.RootNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',14);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fUserConn.RootNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  NodeIdList.Text:=string(Parcel.GetEncryptStringGoods('OnlineNodeIdList'));
            end;
         Parcel.Free;
      end;
end;

function TMsgPoster.PostToOneUser(aTargetUserId: AnsiString; aMsgParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   j: integer;
   TargetUserId,TargetNodeId: AnsiString;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200505';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         result:=false;
         exit;
      end;
   TargetNodeId:=aTargetUserId;
   j:=pos(AnsiString('@'),TargetNodeId);
   TargetUserId:=copy(TargetNodeId,1,j-1);
   delete(TargetNodeId,1,j);
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(TargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',5);
   Parcel.PutEncryptStringGoods('TargetUserId',TargetUserId);
   Parcel.PutEncryptStringGoods('FromUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutIntegerGoods('MessageType',0);              
   Parcel.PutParcelGoods('MessageBody',aMsgParcel);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,TargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnPostSuccess) then
                        fOnPostSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnPostFail) then
                        fOnPostFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0200506';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnPostFail) then
                  fOnPostFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200507';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
      end;
end;

function TMsgPoster.PostToNodeUsers(aTargetNodeId: AnsiString; aUserType: integer; aMsgParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200604';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(aTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',6);
   Parcel.PutEncryptStringGoods('FromUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutIntegerGoods('MessageType',1);              
   Parcel.PutIntegerGoods('TargetUserType',aUserType);   
   Parcel.PutParcelGoods('MessageBody',aMsgParcel);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,aTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnPostSuccess) then
                        fOnPostSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnPostFail) then
                        fOnPostFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0200605';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnPostFail) then
                  fOnPostFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200606';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
      end;
end;

function TMsgPoster.PostToAllUsers(aUserType: integer; aMsgParcel: TQBParcel): boolean;
var
   i: integer;
   NodeIdList: TStringList;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200604';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         result:=false;
         exit;
      end;
   NodeIdList:=TStringList.Create;
   result:=GetOnlineNodeList(NodeIdList);
   result:=(result and (NodeIdList.count>0));
   if not result then
      begin
         LastErrorCode:='0200607';
         LastErrorText:='Get online node list failed.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         exit;
      end;
   for i:=0 to NodeIdList.Count - 1 do
      begin
         PostToNodeUsers(AnsiString(NodeIdList[i]),aUserType,aMsgParcel);
         if Assigned(fOnPostProgress) then
            fOnPostProgress(self,NodeIdList.Count,i+1);
      end;
end;

function TMsgPoster.OneNodeBroadcast(aTargetNodeId: AnsiString; aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200604';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(aTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',6);
   Parcel.PutEncryptStringGoods('FromUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutIntegerGoods('MessageType',2);              
   Parcel.PutIntegerGoods('BroadcastPort',aUDPPort);     
   Parcel.PutParcelGoods('MessageBody',aMsgParcel);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,aTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnPostSuccess) then
                        fOnPostSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnPostFail) then
                        fOnPostFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0200605';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnPostFail) then
                  fOnPostFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200606';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
      end;
end;

function TMsgPoster.AllNodeBroadcast(aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
var
   i: integer;
   NodeIdList: TStringList;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200604';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         result:=false;
         exit;
      end;
   NodeIdList:=TStringList.Create;
   result:=GetOnlineNodeList(NodeIdList);
   result:=(result and (NodeIdList.count>0));
   if not result then
      begin
         LastErrorCode:='0200607';
         LastErrorText:='Get online node list failed.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         exit;
      end;
   for i:=0 to NodeIdList.Count - 1 do
      begin
         OneNodeBroadcast(AnsiString(NodeIdList[i]),aUDPPort,aMsgParcel);
         if Assigned(fOnPostProgress) then
            fOnPostProgress(self,NodeIdList.Count,i+1);
      end;
end;

function TMsgPoster.OneNodeMulticast(aTargetNodeId: AnsiString; aMulticastIp: AnsiString; aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200604';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(aTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',6);
   Parcel.PutEncryptStringGoods('FromUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutIntegerGoods('MessageType',3);              
   Parcel.PutEncryptStringGoods('MulticastAddress',aMulticastIP);
   Parcel.PutIntegerGoods('MulticastPort',aUDPPort);     
   Parcel.PutParcelGoods('MessageBody',aMsgParcel);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,aTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnPostSuccess) then
                        fOnPostSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnPostFail) then
                        fOnPostFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0200605';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnPostFail) then
                  fOnPostFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200606';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
      end;
end;

function TMsgPoster.AllNodeMulticast(aMulticastIp: AnsiString; aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
var
   i: integer;
   NodeIdList: TStringList;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200604';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         result:=false;
         exit;
      end;
   NodeIdList:=TStringList.Create;
   result:=GetOnlineNodeList(NodeIdList);
   result:=(result and (NodeIdList.count>0));
   if not result then
      begin
         LastErrorCode:='0200607';
         LastErrorText:='Get online node list failed.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         exit;
      end;
   for i:=0 to NodeIdList.Count - 1 do
      begin
         OneNodeMulticast(AnsiString(NodeIdList[i]),aMulticastIP,aUDPPort,aMsgParcel);
         if Assigned(fOnPostProgress) then
            fOnPostProgress(self,NodeIdList.Count,i+1);
      end;
end;

function TMsgPoster.OneNodeUnicast(aTargetNodeId: AnsiString; aUnicastAddr: AnsiString; aUdpPort: integer; aMsgParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200604';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(aTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',6);
   Parcel.PutEncryptStringGoods('FromUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutIntegerGoods('MessageType',4);              
   Parcel.PutEncryptStringGoods('UnicastAddress',aUnicastAddr);
   Parcel.PutIntegerGoods('UnicastPort',aUDPPort);       
   Parcel.PutParcelGoods('MessageBody',aMsgParcel);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,aTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnPostSuccess) then
                        fOnPostSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnPostFail) then
                        fOnPostFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0200605';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnPostFail) then
                  fOnPostFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200606';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnPostFail) then
            fOnPostFail(self);
      end;
end;

procedure TMsgPoster.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=string(LastErrorCode);
   ErrorText:=string(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
