//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit MsgReceiver;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  UserConnection, QBParcel, QBTimer, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TMessageEvent = procedure(MsgParcel: TQBParcel; FromId: AnsiString) of object;
   TBackupMsgEvent = procedure(MsgIdList: TStringList) of object;
   TOnNodeRegistered = procedure(aNodeId: AnsiString) of object;
   TOnNodeUpdated = procedure(OldNodeId: AnsiString; NewNodeId: AnsiString) of object;
   TOnNodeRemoved = procedure(aNodeId: AnsiString) of object;
   TOnRootNodeDropped = procedure(Sender: TObject) of object;
   TOnRootNodeConnected = procedure(Sender: TObject) of object;
   TOnNodeOffline = procedure(aNodeId: AnsiString) of object;
   TOnMsgSourceUpdated = procedure(OldSourceId: AnsiString; NewSourceId: AnsiString) of object;
   TOnMsgSourceRemoved = procedure(SourceId: AnsiString) of object;
   TOnMsgSourceMessage = procedure(SourceId: ansistring; MsgParcel: TQBParcel) of object;
   TOnUserGroupUpdated = procedure(OldGroupId: AnsiString; NewGroupId: AnsiString) of object;
   TOnUserGroupRemoved = procedure(aGroupId: AnsiString) of object;
   TOnGroupMemberJoin = procedure(aGroupId: AnsiString; aUserId: AnsiString; aUserName: AnsiString) of object;
   TOnGroupMemberQuit = procedure(aGroupId: AnsiString; aUserId: AnsiString) of object;
   TOnUserGroupMessage = procedure(GroupId: ansistring; MsgParcel: TQBParcel) of object;

type
   TMessageRecord=record
      MsgParcel: TQBParcel;
      FromId: AnsiString;
   end;

type
  TMsgReceiver = class(TComponent)
  private
     fUserConn: TUserConnection;
     fActive: Boolean;
     fTaskTimeout: integer;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     MsgIdList: TStringList;
     EventTigger: TQBTimer;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     fOnOneUserMessage: TMessageEvent;
     fOnMultiUserMessage: TMessageEvent;
     fOnBackupMessages: TBackupMsgEvent;
     fOnNodeRegistered: TOnNodeRegistered;
     fOnNodeUpdated: TOnNodeUpdated;
     fOnNodeRemoved: TOnNodeRemoved;
     fOnRootNodeDropped: TOnRootNodeDropped;
     fOnNodeOffline: TOnNodeOffline;
     fOnRootNodeConnected: TOnRootNodeConnected;
     fOnMsgSourceUpdated: TOnMsgSourceUpdated;
     fOnMsgSourceRemoved: TOnMsgSourceRemoved;
     fOnUserGroupUpdated: TOnUserGroupUpdated;
     fOnUserGroupRemoved: TOnUserGroupRemoved;
     fOnGroupMemberJoin: TOnGroupMemberJoin;
     fOnGroupMemberQuit: TOnGroupMemberQuit;
     fOnMsgSourceMessage: TOnMsgSourceMessage;
     fOnUserGroupMessage: TOnUserGroupMessage;
     MsgQueueSize: integer;
     MsgQueue: array of TMessageRecord;
     MsgQueueFore: integer;
     MsgQueueRear: integer;
     MsgQueueEmpty: boolean;
     MsgQueueFull: boolean;
     MsgQueueCs: TRTLCriticalSection;
     DispatchTimer: TQBTimer;
     DispatcherActive: boolean;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetActive(aValue: Boolean);
     procedure SetTaskTimeout(seconds: integer);
     procedure FireEvent(Sender: TObject);
     function GetBackupMsgList(var MsgIdList: TStringList): boolean;
     function PushMessage(aMsg: TMessageRecord): boolean;
     function PopMessage(var aMsg: TMessageRecord): boolean;
     procedure DispatchMessage(MsgParcel: TQBParcel; FromId: AnsiString);
     procedure DispatchTimerTimer(Sender: TObject);

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     procedure Open;
     procedure Close;
     function FetchMessageBody(MsgId: AnsiString; var MsgParcel: TQBParcel; var FromUserId: AnsiString): boolean;
     function DropMessageBody(MsgId: AnsiString): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property Active: boolean read fActive write SetActive default false;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property OnOneUserMessage: TMessageEvent read fOnOneUserMessage write fOnOneUserMessage;
     property OnMultiUserMessage: TMessageEvent read fOnMultiUserMessage write fOnMultiUserMessage;
     property OnBackupMessages: TBackupMsgEvent read fOnBackupMessages write fOnBackupMessages;
     property OnNodeRegistered: TOnNodeRegistered read fOnNodeRegistered write fOnNodeRegistered;
     property OnNodeUpdated: TOnNodeUpdated read fOnNodeUpdated write fOnNodeUpdated;
     property OnNodeRemoved: TOnNodeRemoved read fOnNodeRemoved write fOnNodeRemoved;
     property OnRootNodeDropped: TOnRootNodeDropped read fOnRootNodeDropped write fOnRootNodeDropped;
     property OnNodeOffline: TOnNodeOffline read fOnNodeOffline write fOnNodeOffline;
     property OnRootNodeConnected: TOnRootNodeConnected read fOnRootNodeConnected write fOnRootNodeConnected;
     property OnMsgSourceUpdated: TOnMsgSourceUpdated read fOnMsgSourceUpdated write fOnMsgSourceUpdated;
     property OnMsgSourceRemoved: TOnMsgSourceRemoved read fOnMsgSourceRemoved write fOnMsgSourceRemoved;
     property OnMsgSourceMessage: TOnMsgSourceMessage read fOnMsgSourceMessage write fOnMsgSourceMessage;
     property OnUserGroupUpdated: TOnUserGroupUpdated read fOnUserGroupUpdated write fOnUserGroupUpdated;
     property OnUserGroupRemoved: TOnUserGroupRemoved read fOnUserGroupRemoved write fOnUserGroupRemoved;
     property OnGroupMemberJoin: TOnGroupMemberJoin read fOnGroupMemberJoin write fOnGroupMemberJoin;
     property OnGroupMemberQuit: TOnGroupMemberQuit read fOnGroupMemberQuit write fOnGroupMemberQuit;
     property OnUserGroupMessage: TOnUserGroupMessage read fOnUserGroupMessage write fOnUserGroupMessage;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TMsgReceiver]);
end;

procedure TMsgReceiver.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TMsgReceiver.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TMsgReceiver.SetActive(aValue: boolean);
var
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         raise(exception.Create('Operation fail before user login.'));
         exit;
      end;
   fActive:=aValue;
   if aValue then
      begin
         fUserConn._MessageBroker:=DispatchMessage;
         ok:=GetBackupMsgList(MsgIdList);
         if ok and (MsgIdList.Count>0) then
            EventTigger.Enabled:=true;
      end
   else
      fUserConn._MessageBroker:=nil;
end;

procedure TMsgReceiver.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

constructor TMsgReceiver.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fActive:=false;
   fTaskTimeout:=15;
   LastErrorCode:='';
   LastErrorText:='';
   MsgIdList:=TStringList.Create;
   EventTigger:=TQBTimer.Create(nil);
   EventTigger.Interval:=500;
   EventTigger.OnTimer:=FireEvent;
   EventTigger.Enabled:=false;
   MsgQueueSize:=512;
   SetLength(MsgQueue,MsgQueueSize);
   MsgQueueFore:=0;
   MsgQueueRear:=0;
   MsgQueueEmpty:=true;
   MsgQueueFull:=false;
   InitializeCriticalSection(MsgQueueCs);
   DispatchTimer:=TQBTimer.Create(nil);
   DispatchTimer.Interval:=100;
   DispatchTimer.OnTimer:=DispatchTimerTimer;
   DispatchTimer.Enabled:=false;
   DispatcherActive:=false;
end;

destructor TMsgReceiver.Destroy;
begin
   EventTigger.OnTimer:=nil;
   DispatchTimer.OnTimer:=nil;
   fOnOneUserMessage:=nil;
   fOnMultiUserMessage:=nil;
   EventTigger.Free;
   DispatchTimer.Free;
   MsgIdList.Free;
   if assigned(fUserConn) then
      fUserConn._MessageBroker:=nil;
   DeleteCriticalSection(MsgQueueCs);
   inherited;
end;

procedure TMsgReceiver.Open;
begin
   SetActive(true);
end;

procedure TMsgReceiver.Close;
begin
   SetActive(false);
end;

procedure TMsgReceiver.FireEvent(Sender: TObject);
begin
   EventTigger.Enabled:=false;
   if Assigned(fOnBackupMessages) then
      begin
         fOnBackupMessages(MsgIdList);
         MsgIdList.Clear;
      end;
end;

function TMsgReceiver.GetBackupMsgList(var MsgIdList: TStringList): boolean;
var
   Parcel: TQBParcel;
begin
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutInt64Goods('SessionId',fUserConn.SessionId);
   Parcel.PutIntegerGoods('TaskId',7);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fUserConn.UserNodeId,RequestBody,fTaskTimeout,ResponseBody);
   MsgIdList.Clear;
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  MsgIdList.Text:=string(Parcel.GetEncryptStringGoods('MessageIdList'));
            end;
         Parcel.Free;
      end;
end;

function TMsgReceiver.PushMessage(aMsg: TMessageRecord): boolean;
begin
   EnterCriticalSection(MsgQueueCs);
   if MsgQueueFull then
      begin
         LeaveCriticalSection(MsgQueueCs);
         result:=false;
         exit;
      end;
   MsgQueue[MsgQueueRear]:=aMsg;
   MsgQueueEmpty:=false;
   inc(MsgQueueRear);
   if MsgQueueRear=MsgQueueSize then
      MsgQueueRear:=0;
   if MsgQueueRear=MsgQueueFore then
      MsgQueueFull:=true;
   LeaveCriticalSection(MsgQueueCs);
   Result:=true;
end;

function TMsgReceiver.PopMessage(var aMsg: TMessageRecord): boolean;
begin
   EnterCriticalSection(MsgQueueCs);
   if MsgQueueEmpty then
      begin
         LeaveCriticalSection(MsgQueueCs);
         Result:=false;
         exit;
      end;
   aMsg:=MsgQueue[MsgQueueFore];
   MsgQueueFull:=false;
   inc(MsgQueueFore);
   if MsgQueueFore=MsgQueueSize then
      MsgQueueFore:=0;
   if MsgQueueFore=MsgQueueRear then
      MsgQueueEmpty:=true;
   LeaveCriticalSection(MsgQueueCs);
   Result:=true;
end;

procedure TMsgReceiver.DispatchMessage(MsgParcel: TQBParcel; FromId: AnsiString);
var
   Msg: TMessageRecord;
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   MsgParcel.SaveToStream(Stream);
   Msg.MsgParcel:=TQBParcel.Create;
   Msg.MsgParcel.EncryptKey:=fUserConn.TransferKey;
   Stream.Position:=0;
   Msg.MsgParcel.LoadFromStream(Stream);
   Stream.Free;
   Msg.FromId:=fromid;
   if PushMessage(Msg) then
      begin
         if not DispatcherActive then
            begin
               DispatcherActive:=true;
               DispatchTimer.Enabled:=true;
            end;
      end;
end;

procedure TMsgReceiver.DispatchTimerTimer(Sender: TObject);
var
   Msg: TMessageRecord;
   MainType: integer;
   NodeId,NodeId1,OldSourceId,SourceId,OldGroupId,GroupId,MemberId,MemberName: AnsiString;
begin
   DispatchTimer.Enabled:=false;
   while true do
      begin
         if not PopMessage(Msg) then
            begin
               DispatcherActive:=false;
               break;
            end;
         MainType:=Msg.MsgParcel.GetIntegerGoods('ResponseId');
         case MainType of
            4: begin
                  if Assigned(fOnOneUserMessage) then
                     fOnOneUserMessage(Msg.MsgParcel,Msg.FromId);
               end;
            5: begin
                  if Assigned(fOnMultiUserMessage) then
                     fOnMultiUserMessage(Msg.MsgParcel,Msg.FromId);
               end;
            6: begin
                  NodeId:=Msg.MsgParcel.GetEncryptStringGoods('NodeId');
                  if Assigned(fOnNodeRegistered) then
                     fOnNodeRegistered(NodeId);
               end;
            7: begin
                  NodeId:=Msg.MsgParcel.GetEncryptStringGoods('OldNodeId');
                  NodeId1:=Msg.MsgParcel.GetEncryptStringGoods('NodeId');
                  if Assigned(fOnNodeUpdated) then
                     fOnNodeUpdated(NodeId,NodeId1);
               end;
            8: begin
                  NodeId:=Msg.MsgParcel.GetEncryptStringGoods('NodeId');
                  if Assigned(fOnNodeRemoved) then
                     fOnNodeRemoved(NodeId);
               end;
            9: begin
                  if Assigned(fOnRootNodeDropped) then
                     fOnRootNodeDropped(self);
               end;
            10: begin
                  NodeId:=Msg.MsgParcel.GetEncryptStringGoods('OfflineNodeId');
                  if Assigned(fOnNodeOffline) then
                     fOnNodeOffline(NodeId);
               end;
            11: begin
                  if Assigned(fOnRootNodeConnected) then
                     fOnRootNodeConnected(self);
               end;
            13: begin
                   OldSourceId:=Msg.MsgParcel.GetEncryptStringGoods('OldSourceId');
                   SourceId:=Msg.MsgParcel.GetEncryptStringGoods('NewSourceId');
                   if Assigned(fOnMsgSourceUpdated) then
                      fOnMsgSourceUpdated(OldSourceId,SourceId);
                end;
            14: begin
                   SourceId:=Msg.MsgParcel.GetEncryptStringGoods('SourceId');
                   if Assigned(fOnMsgSourceRemoved) then
                      fOnMsgSourceRemoved(SourceId);
                end;
            15: begin
                   OldGroupId:=Msg.MsgParcel.GetEncryptStringGoods('OldGroupId');
                   GroupId:=Msg.MsgParcel.GetEncryptStringGoods('NewGroupId');
                   if Assigned(fOnUserGroupUpdated) then
                      fOnUserGroupUpdated(OldGroupId,GroupId);
                end;
            16: begin
                   GroupId:=Msg.MsgParcel.GetEncryptStringGoods('GroupId');
                   if Assigned(fOnUserGroupRemoved) then
                      fOnUserGroupRemoved(GroupId);
                end;
            17: begin
                   GroupId:=Msg.MsgParcel.GetEncryptStringGoods('GroupId');
                   MemberId:=Msg.MsgParcel.GetEncryptStringGoods('JoinUserId');
                   MemberName:=Msg.MsgParcel.GetEncryptStringGoods('JoinUserName');
                   if Assigned(fOnGroupMemberJoin) then
                      fOnGroupMemberJoin(GroupId,MemberId,MemberName);
                end;
            18: begin
                   GroupId:=Msg.MsgParcel.GetEncryptStringGoods('GroupId');
                   MemberId:=Msg.MsgParcel.GetEncryptStringGoods('QuitUserId');
                   if Assigned(fOnGroupMemberQuit) then
                      fOnGroupMemberQuit(GroupId,MemberId);
                end;
            19: begin
                   SourceId:=Msg.MsgParcel.GetEncryptStringGoods('SourceId');
                   if Assigned(fOnMsgSourceMessage) then
                      fOnMsgSourceMessage(SourceId,Msg.MsgParcel);
                end;
            20: begin
                   GroupId:=Msg.MsgParcel.GetEncryptStringGoods('GroupId');
                   if Assigned(fOnUserGroupMessage) then
                      fOnUserGroupMessage(GroupId,Msg.MsgParcel);
                end;
         end;
         Msg.MsgParcel.Free;
      end;
end;

function TMsgReceiver.FetchMessageBody(MsgId: AnsiString; var MsgParcel: TQBParcel; var FromUserId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200803';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutInt64Goods('SessionId',fUserConn.SessionId);
   Parcel.PutIntegerGoods('TaskId',8);
   Parcel.PutEncryptStringGoods('MessageId',MsgId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fUserConn.UserNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               FromUserId:=Parcel.GetEncryptStringGoods('FromUserId');
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     result:=Parcel.GetParcelGoods('MessageBody',MsgParcel);
                     if not result then
                        begin
                           LastErrorCode:='0200806';
                           LastErrorText:='The return message body is invalid.';
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0200804';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200805';
         LastErrorText:='Communication error on commit task.';
      end;
end;

function TMsgReceiver.DropMessageBody(MsgId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200903';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutInt64Goods('SessionId',fUserConn.SessionId);
   Parcel.PutIntegerGoods('TaskId',9);
   Parcel.PutEncryptStringGoods('MessageId',MsgId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fUserConn.UserNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0200904';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200905';
         LastErrorText:='Communication error on commit task.';
      end;
end;

procedure TMsgReceiver.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=string(LastErrorCode);
   ErrorText:=string(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
