//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit MsgSource;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  DbClient, UserConnection, QBParcel, QBCommon, QBZlibExGz, SourceMemberHeartbeat, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TActiveSourceRecord=record
       SourceId: AnsiString;
       LastActiveTime: TDateTime;
   end;

type
   TFetchSourceListSuccessEvent = procedure (Sender: TObject) of object;
   TFetchSourceListFailEvent = procedure (Sender: TObject) of Object;
   TAddSourceSuccessEvent = procedure (Sender: TObject) of object;
   TAddSourceFailEvent = procedure (Sender: TObject) of object;
   TUpdateSourceSuccessEvent = procedure (Sender: TObject) of object;
   TUpdateSourceFailEvent = procedure (Sender: TObject) of object;
   TRemoveSourceSuccessEvent = procedure (Sender: TObject) of object;
   TRemoveSourceFailEvent = procedure (Sender: TObject) of object;
   TFetchMemberListSuccessEvent = procedure (Sender: TObject) of object;
   TFetchMemberListFailEvent = procedure (Sender: TObject) of Object;
   TSubscribeSuccessEvent = procedure (Sender: TObject) of object;
   TSubscribeFailEvent = procedure (Sender: TObject) of object;
   TRecedeSuccessEvent = procedure (Sender: TObject) of object;
   TRecedeFailEvent = procedure (Sender: TObject) of object;
   TImitateSuccessEvent = procedure (Sender: TObject) of object;
   TImitateFailEvent = procedure (Sender: TObject) of object;
   TSourceAbnormalEvent = procedure (Sender: TObject; SourceId: AnsiString) of object;

type
  TMsgSource = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fTargetNodeId: AnsiString;
     fOnFetchSourceListSuccess: TFetchSourceListSuccessEvent;
     fOnFetchSourceListFail: TFetchSourceListFailEvent;
     fOnAddSourceSuccess: TAddSourceSuccessEvent;
     fOnAddSourceFail: TAddSourceFailEvent;
     fOnUpdateSourceSuccess: TUpdateSourceSuccessEvent;
     fOnUpdateSourceFail: TUpdateSourceFailEvent;
     fOnRemoveSourceSuccess: TRemoveSourceSuccessEvent;
     fOnRemoveSourceFail: TRemoveSourceFailEvent;
     fOnFetchMemberListSuccess: TFetchMemberListSuccessEvent;
     fOnFetchMemberListFail: TFetchMemberListFailEvent;
     fOnSubscribeSuccess: TSubscribeSuccessEvent;
     fOnSubscribeFail: TSubscribeFailEvent;
     fOnRecedeSuccess: TRecedeSuccessEvent;
     fOnRecedeFail: TRecedeFailEvent;
     fOnImitateSuccess: TImitateSuccessEvent;
     fOnImitateFail: TImitateFailEvent;
     fOnSourceAbnormal: TSourceAbnormalEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     HeartbeatThread: TSourceMemberHeartbeatThread;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetTargetNodeId(aNodeId: AnsiString);
     procedure AddActiveSource(SourceId: AnsiString);
     procedure RemoveActiveSource(SourceId: AnsiString);

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     ActiveSourceCount: integer;
     ActiveSources: array of TActiveSourceRecord;
     ActiveSourceCs: TRTLCriticalSection;
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     procedure _FireAutoQuitEvent(SourceId: AnsiString);
     function FetchSourceList(aCds: TClientDataset): boolean;
     function AddSource(aSourceId: AnsiString; aSourceName: AnsiString; aSourceType: integer;
              aSourceProtocol: ansistring; aSourceLocalIP: AnsiString; aSourcePort: integer; AllowedMembers: integer): boolean;
     function UpdateSource(OldSourceId: AnsiString; aSourceId: AnsiString; aSourceName: AnsiString; aSourceType: integer;
              aSourceProtocol: ansistring; aSourceLocalIP: AnsiString; aSourcePort: integer; AllowedMembers: integer): boolean;
     function RemoveSource(aSourceId: AnsiString): boolean;
     function FetchReaderList(aSourceId: AnsiString; aCds: TClientDataset): boolean;
     function SubscribeMessages(aSourceId: AnsiString): boolean;
     function RecedeMessages(aSourceId: AnsiString): boolean;
     function ImitateMessage(aSourceId: AnsiString; aMsgBody: AnsiString): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property OnFetchSourceListSuccess: TFetchSourceListSuccessEvent read fOnFetchSourceListSuccess write fOnFetchSourceListSuccess;
     property OnFetchSourceListFail: TFetchSourceListFailEvent read fOnFetchSourceListFail write fOnFetchSourceListFail;
     property OnAddSourceSuccess: TAddSourceSuccessEvent read fOnAddSourceSuccess write fOnAddSourceSuccess;
     property OnAddSourceFail: TAddSourceFailEvent read fOnAddSourceFail write fOnAddSourceFail;
     property OnUpdateSourceSuccess: TUpdateSourceSuccessEvent read fOnUpdateSourceSuccess write fOnUpdateSourceSuccess;
     property OnUpdateSourceFail: TUpdateSourceFailEvent read fOnUpdateSourceFail write fOnUpdateSourceFail;
     property OnRemoveSourceSuccess: TRemoveSourceSuccessEvent read fOnRemoveSourceSuccess write fOnRemoveSourceSuccess;
     property OnRemoveSourceFail: TRemoveSourceFailEvent read fOnRemoveSourceFail write fOnRemoveSourceFail;
     property OnFetchMemberListSuccess: TFetchMemberListSuccessEvent read fOnFetchMemberListSuccess write fOnFetchMemberListSuccess;
     property OnFetchMemberListFail: TFetchMemberListFailEvent read fOnFetchMemberListFail write fOnFetchMemberListFail;
     property OnSubscribeSuccess: TSubscribeSuccessEvent read fOnSubscribeSuccess write fOnSubscribeSuccess;
     property OnSubscribeFail: TSubscribeFailEvent read fOnSubscribeFail write fOnSubscribeFail;
     property OnRecedeSuccess: TRecedeSuccessEvent read fOnRecedeSuccess write fOnRecedeSuccess;
     property OnRecedeFail: TRecedeFailEvent read fOnRecedeFail write fOnRecedeFail;
     property OnImitateSuccess: TImitateSuccessEvent read fOnImitateSuccess write fOnImitateSuccess;
     property OnImitateFail: TImitateFailEvent read fOnImitateFail write fOnImitateFail;
     property OnSourceAbnormal: TSourceAbnormalEvent read fOnSourceAbnormal write fOnSourceAbnormal;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TMsgSource]);
end;

procedure TMsgSource.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TMsgSource.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TMsgSource.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TMsgSource.SetTargetNodeId(aNodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(aNodeId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aNodeId;
end;

constructor TMsgSource.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   LastErrorCode:='';
   LastErrorText:='';
   ActiveSourceCount:=0;
   SetLength(ActiveSources,0);
   InitializeCriticalSection(ActiveSourceCs);
   if not (csDesigning in ComponentState) then
      HeartbeatThread:=TSourceMemberHeartbeatThread.Create(self,true);
end;

destructor TMsgSource.Destroy;
begin
   fOnFetchSourceListSuccess:=nil;
   fOnFetchSourceListFail:=nil;
   fOnAddSourceSuccess:=nil;
   fOnAddSourceFail:=nil;
   fOnUpdateSourceSuccess:=nil;
   fOnUpdateSourceFail:=nil;
   fOnRemoveSourceSuccess:=nil;
   fOnRemoveSourceFail:=nil;
   fOnFetchMemberListSuccess:=nil;
   fOnFetchMemberListFail:=nil;
   fOnSubscribeSuccess:=nil;
   fOnSubscribeFail:=nil;
   fOnRecedeSuccess:=nil;
   fOnRecedeFail:=nil;
   fOnImitateSuccess:=nil;
   fOnImitateFail:=nil;
   fOnSourceAbnormal:=nil;
   if not (csDesigning in ComponentState) then
      begin
         if HeartbeatThread.Suspended then
            begin
               HeartbeatThread.Terminate;
               HeartbeatThread.Resume;
            end
         else
            HeartbeatThread.Terminate;
      end;
   ActiveSourceCount:=0;
   SetLength(ActiveSources,0);
   DeleteCriticalSection(ActiveSourceCs);
  inherited;
end;

procedure TMsgSource.AddActiveSource(SourceId: AnsiString);
var
   i,j: integer;
begin
   EnterCriticalSection(ActiveSourceCs);
   j:=-1;
   for i := 0 to ActiveSourceCount - 1 do
      if StrIComp(PAnsiChar(SourceId),PAnsiChar(ActiveSources[i].SourceId))=0 then
         begin
            j:=i;
            break;
         end;
   if j=-1 then
      begin
         j:=ActiveSourceCount;
         inc(ActiveSourceCount);
         SetLength(ActiveSources,ActiveSourceCount);
      end;
   ActiveSources[j].SourceId:=SourceId;
   ActiveSources[j].LastActiveTime:=now;
   if HeartbeatThread.Suspended then
      HeartbeatThread.Resume;
   LeaveCriticalSection(ActiveSourceCs);
end;

procedure TMsgSource.RemoveActiveSource(SourceId: AnsiString);
var
   i,j: integer;
begin
   EnterCriticalSection(ActiveSourceCs);
   j:=-1;
   for i := 0 to ActiveSourceCount - 1 do
      if StrIComp(PAnsiChar(SourceId),PAnsiChar(ActiveSources[i].SourceId))=0 then
         begin
            j:=i;
            break;
         end;
   if j<>-1 then
      begin
         for i := j to ActiveSourceCount - 2 do
            ActiveSources[i]:=ActiveSources[i+1];
         dec(ActiveSourceCount);
         SetLength(ActiveSources,ActiveSourceCount);
      end;
   if not HeartbeatThread.Suspended then
      HeartbeatThread.Suspend;
   LeaveCriticalSection(ActiveSourceCs);
end;

function TMsgSource.FetchSourceList(aCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0201502';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnFetchSourceListFail) then
            fOnFetchSourceListFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',15);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     aCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('SourceList',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        aCds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if result then
                        begin
                           if Assigned(fOnFetchSourceListSuccess) then
                              fOnFetchSourceListSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0201505';
                           LastErrorText:='Decompress the return dataset fail.';
                           if Assigned(fOnFetchSourceListFail) then
                              fOnFetchSourceListFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnFetchSourceListFail) then
                        fOnFetchSourceListFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0201503';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnFetchSourceListFail) then
                  fOnFetchSourceListFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0201504';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnFetchSourceListFail) then
            fOnFetchSourceListFail(self);
      end;
end;

function TMsgSource.AddSource(aSourceId: AnsiString; aSourceName: AnsiString; aSourceType: integer;
         aSourceProtocol: ansistring; aSourceLocalIP: AnsiString; aSourcePort: integer; AllowedMembers: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0201603';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnAddSourceFail) then
            fOnAddSourceFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',16);
   Parcel.PutEncryptStringGoods('SourceId',aSourceId);
   Parcel.PutEncryptStringGoods('SourceName',aSourceName);
   Parcel.PutIntegerGoods('SourceType',aSourceType);
   Parcel.PutEncryptStringGoods('SourceProtocol',aSourceProtocol);
   Parcel.PutEncryptStringGoods('SourceLocalIP',aSourceLocalIP);
   Parcel.PutIntegerGoods('SourcePort',aSourcePort);
   Parcel.PutIntegerGoods('AllowedMembers',AllowedMembers);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnAddSourceSuccess) then
                        fOnAddSourceSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnAddSourceFail) then
                        fOnAddSourceFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0201605';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnAddSourceFail) then
                  fOnAddSourceFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0201604';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnAddSourceFail) then
            fOnAddSourceFail(self);
      end;
end;

function TMsgSource.UpdateSource(OldSourceId: AnsiString; aSourceId: AnsiString; aSourceName: AnsiString; aSourceType: integer;
         aSourceProtocol: ansistring; aSourceLocalIP: AnsiString; aSourcePort: integer; AllowedMembers: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0201703';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnUpdateSourceFail) then
            fOnUpdateSourceFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',17);
   Parcel.PutEncryptStringGoods('OldSourceId',OldSourceId);
   Parcel.PutEncryptStringGoods('SourceId',aSourceId);
   Parcel.PutEncryptStringGoods('SourceName',aSourceName);
   Parcel.PutIntegerGoods('SourceType',aSourceType);
   Parcel.PutEncryptStringGoods('SourceProtocol',aSourceProtocol);
   Parcel.PutEncryptStringGoods('SourceLocalIP',aSourceLocalIP);
   Parcel.PutIntegerGoods('SourcePort',aSourcePort);
   Parcel.PutIntegerGoods('AllowedMembers',AllowedMembers);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               if StrIComp(PAnsiChar(OldSourceId),PAnsiChar(aSourceId))<>0 then
                  RemoveActiveSource(OldSourceId);
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnUpdateSourceSuccess) then
                        fOnUpdateSourceSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnUpdateSourceFail) then
                        fOnUpdateSourceFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0201705';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnUpdateSourceFail) then
                  fOnUpdateSourceFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0201704';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnUpdateSourceFail) then
            fOnUpdateSourceFail(self);
      end;
end;

function TMsgSource.RemoveSource(aSourceId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0201803';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnRemoveSourceFail) then
            fOnRemoveSourceFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',18);
   Parcel.PutEncryptStringGoods('SourceId',aSourceId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     RemoveActiveSource(aSourceId);
                     if Assigned(fOnRemoveSourceSuccess) then
                        fOnRemoveSourceSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnRemoveSourceFail) then
                        fOnRemoveSourceFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0201805';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnRemoveSourceFail) then
                  fOnRemoveSourceFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0201804';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnRemoveSourceFail) then
            fOnRemoveSourceFail(self);
      end;
end;

function TMsgSource.FetchReaderList(aSourceId: AnsiString; aCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0201902';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnFetchMemberListFail) then
            fOnFetchMemberListFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',19);
   Parcel.PutEncryptStringGoods('SourceId',aSourceId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     aCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('UserList',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        aCds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if result then
                        begin
                           if Assigned(fOnFetchMemberListSuccess) then
                              fOnFetchMemberListSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0201905';
                           LastErrorText:='Decompress the return dataset fail.';
                           if Assigned(fOnFetchMemberListFail) then
                              fOnFetchMemberListFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnFetchMemberListFail) then
                        fOnFetchMemberListFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0201903';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnFetchMemberListFail) then
                  fOnFetchMemberListFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0201904';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnFetchMemberListFail) then
            fOnFetchMemberListFail(self);
      end;
end;

function TMsgSource.SubscribeMessages(aSourceId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0202003';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnSubscribeFail) then
            fOnSubscribeFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',20);
   Parcel.PutEncryptStringGoods('SourceId',aSourceId);
   Parcel.PutEncryptStringGoods('UserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutEncryptStringGoods('UserName',fUserConn.UserName);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     AddActiveSource(aSourceId);
                     if Assigned(fOnSubscribeSuccess) then
                        fOnSubscribeSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnSubscribeFail) then
                        fOnSubscribeFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0202005';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnSubscribeFail) then
                  fOnSubscribeFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0202004';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnSubscribeFail) then
            fOnSubscribeFail(self);
      end;
end;

function TMsgSource.RecedeMessages(aSourceId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0202103';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnRecedeFail) then
            fOnRecedeFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',21);
   Parcel.PutEncryptStringGoods('SourceId',aSourceId);
   Parcel.PutEncryptStringGoods('UserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     RemoveActiveSource(aSourceId);
                     if Assigned(fOnRecedeSuccess) then
                        fOnRecedeSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnRecedeFail) then
                        fOnRecedeFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0202105';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnRecedeFail) then
                  fOnRecedeFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0202104';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnRecedeFail) then
            fOnRecedeFail(self);
      end;
end;

procedure TMsgSource._FireAutoQuitEvent(SourceId: AnsiString);
var
   i,j: integer;
begin
   j:=-1;
   for i := 0 to ActiveSourceCount - 1 do
      if StrIComp(PAnsiChar(SourceId),PAnsiChar(ActiveSources[i].SourceId))=0 then
         begin
            j:=i;
            break;
         end;
   if j<>-1 then
      begin
         for i := j to ActiveSourceCount - 2 do
            ActiveSources[i]:=ActiveSources[i+1];
         dec(ActiveSourceCount);
         SetLength(ActiveSources,ActiveSourceCount);
      end;
   if Assigned(fOnSourceAbnormal) then
      fOnSourceAbnormal(self,SourceId);
end;

function TMsgSource.ImitateMessage(aSourceId: AnsiString; aMsgBody: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0202305';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnRecedeFail) then
            fOnRecedeFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',23);
   Parcel.PutEncryptStringGoods('SourceId',aSourceId);
   Parcel.PutEncryptStringGoods('MessageBody',aMsgBody);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnImitateSuccess) then
                        fOnImitateSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnImitateFail) then
                        fOnImitateFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0202307';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnImitateFail) then
                  fOnImitateFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0202306';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnImitateFail) then
            fOnImitateFail(self);
      end;
   result:=true;
end;

procedure TMsgSource.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=String(LastErrorCode);
   ErrorText:=String(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
