//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit NodeManage;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  DbClient, UserConnection, QBParcel, QBCommon, QBZlibExGz, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TFetchNodeListSuccessEvent = procedure (Sender: TObject) of object;
   TFetchNodeListFailEvent = procedure (Sender: TObject) of Object;
   TAddNodeSuccessEvent = procedure (Sender: TObject) of object;
   TAddNodeFailEvent = procedure (Sender: TObject) of object;
   TUpdateNodeSuccessEvent = procedure (Sender: TObject) of object;
   TUpdateNodeFailEvent = procedure (Sender: TObject) of object;
   TRemoveNodeSuccessEvent = procedure (Sender: TObject) of object;
   TRemoveNodeFailEvent = procedure (Sender: TObject) of object;

type
  TNodeManage = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fOnFetchNodeListSuccess: TFetchNodeListSuccessEvent;
     fOnFetchNodeListFail: TFetchNodeListFailEvent;
     fOnAddNodeSuccess: TAddNodeSuccessEvent;
     fOnAddNodeFail: TAddNodeFailEvent;
     fOnUpdateNodeSuccess: TUpdateNodeSuccessEvent;
     fOnUpdateNodeFail: TUpdateNodeFailEvent;
     fOnRemoveNodeSuccess: TRemoveNodeSuccessEvent;
     fOnRemoveNodeFail: TRemoveNodeFailEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     function FetchNodeList(aCds: TClientDataset; IncludeRootNode: boolean=false): boolean;
     function AddNode(aNodeId: AnsiString; aNodeName: AnsiString; aNodePassword: AnsiString; aNodeType: integer;
              aNodeAddress: AnsiString; aNodeMsgPort: integer; aNodeTaskPort: integer): boolean;
     function UpdateNode(OldNodeId: AnsiString; OldNodePassword: AnsiString; aNodeId: AnsiString; aNodeName: AnsiString;
              aNodePassword: AnsiString; aNodeType: integer; aNodeAddress: AnsiString; aNodeMsgPort: integer; aNodeTaskPort: integer): boolean;
     function RemoveNode(aNodeId: AnsiString; aNodePassword: AnsiString): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property OnFetchNodeListSuccess: TFetchNodeListSuccessEvent read fOnFetchNodeListSuccess write fOnFetchNodeListSuccess;
     property OnFetchNodeListFail: TFetchNodeListFailEvent read fOnFetchNodeListFail write fOnFetchNodeListFail;
     property OnAddNodeSuccess: TAddNodeSuccessEvent read fOnAddNodeSuccess write fOnAddNodeSuccess;
     property OnAddNodeFail: TAddNodeFailEvent read fOnAddNodeFail write fOnAddNodeFail;
     property OnUpdateNodeSuccess: TUpdateNodeSuccessEvent read fOnUpdateNodeSuccess write fOnUpdateNodeSuccess;
     property OnUpdateNodeFail: TUpdateNodeFailEvent read fOnUpdateNodeFail write fOnUpdateNodeFail;
     property OnRemoveNodeSuccess: TRemoveNodeSuccessEvent read fOnRemoveNodeSuccess write fOnRemoveNodeSuccess;
     property OnRemoveNodeFail: TRemoveNodeFailEvent read fOnRemoveNodeFail write fOnRemoveNodeFail;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TNodeManage]);
end;

procedure TNodeManage.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TNodeManage.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TNodeManage.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

constructor TNodeManage.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   LastErrorCode:='';
   LastErrorText:='';
end;

destructor TNodeManage.Destroy;
begin
   fOnFetchNodeListSuccess:=nil;
   fOnFetchNodeListFail:=nil;
   fOnAddNodeSuccess:=nil;
   fOnAddNodeFail:=nil;
   fOnUpdateNodeSuccess:=nil;
   fOnUpdateNodeFail:=nil;
   fOnRemoveNodeSuccess:=nil;
   fOnRemoveNodeFail:=nil;
  inherited;
end;

function TNodeManage.FetchNodeList(aCds: TClientDataset; IncludeRootNode: boolean=false): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0201002';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnFetchNodeListFail) then
            fOnFetchNodeListFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fUserConn.RootNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',10);
   if IncludeRootNode then
      Parcel.PutBooleanGoods('IncludeRootNode',true);
   Parcel.SaveToString(RequestBody);
   RequestBody:=AnsiString(inttostr(Length(RequestBody)))+#13+RequestBody;
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fUserConn.RootNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     aCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('NodeList',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        aCds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if result then
                        begin
                           if Assigned(fOnFetchNodeListSuccess) then
                              fOnFetchNodeListSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0201005';
                           LastErrorText:='Decompress the return dataset fail.';
                           if Assigned(fOnFetchNodeListFail) then
                              fOnFetchNodeListFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnFetchNodeListFail) then
                        fOnFetchNodeListFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0201003';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnFetchNodeListFail) then
                  fOnFetchNodeListFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0201004';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnFetchNodeListFail) then
            fOnFetchNodeListFail(self);
      end;
end;

function TNodeManage.AddNode(aNodeId: AnsiString; aNodeName: AnsiString; aNodePassword: AnsiString; aNodeType: integer;
   aNodeAddress: AnsiString; aNodeMsgPort: integer; aNodeTaskPort: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0201104';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnAddNodeFail) then
            fOnAddNodeFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fUserConn.RootNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',11);
   Parcel.PutEncryptStringGoods('NodeId',aNodeId);
   Parcel.PutEncryptStringGoods('NodeName',aNodeName);
   Parcel.PutEncryptStringGoods('NodePassword',aNodePassword);
   Parcel.PutIntegerGoods('NodeType',aNodeType);
   Parcel.PutEncryptStringGoods('NodeAddress',aNodeAddress);
   Parcel.PutIntegerGoods('NodeMsgPort',aNodeMsgPort);
   Parcel.PutIntegerGoods('NodeTaskPort',aNodeTaskPort);
   Parcel.SaveToString(RequestBody);
   RequestBody:=AnsiString(inttostr(Length(RequestBody)))+#13+RequestBody;
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fUserConn.RootNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnAddNodeSuccess) then
                        fOnAddNodeSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnAddNodeFail) then
                        fOnAddNodeFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0201105';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnAddNodeFail) then
                  fOnAddNodeFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0201106';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnAddNodeFail) then
            fOnAddNodeFail(self);
      end;
end;

function TNodeManage.UpdateNode(OldNodeId: AnsiString; OldNodePassword: AnsiString; aNodeId: AnsiString; aNodeName: AnsiString;
   aNodePassword: AnsiString; aNodeType: integer; aNodeAddress: AnsiString; aNodeMsgPort: integer; aNodeTaskPort: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0201204';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnUpdateNodeFail) then
            fOnUpdateNodeFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fUserConn.RootNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',12);
   Parcel.PutEncryptStringGoods('OldNodeId',OldNodeId);
   Parcel.PutEncryptStringGoods('OldNodePassword',OldNodePassword);
   Parcel.PutEncryptStringGoods('NodeId',aNodeId);
   Parcel.PutEncryptStringGoods('NodeName',aNodeName);
   Parcel.PutEncryptStringGoods('NodePassword',aNodePassword);
   Parcel.PutIntegerGoods('NodeType',aNodeType);
   Parcel.PutEncryptStringGoods('NodeAddress',aNodeAddress);
   Parcel.PutIntegerGoods('NodeMsgPort',aNodeMsgPort);
   Parcel.PutIntegerGoods('NodeTaskPort',aNodeTaskPort);
   Parcel.SaveToString(RequestBody);
   RequestBody:=AnsiString(inttostr(Length(RequestBody)))+#13+RequestBody;
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fUserConn.RootNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnUpdateNodeSuccess) then
                        fOnUpdateNodeSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnUpdateNodeFail) then
                        fOnUpdateNodeFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0201205';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnUpdateNodeFail) then
                  fOnUpdateNodeFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0201206';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnUpdateNodeFail) then
            fOnUpdateNodeFail(self);
      end;
end;

function TNodeManage.RemoveNode(aNodeId: AnsiString; aNodePassword: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0201304';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnRemoveNodeFail) then
            fOnRemoveNodeFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fUserConn.RootNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',13);
   Parcel.PutEncryptStringGoods('NodeId',aNodeId);
   Parcel.PutEncryptStringGoods('NodePassword',aNodePassword);
   Parcel.SaveToString(RequestBody);
   RequestBody:=AnsiString(inttostr(Length(RequestBody)))+#13+RequestBody;
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fUserConn.RootNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnRemoveNodeSuccess) then
                        fOnRemoveNodeSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnRemoveNodeFail) then
                        fOnRemoveNodeFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0201305';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnRemoveNodeFail) then
                  fOnRemoveNodeFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0201306';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnRemoveNodeFail) then
            fOnRemoveNodeFail(self);
      end;
end;

procedure TNodeManage.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=String(LastErrorCode);
   ErrorText:=String(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
