//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit QBBase64;

interface

uses
  SysUtils,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  Classes;

type
  TByteArray2280 = array[0..2279] of byte;

  function StrTobase64(Buf: Ansistring): Ansistring;
  function Base64ToStr(B64: Ansistring): Ansistring;

  function ByteArrayToBase64(var ABuf: TByteArray2280; ASize: integer): Ansistring;
  procedure Base64ToByteArray(B64: Ansistring; var ByteArray: PByteArray; var ASize: integer);

  function StreamToBase64(AStream: TStream): Ansistring;
  function Base64ToStream(B64: Ansistring): TStream;

implementation

const
  Base64Code = AnsiString('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/');
  Pad = '=';

type
  EBase64Error = Exception;

function Char2IDx( c: Ansichar ): byte;
var
  i: integer;
begin
  for i := 1 to Length( Base64Code ) do
    if Base64Code[i] = c then
    begin
      Result := pred( i );
      EXIT;
    end;
  Result := Ord( Pad );
end;

function StreamToBase64(AStream: TStream): Ansistring;
var
  size: integer;
  buf: TByteArray2280;
begin
  result := '';
  AStream.Seek(0, SoFromBeginning);
  size := 2280;
  while size = 2280 do
  begin
    size := AStream.Read(buf, 2280);
    result := result + ByteArrayToBase64(buf, size);
  end;
end;

function Base64ToStream(B64: Ansistring): TStream;
var
  size: integer;
  buf: PByteArray;
begin
  result := TMemoryStream.Create;
  Base64ToByteArray(B64, buf, size);
  try
    result.Seek(0, SoFromBeginning);
    result.Write(buf^, size);
  finally
    FreeMem(buf);
  end;
end;

function ByteArrayToBase64(var ABuf: TByteArray2280; ASize: integer): Ansistring;
var
  i, PadCount, modValue: integer;
  x1, x2, x3, x4: byte;
begin
  result := '';
  PadCount := 0;

  modValue := ASize mod 3;
  if modValue <> 0 then
  begin
    for i := ASize to ASize+3-modValue - 1 do
    begin
      ABuf[i] := 0;
      inc(PadCount);
    end;
  end;

  i := 0;
  while i <= ASize - 3 do
  begin
    x1 := ( ABuf[i] shr 2 ) and $3F;
    x2 := ( ( ABuf[i] shl 4 ) and $3F ) or ABuf[i + 1] shr 4;
    x3 := ( ( ABuf[i + 1] shl 2 ) and $3F ) or ABuf[i + 2] shr 6;
    x4 := ABuf[i + 2] and $3F;
    result := result
      + Base64Code[x1 + 1]
      + Base64Code[x2 + 1]
      + Base64Code[x3 + 1]
      + Base64Code[x4 + 1];
    inc( i, 3 );
  end;

  if PadCount > 0 then
    for i := Length(result) downto 1 do
    begin
      result[i] := Pad;
      dec(PadCount);
      if PadCount = 0 then
        Break;
    end;
end;

procedure Base64ToByteArray(B64: Ansistring; var ByteArray: PByteArray; var ASize: integer);
var
  i: integer;
  str: Ansistring;
begin
  str := Base64ToStr(B64);
  ASize := Length(str);
  ByteArray := PByteArray(AllocMem(ASize));
  for i := 0 to ASize-1 do
  begin
    ByteArray[i] := Ord(str[i+1]);
  end;
end;

function StrTobase64( Buf: Ansistring ): Ansistring;
var
  i             : integer;
  x1, x2, x3, x4: byte;
  PadCount      : integer;
begin
  PadCount := 0;
  while length( Buf ) < 3 do
  begin
    Buf := Buf + #0;
    inc( PadCount );
  end;

  while ( length( Buf ) mod 3 ) <> 0 do
  begin
    Buf := Buf + #0;
    inc( PadCount );
  end;

  Result := '';
  i := 1;

  while i <= length( Buf ) - 2 do
  begin
    x1 := ( Ord( Buf[i] ) shr 2 ) and $3F;
    x2 := ( ( Ord( Buf[i] ) shl 4 ) and $3F ) or Ord( Buf[i + 1] ) shr 4;
    x3 := ( ( Ord( Buf[i + 1] ) shl 2 ) and $3F ) or Ord( Buf[i + 2] ) shr 6;
    x4 := Ord( Buf[i + 2] ) and $3F;
    Result := Result
      + Base64Code[x1 + 1]
      + Base64Code[x2 + 1]
      + Base64Code[x3 + 1]
      + Base64Code[x4 + 1];
    inc( i, 3 );
  end;

  if PadCount > 0 then
    for i := Length( Result ) downto 1 do
    begin
      Result[i] := Pad;
      dec( PadCount );
      if PadCount = 0 then BREAK;
    end;

end;

function Base64ToStr( B64: Ansistring ): Ansistring;
var
  i, PadCount   : integer;
  Block         : string[3];
  x1, x2, x3    : byte;
  c             : AnsiChar;
begin
  if ( Length( B64 ) < 4 )
    or ( Length( B64 ) mod 4 <> 0 ) then
    raise EBase64Error.Create( 'Base64ToStr: illegal input length!' );
    PadCount := 0;
  i := Length( B64 );
  while (B64[i] = Pad) and (i > 0 ) do
  begin
    inc( PadCount );
    dec( i );
  end;

  Result := '';
  i := 1;
  SetLength( Block, 3 );
  while i <= Length( B64 ) - 3 do
  begin
    x1 := ( Char2Idx( B64[i] ) shl 2 ) or ( Char2IDx( B64[i + 1] ) shr 4 );
    move(x1,c,1);
    Result := Result + C;
    x2 := ( Char2Idx( B64[i + 1] ) shl 4 ) or ( Char2IDx( B64[i + 2] ) shr 2 );
    move(x2,c,1);
    Result := Result + c;
    x3 := ( Char2Idx( B64[i + 2] ) shl 6 ) or ( Char2IDx( B64[i + 3] ) );
    move(x3,c,1);
    Result := Result + c;
    inc( i, 4 );
  end;

  while PadCount > 0 do
    begin
      Delete( Result, Length( Result ), 1 );
      dec( PadCount );
    end;

end;

end.
