//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit QBCommon;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
  Windows, SysUtils, Classes, winsock, ComObj, WinInet,
  forms,  AdoDb, QBParcel, winsvc,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  TLHelp32, dbclient
{$IFDEF VER130}
;
{$ELSE}
, Variants,MidasLib;
{$ENDIF}

Const
  MAX_ADAPTER_NAME_LENGTH        = 256;
  MAX_ADAPTER_DESCRIPTION_LENGTH = 128;
  MAX_ADAPTER_ADDRESS_LENGTH     = 8;
  MAX_INTERFACE_NAME_LEN = 256;
  MAXLEN_PHYSADDR = 8;
  MAXLEN_IFDESCR = 256;
  MIB_IF_TYPE_OTHER = 1;
  MIB_IF_TYPE_ETHERNET = 6;
  MIB_IF_TYPE_TOKENRING = 9;
  MIB_IF_TYPE_FDDI = 15;
  MIB_IF_TYPE_PPP = 23;
  MIB_IF_TYPE_LOOPBACK = 24;
  MIB_IF_TYPE_SLIP = 28;
  MIB_IF_ADMIN_STATUS_UP = 1;
  MIB_IF_ADMIN_STATUS_DOWN = 2;
  MIB_IF_ADMIN_STATUS_TESTING = 3;
  MIB_IF_OPER_STATUS_NON_OPERATIONAL = 0;
  MIB_IF_OPER_STATUS_UNREACHABLE = 1;
  MIB_IF_OPER_STATUS_DISCONNECTED = 2;
  MIB_IF_OPER_STATUS_CONNECTING = 3;
  MIB_IF_OPER_STATUS_CONNECTED = 4;
  MIB_IF_OPER_STATUS_OPERATIONAL = 5;

type
  MIB_PHYSADDR = array[0..MAXLEN_PHYSADDR - 1] of Byte;
  MIB_IFDESCR = array[0..MAXLEN_IFDESCR - 1] of AnsiChar;
  PMIB_IFROW = ^MIB_IFROW;
  MIB_IFROW = packed record
    wszName: array[0..MAX_INTERFACE_NAME_LEN - 1] of WCHAR;
    dwIndex,
    dwType,
    dwMtu,
    dwSpeed,
    dwPhysAddrLen: DWORD;
    bPhysAddr: MIB_PHYSADDR;
    dwAdminStatus,
    dwOperStatus,
    dwLastChange,
    dwInOctets,
    dwInUcastPkts,
    dwInNUcastPkts,
    dwInDiscards,
    dwInErrors,
    dwInUnknownProtos,
    dwOutOctets,
    dwOutUcastPkts,
    dwOutNUcastPkts,
    dwOutDiscards,
    dwOutErrors,
    dwOutQLen,
    dwDescrLen: DWORD;
    bDescr: MIB_IFDESCR;
  end;
  PMIB_IFTABLE = ^MIB_IFTABLE;
  MIB_IFTABLE = packed record
    dwNumEntries: DWORD;
    table: array[0..0] of MIB_IFROW;
  end;
  TAdapterStatus = record
    dwType,
    dwOperStatus: DWORD;
    bDescr: MIB_IFDESCR;
  end;
  TAdapterStatuses = array of TAdapterStatus;

Type
  TIPAddressString = Array[0..4*4-1] of AnsiChar;

  PIPAddrString = ^TIPAddrString;
  TIPAddrString = Record
    Next      : PIPAddrString;
    IPAddress : TIPAddressString;
    IPMask    : TIPAddressString;
    Context   : Integer;
  End;

PIPAdapterInfo = ^TIPAdapterInfo;
  TIPAdapterInfo = Record 
    Next                : PIPAdapterInfo;
    ComboIndex          : Integer;
    AdapterName         : Array[0..MAX_ADAPTER_NAME_LENGTH+3] of AnsiChar;
    Description         : Array[0..MAX_ADAPTER_DESCRIPTION_LENGTH+3] of AnsiChar;
    AddressLength       : Integer;
    Address             : Array[1..MAX_ADAPTER_ADDRESS_LENGTH] of Byte;
    Index               : Integer;
    _Type               : Integer;
    DHCPEnabled         : Integer;
    CurrentIPAddress    : PIPAddrString;
    IPAddressList       : TIPAddrString;
    GatewayList         : TIPAddrString;
  End;

function GetLastErrorText: string;
function GetADOLastErrorText(AdoConn: TAdoConnection): string;
Function GetAdaptersInfo(AI: PIPAdapterInfo; Var BufLen: Integer) : Integer; StdCall; External 'iphlpapi.dll' Name 'GetAdaptersInfo';
Function Encrypt(Src: AnsiString; Key: AnsiString): Ansistring;
Function Decrypt (Src: AnsiString; Key: AnsiString): Ansistring;
function UUEncode(const S: Ansistring): Ansistring;
function UUDecode(const S: Ansistring): Ansistring;
function WinCodePage: integer;
function UnicodeToAnsi(const Str: WideString; codePage: Word=0): AnsiString;
function AnsiToUnicode(const Str: AnsiString; codePage: Word=0): WideString;
function PackString(aStr: string; CodePage: Word=0): AnsiString;
function UnpackString(aStr: AnsiString): string;
function EncodeStr(InputString: Ansistring): Ansistring;
function DecodeStr(InputString: Ansistring): Ansistring;
function FishString(aStr: string): ansistring;
function UnfishString(aStr: AnsiString): string;
function MD5Encode(const Input: AnsiString): AnsiString;
Function IsIPInstalled : boolean;
function GetIPFromName(aHostName: string; var Ip: string): Boolean;
function GetNameFromIP(IPAddr: String; var MacName: String): Boolean;
Function LanConnected: Boolean;
function InternetConnected: Boolean;
function UdpPortInuse(Port: integer): boolean;
function TcpPortInuse(Port: integer): boolean;
function GetRandomUdpPort(RangeMin: integer; RangeMax: integer): Integer;
function GetRandomTcpPort(RangeMin: integer; RangeMax: integer): Integer;
function GetLocalIP(): string;
function GetComputerName: string;
function GetLocalIpList(var IpList:TStringList):Integer;
function GetMacAddr(): string;
function GetGatewayIp(): string;
function GetDnsIp : string;
function GetIpMask(): string;
function GetBroadcastIp(): string;
function IsValidIP(aStrIP: AnsiString): Boolean;
function IP2Integer(IP: Ansistring): integer;
function Integer2IP(IntValue: integer): ansistring;
function AddNetConnection(NetPath: string; PassWord: string; LocalPath: string): Boolean;
function RemoveNetConnection(LocalPath: string): Boolean;
function HostIsOnline(IpAddr: string): Boolean;
procedure GetProxyServer(protocol: string; var ProxyServerIp: string; var ProxyServerPort: Integer);
function GetConnectionKind: integer;
function AuthorizedApp(AppName: string; AppFileName: string): boolean;
function AddFirewallPort(PortName: string; Protocol: string; Port: integer): boolean;
function CloseWinFirewall: boolean;
function IsFileInUse(AName: string): boolean;
function GetAbsolutePath(BasePath, RelativePath: string): string;
function AdjustProcessPrivilege(ProcessHandle: THandle; Token_Name: PChar): boolean;
function IsVista: boolean;
function GetWanIp: string;
procedure setLocalDateTime(aDT: TDateTime);
function LastParameter(ParaStr: AnsiString): AnsiString;
procedure SafeDelay(Ms: integer);
function GetIfTable(pIfTable: PMIB_IFTABLE; pdwSize: PULONG; bOrder: BOOL): DWORD; stdcall; external 'iphlpapi.dll';
function GetAdapterTypeString(const dwType: DWORD): Ansistring;
function GetAdapterStatusString(const dwOperStatus: DWORD): Ansistring;
procedure ScanAdapters(var AdapterStatuses: TAdapterStatuses);
procedure FindAProcess(const AFilename: string; const PathMatch: Boolean; var ProcessID: DWORD);
procedure Str2Stream(Str: Ansistring; var aStream: TMemoryStream);
procedure Stream2Str(aStream: TMemoryStream; var aStr: Ansistring);
procedure Str2Cds(Str: Ansistring; var aCds: TClientDataset);
procedure Cds2Str(aCds: TClientDataset; Fmt: TDataPacketFormat; var aStr: Ansistring);
procedure Str2Parcel(Str: Ansistring; var aParcel: TQBParcel);
procedure Parcel2Str(aParcel: TQBParcel; var aStr: Ansistring);
procedure CloneParcel(FromParcel: TQBParcel; var ToParcel: TQBParcel);
function blob2stream(Rs: _RecordSet; FieldName: string; aStream: TMemoryStream): boolean;
procedure VariantToStream (const v: OleVariant; Stream: TMemoryStream);
procedure StreamToVariant(Stream: TMemoryStream; var v: OleVariant);

implementation

var
   LocalCodePage: word;

function GetLastErrorText: string;
var
   dwSize: DWORD;
   lpszTemp: PChar;
begin
   dwSize:=512;
   lpszTemp:=StrAlloc(dwSize);
   try
      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM or FORMAT_MESSAGE_ARGUMENT_ARRAY,nil,GetLastError,LANG_NEUTRAL,lpszTemp,dwSize,nil);
   finally
      Result:=String(lpszTemp);
      StrDispose(lpszTemp);
   end;
end;

function GetADOLastErrorText(AdoConn: TAdoConnection): string;
var
   i: integer;
   Err: TStringList;
begin
   Err:=TStringList.Create;
   for i := 0 to AdoConn.Errors.Count - 1 do
      Err.Add(AdoConn.Errors[i].Description);
   result:=Err.Text;
   Err.Free;
end;

Function Encrypt(Src: AnsiString; Key: AnsiString): Ansistring;
var
   KeyLen: Integer;
   KeyPos: Integer;
   offset: Integer;
   dest: Ansistring;
   SrcPos: Integer;
   SrcAsc: Integer;
   Range: Integer;
begin
   if Src='' then
      begin
         Result:='';
         exit;
      end;
   KeyLen:=Length(Key);
   if KeyLen=0 then
      key:='Invalid';
   KeyPos:=0;
   Range:=256;
   Randomize;
   offset:=Random(Range);
   dest:=AnsiString(format('%1.2x',[offset]));
   for SrcPos:=1 to Length(Src) do
      begin
         SrcAsc:=(Ord(Src[SrcPos])+offset) MOD 255;
         if KeyPos<KeyLen then
            KeyPos:=KeyPos + 1
         else
            KeyPos:=1;
         SrcAsc:=SrcAsc xor Ord(Key[KeyPos]);
         dest:=dest+AnsiString(format('%1.2x',[SrcAsc]));
         offset:=SrcAsc;
      end;
   Result:=Dest;
end;

Function Decrypt(Src: AnsiString; Key: AnsiString): Ansistring;
var
   KeyLen: Integer;
   KeyPos: Integer;
   offset: Integer;
   dest: Ansistring;
   SrcPos: Integer;
   SrcAsc: Integer;
   TmpSrcAsc: Integer;
begin
   if Src='' then
      begin
         Result:='';
         exit;
      end;
   KeyLen:=Length(Key);
   if KeyLen=0 then
      key:='Invalid';
   KeyPos:=0;
   offset:=StrToInt('$'+ string(copy(src,1,2)));
   SrcPos:=3;
   repeat
      SrcAsc:=StrToInt('$'+ string(copy(src,SrcPos,2)));
      if KeyPos<KeyLen Then
         KeyPos:=KeyPos + 1
      else
         KeyPos:=1;
      TmpSrcAsc:=SrcAsc xor Ord(Key[KeyPos]);
      if TmpSrcAsc<=offset then
         TmpSrcAsc:=255+TmpSrcAsc-offset
      else
         TmpSrcAsc:=TmpSrcAsc-offset;
      dest:=dest+AnsiChar(TmpSrcAsc);
      offset:=srcAsc;
      SrcPos:=SrcPos+2;
   until SrcPos>=Length(Src);
   Result:=Dest;
end;

function UUEncode(const S: Ansistring): Ansistring;
   function _UUEncode(const S: Ansistring): Ansistring;
   var
     i: integer;
     InBuf: array[0..2] of byte;
     OutBuf: array[0..3] of byte;
   begin
     SetLength(Result,((Length(S)+2) div 3)*4);
     for i:= 1 to ((Length(S)+2) div 3) do
     begin
       if Length(S)< (i*3) then
         Move(S[(i-1)*3+1],InBuf,Length(S)-(i-1)*3)
       else
         Move(S[(i-1)*3+1],InBuf,3);
       OutBuf[0]:= ((InBuf[0] and $FC) shr 2) + 32;
       OutBuf[1]:= (((InBuf[0] and $03) shl 4) or ((InBuf[1] and $F0) shr 4)) + 32;
       OutBuf[2]:= (((InBuf[1] and $0F) shl 2) or ((InBuf[2] and $C0) shr 6)) + 32;
       OutBuf[3]:= (InBuf[2] and $3F) + 32;
       if OutBuf[0]= 32 then
         OutBuf[0]:= 96;
       if OutBuf[1]= 32 then
         OutBuf[1]:= 96;
       if OutBuf[2]= 32 then
         OutBuf[2]:= 96;
       if OutBuf[3]= 32 then
         OutBuf[3]:= 96;
       Move(OutBuf,Result[(i-1)*4+1],4);
     end;
   end;
begin
  if Length(S)> 45 then
    raise Exception.Create('UUEncode: String too long');
  if Length(S)= 0 then
    Result:= #96
  else
  begin
    Result:= Ansichar(Length(S)+32);
    Result:= Result + _UUEncode(S);
  end;
end;

function UUDecode(const S: Ansistring): Ansistring;
   function _UUDecode(const S: Ansistring): Ansistring;
   var
     i: integer;
     InBuf: array[0..3] of byte;
     OutBuf: array[0..2] of byte;
   begin
     SetLength(Result,((Length(S)+2) div 4)*3);
     for i:= 1 to ((Length(S)+2) div 4) do
     begin
       if Length(S)< (i*4) then
         Move(S[(i-1)*4+1],InBuf,Length(S)-(i-1)*4)
       else
         Move(S[(i-1)*4+1],InBuf,4);
       if InBuf[0]= 96 then
         InBuf[0]:= 0
       else
         Dec(InBuf[0],32);
       if InBuf[1]= 96 then
         InBuf[1]:= 0
       else
         Dec(InBuf[1],32);
       if InBuf[2]= 96 then
         InBuf[2]:= 0
       else
         Dec(InBuf[2],32);
       if InBuf[3]= 96 then
         InBuf[3]:= 0
       else
         Dec(InBuf[3],32);
       OutBuf[0]:= (InBuf[0] shl 2) or ((InBuf[1] shr 4) and $03);
       OutBuf[1]:= (InBuf[1] shl 4) or ((InBuf[2] shr 2) and $0F);
       OutBuf[2]:= (InBuf[2] shl 6) or (InBuf[3] and $3F);
       Move(OutBuf,Result[(i-1)*3+1],3);
     end;
   end;
begin
  if (S[1]< #33) or (S[1]> #97) then
    raise Exception.Create('UUDecode: Incorrect string format');
  Result:= _UUDecode(Copy(S,2,Length(S)-1));
  if S[1]<> #96 then
    SetLength(Result,byte(S[1])-32);
end;

function WinCodePage: integer;
begin
   result:=GetACP;
end;
function UnicodeToAnsi(const Str: WideString; codePage: Word=0): AnsiString;
var
  l,cp: integer;
begin
   if Str='' then
      Result:=''
   else
      begin
         if CodePage=0 then
            cp:=WinCodePage
         else
            cp:=CodePage;
         l:=WideCharToMultiByte(cp,WC_COMPOSITECHECK or WC_DISCARDNS or WC_SEPCHARS or WC_DEFAULTCHAR,@Str[1],-1,nil,0,nil,nil);
         SetLength(Result,l-1);
         if l>1 then
            WideCharToMultiByte(cp,WC_COMPOSITECHECK or WC_DISCARDNS or WC_SEPCHARS or WC_DEFAULTCHAR,@Str[1],-1,@Result[1],l-1,nil,nil);
      end;
end;

function AnsiToUnicode(const Str: AnsiString; codePage: Word=0): WideString;
var
  l,cp: integer;
begin
   if str='' then
      Result:=''
   else
      begin
         if CodePage=0 then
            cp:=WinCodePage
         else
            cp:=CodePage;
         l:=MultiByteToWideChar(cp,MB_PRECOMPOSED,PAnsiChar(@Str[1]),-1,nil,0);
         SetLength(Result, l-1);
         if l > 1 then
            MultiByteToWideChar(cp,MB_PRECOMPOSED,PAnsiChar(@Str[1]),-1,PWideChar(@Result[1]),l-1);
      end;
end;

function PackString(aStr: string; CodePage: Word=0): AnsiString;
var
   Len: integer;
   Crc,CP: word;
{$IFDEF UNICODE}
   tmpstr: AnsiString;
{$ENDIF}
begin
   if CodePage=0 then
      CP:=WinCodePage
   else
      CP:=CodePage;
   Len:=length(aStr);
{$IFDEF UNICODE}
   setlength(result,Len*Sizeof(Char)+12);
   if Len>0 then
      move(aStr[1],Result[9],Len*Sizeof(Char));
   tmpstr:=AnsiString(IntToHex(CP,4));
   move(tmpstr[1],result[1],4);
   tmpstr:=AnsiString(IntToHex(Len,4));
   move(tmpstr[1],result[5],4);
   Crc:=(CP xor Word(Len));
   tmpstr:=AnsiString(IntToHex(Crc,4));
   move(tmpstr[1],result[Len*Sizeof(Char)+9],4);
{$ELSE}
   Crc:=(CP xor Word(Len));
   Result:=AnsiString(IntToHex(CP,4)+IntToHex(Len,4)+aStr+IntToHex(Crc,4));
{$ENDIF}
end;

function UnpackString(aStr: AnsiString): string;
var
   CodePage,Crc: word;
   Len,DataLen: Integer;
{$IFDEF UNICODE}
   tmpstr1: AnsiString;
{$ELSE}
   tmpstr2: WideString;
{$ENDIF}
begin
   try
      CodePage:=strtoint(string('$'+copy(aStr,1,4)));
   except
      CodePage:=0;
   end;
   try
      Len:=strtoint(string('$'+copy(aStr,5,4)));
   except
      Len:=0;
   end;
   DataLen:=Length(aStr)-12;
   if (CodePage=0) or (Len=0) or (DataLen<Len) then
      begin
         Result:='';
         exit;
      end;
   if (DataLen mod Len)<>0 then
      begin
         Result:='';
         exit;
      end;
   try
      Crc:=strtoint(string('$'+copy(aStr,length(aStr)-3,4)));
   except
      result:='';
      exit;
   end;
   if Crc<>(CodePage xor Word(Len)) then
      begin
         result:='';
         exit;
      end;
{$IFDEF UNICODE}
   if DataLen=Len then
      begin
         SetLength(tmpstr1,DataLen);
         move(aStr[9],tmpstr1[1],DataLen);
         Result:=AnsiToUnicode(tmpstr1,CodePage);
      end
   else
      begin
         SetLength(Result,Len);
         move(aStr[9],Result[1],Len*Sizeof(Char));
      end;
{$ELSE}
   if DataLen=Len then
      begin
         SetLength(Result,Len);
         move(aStr[9],Result[1],Len);
      end
   else
      begin
         SetLength(tmpstr2,Len);
         move(aStr[9],tmpstr2[1],Len*SizeOf(WideChar));
         Result:=UnicodeToAnsi(tmpstr2,CodePage);
      end;
{$ENDIF}
end;


Const
   BinToAsc : Array [0..63] of AnsiChar =
              ('+', '-','0','1','2','3','4','5','6','7',
              '8','9','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
              'P','Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d',
              'e','f','g','h','i','j','k','l','m','n','o','p','q','r','s',
              't','u','v','w','x','y','z');
function EncodeStr(InputString: Ansistring): Ansistring;
var
   Counter:      integer;
   ReturnString: Ansistring;
   b:            Byte;
   i:            integer;
   last:         byte;
   Flush:        Boolean;
   LengthInput:  integer;
   c: AnsiChar;
begin
   Counter := 0;
   ReturnString := '';
   Flush := False;
   last := 0;
   if (Length(InputString) mod 3) <> 0 then
      begin
        b:=0;
        move(b,c,1);
        InputString := InputString + c;
        Flush := True;
      end;
   LengthInput := Length(InputString);
   i := 1;
   while (i <= LengthInput) do
     begin
        b := Ord(InputString[i]);
        case Counter of
          0: begin
                ReturnString := ReturnString + BinToAsc[(b shr 2)];
                last := b;
             end;
          1: begin
                ReturnString := ReturnString + BinToAsc[((last and $3) shl 4) or ((b and $F0) shr 4) ];
                last := b;
             end;
          2: begin
                ReturnString := ReturnString + BinToAsc[((last and $F) shl 2) or ((b and $C0) shr 6)];
                if not (Flush and (i = LengthInput)) then
                   ReturnString := ReturnString + BinToAsc[(b and $3F)];
                last := 0;
             end;
        end;
        Inc(Counter);
        if Counter = 3 then
          Counter := 0;
        Inc(i);
     end;
   Result := ReturnString;
end;

function DecodeStr(InputString: Ansistring): Ansistring;
  function DecodeBase64(b: byte): byte;
    begin
      if (b >= Ord('0')) and (b <= Ord('9')) then
         begin
           Result := b - Ord('0') + 2;
           Exit;
         end;
      if (b >= Ord('A')) and (b <= Ord('Z')) then
         begin
           Result := b - Ord('A') + 12;
           Exit;
         end;
      if (b >= Ord('a')) and (b <= Ord('z')) then
         begin
           Result := b - Ord('a') + 38;
           Exit;
         end;
      if b = Ord('+') then
         begin
           Result := 0;
           Exit;
         end;
      if b = Ord('-') then
         begin
           Result := 1;
           Exit;
         end;

      raise EConvertError.Create('Invalid Base64 characterChar('+inttostr(b)+')='+string(AnsiChar(b)));
    end;
var
   Counter:      integer;
   ReturnString: Ansistring;
   c:            AnsiChar;
   last:         byte;
   this:         byte;
   i:          integer;
   k:          byte;
begin
   Counter := 0;
   ReturnString := '';
   last := 0;
   for i := 1 to Length(InputString) do
      begin
        c := InputString[i];
        case Counter of
           0: begin
                 last := DecodeBase64(Ord(c)) shl 2;
              end;
           1: begin
                  this := DecodeBase64(Ord(c));
                  k:=(last or (this shr 4)) and $ff;
                  move(k,c,1);
                  ReturnString := ReturnString + c;
                  last := this shl 4;
              end;
           2: begin
                  this := DecodeBase64(Ord(c));
                  k:=(last or (this shr 2)) and $ff;
                  move(k,c,1);
                  ReturnString := ReturnString + C;
                  last := this shl 6;
             end;
          3: begin
                  this := DecodeBase64(Ord(c));
                  k:=(last or this) and $ff;
                  move(k,c,1);
                  ReturnString := ReturnString + C;
                  last := 0;
             end;
        end;
        Inc(Counter);
        if Counter = 4 then
           Counter := 0;
      end;
   Result := ReturnString;
end;


function FishString(aStr: string): ansistring;
begin
   result:=EncodeStr(packstring(aStr,LocalCodePage));
end;

function UnfishString(aStr: AnsiString): string;
begin
   if astr='' then
      result:=''
   else
      try
         result:=UnpackString(DecodeStr(aStr));
      except
         result:='';
      end;
end;

const
  ADVAPI32    = 'advapi32.dll';
  function CryptAcquireContext(phProv: PULONG; pszContainer: PAnsiChar; pszProvider: PAnsiChar; dwProvType: DWORD; dwFlags: DWORD): BOOL; stdcall; external ADVAPI32 name 'CryptAcquireContextA';
  function CryptCreateHash(hProv: ULONG; Algid: ULONG; hKey: ULONG; dwFlags: DWORD; phHash: PULONG): BOOL; stdcall; external ADVAPI32 name 'CryptCreateHash';
  function CryptHashData(hHash: ULONG; const pbData: PBYTE; dwDataLen: DWORD; dwFlags: DWORD): BOOL; stdcall; external ADVAPI32 name 'CryptHashData';
  function CryptGetHashParam(hHash: ULONG; dwParam: DWORD; pbData: PBYTE; pdwDataLen: PDWORD; dwFlags: DWORD): BOOL; stdcall; external ADVAPI32 name 'CryptGetHashParam';
  function CryptDestroyHash(hHash: ULONG): BOOL; stdcall; external ADVAPI32 name 'CryptDestroyHash';
  function CryptReleaseContext(hProv: ULONG; dwFlags: DWORD): BOOL; stdcall; external ADVAPI32 name 'CryptReleaseContext';

function MD5Encode(const Input: AnsiString): AnsiString;
const
  HP_HASHVAL = $0002;
  PROV_RSA_FULL = 1;
  CRYPT_VERIFYCONTEXT = $F0000000;
  CRYPT_MACHINE_KEYSET = $00000020;
  ALG_CLASS_HASH = (4 SHL 13);
  ALG_TYPE_ANY = 0;
  ALG_SID_MD5 = 3;
  CALG_MD5 = (ALG_CLASS_HASH Or ALG_TYPE_ANY Or ALG_SID_MD5);
var
  hCryptProvider : ULONG;
  hHash : ULONG;
  bHash : Array[0..$7F] Of Byte;
  dwHashLen : DWORD;
  pbContent : PByte;
  I : Integer;
begin
  dwHashLen := 16;
  pbContent := Pointer(PAnsiChar(Input));
  Result := '';
  If CryptAcquireContext(@hCryptProvider,nil,nil,PROV_RSA_FULL,CRYPT_VERIFYCONTEXT Or CRYPT_MACHINE_KEYSET) Then
    begin
      If CryptCreateHash(hCryptProvider,CALG_MD5,0,0,@hHash) Then
        begin
          If CryptHashData(hHash,pbContent,Length(Input),0) Then
            begin
              If CryptGetHashParam(hHash,HP_HASHVAL,@bHash[0],@dwHashLen,0) Then
                begin
                  For I := 0 To dwHashLen - 1 Do
                    begin
                    Result := Result + AnsiString(Format('%.2x',[bHash[I]]));
                    end;
                end;
            end;
          CryptDestroyHash(hHash);
        end;
      CryptReleaseContext(hCryptProvider, 0);
    end;
  Result := AnsiLowerCase(Result);
end;

Function IsIPInstalled : boolean;
var
  WSData: TWSAData;
  ProtoEnt: PProtoEnt;
  Ver: word;
begin
  Result := True;
  try
     {$IFDEF VER130}
     Ver:=1;
     {$ELSE}
     Ver:=2;
     {$ENDIF}
     if WSAStartup(Ver,WSData) = 0 then
        begin
           ProtoEnt := GetProtoByName('IP');
           if ProtoEnt = nil then
              Result := False
        end;
  finally
     WSACleanup;
  end;
end;

function GetIPFromName(aHostName: string; var Ip: string): Boolean;
var
   wsdata : TWSAData;
   hostName : array [0..255] of ansichar;
   hostEnt : PHostEnt;
   addr : PAnsiChar;
begin
   WSAStartup ($0101, wsdata);
   try
      gethostname (hostName, sizeof (hostName));
      StrPCopy(hostName, aHostName);
      hostEnt := gethostbyname (hostName);
      if Assigned (hostEnt) then
      if Assigned (hostEnt^.h_addr_list) then
         begin
           addr := hostEnt^.h_addr_list^;
           if Assigned (addr) then
              begin
                IP := Format ('%d.%d.%d.%d', [byte (addr [0]),
                byte (addr [1]), byte (addr [2]), byte (addr [3])]);
                Result := True;
              end
           else
              Result := False;
         end
      else
         Result := False
      else
       Result := False;
   finally
       WSACleanup;
   end;
end;

function GetNameFromIP(IPAddr: String; var MacName: String): Boolean;
var
  SockAddrIn: TSockAddrIn;
  HostEnt: PHostEnt;
  WSAData: TWSAData;
  Ver: Word;
begin
  Result := False;
  if IpAddr = '' then
     exit;
  try
    {$IFDEF VER130}
     Ver:=1;
    {$ELSE}
     Ver:=2;
    {$ENDIF}
    WSAStartup(Ver, WSAData);
    SockAddrIn.sin_addr.s_addr := inet_addr(PAnsiChar(AnsiString(IPAddr)));
    HostEnt := gethostbyaddr(@SockAddrIn.sin_addr.S_addr, 4, AF_INET);
    if HostEnt <> nil then
      MacName := string(StrPas(Hostent^.h_name))
    else
      MacName := '';
    Result := (MacName<>'');
  finally
    WSACleanup;
  end;
end;

Function LanConnected: Boolean;
begin
  Result := False;
  if GetSystemMetrics(SM_NETWORK) <> 0 then
    Result := True;
end;

function InternetConnected: Boolean;
const
  INTERNET_CONNECTION_MODEM=1;
  INTERNET_CONNECTION_LAN=2;
  INTERNET_CONNECTION_PROXY=4;
  INTERNET_CONNECTION_MODEM_BUSY=8;
var
  dwConnectionTypes : DWORD;
begin
  dwConnectionTypes:=INTERNET_CONNECTION_MODEM+ INTERNET_CONNECTION_LAN+INTERNET_CONNECTION_PROXY;
  Result := InternetGetConnectedState(@dwConnectionTypes, 0);
end;

function UdpPortInuse(Port: integer): boolean;
var
   s:TSocket;
   wsd: TWSAData;
   SockAddrIn: TSockAddrIn;
   Ver: word;
begin
   {$IFDEF VER130}
     Ver:=1;
   {$ELSE}
     Ver:=2;
   {$ENDIF}
   if WSAStartup(Ver,wsd)=0 then
      begin
          s:=Socket(AF_INET,SOCK_DGRAM,0);
          result:=true;
          try
             if (s<>SOCKET_ERROR) then
                begin
                   SockAddrIn.sin_family := AF_INET;
                   SockAddrIn.sin_addr.S_addr := htonl(INADDR_ANY);
                   SockAddrIn.sin_port := htons(port);
                   result:=(Bind(s,SockAddrIn,SizeOf(SockAddrIn))<>0);
                end;
          finally
              CloseSocket(s);
              WSACleanup();
          end;
      end
   else
      result:=true;
end;

function TcpPortInuse(Port: integer): boolean;
var
   s:TSocket;
   wsd: TWSAData;
   SockAddrIn: TSockAddrIn;
   Ver: word;
begin
   {$IFDEF VER130}
     Ver:=1;
   {$ELSE}
     Ver:=2;
   {$ENDIF}
   if WSAStartup(Ver,wsd)=0 then
      begin
          s:=Socket(AF_INET,SOCK_STREAM,0);
          result:=true;
          try
             if (s<>SOCKET_ERROR) then
                begin
                   SockAddrIn.sin_family := AF_INET;
                   SockAddrIn.sin_addr.S_addr := htonl(INADDR_ANY);
                   SockAddrIn.sin_port := htons(port);
                   result:=(Bind(s,SockAddrIn,SizeOf(SockAddrIn))<>0);
                end;
          finally
              CloseSocket(s);
              WSACleanup();
          end;
      end
   else
      result:=true;
end;

function GetRandomUdpPort(RangeMin: integer; RangeMax: integer): Integer;
var
   err,i,j: Integer;
   sockHandle: WinSock.TSocket;
   WData: TWSAData;
   Addr: TSockAddr;
   Ver: word;
begin
   {$IFDEF VER130}
     Ver:=1;
   {$ELSE}
     Ver:=2;
   {$ENDIF}
   err:=WSAStartup(Ver,WData);
   if err<>0 then
      begin
         result:=0;
         exit;
      end;
   sockHandle:=socket(AF_INET,SOCK_DGRAM,0);
   if sockHandle=SOCKET_ERROR then
      begin
         WSACleanup;
         result:=0;
         exit;
      end;
   result:=0;
   for i:=RangeMin to RangeMax do
      begin
         j:=random(RangeMax-RangeMin)+RangeMin;
         Addr.sin_family:=AF_INET;
         Addr.sin_addr.s_addr:=INADDR_ANY;
         Addr.sin_port:=htons(j);
         if bind(SockHandle,Addr,SizeOf(Addr))=SOCKET_ERROR then
            continue;
         CloseSocket(sockHandle);
         result:=j;
         break;
      end;
   WSACleanup;
end;

function GetRandomTcpPort(RangeMin: integer; RangeMax: integer): Integer;
var
   err,i,j: Integer;
   sockHandle: WinSock.TSocket;
   WData: TWSAData;
   Addr: TSockAddr;
   ver: word;
begin
   {$IFDEF VER130}
     Ver:=1;
   {$ELSE}
     Ver:=2;
   {$ENDIF}
   err:=WSAStartup(ver,WData);
   if err<>0 then
      begin
         result:=0;
         exit;
      end;
   sockHandle:=socket(AF_INET,SOCK_STREAM,0);
   if sockHandle=SOCKET_ERROR then
      begin
         WSACleanup;
         result:=0;
         exit;
      end;
   result:=0;
   for i:=RangeMin to RangeMax do
      begin
         j:=random(RangeMax-RangeMin)+RangeMin;
         Addr.sin_family:=AF_INET;
         Addr.sin_addr.s_addr:=INADDR_ANY;
         Addr.sin_port:=htons(j);
         if bind(SockHandle,Addr,SizeOf(Addr))=SOCKET_ERROR then
            continue;
         CloseSocket(sockHandle);
         result:=j;
         break;
      end;
   WSACleanup;
end;

function GetLocalIP(): string;
var
   WSAData: TWSAData;
   HostEnt: PHostEnt;
   HostName: Ansistring;
   ver: word;
begin
   {$IFDEF VER130}
     Ver:=1;
   {$ELSE}
     Ver:=2;
   {$ENDIF}
   WSAStartup(ver,WsaData);
   SetLength(HostName,255);
   Gethostname(PAnsichar(HostName),255);
   SetLength(HostName,StrLen(PAnsiChar(HostName)));
   HostEnt:=gethostbyname(PAnsiChar(HostName));
   with HostEnt^ do
      Result:=Format('%d.%d.%d.%d',[Byte(h_addr^[0]),Byte(h_addr^[1]),Byte(h_addr^[2]),Byte(h_addr^[3])]);
   WSACleanup;
end;

function GetComputerName: string;
var
   FStr: PChar;
   FSize: Cardinal;
begin
   FSize := 255;
   GetMem(FStr, FSize);
   Windows.GetComputerName(FStr, FSize);
   Result := FStr;
   FreeMem(FStr);
end;

function GetLocalIpList(var IpList: TStringList):Integer;
type
  TaPInAddr = array[0..10] of PInAddr; 
  PaPInAddr = ^TaPInAddr; 
var 
  HostName : array [0..MAX_PATH] of Ansichar;
  NameLen:Integer; 
  WSData: TWSAData; 
  lpHostEnt:PHostEnt; 
  I:Integer; 
  pptr: PaPInAddr;
  Ver: word;
begin 
  Result := 0; 
   {$IFDEF VER130}
     Ver:=1;
   {$ELSE}
     Ver:=2;
   {$ENDIF}
  if WSAStartup(Ver, WSData) <>0 then
    Exit; 
  try 
    NameLen := sizeof(HostName); 
    fillchar(HostName,NameLen,0); 
    NameLen:=GetHostName(HostName,NameLen); 
    if NameLen = SOCKET_ERROR then 
      Exit; 
    lpHostEnt := GetHostByName(HostName); 
    if lpHostEnt = Nil then 
      Exit; 
    I := 0; 
    pPtr := PaPInAddr(lpHostEnt^.h_addr_list); 
    IpList.Clear; 
    while pPtr^[I] <> nil  do 
      begin 
        IpList.ADD(String(StrPas(inet_ntoa(pptr^[I]^))));
        Inc(I); 
      end; 
    Result := IpList.Count; 
  finally 
    WSACleanup; 
  end; 
end;

function GetMacAddr(): string;
var
  AI,Work : PIPAdapterInfo;
  Size    : Integer;
  Res     : Integer;
  Function MACToStr(ByteArr : PByte; Len : Integer) : String;
    Begin
      Result := '';
      While (Len > 0) do Begin
        Result := Result+IntToHex(ByteArr^,2)+'-';
        ByteArr := Pointer(Integer(ByteArr)+SizeOf(Byte));
        Dec(Len);
      End;
      SetLength(Result,Length(Result)-1);
    End;
begin
   Size := 5120;
   GetMem(AI,Size);
   work:=ai;
   Res := GetAdaptersInfo(AI,Size);
   If (Res <> ERROR_SUCCESS) Then
      Begin
         result:='';
         FreeMem(Ai);
         exit;
      End;
   Result:=MACToStr(@Work^.Address,Work^.AddressLength);
   FreeMem(Ai);
end;

function GetGatewayIp(): string;
var
  AI,Work : PIPAdapterInfo;
  Size    : Integer;
  Res     : Integer;
begin
   Size := 5120;
   GetMem(AI,Size);
   work:=ai;
   Res := GetAdaptersInfo(AI,Size);
   If (Res <> ERROR_SUCCESS) Then
      Begin
         result:='0.0.0.0';
         FreeMem(Ai);
         exit;
      End;
   result:=string(AnsiString(work.GatewayList.IPAddress));
   FreeMem(Ai);
end;


function GetDnsIp : string;
const
   IPCFG_DUMMY_FILE = '_dmytmpdns.tmp';
   IPCFG_WIN9X = 'winipcfg.exe /all /batch ';
   IPCFG_WINNT = 'ipconfig.exe /all';
   IPCFG_DNS_SERVER_LINE = 'DNS Servers';
   REG_NT_NAMESERVER_PATH ='System\CurrentControlSet\Services\Tcpip\Parameters';
   REG_NT_NAMESERVER = 'DhcpNameServer';
   REG_9X_NAMESERVER_PATH = 'System\CurrentControlSet\Services\MSTCP';
   REG_9X_NAMESERVER = 'NameServer';

   function BackSlashStr (const s : Ansistring) : Ansistring;
   begin
      Result := s;
      if Result[Length(Result)] <> '\' then
          Result := Result + '\';
   end;

   function GetWindowsPath: Ansistring;
   var
      Temp : array [0..MAX_PATH] of char;
   begin
      GetWindowsDirectory (Temp, SizeOf(Temp));
      Result := BackSlashStr(AnsiString(String(Temp)));
   end;

   function GetSystemPath: Ansistring;
   var
      Temp : array [0..MAX_PATH] of char;
   begin
      GetSystemDirectory (Temp, SizeOf(Temp));
   end;

   function LooksLikeIP(StrIn: string): boolean;
   var
      IPAddr : string;
      period, octet, i : Integer;
   begin
      result := false;
      IPAddr := StrIn;
      for i := 1 to 4 do
         begin
             if i = 4 then
                period := 255
             else
                period := pos('.',IPAddr);
             if period = 0 then
                exit;
             try
               octet := StrToInt(copy(IPAddr,1,period - 1));
             except
               exit;
             end;
             if (octet < (1 div i)) or (octet > 254) then
                exit;
             if i = 4 then
                result := true
             else
                IPAddr := copy(IPAddr,period+1,255);
         end;
   end;

   procedure GetConsoleOutput (const CommandLine : string; var Output : TStringList);
   var
      SA: TSecurityAttributes;
      SI: TStartupInfo;
      PI: TProcessInformation;
      StdOutFile, AppProcess, AppThread : THandle;
      RootDir, WorkDir, StdOutFileName:string;
   const
      FUNC_NAME = 'GetConsoleOuput';
   begin
       try
          StdOutFile:=0;
          AppProcess:=0;
          AppThread:=0;
          RootDir:=ExtractFilePath(ParamStr(0));
          WorkDir:=ExtractFilePath(CommandLine);
          if not (FileSearch(ExtractFileName(CommandLine),WorkDir)<>'') then
            WorkDir:=RootDir;
          FillChar(SA,SizeOf(SA),#0);
          SA.nLength:=SizeOf(SA);
          SA.lpSecurityDescriptor:=nil;
          SA.bInheritHandle:=True;
          StdOutFileName:=RootDir+'output.tmp';
          StdOutFile:=CreateFile(PChar(StdOutFileName),
                         GENERIC_READ or GENERIC_WRITE,
                         FILE_SHARE_READ or FILE_SHARE_WRITE,
                         @SA,
                         Create_ALWAYS,
                         FILE_ATTRIBUTE_TEMPORARY or
                         FILE_FLAG_WRITE_THROUGH,
                         0);
          if StdOutFile = INVALID_HANDLE_VALUE then
            raise Exception.CreateFmt('Function %s() failed!' + #10#13 +'Command line = %s',[FUNC_NAME,CommandLine]);
          FillChar(SI,SizeOf(SI),#0);
          with SI do
             begin
               cb:=SizeOf(SI);
               dwFlags:=STARTF_USESHOWWINDOW or STARTF_USESTDHANDLES;
               wShowWindow:=SW_HIDE;
               hStdInput:=GetStdHandle(STD_INPUT_HANDLE);
               hStdError:=StdOutFile;
               hStdOutput:=StdOutFile;
            end;
          if CreateProcess(nil, PChar(CommandLine), nil, nil,
                           True, 0, nil,
                           PChar(WorkDir), SI, PI) then
            begin
               WaitForSingleObject(PI.hProcess,INFINITE);
               AppProcess:=PI.hProcess;
               AppThread:=PI.hThread;
            end
          else
            raise Exception.CreateFmt('CreateProcess() in function %s() failed!' + #10#13 + 'Command line = %s',[FUNC_NAME,CommandLine]);
          CloseHandle(StdOutFile);
          StdOutFile:=0;
          Output.Clear;
          Output.LoadFromFile (StdOutFileName);
       finally
          if StdOutFile <> 0 then
             CloseHandle(StdOutFile);
          if AppProcess <> 0 then
             CloseHandle(AppProcess);
          if AppThread <> 0 then
             CloseHandle(AppThread);
          if FileExists(StdOutFileName) then
            SysUtils.DeleteFile(StdOutFileName);
       end;
   end;

   function GetBasicOsType : LongWord;
   var
      VerInfo : TOsVersionInfo;
   begin
      VerInfo.dwOSVersionInfoSize := SizeOf(VerInfo);
      GetVersionEx (VerInfo);
      Result := VerInfo.dwPlatformId;
   end;

   function GetIpCfg9xOutPath : string;
   begin
      Result := String(GetWindowsPath) + IPCFG_DUMMY_FILE;
   end;

   function GetIpCfgExePath : Ansistring;
   begin
      Result := '';
      Case GetBasicOsType of
          VER_PLATFORM_WIN32_WINDOWS : Result := GetWindowsPath + AnsiString(IPCFG_WIN9X + GetIpCfg9xOutPath);
          VER_PLATFORM_WIN32_NT : Result := GetSystemPath + IPCFG_WINNT;
      end;
   end;

   function GetDnsIpFromReg: string;
   var
      OpenKey : HKEY;
      Vn, SubKey : PChar;
      DataType, DataSize : integer;
      Temp : array [0..2048] of char;
   begin
      Result := '';
      SubKey := '';
      Vn := '';
      case GetBasicOsType of
          VER_PLATFORM_WIN32_WINDOWS :
             begin
               SubKey := REG_9X_NAMESERVER_PATH;
               Vn := REG_9X_NAMESERVER;
             end;
          VER_PLATFORM_WIN32_NT :
             begin
               SubKey := REG_NT_NAMESERVER_PATH;
               Vn := REG_NT_NAMESERVER;
             end;
      end;
      if RegOpenKeyEx (HKEY_LOCAL_MACHINE, SubKey, REG_OPTION_NON_VOLATILE, KEY_READ, OpenKey) = ERROR_SUCCESS then
         begin
             DataType := REG_SZ;
             DataSize := SizeOf(Temp);
             if RegQueryValueEx (OpenKey, Vn, nil, @DataType, @Temp,
               @DataSize) = ERROR_SUCCESS then
               Result := string(Temp);
             RegCloseKey (OpenKey);
         end;
   end;

   function GetDnsIpFromIpCfgOut (const Output : TStringList; var DnsIp : string) : boolean;
   var
      i : integer;
   begin
      Result := FALSE;
      if Output.Count >= 1 then
          for i := 0 to Output.Count - 1 do
          begin
            if Pos(IPCFG_DNS_SERVER_LINE, Output[i]) > 0 then
            begin
              DnsIp := Trim(Copy (Output[i], Pos(':', Output[i])+1,
                Length(Output[i])));
              Result := LooksLikeIp (DnsIp);
            end;
          end;
   end;

var
   Output : TStringList;
   DnsIp: string;
   CmdLine : string;
begin
  CmdLine := string(GetIpCfgExePath);
  if CmdLine <> '' then
     begin
       Output := TStringList.Create;
       try
         case GetBasicOsType of
           VER_PLATFORM_WIN32_WINDOWS :
              begin
                GetConsoleOutput (CmdLine, Output);
                Output.LoadFromFile (GetIpCfg9xOutPath);
              end;
           else
              GetConsoleOutput (CmdLine, Output);
         end;
         if GetDnsIpFromIpCfgOut (Output, DnsIp) then
            Result := DnsIp
         else
            Result := GetDnsIpFromReg;
       finally
         Output.Free;
       end;
     end;
end;

function GetIpMask(): string;
var
   tmpstr,s1,s2,s3: string;
   i: integer;
begin
   tmpstr:=GetLocalIp();
   i:=pos('.',tmpstr);
   s1:=copy(tmpstr,1,i-1);
   delete(tmpstr,1,i);
   i:=pos('.',tmpstr);
   s2:=copy(tmpstr,1,i-1);
   delete(tmpstr,1,i);
   i:=pos('.',tmpstr);
   s3:=copy(tmpstr,1,i-1);
   delete(tmpstr,1,i);
   if strtoint(s1) in [1..126] then
      result:='255.0.0.0'
   else
      begin
         if strtoint(s1) in [128..191] then
            result:='255.255.0.0'
         else
            result:='255.255.255.0';
      end;
end;

function GetBroadcastIp(): string;
var
   tmpstr,s1,s2,s3: string;
   i: integer;
begin
   tmpstr:=GetLocalIp();
   i:=pos('.',tmpstr);
   s1:=copy(tmpstr,1,i-1);
   delete(tmpstr,1,i);
   i:=pos('.',tmpstr);
   s2:=copy(tmpstr,1,i-1);
   delete(tmpstr,1,i);
   i:=pos('.',tmpstr);
   s3:=copy(tmpstr,1,i-1);
   delete(tmpstr,1,i);
   if strtoint(s1) in [1..126] then
      result:=s1+'.255.255.255'
   else
      begin
         if strtoint(s1) in [128..191] then
            result:=s1+'.'+s2+'.255.255'
         else
            result:=s1+'.'+s2+'.'+s3+'.255';
      end;
end;

function IsValidIP(aStrIP: AnsiString): Boolean;
var
   iCount:Integer;
   iIPLength:Integer;
   iFieldLength:Integer;
   iFieldStart:Integer;
   iDotCount:Integer;
   strTemp: AnsiString;
begin
   iIPLength:=Length(astrIP);
   if (iIPLength>15) or (iIPLength<7) or (aStrIP='0.0.0.0') then
      begin
         Result:=False;
         Exit;
      end;
   iDotCount:=0;
   iFieldLength:=0;
   iFieldStart:=1;
   for iCount:=1 to iIPLength do
      begin
         case aStrIP[iCount] of
         '0','1','2','3','4','5','6','7','8','9':
            begin
               iFieldLength:=iFieldLength+1;
               if (3<iFieldLength)   then
                  begin
                     Result:=False;
                     Exit;
                  end;
            end;
         '.':
            begin
               if 0=iFieldLength then
                  begin
                     Result:=False;
                     Exit;
                  end;
               strTemp:=copy(astrIp,iFieldStart,iCount-iFieldStart);
               if(255<StrToInt(strTemp))then
                  begin
                     Result:=False;
                     Exit;
                  end;
               iDotCount:=iDotCount+1;
               if 3<iDotCount then
                  begin
                     Result:=False;
                     Exit;
                  end;
               iFieldLength:=0;
               iFieldStart:=iCount+1;
            end;
         else
            begin
               Result:=False;
               Exit;
            end;
         end;
      end;
   Result:=True;
end;

function IP2Integer(IP: Ansistring): integer;
begin
   if IsValidIp(IP) then
      result:=inet_addr(pAnsichar(ip))
   else
      Result:=0;
end;

function Integer2IP(IntValue: integer): ansistring;
type 
    PIp_Addr=^IP_Addr; 
    Ip_Addr=packed record
          a,b,c,d: byte;
    end; 
var
   Ip: PIp_Addr;
begin 
    Ip:=@IntValue;
    result:=AnsiString(format('%d.%d.%d.%d',[IP.a,Ip.b,Ip.c,Ip.d]));
end;

function AddNetConnection(NetPath: string; PassWord: string; LocalPath: string): Boolean;
var
  Res: Dword;
  NetP,LocalP,Psw: pchar;
begin
  Result := False;
  NetP:=PChar(netpath);
  LocalP:=PChar(LocalPath);
  Psw:=PChar(PassWord);
  Res := WNetAddConnection(NetP,Psw,LocalP);
  if Res <> No_Error then
     exit;
  Result := True;
end;

function RemoveNetConnection(LocalPath: string): Boolean;
var
  strLocalName : PChar;
begin
  strLocalName:=PChar(LocalPath);
  WNetCancelConnection2(strLocalName,CONNECT_UPDATE_PROFILE,False);
  Result:=true;
end;

Function HostIsOnline(IpAddr: string): Boolean;
type
  PIPOptionInformation = ^TIPOptionInformation;
  TIPOptionInformation = packed record
     TTL:         Byte;      // Time To Live (used for traceroute)
     TOS:         Byte;      // Type Of Service (usually 0)
     Flags:       Byte;      // IP header flags (usually 0)
     OptionsSize: Byte;      // Size of options data (usually 0, max 40)
     OptionsData: PChar;     // Options data buffer
  end;

  PIcmpEchoReply = ^TIcmpEchoReply;
  TIcmpEchoReply = packed record
     Address:       DWord;                // replying address
     Status:        DWord;                // IP status value (see below)
     RTT:           DWord;                // Round Trip Time in milliseconds
     DataSize:      Word;                 // reply data size
     Reserved:      Word;
     Data:          Pointer;              // pointer to reply data buffer
     Options:       TIPOptionInformation; // reply options
  end;

  TIcmpCreateFile = function: THandle; stdcall;
  TIcmpCloseHandle = function(IcmpHandle: THandle): Boolean; stdcall;
  TIcmpSendEcho = function(
     IcmpHandle:          THandle;
     DestinationAddress:  DWord;
     RequestData:         Pointer;
     RequestSize:         Word;
     RequestOptions:      PIPOptionInformation;
     ReplyBuffer:         Pointer;
     ReplySize:           DWord;
     Timeout:             DWord): DWord; stdcall;

const
  Size = 32;
  TimeOut = 1000;
var
  wsadata: TWSAData;
  Address: DWord;                     // Address of host to contact
  HostName: Ansistring;
  HostIP: AnsiString;                 // Name and dotted IP of host to contact
  Phe: PHostEnt;                      // HostEntry buffer for name lookup
  BufferSize, nPkts: Integer;
  pReqData, pData: Pointer;
  pIPE: PIcmpEchoReply;               // ICMP Echo reply buffer
  IPOpt: TIPOptionInformation;        // IP Options for packet to send
const
  IcmpDLL = 'icmp.dll';

var
  hICMPlib: HModule;
  IcmpCreateFile : TIcmpCreateFile;
  IcmpCloseHandle: TIcmpCloseHandle;
  IcmpSendEcho:    TIcmpSendEcho;
  hICMP: THandle;                     // Handle for the ICMP Calls
  Ver: word;
begin
  Result:=True;
   {$IFDEF VER130}
     Ver:=1;
   {$ELSE}
     Ver:=2;
   {$ENDIF}
  if WSAStartup(Ver,wsadata) <> 0 then begin
     Result:=False;
     halt;
  end;
  hICMPlib := loadlibrary(icmpDLL);
  if hICMPlib <> 0 then
     begin
        @ICMPCreateFile := GetProcAddress(hICMPlib, 'IcmpCreateFile');
        @IcmpCloseHandle:= GetProcAddress(hICMPlib, 'IcmpCloseHandle');
        @IcmpSendEcho:= GetProcAddress(hICMPlib, 'IcmpSendEcho');
        if (@ICMPCreateFile = Nil) or (@IcmpCloseHandle = Nil) or (@IcmpSendEcho = Nil) then
           begin
              Result:=False;
              halt;
          end;
        hICMP := IcmpCreateFile;
        if hICMP = INVALID_HANDLE_VALUE then
           begin
              Result:=False;
              halt;
           end;
     end
  else
     begin
       Result:=False;
       halt;
     end;
  Address := inet_addr(PAnsiChar(AnsiString(IpAddr)));
  if Address = INADDR_NONE then
    begin
      Phe := GetHostByName(PAnsiChar(AnsiString(IpAddr)));
      if Phe = Nil then
        Result:=False
      else
        begin
          Address := longint(plongint(Phe^.h_addr_list^)^);
          HostName := Phe^.h_name;
          HostIP := StrPas(inet_ntoa(TInAddr(Address)));
        end;
    end
  else
    begin
      Phe := GetHostByAddr(@Address, 4, PF_INET);
      if Phe = Nil then
         Result:=False;
    end;

  if Address = INADDR_NONE then
    Result:=False;
  BufferSize := SizeOf(TICMPEchoReply) + Size;
  GetMem(pReqData, Size);
  GetMem(pData, Size);
  GetMem(pIPE, BufferSize);
  FillChar(pReqData^, Size, $AA);
  pIPE^.Data := pData;
  FillChar(IPOpt, SizeOf(IPOpt), 0);
  IPOpt.TTL := 64;
  NPkts := IcmpSendEcho(hICMP, Address, pReqData, Size,
                        @IPOpt, pIPE, BufferSize, TimeOut);
  if NPkts = 0 then
     Result:=False;
  FreeMem(pIPE); FreeMem(pData); FreeMem(pReqData);
  IcmpCloseHandle(hICMP);
  FreeLibrary(hICMPlib);
  if WSACleanup <> 0 then
     Result:=False;
end;

procedure GetProxyServer(protocol: string; var ProxyServerIp: string; var ProxyServerPort: Integer);
   function GetProxyInformation: string;
   var
      ProxyInfo: PInternetProxyInfo;
      Len: LongWord;
   begin
      Result := '';
      Len := 4096;
      GetMem(ProxyInfo, Len);
      try
         if InternetQueryOption(nil, INTERNET_OPTION_PROXY, ProxyInfo, Len) then
            if ProxyInfo^.dwAccessType = INTERNET_OPEN_TYPE_PROXY then
               Result := String(AnsiString(ProxyInfo^.lpszProxy));
      finally
         FreeMem(ProxyInfo);
      end;
   end;
var
   i: Integer;
   proxyinfo: string;
begin
   ProxyServerIp := '';
   ProxyServerPort := 0;
   proxyinfo := GetProxyInformation;
   if proxyinfo = '' then
      Exit;
   protocol := protocol + '=';
   i := Pos(protocol, proxyinfo);
   if i > 0 then
      begin
         Delete(proxyinfo, 1, i + Length(protocol));
         i := Pos(AnsiString(';'), ProxyServerIp);
         if i > 0 then
            proxyinfo := Copy(proxyinfo, 1, i - 1);
      end;
   i := Pos(AnsiString(':'), proxyinfo);
   if i > 0 then
      begin
         ProxyServerPort := StrToIntDef(Copy(proxyinfo, i + 1, Length(proxyinfo) - i), 0);
         ProxyServerIp := Copy(proxyinfo, 1, i - 1);
      end;
end;

const
  MODEM = 1;
  LAN = 2;
  PROXY = 4;
  BUSY = 8;
function GetConnectionKind: integer;
var
  flags: DWORD;
  ok: boolean;
begin
  result:=0;
  ok := InternetGetConnectedState(@flags, 0);
  if ok then
     begin
        if (flags and MODEM) = MODEM then
           result:=1;
        if (flags and LAN) = LAN then
           result:=2;
        if (flags and PROXY) = PROXY then
           result:=4;
        if (flags and BUSY) = BUSY then
           result:=8;
     end;
end;

function AuthorizedApp(AppName: string; AppFileName: string): boolean;
var
   FwMgr,Profile,FwApp: variant;
begin
   try
      FwMgr := CreateOLEObject('HNetCfg.FwMgr');
      Profile := FwMgr.LocalPolicy.CurrentProfile;
      FwApp := CreateOLEObject('HNetCfg.FwAuthorizedApplication');
      FwApp.Name :=AppName;
      FwApp.ProcessImageFileName := AppFileName;
      FwApp.Enabled  := true;
      Profile.AuthorizedApplications.Add(FwApp);
      result:=true;
   except
      result:=false;
   end;
end;

function AddFirewallPort(PortName: string; Protocol: string; Port: integer): boolean;
var
   FwMgr,Profile,FwPort: variant;
   P: integer;
begin
   if uppercase(Protocol)='TCP' then
      p:=6
   else
      p:=17;
   try
      FwMgr := CreateOLEObject('HNetCfg.FwMgr');
      Profile := FwMgr.LocalPolicy.CurrentProfile;
      FwPort := CreateOLEObject('HNetCfg.FWOpenPort');
      FwPort.Name :=PortName;
      FwPort.Protocol := p;
      FwPort.Port  := port;
      Profile.GloballyOpenPorts.Add(FwPort);
      result:=true;
   except
      result:=false;
   end;
end;

function CloseWinFirewall: boolean;
var
  SCM, hService: LongWord;
  sStatus: TServiceStatus;
begin
  SCM := OpenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
  if scm<>0 then
     begin
        hService := OpenService(SCM, PChar('SharedAccess'), SERVICE_ALL_ACCESS);
        if hService<>0 then
           begin
              ControlService(hService, SERVICE_CONTROL_STOP, sStatus);
              result:=true;
           end
        else
           result:=false;
        CloseServiceHandle(hService);
     end
  else
     result:=false;
end;

function IsFileInUse(AName: string): boolean;
var
   hFileRes: HFILE;
begin
   Result := False;
   if not FileExists(AName) then
      exit;
   hFileRes := CreateFile(PChar(AName), GENERIC_READ or GENERIC_WRITE, 0,
       nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
   Result   := hFileRes = INVALID_HANDLE_VALUE;
   if not Result then
       CloseHandle(hFileRes);
end;

function GetAbsolutePath(BasePath, RelativePath: string): string;
var
   base,dir: string;
   j: integer;
begin
   if RelativePath='' then
      begin
         result:='';
         exit;
      end;
   if copy(RelativePath,2,1)=':' then
      begin
         result:=RelativePath;
         exit;
      end;
   base:=BasePath;
   if copy(base,length(base),1)='\' then
      delete(base,length(base),1);
   dir:=RelativePath;
   while true do
      begin
         if copy(dir,1,2)='.\' then
            begin
               delete(dir,1,2);
               continue;
            end;
         if copy(dir,1,3)='..\' then
            begin
               delete(dir,1,3);
               j:=length(base);
               while j>0 do
                  begin
                     if base[j]='\' then
                        begin
                           delete(base,j,1);
                           break;
                        end;
                     delete(base,j,1);
                     dec(j);
                  end;
               continue;
            end;
         break;
      end;
   if copy(dir,1,1)='\' then
      result:=base+dir
   else
      result:=base+'\'+dir;
end;
function AdjustProcessPrivilege(ProcessHandle: THandle; Token_Name: PChar): boolean;
var
   Token:       cardinal;
   TokenPri:    _TOKEN_PRIVILEGES;
   ProcessDest: int64;
   l:           DWORD;
begin
   Result := False;
   if OpenProcessToken(ProcessHandle, TOKEN_Adjust_Privileges, Token) then
   begin
       if LookupPrivilegeValue(nil, Token_Name, ProcessDest) then
       begin
         TokenPri.PrivilegeCount := 1;
         TokenPri.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
         TokenPri.Privileges[0].Luid := ProcessDest;
         l := 0;
         if AdjustTokenPrivileges(Token, False, TokenPri, sizeof(TokenPri), nil, l) then
           Result := True;
       end;
   end;
end;

function IsVista: boolean;
begin
   Result := not (Win32MajorVersion <6);
end;

function GetWanIp: string;
var
   xml : OleVariant;
   r: string;
   p1,p2 : Integer;
begin
   xml := CreateOleObject('Microsoft.XMLHTTP');
   xml.Open('GET','http://www.net.cn/static/customercare/yourIP.asp', False);
   xml.Send;
   r := String(xml.responseText);
   p1:=Pos('<h2>',r);
   p2:=Pos('</h2>',r);
   Result := Copy(r, p1+4, p2-p1-4);
end;

procedure setLocalDateTime(aDT: TDateTime);
var
   lSystemDateTime : TSystemTime;
begin
   DateTimeToSystemTime(aDT,lSystemDateTime);
   SetLocalTime(lSystemDateTime);
end;

function LastParameter(ParaStr: AnsiString): AnsiString;
var
   j: integer;
begin
   j:=pos(AnsiString(#13),ParaStr);
   if j>0 then
      result:=copy(ParaStr,1,j-1)
   else
      begin
         j:=pos(AnsiString(#14),ParaStr);
         if j>0 then
            result:=copy(ParaStr,1,j-1)
         else
            result:=ParaStr;
      end;
end;

procedure SafeDelay(Ms: integer);
var
   t0: DWORD;
begin
   t0:=gettickcount;
   application.processmessages;
   while abs(GetTickCount-t0)<ms do
      application.processmessages;
end;

var
  dwSize: DWORD;
  pMibIfTable: PMIB_IFTABLE;
function GetAdapterTypeString(const dwType: DWORD): Ansistring;
begin
  case dwType of
    MIB_IF_TYPE_OTHER: Result := '';
    MIB_IF_TYPE_ETHERNET: Result := '̫';
    MIB_IF_TYPE_TOKENRING: Result := 'ƻ';
    MIB_IF_TYPE_FDDI: Result := 'FDDI';
    MIB_IF_TYPE_PPP: Result := 'PPP';
    MIB_IF_TYPE_LOOPBACK: Result := '·';
    MIB_IF_TYPE_SLIP: Result := 'SLIP';
  end;
end;

function GetAdapterStatusString(const dwOperStatus: DWORD): Ansistring;
begin
  case dwOperStatus of
    MIB_IF_OPER_STATUS_NON_OPERATIONAL: Result := '';
    MIB_IF_OPER_STATUS_UNREACHABLE: Result := 'ɴ';
    MIB_IF_OPER_STATUS_DISCONNECTED: Result := 'Ͽ';
    MIB_IF_OPER_STATUS_CONNECTING: Result := '';
    MIB_IF_OPER_STATUS_CONNECTED: Result := '';
    MIB_IF_OPER_STATUS_OPERATIONAL: Result := 'ͨ';
  end;
end;

procedure ScanAdapters(var AdapterStatuses: TAdapterStatuses);
var
  dwRetVal: DWORD;
  num, i: Longint;
begin
  dwRetVal := GetIfTable(pMibIfTable, @dwSize, False);
  if dwRetVal = NO_ERROR then
  begin
    num := pMibIfTable^.dwNumEntries;
    if Length(AdapterStatuses) <> num then
      SetLength(AdapterStatuses, num);
    for i := Low(AdapterStatuses) to High(AdapterStatuses) do
    begin
      AdapterStatuses[i].dwType := pMibIfTable^.table[i].dwType;
      AdapterStatuses[i].dwOperStatus := pMibIfTable^.table[i].dwOperStatus;
      AdapterStatuses[i].bDescr := pMibIfTable^.table[i].bDescr;
    end;
  end;
end;

procedure FindAProcess(const AFilename: string; const PathMatch: Boolean; var ProcessID: DWORD);
var
   lppe: TProcessEntry32;
   SsHandle: Thandle;
   FoundAProc, FoundOK: boolean;
begin
   SsHandle:=CreateToolHelp32SnapShot(TH32CS_SNAPALL,0);
   FoundAProc:=Process32First(Sshandle,lppe);
   ProcessId:=0;
   while FoundAProc do
      begin
       if PathMatch then
          FoundOK:=Stricomp(lppe.szExefile,PChar(AFilename))=0
       else
          FoundOK:=Stricomp(PChar(ExtractFilename(lppe.szExefile)),PChar(ExtractFilename(AFilename)))=0;
       if FoundOK then
          begin
             ProcessID:=lppe.th32ProcessID;
             break;
          end;
       FoundAProc:=Process32Next(SsHandle,lppe);
   end;
   CloseHandle(SsHandle);
end;

procedure Str2Stream(Str: Ansistring; var aStream: TMemoryStream);
begin
   if aStream<>nil then
      aStream.Clear
   else
      aStream:=TMemoryStream.Create;
   if str='' then
      exit;
   aStream.WriteBuffer(Str[1],length(str));
   aStream.Position:=0;
end;

procedure Stream2Str(aStream: TMemoryStream; var aStr: Ansistring);
begin
   aStream.Position:=0;
   setlength(aStr,aStream.size);
   aStream.ReadBuffer(aStr[1],aStream.Size);
end;

procedure Str2Cds(Str: Ansistring; var aCds: TClientDataset);
var
   stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   Stream.WriteBuffer(Str[1],length(str));
   Stream.Position:=0;
   if aCds<>nil then
      aCds.Close
   else
      aCds:=TClientDataset.Create(nil);
   aCds.LoadFromStream(Stream);
   Stream.Free
end;

procedure Cds2Str(aCds: TClientDataset; Fmt: TDataPacketFormat; var aStr: Ansistring);
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   aCds.SaveToStream(Stream,Fmt);
   Stream2Str(Stream,aStr);
   Stream.Free;
end;

procedure Str2Parcel(Str: Ansistring; var aParcel: TQBParcel);
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   Stream.WriteBuffer(Str[1],length(str));
   Stream.Position:=0;
   if aParcel<>nil then
      aParcel.Clear
   else
      aParcel:=TQBParcel.Create;
   aParcel.LoadFromStream(Stream);
   Stream.Free;
end;

procedure Parcel2Str(aParcel: TQBParcel; var aStr: Ansistring);
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   aParcel.SaveToStream(Stream);
   Stream2Str(Stream,aStr);
   Stream.Free;
end;

procedure CloneParcel(FromParcel: TQBParcel; var ToParcel: TQBParcel);
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   FromParcel.SaveToStream(Stream);
   if ToParcel=nil then
      ToParcel:=TQBParcel.Create
   else
      ToParcel.Clear;
   Stream.Position:=0;
   ToParcel.LoadFromStream(Stream);
   Stream.Free;
end;

function blob2stream(Rs: _RecordSet; FieldName: string; aStream: TMemoryStream): boolean;
var
   Buf: Variant;   
   pBuf: PByte;   
   BufLen: Integer;   
begin   
   if (Rs=nil) or (FieldName='') or (aStream=nil) then
      begin
         result:=false;
         exit;
      end;
   while True do 
      begin   
         Buf:=Rs.Fields[FieldName].GetChunk(4096);   
         if VarIsNull(Buf) then   
            Break;
         BufLen:=VarArrayHighBound(Buf,1);
         pBuf:=VarArrayLock(Buf);
         aStream.Write(pBuf^,BufLen);   
         VarArrayUnlock(Buf);   
      end;  
   result:=true; 
end;   

procedure VariantToStream (const v: OleVariant; Stream: TMemoryStream);
var
  p: pointer;
begin
  try
     Stream.Position:=0;
     Stream.Size:=VarArrayHighBound(v,1)-VarArrayLowBound(v,1)+1;
     p:=VarArrayLock(v);
     Stream.Write(p^,Stream.Size);
     VarArrayUnlock(v);
     Stream.Position:=0;
  except
     Stream.clear;
  end;
end;

procedure StreamToVariant(Stream: TMemoryStream; var v: OleVariant);
var
  p: pointer;
begin
  v:=VarArrayCreate([0,Stream.Size-1],varByte);
  p:=VarArrayLock(v);
  Stream.Position:=0;
  Stream.Read(p^,Stream.Size);
  VarArrayUnlock(v);
end;


initialization
  GetIfTable(nil, @dwSize, False);
  GetMem(pMibIfTable, dwSize);
  LocalCodePage:=WinCodePage;

finalization
  FreeMem(pMibIfTable);

end.
