//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit QBDBJson;

interface

uses
  Windows, SysUtils, Classes, QBJson, DbClient, AdoDB,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  QBParcel, db
{$IFDEF VER130}
;
{$ELSE}
,MidasLib;
{$ENDIF}

type
   TStructureRecord=Record
      FieldName: Ansistring;
      FieldType: Ansistring;
      JsonType: Ansistring;
      FieldSize: integer;
end;

Type
   TFieldAttrJson=class(TQBJson)
      class function CreateIt(aFieldName: Ansistring; aFieldType: Ansistring; aJsonType: Ansistring; aFieldSize: integer): TFieldAttrJson; Overload;
      class function CreateIt(aCds: TClientDataset; Index: integer): TFieldAttrJson; Overload;
      class function CreateIt(aDs: TAdoDataset; Index: integer): TFieldAttrJson; Overload;

      procedure ResolveTo(var aFieldName: AnsiString; var aFieldType: AnsiString; var aJsonType: AnsiString; var aFieldSize: integer);
   end;

   TFieldAttrListJson=class(TQBJson)
      class function CreateIt(aCds: TClientDataset): TFieldAttrListJson; Overload;
      class function CreateIt(aDs: TAdoDataset): TFieldAttrListJson; Overload;

      function ResolveTo: TQBJsonArray;
      function ConvertToCds(aCds: TClientDataset): boolean;
   end;

   TRecordJson=class(TQBJson)
      class function CreateIt(aFAListJson: TFieldAttrListJson): TRecordJson; Overload;
      class function CreateIt(aCds: TClientDataset): TRecordJson; Overload;
      class function CreateIt(aDs: TAdoDataset): TRecordJson; Overload;

      function ResolveTo(aDs: TAdoDataset): boolean; Overload;
      function ResolveTo(aCds: TClientDataset): boolean; Overload;
   end;

   TRecordListJson=class(TQBJson)
      class function CreateIt(aCds: TClientDataset): TRecordListJson; Overload;
      class function CreateIt(aDs: TAdoDataset): TRecordListJson; Overload;

      procedure PutRecordJson(aRecordJson: TRecordJson); Overload;
      function GetRecordJson(index: integer): TRecordJson;

      function ResolveTo(aDs: TAdoDataset): boolean; Overload;
      function ResolveTo(aCds: TClientDataset): boolean; Overload;
   end;

   TTableJson=class(TQBJson)
      class function CreateIt(TableName: AnsiString; aCds: TClientDataset): TTableJson; Overload;
      class function CreateIt(TableName: AnsiString; aDs: TAdoDataset): TTableJson; Overload;

      function ResolveTo(aDs: TAdoDataset): boolean; Overload;
      function ResolveTo(aCds: TClientDataset): boolean; Overload;
   end;

implementation

procedure FieldTypeToJsonType(FieldType: TFieldType; var TypeStr: AnsiString; var JsonType: AnsiString);
begin
   Case FieldType of
      ftUnknown:
         begin
            TypeStr:='Unknown';
            JsonType:='Variant';
         end;
      ftString:
         begin
            TypeStr:='String';
            JsonType:='String';
         end;
      ftSmallint:
         begin
            TypeStr:='SmallInt';
            JsonType:='Integer';
         end;
      ftInteger:
         begin
            TypeStr:='Integer';
            JsonType:='Integer';
         end;
      ftWord:
         begin
            TypeStr:='Word';
            JsonType:='Integer';
         end;
      ftBoolean:
         begin
            TypeStr:='Boolean';
            JsonType:='Boolean';
         end;
      ftFloat:
         begin
            TypeStr:='Float';
            JsonType:='Double';
         end;
      ftCurrency:
         begin
            TypeStr:='Currency';
            JsonType:='Double';
         end;
      ftBCD:
         begin
            TypeStr:='BCD';
            JsonType:='Double';
         end;
      ftDate:
         begin
            TypeStr:='Date';
            JsonType:='String';
         end;
      ftTime:
         begin
            TypeStr:='Time';
            JsonType:='String';
         end;
      ftDateTime:
         begin
            TypeStr:='DateTime';
            JsonType:='String';
         end;
      ftBytes:
         begin
            TypeStr:='Bytes';
            JsonType:='Integer';
         end;
      ftVarBytes:
         begin
            TypeStr:='VarBytes';
            JsonType:='Variant';
         end;
      ftAutoInc:
         begin
            TypeStr:='AutoInc';
            JsonType:='Integer';
         end;
      ftBlob:
         begin
            TypeStr:='Blob';
            JsonType:='Variant';
         end;
      ftMemo:
         begin
            TypeStr:='Memo';
            JsonType:='String';
         end;
      ftGraphic:
         begin
            TypeStr:='Graphic';
            JsonType:='String';
         end;
      ftFmtMemo:
         begin
            TypeStr:='FmtMemo';
            JsonType:='String';
         end;
      ftParadoxOle:
         begin
            TypeStr:='ParadoxOle';
            JsonType:='Variant';
         end;
      ftDBaseOle:
         begin
            TypeStr:='DBaseOle';
            JsonType:='Variant';
         end;
      ftTypedBinary:
         begin
            TypeStr:='TypedBinary';
            JsonType:='Variant';
         end;
      ftCursor:
         begin
            TypeStr:='Cursor';
            JsonType:='Variant';
         end;
      ftFixedChar:
         begin
            TypeStr:='FixedChar';
            JsonType:='String';
         end;
      ftWideString:
         begin
            TypeStr:='WideString';
            JsonType:='String';
         end;
      ftLargeint:
         begin
            TypeStr:='Largeint';
            JsonType:='Integer';
         end;
      ftADT:
         begin
            TypeStr:='ADT';
            JsonType:='Variant';
         end;
      ftArray:
         begin
            TypeStr:='Array';
            JsonType:='Variant';
         end;
      ftReference:
         begin
            TypeStr:='Reference';
            JsonType:='Variant';
         end;
      ftDataSet:
         begin
            TypeStr:='DataSet';
            JsonType:='Variant';
         end;
      ftOraBlob:
         begin
            TypeStr:='OraBlob';
            JsonType:='Variant';
         end;
      ftOraClob:
         begin
            TypeStr:='OraClob';
            JsonType:='Variant';
         end;
      ftVariant:
         begin
            TypeStr:='Variant';
            JsonType:='Variant';
         end;
      ftInterface:
         begin
            TypeStr:='Interface';
            JsonType:='Variant';
         end;
      ftIDispatch:
         begin
            TypeStr:='IDispatch';
            JsonType:='Variant';
         end;
      ftGuid:
         begin
            TypeStr:='Guid';
            JsonType:='String';
         end;
      else
         begin
            TypeStr:='Unknown';
            JsonType:='Variant';
         end;
   end;
end;

{TFieldAttrJson}

class function TFieldAttrJson.CreateIt(aFieldName: AnsiString; aFieldType: AnsiString; aJsonType: AnsiString; aFieldSize: integer): TFieldAttrJson;
begin
   Result:=TFieldAttrJson.Create;
   Result.Put('FieldName',aFieldName);
   Result.Put('FieldType',aFieldType);
   Result.Put('JsonType',aJsonType);
   Result.Put('FieldSize',aFieldSize);
end;

class function TFieldAttrJson.CreateIt(aCds: TClientDataset; Index: integer): TFieldAttrJson;
var
   FieldName,FieldType,JsonType: AnsiString;
   FieldSize: integer;
begin
   if (not aCds.Active) or (index<0) or (index>=aCds.Fields.Count) then
      begin
         Result:=nil;
         exit;
      end;
   Result:=TFieldAttrJson.Create;
   FieldName:=AnsiString(aCds.Fields[Index].FieldName);
   FieldSize:=aCds.Fields[Index].Size;
   FieldTypeToJsonType(aCds.Fields[Index].DataType,FieldType,JsonType);
   Result.Put('FieldName',FieldName);
   Result.Put('FieldType',FieldType);
   Result.Put('JsonType',JsonType);
   Result.Put('FieldSize',FieldSize);
end;

class function TFieldAttrJson.CreateIt(aDs: TAdoDataset; Index: integer): TFieldAttrJson;
var
   FieldName,FieldType, JsonType: AnsiString;
   FieldSize: integer;
begin
   if (not aDs.Active) or (index<0) or (index>=aDs.Fields.Count) then
      begin
         Result:=nil;
         exit;
      end;
   Result:=TFieldAttrJson.Create;
   FieldName:=AnsiString(aDs.Fields[Index].FieldName);
   FieldSize:=aDs.Fields[Index].Size;
   FieldTypeToJsonType(aDs.Fields[Index].DataType,FieldType,JsonType);
   Result.Put('FieldName',FieldName);
   Result.Put('FieldType',FieldType);
   Result.Put('JsonType',JsonType);
   Result.Put('FieldSize',FieldSize);
end;

procedure TFieldAttrJson.ResolveTo(var aFieldName: AnsiString; var aFieldType: AnsiString; var aJsonType: AnsiString; var aFieldSize: integer);
begin
   aFieldName:=Self.GetString('FieldName');
   aFieldType:=Self.GetString('FieldType');
   aJsonType:=Self.GetString('JsonType');
   aFieldSize:=Self.GetInt('FieldSize');
end;


{TFieldAttrListJson}

class function TFieldAttrListJson.CreateIt(aCds: TClientDataset): TFieldAttrListJson;
var
   i: integer;
   FieldCount: integer;
   FAJson: TFieldAttrJson;
   FieldList: TQBJsonArray;
begin
   if not aCds.Active then
      begin
         Result:=nil;
         exit;
      end;
   FieldCount:=aCds.Fields.Count;
   FieldList:=TQBJsonArray.create;
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson.CreateIt(aCds,i);
         FieldList.put(FAJson);
      end;
   Result:=TFieldAttrListJson.Create;
   Result.Put('FieldAttrCount',aCds.Fields.Count);
   Result.Put('FieldAttrList',FieldList);
end;

class function TFieldAttrListJson.CreateIt(aDs: TAdoDataset): TFieldAttrListJson;
var
   i: integer;
   FAJson: TFieldAttrJson;
   FieldList: TQBJsonArray;
begin
   if not aDs.Active then
      begin
         Result:=nil;
         exit;
      end;
   FieldList:=TQBJsonArray.create;
   for i:=0 to aDs.Fields.Count-1 do
      begin
         FAJson:=TFieldAttrJson.CreateIt(aDs,i);
         FieldList.put(FAJson);
      end;
   Result:=TFieldAttrListJson.Create;
   Result.Put('FieldAttrCount',aDs.Fields.Count);
   Result.Put('FieldAttrList',FieldList);
end;

function TFieldAttrListJson.ResolveTo: TQBJsonArray;
begin
   Result:=Self.GetJSonArray('FieldAttrList');
end;

function TFieldAttrListJson.ConvertToCds(aCds: TClientDataset): boolean;
var
   FieldCount: integer;
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   FieldSize,i: integer;
   FieldName,FieldType: AnsiString;
begin
   if aCds=nil then
      begin
         Result:=false;
         exit;
      end;
   aCds.Close;
   FieldCount:=Self.GetInt('FieldAttrCount');
   FieldList:=Self.GetJSonArray('FieldAttrList');
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.get(i));
         FieldName:=FAJson.GetString('FieldName');
         FieldType:=FAJson.GetString('FieldType');
         FieldSize:=FAJson.GetInt('FieldSize');
         with aCds.FieldDefs.AddFieldDef do
           begin
              Name:=string(FieldName);
              Size:=FieldSize;
              DataType:=ftVariant;
              if FieldType='Unknown' then
                 DataType:=ftVariant;
              if FieldType='String' then
                 DataType:=ftString;
              if FieldType='SmallInt' then
                 DataType:=ftSmallInt;
              if FieldType='Integer' then
                 DataType:=ftInteger;
              if FieldType='Word' then
                 DataType:=ftWord;
              if FieldType='Boolean' then
                 DataType:=ftBoolean;
              if FieldType='Float' then
                 DataType:=ftFloat;
              if FieldType='Currency' then
                 DataType:=ftCurrency;
              if FieldType='BCD' then
                 DataType:=ftBCD;
              if FieldType='Date' then
                 DataType:=ftDate;
              if FieldType='Time' then
                 DataType:=ftTime;
              if FieldType='DateTime' then
                 DataType:=ftDateTime;
              if FieldType='Bytes' then
                 DataType:=ftBytes;
              if FieldType='VarBytes' then
                 DataType:=ftVarBytes;
              if FieldType='AutoInc' then
                 DataType:=ftAutoInc;
              if FieldType='Blob' then
                 DataType:=ftBlob;
              if FieldType='Memo' then
                 DataType:=ftMemo;
              if FieldType='Graphic' then
                 DataType:=ftGraphic;
              if FieldType='FmtMemo' then
                 DataType:=ftFmtMemo;
              if FieldType='ParadoxOle' then
                 DataType:=ftParadoxOle;
              if FieldType='DBaseOle' then
                 DataType:=ftDBaseOle;
              if FieldType='TypedBinary' then
                 DataType:=ftTypedBinary;
              if FieldType='Cursor' then
                 DataType:=ftCursor;
              if FieldType='FixedChar' then
                 DataType:=ftFixedChar;
              if FieldType='WideString' then
                 DataType:=ftWideString;
              if FieldType='Largeint' then
                 DataType:=ftLargeint;
              if FieldType='ADT' then
                 DataType:=ftADT;
              if FieldType='Array' then
                 DataType:=ftArray;
              if FieldType='Reference' then
                 DataType:=ftReference;
              if FieldType='DataSet' then
                 DataType:=ftDataSet;
              if FieldType='DOraBlob' then
                 DataType:=ftOraBlob;
              if FieldType='OraClob' then
                 DataType:=ftOraClob;
              if FieldType='Variant' then
                 DataType:=ftVariant;
              if FieldType='Interface' then
                 DataType:=ftInterface;
              if FieldType='IDispatch' then
                 DataType:=ftIDispatch;
              if FieldType='Guid' then
                 DataType:=ftGuid;
           end;
      end;
   aCds.CreateDataSet;
   aCds.Open;
   Result:=true;
end;


{TRecordJson}

class function TRecordJson.CreateIt(aFAListJson: TFieldAttrListJson): TRecordJson;
var
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   FieldCount,FieldSize,i,j: integer;
   FieldName,FieldType, JsonType: AnsiString;
begin
   if (aFAListJson=nil) or (aFAListJson.ToString='') then
      begin
         Result:=nil;
         exit;
      end;
   FieldCount:=aFAListJson.GetInt('FieldAttrCount');
   FieldList:=aFAListJson.GetJSonArray('FieldAttrList');
   if (FieldCount<=0) or (FieldList=nil) then
      begin
         Result:=nil;
         exit;
      end;
   Result:=TRecordJson.Create;
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.get(i));
         FAJson.ResolveTo(FieldName,FieldType,JsonType,FieldSize);
         FAJson.Free;
         j:=pos(JsonType,AnsiString('Integer,Double,Boolean,String'));
         case j of
            1:  Result.Put(FieldName,0);
            9:  Result.Put(FieldName,0.0);
            16: Result.Put(FieldName,false);
            24: Result.Put(FieldName,'');
         end;
      end;
end;

class function TRecordJson.CreateIt(aCds: TClientDataset): TRecordJson;
var
   FAListJson: TFieldAttrListJson;
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   i,FieldCount,FieldSize,j: integer;
   FieldName,FieldType,JsonType: AnsiString;
begin
   if (not aCds.Active) or (aCds.RecordCount<=0) then
      begin
         Result:=nil;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aCds);
   FieldCount:=FAListJson.GetInt('FieldAttrCount');
   FieldList:=TQBJsonArray(FAListJson.Get('FieldAttrList'));
   if (FieldCount<=0) or (FieldList=nil) then
      begin
         FAListJson.Free;
         Result:=nil;
         exit;
      end;
   Result:=TRecordJson.Create;
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.get(i));
         FAJson.ResolveTo(FieldName,FieldType,JsonType,FieldSize);
         j:=pos(JsonType,AnsiString('Integer,Double,Boolean,String'));
         case j of
            1:  Result.Put(FieldName,aCds.fieldbyname(string(FieldName)).asinteger);
            9:  Result.Put(FieldName,aCds.fieldbyname(string(FieldName)).asfloat);
            16: Result.Put(FieldName,aCds.fieldbyname(string(FieldName)).AsBoolean);
            24: Result.Put(FieldName,AnsiString(trim(aCds.fieldbyname(string(FieldName)).asstring)));
         end;
      end;
   FAListJson.Free;
end;

class function TRecordJson.CreateIt(aDs: TAdoDataset): TRecordJson;
var
   FAListJson: TFieldAttrListJson;
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   i,FieldCount,FieldSize,j: integer;
   FieldName,FieldType,JsonType: AnsiString;
begin
   if (not aDs.Active) or (aDs.RecordCount<=0) then
      begin
         Result:=nil;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aDs);
   FieldList:=TQBJsonArray(FAListJson.Get('FieldAttrList'));
   FieldCount:=FAListJson.GetInt('FieldAttrCount');
   if (FieldCount<=0) or (FieldList=nil) then
      begin
         FAListJson.Free;
         Result:=nil;
         exit;
      end;
   Result:=TRecordJson.Create;
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.get(i));
         FAJson.ResolveTo(FieldName,FieldType,JsonType,FieldSize);
         j:=pos(JsonType,AnsiString('Integer,Double,Boolean,String'));
         case j of
            1:  Result.Put(FieldName,aDs.fieldbyname(string(FieldName)).asinteger);
            9:  Result.Put(FieldName,aDs.fieldbyname(string(FieldName)).asfloat);
            16: Result.Put(FieldName,aDs.fieldbyname(string(FieldName)).AsBoolean);
            24: Result.Put(FieldName,AnsiString(trim(aDs.fieldbyname(string(FieldName)).asstring)));
         end;
      end;
   FAListJson.Free;
end;

function TRecordJson.ResolveTo(aDs: TAdoDataset): boolean;
var
   FAListJson: TFieldAttrListJson;
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   FieldCount,i,FieldSize,j: integer;
   FieldName,FieldType,JsonType: AnsiString;
begin
   if (not aDs.Active) or (self.PropCount<=0) then
      begin
         Result:=false;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aDs);
   FieldList:=FAListJson.GetJSonArray('FieldAttrList');
   FieldCount:=FAListJson.GetInt('FieldAttrCount');
   aDs.Append;
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.getJSon(i));
         FAJson.ResolveTo(FieldName,FieldType,JsonType,FieldSize);
         j:=pos(JsonType,AnsiString('Integer,Double,Boolean,String'));
         case j of
            1:  aDs.FieldValues[string(FieldName)]:=Self.GetInt(FieldName);
            9:  aDs.FieldValues[string(FieldName)]:=Self.GetDouble(FieldName);
            16: aDs.FieldValues[string(FieldName)]:=Self.GetBoolean(FieldName);
            24: begin
                   if FieldType='DateTime' then
                      aDs.FieldValues[string(FieldName)]:=StrToDateTime(string(Self.GetString(FieldName)))
                   else
                      begin
                         if FieldType='Date' then
                            aDs.FieldValues[string(FieldName)]:=StrToDate(string(Self.GetString(FieldName)))
                         else
                            begin
                               if FieldType='Date' then
                                  aDs.FieldValues[string(FieldName)]:=StrToTime(string(Self.GetString(FieldName)))
                               else
                                  aDs.FieldValues[string(FieldName)]:=Self.GetString(FieldName);
                            end;
                      end;
                end;
         end;
      end;
   aDs.Post;
   FAListJson.free;
   Result:=true;
end;

function TRecordJson.ResolveTo(aCds: TClientDataset): boolean;
var
   FAListJson: TFieldAttrListJson;
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   FieldCount,i,FieldSize,j: integer;
   FieldName,FieldType,JsonType: AnsiString;
begin
   if (not aCds.Active) or (self.PropCount<=0) then
      begin
         Result:=false;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aCds);
   FieldList:=TQBJsonArray(FAListJson.Get('FieldAttrList'));
   FieldCount:=FAListJson.GetInt('FieldAttrCount');
   aCds.Append;
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.get(i));
         FAJson.ResolveTo(FieldName,FieldType,JsonType,FieldSize);
         j:=pos(JsonType,AnsiString('Integer,Double,Boolean,String'));
         case j of
            1:  aCds.FieldValues[string(FieldName)]:=Self.GetInt(FieldName);
            9:  aCds.FieldValues[string(FieldName)]:=Self.GetDouble(FieldName);
            16: aCds.FieldValues[string(FieldName)]:=Self.GetBoolean(FieldName);
            24: begin
                   if FieldType='DateTime' then
                      aCds.FieldValues[string(FieldName)]:=StrToDateTime(string(Self.GetString(FieldName)))
                   else
                      begin
                         if FieldType='Date' then
                            aCds.FieldValues[string(FieldName)]:=StrToDate(string(Self.GetString(FieldName)))
                         else
                            begin
                               if FieldType='Date' then
                                  aCds.FieldValues[string(FieldName)]:=StrToTime(string(Self.GetString(FieldName)))
                               else
                                  aCds.FieldValues[string(FieldName)]:=Self.GetString(FieldName);
                            end;
                      end;
                end;
         end;
      end;
   aCds.Post;
   FAListJson.free;
   result:=true;
end;


{TRecordListJson}

class function TRecordListJson.CreateIt(aCds: TClientDataset): TRecordListJson;
var
   FAListJson: TFieldAttrListJson;
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   i,FieldCount,FieldSize,j: integer;
   FieldName,FieldType,JsonType: AnsiString;
   RecordJson: TRecordJson;
   RecordArray: TQBJsonArray;
   fFields: array of TStructureRecord;
begin
   if (not aCds.Active) or (aCds.RecordCount<=0) then
      begin
         Result:=nil;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aCds);
   FieldList:=TQBJsonArray(FAListJson.Get('FieldAttrList'));
   FieldCount:=FAListJson.GetInt('FieldAttrCount');
   if (FieldCount<=0) or (FieldList=nil) then
      begin
         FAListJson.free;
         Result:=nil;
         exit;
      end;
   SetLength(fFields,FieldCount);
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.get(i));
         FAJson.ResolveTo(FieldName,FieldType,JsonType,FieldSize);
         fFields[i].FieldName:=FieldName;
         fFields[i].FieldType:=FieldType;
         fFields[i].JsonType:=JsonType;
         fFields[i].FieldSize:=FieldSize;
      end;
   Result:=TRecordListJson.Create;
   Result.Put('RecordCount',aCds.RecordCount);
   RecordArray:=TQBJsonArray.create;
   aCds.First;
   while not aCds.Eof do
      begin
         RecordJson:=TRecordJson.Create;
         for i:=0 to FieldCount-1 do
            begin
               j:=pos(fFields[i].JsonType,AnsiString('Integer,Double,Boolean,String'));
               case j of
                  1:  RecordJson.Put(fFields[i].FieldName,aCds.fieldbyname(string(fFields[i].FieldName)).asinteger);
                  9:  RecordJson.Put(fFields[i].FieldName,aCds.fieldbyname(string(fFields[i].FieldName)).asfloat);
                  16: RecordJson.Put(fFields[i].FieldName,aCds.fieldbyname(string(fFields[i].FieldName)).AsBoolean);
                  24: RecordJson.Put(fFields[i].FieldName,AnsiString(trim(aCds.fieldbyname(string(fFields[i].FieldName)).asstring)));
               end;
            end;
         RecordArray.put(RecordJson);
         aCds.Next;
      end;
   Result.Put('RecordList',RecordArray);
   FAListJson.free;
end;

class function TRecordListJson.CreateIt(aDs: TAdoDataset): TRecordListJson;
var
   FAListJson: TFieldAttrListJson;
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   i,FieldCount,FieldSize,j: integer;
   FieldName,FieldType,JsonType: AnsiString;
   RecordJson: TRecordJson;
   RecordArray: TQBJsonArray;
   fFields: array of TStructureRecord;
begin
   if (not aDs.Active) or (aDs.RecordCount<=0) then
      begin
         Result:=nil;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aDs);
   FieldList:=TQBJsonArray(FAListJson.Get('FieldAttrList'));
   FieldCount:=FAListJson.GetInt('FieldAttrCount');
   if (FieldCount<=0) or (FieldList=nil) then
      begin
         FAListJson.free;
         Result:=nil;
         exit;
      end;
   SetLength(fFields,FieldCount);
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.get(i));
         FAJson.ResolveTo(FieldName,FieldType,JsonType,FieldSize);
         fFields[i].FieldName:=FieldName;
         fFields[i].FieldType:=FieldType;
         fFields[i].JsonType:=JsonType;
         fFields[i].FieldSize:=FieldSize;
      end;
   Result:=TRecordListJson.Create;
   Result.Put('RecordCount',aDs.RecordCount);
   RecordArray:=TQBJsonArray.create;
   aDs.First;
   while not aDs.Eof do
      begin
         RecordJson:=TRecordJson.Create;
         for i:=0 to FieldCount-1 do
            begin
               j:=pos(fFields[i].JsonType,AnsiString('Integer,Double,Boolean,String'));
               case j of
                  1:  RecordJson.Put(fFields[i].FieldName,aDs.fieldbyname(string(fFields[i].FieldName)).asinteger);
                  9:  RecordJson.Put(fFields[i].FieldName,aDs.fieldbyname(string(fFields[i].FieldName)).asfloat);
                  16: RecordJson.Put(fFields[i].FieldName,aDs.fieldbyname(string(fFields[i].FieldName)).AsBoolean);
                  24: RecordJson.Put(fFields[i].FieldName,AnsiString(trim(aDs.fieldbyname(string(fFields[i].FieldName)).asstring)));
               end;
            end;
         RecordArray.put(RecordJson);
         aDs.Next;
      end;
   Result.Put('RecordList',RecordArray);
   FAListJson.free;
end;

procedure TRecordListJson.PutRecordJson(aRecordJson: TRecordJson);
var
   RecordList: TQBJsonArray;
begin
   RecordList:=TQBJsonArray(self.Get('RecordList'));
   RecordList.put(aRecordJson);
   Self.Remove('RecordList');
   Self.Put('RecordList',RecordList);
end;

function TRecordListJson.GetRecordJson(index: integer): TRecordJson;
var
   RecordCount: integer;
   RecordList: TQBJsonArray;
begin
   RecordCount:=Self.GetInt('RecordCount');
   RecordList:=TQBJsonArray(self.Get('RecordList'));
   if index>=RecordCount then
      begin
         result:=nil;
         exit;
      end;
   Result:=TRecordJson(RecordList.getJSon(index));
end;

function TRecordListJson.ResolveTo(aDs: TAdoDataset): boolean;
var
   FAListJson: TFieldAttrListJson;
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   FieldCount,i,FieldSize,j,k: integer;
   FieldName,FieldType,JsonType: AnsiString;
   fFields: array of TStructureRecord;
   RecordCount: integer;
   RecordList: TQBJsonArray;
   RecordJson: TRecordJson;
begin
   RecordCount:=Self.GetInt('RecordCount');
   if RecordCount<=0 then
      begin
         result:=false;
         exit;
      end;
   RecordList:=TQBJsonArray(self.Get('RecordList'));
   if (not aDs.Active) or (self.PropCount<=0) then
      begin
         Result:=false;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aDs);
   FieldList:=TQBJsonArray(FAListJson.Get('FieldAttrList'));
   FieldCount:=FAListJson.GetInt('FieldAttrCount');
   SetLength(fFields,FieldCount);
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.get(i));
         FAJson.ResolveTo(FieldName,FieldType,JsonType,FieldSize);
         fFields[i].FieldName:=FieldName;
         fFields[i].FieldType:=FieldType;
         fFields[i].JsonType:=JsonType;
         fFields[i].FieldSize:=FieldSize;
      end;
   for k:=0 to RecordCount-1 do
      begin
         RecordJson:=TRecordJson(RecordList.get(k));
         aDs.Append;
         for i:=0 to FieldCount-1 do
            begin
               j:=pos(fFields[i].JsonType,AnsiString('Integer,Double,Boolean,String'));
               case j of
                  1:  aDs.FieldValues[string(fFields[i].FieldName)]:=RecordJson.GetInt(fFields[i].FieldName);
                  9:  aDs.FieldValues[string(fFields[i].FieldName)]:=RecordJson.GetDouble(fFields[i].FieldName);
                  16: aDs.FieldValues[string(fFields[i].FieldName)]:=RecordJson.GetBoolean(fFields[i].FieldName);
                  24: begin
                         if FieldType='DateTime' then
                            aDs.FieldValues[string(fFields[i].FieldName)]:=StrToDateTime(string(RecordJson.GetString(fFields[i].FieldName)))
                         else
                            begin
                               if FieldType='Date' then
                                  aDs.FieldValues[string(fFields[i].FieldName)]:=StrToDate(string(RecordJson.GetString(fFields[i].FieldName)))
                               else
                                  begin
                                     if FieldType='Date' then
                                        aDs.FieldValues[string(fFields[i].FieldName)]:=StrToTime(string(RecordJson.GetString(fFields[i].FieldName)))
                                     else
                                        aDs.FieldValues[string(fFields[i].FieldName)]:=RecordJson.GetString(fFields[i].FieldName);
                                  end;
                            end;
                      end;
               end;
            end;
         aDs.Post;
         RecordJson.Free;
      end;
   FAListJson.free;
   Result:=true;
end;

function TRecordListJson.ResolveTo(aCds: TClientDataset): boolean;
var
   FAListJson: TFieldAttrListJson;
   FieldList: TQBJsonArray;
   FAJson: TFieldAttrJson;
   FieldCount,i,FieldSize,j,k: integer;
   FieldName,FieldType,JsonType: AnsiString;
   fFields: array of TStructureRecord;
   RecordCount: integer;
   RecordList: TQBJsonArray;
   RecordJson: TRecordJson;
begin
   RecordCount:=Self.GetInt('RecordCount');
   if RecordCount<=0 then
      begin
         result:=false;
         exit;
      end;
   RecordList:=TQBJsonArray(self.Get('RecordList'));
   if (not aCds.Active) or (self.PropCount<=0) then
      begin
         Result:=false;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aCds);
   FieldList:=TQBJsonArray(FAListJson.Get('FieldAttrList'));
   FieldCount:=FAListJson.GetInt('FieldAttrCount');
   SetLength(fFields,FieldCount);
   for i:=0 to FieldCount-1 do
      begin
         FAJson:=TFieldAttrJson(FieldList.get(i));
         FAJson.ResolveTo(FieldName,FieldType,JsonType,FieldSize);
         fFields[i].FieldName:=FieldName;
         fFields[i].FieldType:=FieldType;
         fFields[i].JsonType:=JsonType;
         fFields[i].FieldSize:=FieldSize;
      end;
   for k:=0 to RecordCount-1 do
      begin
         RecordJson:=TRecordJson(RecordList.get(k));
         aCds.Append;
         for i:=0 to FieldCount-1 do
            begin
               j:=pos(fFields[i].JsonType,AnsiString('Integer,Double,Boolean,String'));
               case j of
                  1:  aCds.FieldValues[string(fFields[i].FieldName)]:=RecordJson.GetInt(fFields[i].FieldName);
                  9:  aCds.FieldValues[string(fFields[i].FieldName)]:=RecordJson.GetDouble(fFields[i].FieldName);
                  16: aCds.FieldValues[string(fFields[i].FieldName)]:=RecordJson.GetBoolean(fFields[i].FieldName);
                  24: begin
                         if FieldType='DateTime' then
                            aCds.FieldValues[string(fFields[i].FieldName)]:=StrToDateTime(string(RecordJson.GetString(fFields[i].FieldName)))
                         else
                            begin
                               if FieldType='Date' then
                                  aCds.FieldValues[string(fFields[i].FieldName)]:=StrToDate(string(RecordJson.GetString(fFields[i].FieldName)))
                               else
                                  begin
                                     if FieldType='Date' then
                                        aCds.FieldValues[string(fFields[i].FieldName)]:=StrToTime(string(RecordJson.GetString(fFields[i].FieldName)))
                                     else
                                        aCds.FieldValues[string(fFields[i].FieldName)]:=RecordJson.GetString(fFields[i].FieldName);
                                  end;
                            end;
                      end;
               end;
            end;
         aCds.Post;
      end;
   FAListJson.free;
   Result:=true;
end;

class function TTableJson.CreateIt(TableName: AnsiString; aCds: TClientDataset): TTableJson;
var
   FAListJson: TFieldAttrListJson;
   RecordListJson: TRecordListJson;
begin
   if not aCds.Active then
      begin
         Result:=nil;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aCds);
   RecordListJson:=TRecordListJson.CreateIt(aCds);
   Result:=TTableJson.Create;
   Result.Put('TableName',TableName);
   Result.Put('TableHead',FAListJson);
   Result.Put('TableBody',RecordListJson);
end;

class function TTableJson.CreateIt(TableName: AnsiString; aDs: TAdoDataset): TTableJson;
var
   FAListJson: TFieldAttrListJson;
   RecordListJson: TRecordListJson;
begin
   if not aDs.Active then
      begin
         Result:=nil;
         exit;
      end;
   FAListJson:=TFieldAttrListJson.CreateIt(aDs);
   RecordListJson:=TRecordListJson.CreateIt(aDs);
   Result:=TTableJson.Create;
   Result.Put('TableName',TableName);
   Result.Put('TableHead',FAListJson);
   Result.Put('TableBody',RecordListJson);
end;

function TTableJson.ResolveTo(aDs: TAdoDataset): boolean;
var
   RecordListJson: TRecordListJson;
begin
   if not aDs.Active then
      begin
         Result:=false;
         exit;
      end;
   RecordListJson:=TRecordListJson(Self.Get('TableBody'));
   Result:=RecordListJson.ResolveTo(aDs);
end;

function TTableJson.ResolveTo(aCds: TClientDataset): boolean;
var
   FAListJson: TFieldAttrListJson;
   RecordListJson: TRecordListJson;
begin
   if aCds=nil then
      begin
         Result:=false;
         exit;
      end;
   FAListJson:=TFieldAttrListJson(Self.Get('TableHead'));
   FAListJson.ConvertToCds(aCds);
   RecordListJson:=TRecordListJson(Self.Get('TableBody'));
   Result:=RecordListJson.ResolveTo(aCds);
end;

end.
