//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit QBDate; 

{$HINTS OFF}
{$WARNINGS OFF}

interface 

uses
   Windows, SysUtils, QBMisc,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
   DateUtils;

const 
   DayOfWeekStrings: array [1..7] of AnsiString = ('SUNDAY', 'MONDAY',
                     'TUESDAY', 'WEDNESDAY', 'THURSDAY', 'FRIDAY', 'SATURDAY'); 

   MonthStrings: array [1..12] of AnsiString = ('JANUARY', 'FEBRUARY',
                 'MARCH', 'APRIL','MAY', 'JUNE', 'JULY', 'AUGUST', 'SEPTEMBER', 
                 'OCTOBER', 'NOVEMBER', 'DECEMBER'); 

   DayOfCWeekStrings: array [1..7] of AnsiString = ('','һ',
                      'ڶ','','','',''); 

   MonthCStrings: array [1..12] of AnsiString = ('һ', '', '','','',
                  '', '', '', '', 'ʮ','ʮһ', 'ʮ'); 
                                             
   OneDay         = 1.0; 
   OneHour        = OneDay / 24.0;
   OneMinute      = OneHour / 60.0;
   OneSecond      = OneMinute / 60.0; 
   OneMillisecond = OneSecond / 1000.0; 

   function IsLeapYear (Year: Word): Boolean;
   function GetFirstDayOfYear (const Year: Word): TDateTime;
   function GetLastDayOfYear (const Year: Word): TDateTime;
   function MinutesApart (const DT1, DT2: TDateTime): Word;
   function DaysApart (const DT1, DT2: TDateTime): LongInt;
   function WeeksApart (const DT1, DT2: TDateTime): LongInt;
   function AdjustDateYear (const D: TDateTime; const Year: Word): TDateTime;
   function AddMins (const DT: TDateTime; const Mins: Extended): TDateTime;
   function AddHrs (const DT: TDateTime; const Hrs: Extended): TDateTime;
   function AddWeeks (const DT: TDateTime; const Weeks: Extended): TDateTime;
   function AddMonths (const DT: TDateTime; const Months: Extended): TDateTime;
   function AddYrs (const DT: TDateTime; const Yrs: Extended): TDateTime;
   function StartOfWeek (const DT: TDateTime): TDateTime;
   function EndOfWeek (const DT: TDateTime): TDateTime;
   function DatesInSameMonth (const DT1, DT2: TDateTime): Boolean;
   function DatesInSameYear (const DT1, DT2: TDateTime): Boolean;
   function DatesInSameMonthYear (const DT1, DT2: TDateTime): Boolean;
   function DaysLeftInMonth (const DT: TDateTime): Byte;
   function DaysInMonth (const DT: TDateTime): Byte;
   function DaysInYear (const DT: TDateTime): Word;
   function DayOfYear (const DT: TDateTime): Word;
   function DaysLeftInYear (const DT: TDateTime): Word;
   function IsAM (const DT: TDateTime): Boolean;
   function IsPM (const DT: TDateTime): Boolean;
   function IsNoon (const DT: TDateTime): Boolean;
   function IsMidnight (const DT: TDateTime): Boolean;
   function IsWeekend (const DT: TDateTime): Boolean;
   function GetFirstDayofMonth (const DT: TDateTime): TDateTime;
   function GetLastDayofMonth (const DT: TDateTime): TDateTime;
   function DateTimeToGMTString(DateTime: TDateTime): string;
   function GMTToDateTime(GMTString: string): TDateTime;
   function GetFileCreateDate(FHandle: Integer): string;
   function GetFileModifiedDate(filename: string): string;

implementation 

function IsLeapYear(Year: Word): Boolean;
begin 
   Result := ((Year and 3) = 0) and ((Year mod 100 > 0) or (Year mod 400 = 0));
end; 

function MinutesApart (const DT1, DT2: TDateTime): Word;
var
   Hr1, Min1, Sec1, MSec1: Word;
   Hr2, Min2, Sec2, MSec2: Word;
begin
   DecodeTime (DT1, Hr1, Min1, Sec1, MSec1);
   DecodeTime (DT2, Hr2, Min2, Sec2, MSec2);
   if Min2 < Min1 then
      begin
         Min2 := Min2 + 60;
         Dec (Hr2);
      end;
   if Hr1 > Hr2 then
      Hr2 := Hr2 + 24;
   Result := (Hr2 - Hr1) * 60 + (Min2 - Min1);
end; 

function WeeksApart (const DT1, DT2: TDateTime): Integer; 
begin 
   Result := DaysApart (DT1, DT2) div 7;;
end;

function DaysApart (const DT1, DT2: TDateTime): LongInt;
begin
   if dt2>dt1 then
      Result := Trunc (DT2) - Trunc (DT1)
   else
      Result := Trunc (DT1) - Trunc (DT2)
end;

function AdjustDateYear (const D: TDateTime; const Year: Word): TDateTime; 
var 
   Day, Month, OldYear: Word;
begin
   DecodeDate (D, OldYear, Month, Day);
   if Year = OldYear then
      begin
         Result := Int (D);
         Exit;
      end;
   if not IsLeapYear (Year) and (Month = 2) and (Day = 29) then
      begin
         Month := 3;
         Day := 1;
      end;
   Result := EncodeDate (Year, Month, Day);
end; 

function AddMins (const DT: TDateTime; const Mins: Extended): TDateTime;
begin 
   Result := DT + Mins / (60 * 24)
end; 

function AddHrs (const DT: TDateTime; const Hrs: Extended): TDateTime; 
begin 
   Result := DT + Hrs / 24.0
end; 

function AddWeeks (const DT: TDateTime; const Weeks: Extended): TDateTime; 
begin 
   Result := DT + Weeks * 7;
end; 

function AddMonths (const DT: TDateTime; const Months: Extended): TDateTime; 
var 
   Day, Month, Year: Word;
   IMonth: Integer;
begin 
   DecodeDate (DT, Year, Month, Day);
   IMonth := Month + Trunc (Months);

   if IMonth > 12 then
      begin
         Year := Year + (IMonth - 1) div 12;
         IMonth := IMonth mod 12;
         if IMonth = 0 then
            IMonth := 12;
      end
   else
      if IMonth < 1 then
         begin
            Year := Year + (IMonth div 12) - 1;
            IMonth := 12 - abs (IMonth) mod 12;
         end;
   Month := IMonth;

   if Month = 2 then
      begin
         if IsLeapYear (Year) and (Day > 29) then
            Day := 29
         else
            begin
               if not IsLeapYear (Year) and (Day > 28) then
                  Day := 28;
            end;
      end
   else
      begin
         if (Month in [9, 4, 6, 11]) and (Day = 31) then
            Day := 30;
      end;

   Result := EncodeDate (Year, Month, Day) + Frac(Months) * 30 + Frac(DT);
end; 

function AddYrs (const DT: TDateTime; const Yrs: Extended): TDateTime; 
var 
   Day, Month, Year: Word;
begin 
   DecodeDate (DT, Year, Month, Day);
   Year := Year + Trunc (Yrs);
   if not IsLeapYear (Year) and (Month = 2) and (Day = 29) then
      Day := 28;
   Result := EncodeDate (Year, Month, Day) + Frac (Yrs) * 365.25+ Frac (DT);
end; 

function GetLastDayofMonth (const DT: TDateTime): TDateTime; 
var 
   D, M, Y: Word;
begin 
   DecodeDate (DT, Y, M, D);
   case M of
      2: begin
            if IsLeapYear (Y) then
               D := 29
            else
               D := 28;
         end;
      4, 6, 9, 11: D := 30
      else
         D := 31;
   end;
   Result := EncodeDate (Y, M, D) + Frac (DT); 
end; 

function GetFirstDayofMonth (const DT: TDateTime): TDateTime;
var
   D, M, Y: Word;
begin 
   DecodeDate (DT, Y, M, D);
   Result := EncodeDate (Y, M, 1) + Frac (DT);
end; 

function StartOfWeek (const DT: TDateTime): TDateTime;
begin
   Result := DT - DayOfWeek (DT) + 1;
end; 

function EndOfWeek (const DT: TDateTime): TDateTime; 
begin 
   Result := DT - DayOfWeek (DT) + 7;
end; 

function DatesInSameMonth (const DT1, DT2: TDateTime): Boolean;
begin
   Result := (Monthof(DT1) = Monthof(DT2));
end;

function DatesInSameYear (const DT1, DT2: TDateTime): Boolean; 
begin 
   Result := (Yearof(DT1) = Yearof(DT2));
end; 

function DatesInSameMonthYear (const DT1, DT2: TDateTime): Boolean; 
begin 
   Result := (DatesInSameMonth (DT1, DT2) and DatesInSameYear (DT1, DT2)); 
end; 

function IsAM (const DT: TDateTime): Boolean; 
begin 
   Result := Frac (DT) < 0.5
end; 

function IsPM (const DT: TDateTime): Boolean; 
begin 
   Result := not IsAM (DT);
end; 

function IsNoon (const DT: TDateTime): Boolean; 
begin 
   Result := Frac (DT) = 0.5;
end; 

function IsMidnight (const DT: TDateTime): Boolean; 
begin 
   Result := Frac (DT) = 0.0;
end; 

function IsWeekend (const DT: TDateTime): Boolean;
begin
   Result := DayOfWeek (DT) in [1, 7];
end;

function DaysInMonth (const DT: TDateTime): Byte;
begin 
   case monthof(DT) of
      2: if IsLeapYear(yearof(DT)) then
            Result := 29
         else
            Result := 28;
      4, 6, 9, 11: Result := 30;
   else
      Result := 31;
   end; 
end; 

function DaysLeftInMonth (const DT: TDateTime): Byte; 
begin
   Result := DaysInMonth (DT) - DayOf(DT);
end; 

function DaysInYear (const DT: TDateTime): Word; 
begin 
   if IsLeapYear(Yearof(DT)) then
      Result := 366
   else
      Result := 365;
end; 

function DayOfYear (const DT: TDateTime): Word; 
begin 
   Result := Trunc(DT) - Trunc(EncodeDate(Yearof(DT), 1, 1)) + 1;
end; 

function DaysLeftInYear (const DT: TDateTime): Word; 
begin 
   Result := DaysInYear(DT) - DayOfYear (DT); 
end; 

function GetFirstDayOfYear (const Year: Word): TDateTime;
begin 
   Result := EncodeDate (Year, 1, 1);
end; 

function GetLastDayOfYear (const Year: Word): TDateTime; 
begin 
   Result := EncodeDate (Year, 12, 31);
end;

function DateTimeToGMTString(DateTime: TDateTime): string;
var
   d: string;
   SDateTime: TSystemTime;
begin
   DateTimeToSystemTime(datetime,SDateTime);                   
   result:=copy('SunMonTueWedThuFriSat',sdatetime.wDayOfWeek*3+1,3)+', ';                
   d:=inttostr(sdatetime.wDay);
   if length(d)<2 then
      d:='0'+d;
   result:=result+d+' '+copy('JanFebMarAprMayJunJulAugSepOctNovDec',(sdatetime.wmonth-1)*3+1,3)+' '+inttostr(Sdatetime.wyear)+' ';    
   d:=inttostr(sdatetime.whour);
   if length(d)<2 then
      d:='0'+d;                         
   result:=result+d+':';
   d:=inttostr(sdatetime.wMinute);
   if length(d)<2 then
      d:='0'+d;                         
   result:=result+d+':';
   d:=inttostr(sdatetime.wSecond);
   if length(d)<2 then
      d:='0'+d;                         
   result:=result+d+' GMT';
end;

function GMTToDateTime(GMTString: string): TDateTime;
var
   SDateTime: TSystemTime;
   tmpstr,tmp: string;
   j: integer;
begin
   datetimetosystemtime(now,sdatetime);
   tmpstr:=gmtstring;
   j:=pos(AnsiString(','),tmpstr);
   tmp:=trim(copy(tmpstr,1,j-1));
   delete(tmpstr,1,j);
   tmpstr:=trim(tmpstr);
   j:=pos(tmp,AnsiString('SunMonTueWedThuFriSat'));
   if j<=0 then
      j:=-1
   else
      j:=(j-1) div 3;
   if j<>-1 then
      sdatetime.wDayOfWeek:=j;
   j:=pos(AnsiString(' '),tmpstr);
   tmp:=copy(tmpstr,1,j-1);
   delete(tmpstr,1,j);
   if trim(tmp)<>'' then
      try
         j:=strtoint(string(tmp));
         sdatetime.wDay:=j;
      except
      end;
   j:=pos(AnsiString(' '),tmpstr);
   tmp:=trim(copy(tmpstr,1,j-1));
   delete(tmpstr,1,j);
   j:=pos(tmp,AnsiString('JanFebMarAprMayJunJulAugSepOctNovDec'));
   if j<=0 then
      j:=-1
   else
      j:=((j-1) div 3)+1;
   if j<>-1 then
      sdatetime.wMonth:=j;
   j:=pos(AnsiString(' '),tmpstr);
   tmp:=trim(copy(tmpstr,1,j-1));
   delete(tmpstr,1,j);
   if tmp<>'' then
      try
         j:=strtoint(string(tmp));
         sdatetime.wYear:=j;
      except
      end;
   j:=pos(AnsiString(':'),tmpstr);
   tmp:=trim(copy(tmpstr,1,j-1));
   delete(tmpstr,1,j);
   if tmp<>'' then
      try
         j:=strtoint(string(tmp));
         sdatetime.wHour:=j;
      except
      end;
   j:=pos(AnsiString(':'),tmpstr);
   tmp:=trim(copy(tmpstr,1,j-1));
   delete(tmpstr,1,j);
   if tmp<>'' then
      try
         j:=strtoint(String(tmp));
         sdatetime.wMinute:=j;
      except
      end;
   j:=pos(AnsiString(' '),tmpstr);
   tmp:=trim(copy(tmpstr,1,j-1));
   delete(tmpstr,1,j);
   if tmp<>'' then
      try
         j:=strtoint(string(tmp));
         sdatetime.wSecond:=j;
      except
      end;
   result:=SystemTimeToDateTime(sdatetime);
end;

function GetFileCreateDate(FHandle: Integer): string;
var
   j: integer;
   FileDateTime: TDateTime;
begin
   j:=filegetdate(fhandle);
   if j=-1 then
      filedatetime:=now
   else
      filedatetime:=FileDateToDateTime(j);                     
   filedatetime:=filedatetime+8/24;                            
   result:=DateTimeToGMTString(fileDateTime);
end;

function GetFileModifiedDate(filename: string): string;
var
   j: integer;
   FileDateTime: TDateTime;
begin
   j:=fileage(filename);
   if j=-1 then
      filedatetime:=now
   else
      filedatetime:=FileDateToDateTime(j);         
   filedatetime:=filedatetime+8/24;                            
   result:=DateTimeToGMTString(fileDateTime);
end;

end.
