//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//

unit QBJSon;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
  Windows, SysUtils, Classes,  TypInfo,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  QBParcel, QBCommon;

Type
  TQBJSonAbstractObject = class
    function equals(const Value: TQBJSonAbstractObject): Boolean; virtual;
    function hash: LongInt;
    function Clone: TQBJSonAbstractObject; virtual;
    function toString: Ansistring; virtual;
    function instanceOf(const Value: TQBJSonAbstractObject): Boolean;
  end;
    
  ClassCastException = class (Exception);
  NoSuchElementException = class (Exception);
  NumberFormatException = class (Exception);
  NullPointerException = class (Exception);
  NotImplmentedFeature = class (Exception);
  TQBJSonArray = class;
  _Number =  class;
  _String = class;
  _Double = class;
  _NULL = class;

  ParseException = class (Exception)
     constructor create (_message : Ansistring ; index : integer);
  end;

  TQBJSonTokener = class  (TQBJSonAbstractObject)
  public
    constructor create (s: Ansistring) ;
    procedure back();
    class function dehexchar(c : Ansichar) :integer;
    function more :boolean;
    function next() : Ansichar; overload ;
    function next (c: Ansichar ) : Ansichar; overload ;
    function next (n:integer) : Ansistring; overload ;
    function nextClean () : Ansichar;
    function nextString (quote : Ansichar) : Ansistring;
    function nextTo (d : Ansichar) : Ansistring;  overload ;
    function nextTo (delimiters : Ansistring) : Ansichar;   overload ;
    function nextValue () : TQBJSonAbstractObject ;
    procedure skipPast (_to : Ansistring ) ;
    function skipTo (_to : Ansichar ): Ansichar;
    function syntaxError (_message : Ansistring) : ParseException;
    function toString : Ansistring;  override;
    function unescape (s : Ansistring): Ansistring;
  private
    myIndex : integer;
    mySource : Ansistring;
  end;

  TQBJSon = class (TQBJSonAbstractObject)
  private
    myHashMap : TStringList;
    function GetPropValues(const Key: AnsiString): AnsiString;
    procedure SetPropValues(const Key: AnsiString; const Value: AnsiString);
    procedure SetAsString(const Value: AnsiString);
    function GetKeyByIndex(index: Integer): AnsiString;
    procedure SetCascadeValueEx(const Value: AnsiString; const Keys: array of AnsiString;
      StartIdx: Integer);
    function GetValByIndex(index: Integer): AnsiString;
    procedure UpdateByTokener(x: TQBJSonTokener);
  public
    constructor Create;  overload;
    constructor Create  (jo : TQBJSon; sa : array of Ansistring); overload;
    constructor Create (x : TQBJSonTokener); overload;
    constructor Create (map : TStringList); overload;
    constructor Create (s : Ansistring); overload;
    constructor Create (aStream: TStream; Bytes: integer); overLoad;
    constructor Create (aFilename: string; FailIfNoFile: boolean); overload;
    constructor create (aQBParcel: TQBParcel; GoodsName: Ansistring); overload;

    procedure Clean;
    function Clone : TQBJSonAbstractObject; override;
    function Accumulate (key : Ansistring; value : TQBJSonAbstractObject): TQBJSon;
    function Get (key : Ansistring) : TQBJSonAbstractObject;
    function GetBoolean (key : Ansistring): boolean;
    function GetDouble (key : Ansistring): double;
    function GetInt (key : Ansistring): integer;
    function GetJSonArray (key : Ansistring) : TQBJSonArray;
    function GetJSon (key : Ansistring) : TQBJSon;
    function GetString (key : Ansistring): Ansistring;
    function Has (key : Ansistring) : boolean;
    function IsNull (key : Ansistring) : boolean;
    function Keys : TStringList ;
    function Length : integer;
    function Names : TQBJSonArray;
    class function NumberToString (n: _Number): Ansistring;
    class function ValueToString(value : TQBJSonAbstractObject) : Ansistring; overload;
    class function ValueToString(value : TQBJSonAbstractObject; indentFactor, indent : integer) : Ansistring; overload;

    function Opt (key : Ansistring) : TQBJSonAbstractObject;
    function OptBoolean (key : Ansistring): boolean; overload;
    function OptBoolean (key : Ansistring; defaultValue : boolean): boolean; overload;
    function OptDouble (key : Ansistring): double; overload;
    function OptDouble (key : Ansistring; defaultValue : double): double; overload;
    function OptInt (key : Ansistring): integer; overload;
    function OptInt (key : Ansistring; defaultValue : integer): integer; overload;
    function OptString (key : Ansistring): Ansistring; overload;
    function OptString (key : Ansistring; defaultValue : Ansistring): Ansistring; overload;

    function OptJSonArray (key : Ansistring): TQBJSonArray; overload;
    function OptJSon (key : Ansistring): TQBJSon; overload;

    procedure Put(key: Ansistring; value: boolean); overload;
    procedure Put(key: Ansistring; value: double); overload;
    procedure Put(key: Ansistring; value: integer); overload;
    procedure Put(key: Ansistring; value: Ansistring); overload;
    procedure Put(key: Ansistring; value: TQBJSonAbstractObject); overload;

    procedure PutOpt (key : Ansistring; value : TQBJSonAbstractObject);
    class function quote (s : Ansistring): Ansistring;
    function Remove (key : Ansistring): TQBJSonAbstractObject;
    procedure AssignTo(JSon: TQBJSon);

    function ToJSonArray (names : TQBJSonArray) : TQBJSonArray;
    function ToString (): Ansistring ; overload; override;
    function ToString2 (indentFactor : integer): Ansistring; overload;
    function ToString3 (indentFactor, indent : integer): Ansistring; overload;

    property PropValues[const Key: AnsiString]: AnsiString read GetPropValues write SetPropValues; default;
    property KeyByIndex[index: Integer]: AnsiString read GetKeyByIndex;
    property ValByIndex[index: Integer]: AnsiString read GetValByIndex;
    property AsString: AnsiString read ToString write SetAsString;
    procedure Assign(Source: TQBJSon);
    function GetCascadeValue(const Keys: array of AnsiString): AnsiString;
    procedure SetCascadeValue(const Value: AnsiString; const Keys: array of AnsiString);
    function GetDiffFrom(Source: TQBJSon): TQBJSon;
    procedure RemoveByKeyHeader(const Header: AnsiString='~');
    procedure CleanKey(const Key: AnsiString);
    function PropCount:Integer;
    function KeyByVal(const Value: AnsiString): AnsiString;

    destructor Destroy;override;
    class function NULL : _NULL;

    function SaveToStream(aStream: TStream; OutFormat: integer): boolean;
    function SaveToFile(aFilename: string; OutFormat: integer): boolean;
    function SaveToParcel(aQBParcel: TQBParcel; GoodsName: Ansistring; OutFormat: integer): boolean;

  end;

  TQBJSonArray = class (TQBJSonAbstractObject)
  public
    destructor destroy ; override;
    constructor create ; overload;
    constructor create (collection : TList); overload;
    constructor create (x : TQBJSonTokener); overload;
    constructor create (s : Ansistring);  overload;
    constructor Create (aStream: TStream; Bytes: integer); overLoad;
    constructor Create (aFilename: string; FailIfNoFile: boolean); overload;
    constructor create (aQBParcel: TQBParcel; GoodsName: Ansistring); overload;

    function Clone : TQBJSonAbstractObject; override;
    function get (index : integer) : TQBJSonAbstractObject;
    function getBoolean (index : integer) : boolean;
    function getDouble (index : integer) : double;
    function getInt (index : integer): integer;
    function getJSonArray (index : integer) : TQBJSonArray;
    function getJSon (index : integer) : TQBJSon;
    function getString (index : integer) : Ansistring;
    function isNull (index : integer): boolean;
    function join (separator : Ansistring) : Ansistring;
    function length : integer;
    function opt (index : integer) : TQBJSonAbstractObject;
    function optBoolean ( index : integer) : boolean; overload;
    function optBoolean ( index : integer; defaultValue : boolean) : boolean; overload;
    function optDouble (index : integer) : double; overload;
    function optDouble (index : integer; defaultValue :double ) : double ; overload;
    function optInt (index : integer) : integer; overload;
    function optInt (index : integer; defaultValue : integer) : integer; overload;
    function optJSonArray (index : integer) : TQBJSonArray ; overload;
    function optJSon (index : integer) : TQBJSon ; overload;
    function optString (index : integer) : Ansistring; overload;
    function optString (index : integer; defaultValue : Ansistring) : Ansistring; overload;

    procedure put ( value : boolean); overload ;
    procedure put ( value : double );   overload ;
    procedure put ( value : integer);   overload ;
    procedure put ( value : TQBJSonAbstractObject);  overload ;
    procedure put ( value: Ansistring); overload;
    procedure put ( index : integer ; value : boolean);  overload ;
    procedure put ( index : integer ; value : double);  overload ;
    procedure put ( index : integer ; value : integer);  overload ;
    procedure put ( index : integer ; value : TQBJSonAbstractObject);  overload ;
    procedure put ( index: integer; value: Ansistring); overload;

    function toJSon (names : TQBJSonArray ) : TQBJSon ;  overload ;
    function toString : Ansistring; overload; override;
    function toString2 (indentFactor : integer) : Ansistring; overload;
    function toString3 (indentFactor, indent : integer) : Ansistring; overload;
    function appendJSonArray( value : TQBJSonArray): Integer ;

    function SaveToSteam(aStream: TStream; OutFormat: integer): boolean;
    function SaveToFile(aFilename: Ansistring; OutFormat: integer): boolean;
    function SaveToParcel(aQBParcel: TQBParcel; GoodsName: Ansistring; OutFormat: integer): boolean;

  private
    myArrayList : TList;
  end;

  _Number =  class (TQBJSonAbstractObject)
    function doubleValue : double; virtual; abstract;
    function intValue : integer; virtual; abstract;
  end;

  _Boolean = class (TQBJSonAbstractObject)
    class function _TRUE () : _Boolean;
    class function _FALSE () : _Boolean;
    class function valueOf (b : boolean) : _Boolean;
    constructor create (b : boolean);
    function boolValue : Boolean;
    function toString () : Ansistring; override;
    function clone :TQBJSonAbstractObject;  override;
  private
    fvalue : boolean;
  end;

  _Double = class (_Number)
    constructor create (s : Ansistring); overload;
    constructor create (s : _String); overload;
    constructor create (d : double); overload;
    function doubleValue : double; override;
    function intValue : integer;  override;
    function toString () : Ansistring ; override;
    class function NaN : double;
    function clone :TQBJSonAbstractObject; override;
  private
    fvalue : double;
  end;

  _Integer = class (_Number)
    class function parseInt (s : Ansistring; i : integer): integer; overload;
    class function parseInt (s : _String): integer; overload;
    class function toHexString (c : Ansichar) : Ansistring;
    constructor create (i : integer); overload;
    constructor create (s : Ansistring); overload;
    function doubleValue : double; override;
    function intValue : integer;  override;
    function toString () : Ansistring; override;
    function clone :TQBJSonAbstractObject; override;
  private
    fvalue : integer;
  end;

  _String = class (TQBJSonAbstractObject)
    constructor create (s : Ansistring);
    function equalsIgnoreCase (s: Ansistring) : boolean;
    function Equals(const Value: TQBJSonAbstractObject): Boolean; override;
    function toString() : Ansistring; override;
    function clone :TQBJSonAbstractObject; override;
  private
    fvalue : Ansistring;
  end;

  _NULL = class (TQBJSonAbstractObject)
    function Equals(const Value: TQBJSonAbstractObject): Boolean; override;
    function toString() : Ansistring; override;
  end;

var
  gcLista : TList;
  CNULL : _NULL;

implementation

const
  CROTINA_NAO_IMPLEMENTADA: Ansistring = 'Not imp';

var
  CONST_FALSE : _Boolean ;
  CONST_TRUE : _Boolean;

function IsSimpString(const Str: AnsiString):Boolean;
var
  i:Integer;
begin
  Result:=true;
  for i:=1 to Length(Str) do
     begin
        Result:=Str[i] in ['0'..'9','a'..'z','A'..'Z','_'];
        if not Result then
           exit;
     end;
end;

procedure newNotImplmentedFeature () ;
begin
  raise NotImplmentedFeature.create(string(CROTINA_NAO_IMPLEMENTADA));
end;

function HexToInt(S: String): Integer;
var
  I, E, F, G: Integer;

  function DigitValue(C: Char): Integer;
  begin
    case C of
      'A': Result := 10;
      'B': Result := 11;
      'C': Result := 12;
      'D': Result := 13;
      'E': Result := 14;
      'F': Result := 15;
    else
      Result := StrToInt(C);
    end;
  end;

begin
  S := UpperCase(S);
  if S[1] = '$' then
     Delete(S, 1, 1);
  if S[2] = 'X' then
     Delete(S, 1, 2);
  E := -1; Result := 0;
  for I := Length(S) downto 1 do
     begin
        G := 1;
        for F := 0 to E do
           G := G*16;
        Result := Result+(DigitValue(S[I])*G);
        Inc(E);
     end;
end;



constructor TQBJSonTokener.create(s: Ansistring);
begin
   self.myIndex := 1;
   self.mySource := s;
end;

procedure TQBJSonTokener.back;
begin
  if (self.myIndex > 1) then
     self.myIndex := self.myIndex - 1;
end;

class function TQBJSonTokener.dehexchar(c: Ansichar): integer;
begin
  if ((c >= '0') and (c <= '9')) then
     begin
        Result :=  (ord(c) - ord('0'));
        exit;
     end;
  if ((c >= 'A') and (c <= 'F')) then
     begin
        Result :=  (ord(c) + 10 - ord('A'));
        exit;
     end;
  if ((c >= 'a') and (c <= 'f')) then
     begin
        Result := ord(c) + 10 - ord('a');
        exit;
     end;
  Result := -1;
end;

function TQBJSonTokener.more: boolean;
begin
  Result := (self.myIndex <= System.length(self.mySource)+1);
end;

function TQBJSonTokener.next: Ansichar;
begin
  if more() then
     begin
        Result := self.mySource[self.myIndex];
        self.myIndex := self.myIndex + 1;
        exit;
     end;
  Result := chr(0);
end;

function TQBJSonTokener.next(c: Ansichar): Ansichar;
begin
  Result := next();
  if Result <> c then
     raise syntaxError(AnsiString('Expected ') + c + AnsiString(' and instead saw ') + Result + AnsiString('.'));
end;

function TQBJSonTokener.next(n: integer): Ansistring;
var
 i,j : integer;
begin
  i := self.myIndex;
  j := i + n;
  if (j > System.length(self.mySource)) then
     raise syntaxError('Substring bounds error');
  self.myIndex := self.myIndex + n;
  Result := copy (self.mySource,i,n);
end;

function TQBJSonTokener.nextClean: Ansichar;
var
  c: Ansichar;

begin
  while (true) do
     begin
        c := next();
        if (c = '/') then
           begin
              case (next()) of
                 '/': begin
                         repeat
                            c := next();
                         until (not ((c <> #10) and (c <> #13) and (c <> #0)));
                      end ;
                 '*': begin
                         while (true) do
                            begin
                               c := next();
                               if (c = #0) then
                                  raise syntaxError('Unclosed comment.');
                               if (c = '*') then
                                  begin
                                     if (next() = '/') then
                                        break;
                                     back();
                                  end;
                            end;
                      end;
                 else
                      begin
                         back();
                         Result := '/';
                         exit;
                      end;
              end;
           end
        else
           if (c = '#') then
              begin
                 repeat
                    c := next();
                 until (not ((c <> #10) and (c <> #13) and (c <> #0)));
              end
           else
              begin
                 if ((c = #0) or (c > ' ')) then
                    begin
                       Result := c;
                       exit;
                    end;
              end;
     end;
end;

function TQBJSonTokener.nextString (quote : Ansichar): Ansistring;
var
  c : Ansichar;
  sb : Ansistring;
begin
  sb := '';
  while (true) do
     begin
        c := next();
        case (c) of
           #0, #10, #13: continue;
           '\': begin
                   c := next();
                   case (c) of
                      't': sb := sb + #9;
                      'n': sb := sb + #10;
                      'f': sb := sb + #12;
                      'r': sb := sb + #13;
                      else
                         sb := sb + c
                   end;
                end;
             else
                begin
                   if (c = quote) then
                      begin
                         Result := sb;
                         exit;
                      end;
                   sb := sb + c
                end;
        end;
     end;
end;

function TQBJSonTokener.nextTo(d: Ansichar): Ansistring;
var
  sb : Ansistring;
  c : Ansichar;
begin
  sb := '';
  while (true) do
     begin
        c := next();
        if ((c = d) or (c = #0) or (c = #10) or (c = #13)) then
           begin
              if (c <> #0) then
                 back();
              Result := trim (sb);
              exit;
           end;
        sb := sb + c;
     end;
end;

function TQBJSonTokener.nextTo(delimiters: Ansistring): Ansichar;
var
  c : Ansichar;
  sb : Ansistring;
begin
  Result:=#0;
  sb := '';
  while (true) do
     begin
        c := next();
        if ((pos (c,delimiters) > 0) or (c = #0) or (c = #10) or (c = #13)) then
           begin
              if (c <> #0) then
                 back();
              sb := trim(sb);
              if (System.length(sb) > 0) then
                 Result := sb[1];
              exit;
           end;
        sb := sb + c;
     end;
end;

function TQBJSonTokener.nextValue: TQBJSonAbstractObject;
var
  c, b : Ansichar;
  s , sb: Ansistring;
begin
  c := nextClean();
  case (c) of
    '"', #39: begin
        Result := _String.create (nextString(c));
        exit;
    end;
    '{': begin
        back();
        Result := TQBJSon.create(self);
        exit;
    end;
    '[': begin
        back();
        Result := TQBJSonArray.create(self);
        exit;
    end;
  end;

  sb := '';
  b := c;
  while ((ord(c) >= ord(' ')) and (pos (c,AnsiString(',:]}/\\\"[{;=#')) = 0)) do
     begin
        sb := sb + c;
        c := next();
     end;
  back();

  s := trim (sb);
  if (s = '') then
     raise syntaxError('Missing value.');

  if (AnsiLowerCase (s) = 'true') then
     begin
        Result :=  _Boolean._TRUE;
        exit;
     end;

  if (AnsiLowerCase (s) = 'false') then
     begin
        Result := _Boolean._FALSE;
        exit;
     end;

  if (AnsiLowerCase (s) = 'null') then
     begin
        Result := TQBJSon.NULL;
        exit;
     end;

  if ( ((b >= '0') and (b <= '9')) or (b = '.') or (b = '-') or (b = '+')) then
     begin
        if (b = '0') then
           begin
              if ((System.length(s) > 2) and ((s[2] = 'x') or (s[2] = 'X'))) then
                 begin
                    try
                       Result := _Integer.create(_Integer.parseInt(copy(s,3,System.length(s)), 16));
                       exit;
                    Except
                       on e:Exception do
                          begin
                          end;
                    end;
                 end
              else
                 begin
                    try
                       Result := _Integer.create(_Integer.parseInt(s, 8));
                       exit;
                    Except
                       on e:Exception do
                          begin
                          end;
                    end;
                 end;
           end;

        if Pos(AnsiString('.'),s)<0 then
           try
              Result := _Integer.create(s);
              exit;
           Except
              on e:Exception do
                 begin
                 end;
           end;

        try
           Result := _Double.create(s);
           exit;
        Except
           on e:Exception do
              begin
              end;
        end;
     end;
  Result := _String.create(s);
end;

function TQBJSonTokener.skipTo(_to: Ansichar): Ansichar;
var
  c : Ansichar;
  index : integer;
begin
  index := self.myIndex;
  repeat
     c := next();
     if (c = #0) then
        begin
           self.myIndex := index;
           Result := c;
           exit;
        end;
  until (not (c <> _to));
  back();
  Result := c;
end;

procedure TQBJSonTokener.skipPast(_to: Ansistring);
begin
   self.myIndex := pos(_to, copy(mySource, self.myIndex, System.length(mySource)));
   if (self.myIndex < 0) then
      self.myIndex := System.length(self.mySource)+1
   else
      self.myIndex := self.myIndex + System.length(_to);
end;

function TQBJSonTokener.syntaxError(_message: Ansistring): ParseException;
begin
   Result := ParseException.create (_message + toString()+' syntax error : ' + copy (toString(),self.myIndex,10), self.myIndex);
end;

function TQBJSonTokener.toString: Ansistring;
begin
  Result := ' at character ' + AnsiString(intToStr(self.myIndex)) + ' of ' + self.mySource;
end;

function TQBJSonTokener.unescape(s: Ansistring): Ansistring;
var
  len, i,d,e : integer;
  b : Ansistring;
  c : Ansichar;
begin
  len := System.length(s);
  b := '';
  i := 1;
  while ( i <= len ) do
     begin
        c := s[i];
        if (c = '+') then
           c := ' '
        else
           if ((c = '%') and ((i + 2) <= len)) then
              begin
                 d := dehexchar(s[i + 1]);
                 e := dehexchar(s[i + 2]);
                 if ((d >= 0) and (e >= 0)) then
                    begin
                       c := AnsiChar(d * 16 + e);
                       i := i + 2;
                    end;
              end;
        b := b + c;
        i := i + 1;
     end;
  Result := b ;
end;

constructor TQBJSon.create;
begin
  myHashMap := TStringList.create;
end;

constructor TQBJSon.create(jo: TQBJSon; sa: array of Ansistring);
var
 i : integer;
begin
  create();
  for i :=low(sa) to high(sa) do
     putOpt(sa[i], jo.opt(sa[i]).Clone);
end;

constructor TQBJSon.create(x: TQBJSonTokener);
begin
  create ;
  UpdateByTokener(x);
end;

constructor TQBJSon.create(map: TStringList);
var
 i : integer;
begin
  self.myHashMap := TStringlist.create;
  for i := 0 to map.Count -1 do
     self.myHashMap.AddObject(map[i],map.Objects[i]);
end;

constructor TQBJSon.create(s: Ansistring);
var
  Tokener : TQBJSonTokener;
begin
  if s='' then
     begin
       create();
       exit;
     end;
  Tokener := TQBJSonTokener.create(s);
  try
     create(Tokener);
  finally
     Tokener.free;
  end;
end;

constructor TQBJSon.Create(aStream: TStream; Bytes: integer);
var
   str: AnsiString;
begin
   setlength(str,bytes);
   aStream.ReadBuffer(str[1],bytes);
   create(str);
end;

constructor TQBJSon.Create (aFilename: string; FailIfNoFile: boolean);
var
   Stream: TMemoryStream;
begin
   if not fileexists(aFileName) then
      begin
         if FailIfNoFile then
            raise Exception.create('File not found!')
         else
            Create;
      end
   else
      begin
         Stream:=TMemoryStream.Create;
         try
            try
               Stream.LoadFromFile(aFileName);
               Stream.Position:=0;
               Create(Stream,Stream.Size);
            except
               raise Exception.create('Read from file error!');
            end;
         finally
            Stream.Free;
         end;
      end;
end;

constructor TQBJSon.create (aQBParcel: TQBParcel; GoodsName: Ansistring);
var
   Str: Ansistring;
begin
   str:=aQBParcel.GetAnsiStringGoods(GoodsName);
   Create(str);
end;

function TQBJSon.accumulate(key: Ansistring; value: TQBJSonAbstractObject): TQBJSon;
var
  a : TQBJSonArray;
  o : TQBJSonAbstractObject;
begin
  o := opt(key);
  if (o = nil) then
     put(key, value)
  else
     if (o is TQBJSonArray) then
        begin
           a := TQBJSonArray(o);
           a.put(value);
        end
     else
        begin
           a := TQBJSonArray.create;
           a.put(o.clone);
           a.put(value);
           put(key, a);
       end;
  Result := self;
end;

function TQBJSon.get(key: Ansistring): TQBJSonAbstractObject;
var
  o : TQBJSonAbstractObject;
begin
  o := opt(key);
  if (o = nil) then
     raise NoSuchElementException.create('TQBJSon[' + string(quote(key)) + '] not found.');
  Result := o;
end;

function TQBJSon.getBoolean(key: Ansistring): boolean;
var
  o : TQBJSonAbstractObject;
begin
    o := get(key);
    if (o.equals(_Boolean._FALSE) or ((o is _String) and (_String(o)).equalsIgnoreCase('false'))) then
       begin
          Result := false;
          exit;
       end
    else
       if (o.equals(_Boolean._TRUE) or ((o is _String) and (_String(o)).equalsIgnoreCase('true'))) then
          begin
             Result := true;
             exit;
          end;
    raise ClassCastException.create('TQBJSon[' + string(quote(key)) + '] is not a Boolean.');
end;

function TQBJSon.getDouble(key: Ansistring): double;
var
  o : TQBJSonAbstractObject;
begin
   o := get(key);
   if (o is _Number) then
      begin
         Result := _Number (o).doubleValue();
         exit;
      end ;
   if (o is _String) then
      begin
         Result := StrToFloat(string(_String(o).toString()));
         exit;
      end;
   raise NumberFormatException.create('TQBJSon[' + string(quote(key)) + '] is not a number.');
end;

function TQBJSon.getInt(key: Ansistring): integer;
var
  o : TQBJSonAbstractObject;
begin
   o := get(key);
   if (o is _Number) then
      Result :=  _Number(o).intValue()
   else
      Result :=  Round(getDouble(key));
end;

function TQBJSon.getJSonArray(key: Ansistring): TQBJSonArray;
var
  o : TQBJSonAbstractObject;
begin
  o := get(key);
  if (o is TQBJSonArray) then
     Result := TQBJSonArray(o)
  else
     raise  NoSuchElementException.create('TQBJSon[' + string(quote(key)) + '] is not a TQBJSonArray.');
end;

function TQBJSon.getJSon(key: Ansistring): TQBJSon;
var
  o : TQBJSonAbstractObject;
begin
   o := get(key);
   if (o is TQBJSon) then
      Result := TQBJSon(o)
   else
      raise NoSuchElementException.create('TQBJSon[' + string(quote(key)) + '] is not a TQBJSon.');
end;

function TQBJSon.getString(key: Ansistring): Ansistring;
begin
   Result := get(key).toString();
end;

function TQBJSon.has(key: Ansistring): boolean;
begin
   Result := (self.myHashMap.IndexOf(string(key)) >= 0);
end;

function TQBJSon.isNull(key: Ansistring): boolean;
begin
   Result := NULL.equals(opt(key));
end;

function TQBJSon.keys: TStringList;
var
 i : integer;
begin
  Result := TStringList.Create;
  for i := 0 to myHashMap.Count -1 do begin
    Result.add (myHashMap[i]);
  end;
end;

function TQBJSon.Length: integer;
begin
   Result := myHashMap.Count;
end;

function TQBJSon.names: TQBJSonArray;
var
  ja : TQBJSonArray;
  i : integer;
  k : TStringList;
begin
   ja := TQBJSonArray.create;
   k := keys;
   try
      for i := 0 to k.Count -1 do
         ja.put(_String.create(AnsiString(k[i])));
      if (ja.length = 0) then
         Result := nil
      else
         Result := ja;
   finally
      k.free;
   end;
end;

class function TQBJSon.numberToString(n: _Number): Ansistring;
begin
   if (n = nil) then
     Result := ''
   else
      begin
         if (n is _Integer) then
            Result := AnsiString(IntToStr(n.intValue))
         else
            Result := AnsiString(FloatToStr(n.doubleValue));
      end;
end;

function TQBJSon.opt(key: Ansistring): TQBJSonAbstractObject;
begin
   if (key = '') then
      raise NullPointerException.create('Null key')
   else
      begin
        if myHashMap.IndexOf(string(key)) < 0 then
           Result := nil
        else
           Result := TQBJSonAbstractObject(myHashMap.Objects[myHashMap.IndexOf(string(key))]);
     end;
end;

function TQBJSon.optBoolean(key: Ansistring): boolean;
begin
  Result := optBoolean(key, false);
end;

function TQBJSon.optBoolean(key: Ansistring; defaultValue: boolean): boolean;
var
  o : TQBJSonAbstractObject;
begin
  o := opt(key);
  if (o <> nil) then
     begin
        if o.ClassType=_Boolean then
           begin
              Result:=_Boolean(o).fvalue;
              exit;
           end
        else
           if ((o is _String) and (_String(o).equalsIgnoreCase('false'))) then
              begin
                 Result := false;
                 exit;
              end
           else
              if ((o is _String) and (_String(o).equalsIgnoreCase('true'))) then
                 begin
                    Result := true;
                    exit;
                 end;
     end;
  Result := defaultValue;
end;

function TQBJSon.optDouble(key: Ansistring): double;
begin
  Result := optDouble(key, _Double.NaN);
end;

function TQBJSon.optDouble(key: Ansistring; defaultValue: double): double;
var
  o: TQBJSonAbstractObject;
begin
   o := opt(key);
   if (o <> nil) then
      begin
         if (o is _Number) then
            begin
               Result := (_Number(o)).doubleValue();
               exit;
            end ;
         try
            Result := _Double.create(_String(o)).doubleValue();
            exit;
         except
            on e:Exception  do
               begin
                  Result := defaultValue;
                  exit;
               end;
         end;
      end;
   Result := defaultValue;
end;

function TQBJSon.optInt(key: Ansistring): integer;
begin
  Result := optInt (key, 0);
end;

function TQBJSon.optInt(key: Ansistring; defaultValue: integer): integer;
var
   o : TQBJSonAbstractObject;
begin
   o := opt(key);
   if (o <> null) and ( o <> nil ) then
      begin
         if (o is _Number) then
            begin
               Result :=  (_Number(o)).intValue();
               exit;
            end;
         try
            Result := _Integer.parseInt(_String(o));
         except
            on e:Exception do
               Result := defaultValue;
         end;
      end
   else
      Result := defaultValue;
end;

function TQBJSon.optJSonArray(key: Ansistring): TQBJSonArray;
var
 o : TQBJSonAbstractObject ;
begin
    o := opt(key);
    if (o is TQBJSonArray) then
       Result := TQBJSonArray(o)
    else
       Result := nil;
end;

function TQBJSon.optJSon(key: Ansistring): TQBJSon;
var
  o : TQBJSonAbstractObject ;
begin
   o := opt(key);
   if (o is TQBJSon) then
      Result := TQBJSon(o)
   else
      Result := nil;
end;

function TQBJSon.optString(key: Ansistring): Ansistring;
begin
  Result := optString(key, '');
end;

function TQBJSon.optString(key, defaultValue: Ansistring): Ansistring;
var
  o : TQBJSonAbstractObject ;
begin
   o := opt(key);
   if (o <> nil) then
      Result := o.toString()
   else
      Result := defaultValue;
end;

procedure TQBJSon.put(key: AnsiString; value: boolean);
begin
   put(key, _Boolean.valueOf(value));
end;

procedure TQBJSon.put(key: AnsiString; value: double);
begin
   put(key, _Double.create(value));
end;

procedure TQBJSon.put(key: AnsiString; value: integer);
begin
   put(key, _Integer.create(value));
end;

procedure TQBJSon.put(key: AnsiString; value: TQBJSonAbstractObject);
var
  temp : TObject;
  i : integer;
begin
    if (key = '') then
       raise NullPointerException.create('Null key.');
    if (value <> nil) then
       begin
          i := self.myHashMap.IndexOf(string(key));
          if ( i >= 0) then
             begin
                temp := self.myHashMap.Objects [i];
                self.myHashMap.Objects[i]  := value;
                temp.free;
             end
          else
             self.myHashMap.AddObject(string(key), value);
       end
    else
       begin
          temp := remove(key);
          if (temp <> nil) then
             temp.free;
       end;
end;

procedure TQBJSon.put(key, value: AnsiString);
begin
   put(key, _String.create(value));
end;

procedure TQBJSon.putOpt(key: AnsiString; value: TQBJSonAbstractObject);
begin
   if (value <> nil) then
      put(key, value);
end;

class function TQBJSon.quote(s: AnsiString): AnsiString;
var
   b,c : AnsiChar;
   i, len : integer;
   sb, t : AnsiString;
begin
  if ((s = '') or (System.Length(s) = 0)) then
     begin
        Result :=  '""';
        exit;
     end;

  c := #0;
  len := System.length(s);
  t := '';

  sb := sb +'"';
  for i := 1 to len do
     begin
        b := c;
        c := s[i];
        case (c) of
           '\', '"':
               begin
                  sb := sb + '\';
                  sb := sb + c;
               end;
           '/':
               begin
                  if (b = '<') then
                     sb := sb + '\';
                  sb := sb + c;
               end;
           #8, #9, #10, #12, #13:
               sb := sb + c;
           else
               begin
                  if (c < ' ') then
                     begin
                        t := AnsiString('000') + _Integer.toHexString(c);
                        sb := sb + '\u' + copy (t,System.length(t)-3,4);
                     end
                  else
                     sb := sb + c;
               end;
        end;
     end;
  sb := sb + '"';
  Result := sb;
end;

function TQBJSon.remove(key: AnsiString): TQBJSonAbstractObject;
var
  i:Integer;
begin
  i:=myHashMap.IndexOf(string(key));
  if ( i < 0) then
     Result:=nil
  else
     begin
        Result:=TQBJSonAbstractObject(myHashMap.Objects [i]);
        self.myHashMap.delete(i);
     end;
end;

function TQBJSon.toJSonArray(names: TQBJSonArray): TQBJSonArray;
var
 i : integer;
 ja : TQBJSonArray ;
begin
  if ((names = nil) or (names.length() = 0)) then
     begin
        Result := nil;
        exit;
     end;
  ja := TQBJSonArray.create;
  for i := 0 to names.length -1  do
     ja.put(self.opt(names.getString(i)));
  Result := ja;
end;

function TQBJSon.toString: AnsiString;
var
  _keys : TStringList;
  sb : AnsiString;
  o : AnsiString;
  i :integer;
begin
  _keys := keys();
  try
    sb := '{';

    for i := 0 to _keys.count -1 do
    begin
      if (System.length(sb) > 1) then
      begin
        sb:= sb + ',';
      end;
      o := AnsiString(_keys[i]);
      if IsSimpString(o) then
        sb := sb + o
      else
        sb := sb + quote(o);
      sb := sb + ':';
      sb:= sb + valueToString(TQBJSonAbstractObject(myHashMap.Objects[myHashMap.IndexOf(string(o))]));
    end;
    sb := sb + '}';
    Result := sb;
  finally
    _keys.free;
  end;
end;

function TQBJSon.toString2(indentFactor: integer): AnsiString;
begin
  Result := toString3(indentFactor, 0);
end;

function TQBJSon.toString3(indentFactor, indent: integer): AnsiString;
var
 j , i , n , newindent: integer;
 _keys : TStringList;
 o, sb : AnsiString;
begin
  n := length();
  if (n = 0) then
     begin
       Result := '';
       exit;
     end;
  _keys := keys();
  sb := sb + '{';
  newindent := indent + indentFactor;
  if (n = 1) then
     begin
       o := AnsiString(_keys[0]);
       sb:= sb + quote(o);
       sb:= sb + ': ';
       sb:= sb + valueToString(TQBJSonAbstractObject(myHashMap.Objects[myHashMap.IndexOf(string(o))]), indentFactor, indent);
     end
  else
     begin
       for j := 0 to _keys.count -1 do
          begin
            o := AnsiString(_keys[j]);
            if (system.length(sb) > 1) then
               sb := sb + ','+ #13#10
            else
               sb:= sb + #13#10;
            for i := 0 to newindent -1 do
              sb:= sb + ' ';
            sb:= sb + quote(o);
            sb:= sb + ': ';
            sb:= sb + valueToString(TQBJSonAbstractObject(myHashMap
            .Objects[myHashMap.IndexOf(string(o))])
            , indentFactor, newindent);
          end;
       if (System.length(sb) > 1) then
          begin
            sb := sb + #13#10;
            for i := 0 to indent -1 do
            begin
              sb:= sb + ' ';
            end;
          end;
     end;
  _keys.Free;
  sb:= sb + '}';
  Result := sb;
end;

class function TQBJSon.NULL: _NULL;
begin
  Result := CNULL;
end;

class function TQBJSon.valueToString(value: TQBJSonAbstractObject): AnsiString;
begin
  if ((value = nil) or (value.equals(null))) then
     begin
        Result := 'null';
        exit;
     end;
  if (value is _Number) then
     begin
        Result := numberToString(_Number(value));
        exit;
     end;
  if ((value is _Boolean) or (value is TQBJSon) or (value is TQBJSonArray)) then
     begin
        Result := value.toString();
        exit;
     end;
  Result := quote(value.toString());
end;

class function TQBJSon.valueToString(value: TQBJSonAbstractObject; indentFactor, indent: integer): AnsiString;
begin
   if ((value = nil) or (value.equals(nil))) then
      begin
         Result := 'null';
         exit;
      end;
    if (value is _Number) then
       begin
          Result := numberToString(_Number(value));
          exit;
       end;
    if (value is _Boolean) then
       begin
          Result :=  value.toString();
          exit;
      end;
    if (value is TQBJSon) then
       begin
          Result := ((TQBJSon(value)).toString3(indentFactor, indent));
          exit;
       end;
    if (value is TQBJSonArray) then
       begin
          Result := ((TQBJSonArray(value)).toString3(indentFactor, indent));
          exit;
       end;
    Result := quote(value.toString());
end;

procedure TQBJSon.clean;
var
  sl : TStringList;
  i : integer;
  obj : TObject;
begin
   sl := keys;
   for i := 0 to sl.count -1 do
      begin
         obj := remove(AnsiString(sl[i]));
         if (obj <> nil) then
            FreeAndNil (obj);
      end;
   sl.free;
end;

procedure TQBJSon.assignTo (JSon: TQBJSon) ;
var
 _keys : TStringList;
 i : integer;
begin
  _keys := keys;
  try
    for i := 0 to _keys.Count -1 do
    begin
      JSon.put (AnsiString(_keys[i]),get(AnsiString(_keys[i])).clone);
    end;
  finally
   _keys.free;
  end;
end;

function TQBJSon.clone: TQBJSonAbstractObject;
begin
  Result := TQBJSon.create(self.toString());
end;

function TQBJSon.GetPropValues(const Key: AnsiString): AnsiString;
begin
  Result:=OptString(Key);
end;

procedure TQBJSon.SetPropValues(const Key: AnsiString; const Value: AnsiString);
begin
  Put(Key, Value);
end;

function TQBJSon.GetCascadeValue(const Keys: array of AnsiString): AnsiString;
var
  i:Integer;
  TmpProp:TQBJSon;
begin
  Result:='';
  TmpProp:=Self;
  for i:=Low(Keys) to High(Keys) do
     begin
        if i=High(Keys) then
           begin
              Result:=TmpProp.PropValues[Keys[i]];
              exit;
           end;
        TmpProp:=TmpProp.OptJSon(Keys[i]);
        if TmpProp=nil then
           exit;
     end;
end;

procedure TQBJSon.SetAsString(const Value: AnsiString);
var
  Tokener: TQBJSonTokener;
  i : integer;
  MyObj: TObject;
begin
  for i:=Pred(myHashMap.Count) downto 0 do
     begin
        MyObj:=myHashMap.Objects[i];
        if (MyObj <> CONST_FALSE) and (MyObj <> CONST_TRUE) and (MyObj <> CNULL) then
           MyObj.Free;
     end;
  myHashMap.Clear;
  if System.Length(Value)<=2 then
     exit;
  Tokener:=TQBJSonTokener.create(Value);
  try
     UpdateByTokener(Tokener);
  finally
     Tokener.free;
  end;
end;

function TQBJSon.GetDiffFrom(Source: TQBJSon): TQBJSon;
var
  i:Integer;
begin
  Result:=Create;
  with Source.Keys do
     begin
        for i:=0 to Pred(Count) do
           begin
              if Source.PropValues[AnsiString(Strings[i])]=PropValues[AnsiString(Strings[i])] then
                 continue;
              Result.PropValues[AnsiString(Strings[i])]:=Source.PropValues[AnsiString(Strings[i])];
           end;
        Free;
     end;
end;

procedure TQBJSon.RemoveByKeyHeader(const Header: AnsiString);
var
  i:Integer;
begin
  with Keys do
     begin
        for i:=Pred(Count) downto 0 do
           begin
              if Pos(Header,AnsiString(Strings[i]))=1 then
                 CleanKey(AnsiString(Strings[i]));
           end;
        Free;
     end;
end;

function TQBJSon.PropCount: Integer;
begin
  Result:=myHashMap.Count;
end;

function TQBJSon.KeyByVal(const Value: AnsiString): AnsiString;
var
  i:Integer;
begin
  for i:=0 to Pred(myHashMap.Count) do
     begin
        with TQBJSonAbstractObject(myHashMap.Objects[i]) do
           begin
              if toString=Value then
                 begin
                    Result:=AnsiString(myHashMap[i]);
                    exit;
                 end;
           end;
     end;
  Result:='';
end;

procedure TQBJSon.Assign(Source: TQBJSon);
begin
  if Source=nil then
     Clean
  else
     AsString:=Source.AsString;
end;

function TQBJSon.GetKeyByIndex(index: Integer): AnsiString;
begin
  Result:=AnsiString(myHashMap[index]);
end;

procedure TQBJSon.SetCascadeValue(const Value: AnsiString; const Keys: array of AnsiString);
begin
  SetCascadeValueEx(Value,Keys,0);
end;

procedure TQBJSon.SetCascadeValueEx(const Value: AnsiString; const Keys: array of AnsiString; StartIdx: Integer);
var
  JObj:TQBJSon;
begin
  if High(Keys)<StartIdx then
     exit;
  if High(Keys)=StartIdx then
     begin
        Self.Put(Keys[StartIdx],Value);
        exit;
     end;
  JObj:=OptJSon(Keys[StartIdx]);
  if JObj=nil then
     begin
        JObj:=TQBJSon.Create;
        Self.Put(Keys[StartIdx],JObj);
     end;
  JObj.SetCascadeValueEx(Value,Keys,StartIdx+1);
end;

function TQBJSon.GetValByIndex(index: Integer): AnsiString;
begin
  Result:=TQBJSonAbstractObject(myHashMap.Objects[index]).toString;
end;

procedure TQBJSon.CleanKey(const Key: AnsiString);
var
  i:Integer;
begin
  i:=myHashMap.IndexOf(string(key));
  if i<0 then
     exit;
  TQBJSonAbstractObject(myHashMap.Objects[i]).Free;
  myHashMap.delete(i);
end;

procedure TQBJSon.UpdateByTokener(x: TQBJSonTokener);
var
  c : AnsiChar;
  key : AnsiString;
begin
  key := '';

  if (x.nextClean() <> '{') then
     raise x.syntaxError('A TQBJSon must begin with "{"');
  while (true) do
     begin
        c := x.nextClean();
        case (c) of
           #0:
              raise x.syntaxError('A TQBJSon must end with "}"');
           '}':
              exit;
           else
              begin
                 x.back();
                 with x.nextValue() do
                    begin
                       key := toString();
                       Free;
                    end;
              end
        end;

        c := x.nextClean();
        if (c = '=') then
           begin
              if (x.next() <> '>') then
                 x.back();
           end
        else
           begin
              if (c <> ':') then
                 raise x.syntaxError('Expected a ":" after a key');
           end;
        self.myHashMap.AddObject(string(key), x.nextValue());

        case (x.nextClean()) of
           ';', ',':
               begin
                  if (x.nextClean() = '}') then
                    exit;
                  x.back();
               end;
           '}':
               exit;
           else
               raise x.syntaxError('Expected a "," or "}"');
        end;
     end;
end;

function TQBJSon.SaveToStream(aStream: TStream; OutFormat: integer): boolean;
var
   Str: AnsiString;
begin
   if OutFormat=1 then
      Str:=Self.ToString
   else
      begin
         if OutFormat=2 then
            Str:=Self.ToString2(3)
         else
            Str:=Self.ToString3(3,3);
      end;
   aStream.WriteBuffer(Str[1],System.Length(Str));
   result:=true;
end;

function TQBJSon.SaveToFile(aFilename: String; OutFormat: integer): boolean;
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   try
      Self.SaveToStream(Stream,OutFormat);
      Stream.Position:=0;
      Stream.SaveToFile(string(aFileName));
      result:=true;
   finally
      Stream.Free;
   end;
end;

function TQBJSon.SaveToParcel(aQBParcel: TQBParcel; GoodsName: AnsiString; OutFormat: integer): boolean;
var
   Str: AnsiString;
begin
   if OutFormat=1 then
      Str:=Self.ToString
   else
      begin
         if OutFormat=2 then
            Str:=Self.ToString2(3)
         else
            Str:=Self.ToString3(3,3);
      end;
   aQBParcel.PutStringGoods(GoodsName,string(str));
   result:=true;
end;



function _Boolean.boolValue: Boolean;
begin
  Result := fvalue;
end;


function _Boolean.clone: TQBJSonAbstractObject;
begin
  Result := _Boolean.create(Self.fvalue);
end;

constructor _Boolean.create(b: boolean);
begin
   fvalue := b;
end;

function _Boolean.toString: AnsiString;
begin
  if fvalue then
     Result := 'true'
  else
     Result := 'false';
end;

class function _Boolean.valueOf(b: boolean): _Boolean;
begin
  if (b) then
     Result := _TRUE
  else
     Result := _FALSE;
end;

class function _Boolean._FALSE: _Boolean;
begin
  Result := CONST_FALSE;
end;

class function _Boolean._TRUE: _Boolean;
begin
  Result := CONST_TRUE;
end;



function _String.clone: TQBJSonAbstractObject;
begin
  Result := _String.create (self.fvalue);
end;

constructor _String.create(s: AnsiString);
begin
  fvalue := s;
end;

function _String.equals(const Value: TQBJSonAbstractObject): Boolean;
begin
    Result := (value is _String) and (_String (value).fvalue = fvalue);
end;

function _String.equalsIgnoreCase(s: AnsiString): boolean;
begin
   Result := AnsiLowerCase (s) = AnsiLowerCase (fvalue);
end;

function _String.toString: AnsiString;
begin
  Result := fvalue;
end;



constructor ParseException.create(_message: AnsiString; index: integer);
begin
   inherited createFmt(string(_message)+#13#10' error no caracter : %d',[index]);
end;



constructor _Integer.create(i: integer);
begin
  fvalue := i;
end;

function _Integer.clone: TQBJSonAbstractObject;
begin
  Result := _Integer.create (self.fvalue);
end;

constructor _Integer.create(s: AnsiString);
begin
  fvalue := strToInt(string(s));
end;

function _Integer.doubleValue: double;
begin
  Result := fvalue;
end;

function _Integer.intValue: integer;
begin
  Result := fvalue;
end;

class function _Integer.parseInt(s: AnsiString; i: integer): integer;
begin
  Result:=0;
  case i of
    10: Result := strToInt(string(s));
    16: Result := hexToInt(string(s));
    8:  begin
           if s='0' then
              exit;
           newNotImplmentedFeature () ;
        end;
    else
       newNotImplmentedFeature () ;
  end;
end;

class function _Integer.parseInt(s: _String): integer;
begin
  Result := _Integer.parseInt (s.toString, 10);
end;

class function _Integer.toHexString(c: AnsiChar): AnsiString;
begin
  Result := AnsiString(IntToHex(ord(c),2));
end;

function _Integer.toString: AnsiString;
begin
  Result := AnsiString(intToStr (fvalue));
end;



constructor _Double.create(s: AnsiString);
begin
  fvalue := StrToFloat(string(s));
end;

constructor _Double.create(s: _String);
begin
  create (s.toString);
end;

function _Double.clone: TQBJSonAbstractObject;
begin
  Result := _Double.create (Self.fvalue);
end;

constructor _Double.create(d: double);
begin
  fvalue := d;
end;

function _Double.doubleValue: double;
begin
  Result := fvalue;
end;

function _Double.intValue: integer;
begin
  Result := trunc (fvalue);
end;

class function _Double.NaN: double;
begin
  Result := 3.6e-4951;
end;

function _Double.toString: AnsiString;
begin
  Result := AnsiString(floatToStr(fvalue));
end;



constructor TQBJSonArray.create(x: TQBJSonTokener);
begin
  create;
  if (x.nextClean() <> '[') then
     raise x.syntaxError('A TQBJSonArray must start with "["');
  if (x.nextClean() = ']') then
     exit;
  x.back();
  while (true) do
     begin
        if (x.nextClean() = ',') then
           begin
              x.back();
              myArrayList.add(nil);
           end
        else
           begin
              x.back();
              myArrayList.add(x.nextValue());
           end;
        case (x.nextClean()) of
           ';',',':
              begin
                 if (x.nextClean() = ']') then
                    exit;
                 x.back();
              end;
           ']':
              exit;
           else
              raise x.syntaxError('Expected a "," or "]"');
        end;
     end;
end;

destructor TQBJSon.destroy;
var
  i :integer;
  MyObj:TObject;
begin
  for i:=Pred(myHashMap.Count) downto 0 do
     begin
        MyObj:=myHashMap.Objects[i];
        if (MyObj <> CONST_FALSE) and (MyObj <> CONST_TRUE) and (MyObj <> CNULL) then
           MyObj.Free;
     end;
  myHashMap.Free;
  inherited;
end;

constructor TQBJSonArray.create(collection: TList);
var
  i : integer;
begin
  myArrayList := TList.create ();
  for i := 0 to collection.count -1 do
     myArrayList.add (collection[i]);
end;

constructor TQBJSonArray.create;
begin
   myArrayList := TList.create;
end;

constructor TQBJSonArray.create(s: AnsiString);
var
  token: TQBJSonTokener;
begin
  token:=TQBJSonTokener.create(s);
  try
     create(token);
  finally
     token.free;
  end;
end;

constructor TQBJSonArray.Create(aStream: TStream; Bytes: integer);
var
   str: AnsiString;
begin
   setlength(str,bytes);
   aStream.ReadBuffer(str[1],bytes);
   create(str);
end;

constructor TQBJSonArray.Create (aFilename: String; FailIfNoFile: boolean);
var
   Stream: TMemoryStream;
begin
   if not fileexists(aFileName) then
      begin
         if FailIfNoFile then
            raise Exception.create('File not found!')
         else
            Create;
      end
   else
      begin
         Stream:=TMemoryStream.Create;
         try
            try
               Stream.LoadFromFile(aFileName);
               Stream.Position:=0;
               Create(Stream,Stream.Size);
            except
               raise Exception.create('Read from file error!');
            end;
         finally
            Stream.Free;
         end;
      end;
end;

constructor TQBJSonArray.create (aQBParcel: TQBParcel; GoodsName: AnsiString);
var
   Str: AnsiString;
begin
   str:=aQBParcel.GetAnsiStringGoods(GoodsName);
   Create(str);
end;

destructor TQBJSonArray.destroy;
var
  i : integer;
  obj : TObject;
begin
  for i:=Pred(myArrayList.Count) downto 0 do
     begin
        obj:=myArrayList[i];
        if (obj <> CONST_FALSE) and (obj <> CONST_TRUE) and (obj <> CNULL) then
           obj.Free;
     end;
  myArrayList.Free;
  inherited;
end;

function TQBJSonArray.Clone: TQBJSonAbstractObject;
begin
  Result:=TQBJSonArray.create(Self.toString);
end;

function TQBJSonArray.appendJSonArray(value: TQBJSonArray): Integer;
var
  i:Integer;
begin
  if value=nil then
     begin
        Result:=0;
        exit;
     end;
  Result:=value.length;
  for i:=0 to Pred(Result) do
    put(value.get(i).Clone);
end;

function TQBJSonArray.get(index: integer): TQBJSonAbstractObject;
var
  o : TQBJSonAbstractObject;
begin
  o := opt(index);
  if (o = nil) then
     raise NoSuchElementException.create('TQBJSonArray[' + intToStr(index) + '] not found.');
  Result := o;
end;

function TQBJSonArray.getBoolean(index: integer): boolean;
var
  o : TQBJSonAbstractObject;
begin
  o := get(index);
  if ((o.equals(_Boolean._FALSE) or ((o is _String) and (_String(o)).equalsIgnoreCase('false')))) then
     begin
        Result := false;
        exit;
     end
  else
     if ((o.equals(_Boolean._TRUE) or ((o is _String) and (_String(o)).equalsIgnoreCase('true')))) then
        begin
           Result := true;
           exit;
        end;
  raise ClassCastException.create('TQBJSonArray[' + intToStr(index) + '] not a Boolean.');
end;

function TQBJSonArray.getDouble(index: integer): double;
var
  o : TQBJSonAbstractObject;
  d : _Double;
begin
  o := get(index);
  if (o is _Number) then
     begin
        Result := (_Number(o)).doubleValue();
        exit;
     end;
  if (o is _String) then
     begin
        d :=  _Double.create(_String(o));
        try
           Result := d.doubleValue();
           exit;
        finally
           d.Free;
        end;
     end;
  raise NumberFormatException.create('TQBJSon[' + intToStr(index) + '] is not a number.');
end;

function TQBJSonArray.getInt(index: integer): integer;
var
  o : TQBJSonAbstractObject;
begin
  o := get(index);
  if (o is _Number) then
     Result := _Number(o).intValue()
  else
     Result := trunc (getDouble (index));
end;

function TQBJSonArray.getJSonArray(index: integer): TQBJSonArray;
var
  o: TQBJSonAbstractObject;
begin
  o := get(index);
  if (o is TQBJSonArray) then
     begin
        Result := TQBJSonArray(o);
        exit;
     end;
  raise NoSuchElementException.create('TQBJSonArray[' + intToStr(index) + '] is not a TQBJSonArray.');
end;

function TQBJSonArray.getJSon(index: integer): TQBJSon;
var
  o : TQBJSonAbstractObject;
  s : AnsiString;
begin
  o := get(index);
  if (o is TQBJSon) then
     Result := TQBJSon(o)
  else
     begin
        if o <> nil then
           s := AnsiString(o.ClassName)
        else
           s := 'nil';
        raise NoSuchElementException.create('TQBJSonArray[' + intToStr(index) + '] is not a TQBJSon is ' + string(s));
    end;
end;

function TQBJSonArray.getString(index: integer): AnsiString;
begin
  Result := get(index).toString();
end;

function TQBJSonArray.isNull(index: integer): boolean;
var
 o : TQBJSonAbstractObject;
begin
    o := opt(index);
    Result := (o = nil) or (o.equals(nil));
end;

function TQBJSonArray.join(separator: AnsiString): AnsiString;
var
  len, i : integer;
  sb : AnsiString ;
begin
		len := length();
    sb := '';
    for i := 0 to len -1 do
       begin
          if (i > 0) then
             sb := sb + separator;
          sb:= sb + TQBJSon.valueToString(TQBJSonAbstractObject(myArrayList[i]));
       end;
    Result := sb;
end;

function TQBJSonArray.length: integer;
begin
  Result := myArrayList.Count ;
end;

function TQBJSonArray.opt(index: integer): TQBJSonAbstractObject;
begin
    if ((index < 0) or (index >= length()) ) then
       Result := nil
    else
      Result := TQBJSonAbstractObject (myArrayList[index]);
end;

function TQBJSonArray.optBoolean(index: integer): boolean;
begin
  Result := optBoolean(index, false);
end;

function TQBJSonArray.optBoolean(index: integer; defaultValue: boolean): boolean;
var
 o : TQBJSonAbstractObject;
begin
  o := opt(index);
  if (o <> nil) then begin
      if ((o.equals(_Boolean._FALSE) or
              ((o is _String) and
              (_String(o)).equalsIgnoreCase('false')))) then begin
          Result := false;
          exit;
      end else if ((o.equals(_Boolean._TRUE) or
              ((o is _String) and
              (_String(o)).equalsIgnoreCase('true')))) then begin
          Result := true;
          exit;
      end;
  end;
  Result := defaultValue;
end;

function TQBJSonArray.optDouble(index: integer): double;
begin
   Result := optDouble(index, _Double.NaN);
end;

function TQBJSonArray.optDouble(index: integer; defaultValue :double): double;
var
 o : TQBJSonAbstractObject;
 d : _Double;
begin
  o := opt(index);
  if (o <> nil) then
     begin
        if (o is _Number) then
           begin
              Result := (_Number(o)).doubleValue();
              exit;
           end;
        try
           d := _Double.create (_String (o));
           Result := d.doubleValue ;
           d.Free;
           exit;
        except
           on e:Exception do
              begin
                 Result := defaultValue;
                 exit;
              end;
        end;
     end;
  Result := defaultValue;
end;

function TQBJSonArray.optInt(index: integer): integer;
begin
  Result := optInt(index, 0);
end;

function TQBJSonArray.optInt(index, defaultValue: integer): integer;
var
  o : TQBJSonAbstractObject;
begin
  o := opt(index);
  if (o <> nil) then
     begin
        if (o is _Number) then
           begin
              Result :=  (_Number(o)).intValue();
              exit;
           end;
        try
           Result := _Integer.parseInt(_String(o));
           exit;
        except
           on e: exception do
              begin
                 Result := defaultValue;
                 exit;
              end;
        end;
     end;
  Result := defaultValue;
end;

function TQBJSonArray.optJSonArray(index: integer): TQBJSonArray;
var
  o: TQBJSonAbstractObject;
begin
  o := opt(index);
  if (o is TQBJSonArray) then
     Result := TQBJSonArray (o)
  else
     Result := nil;
end;

function TQBJSonArray.optJSon(index: integer): TQBJSon;
var
  o: TQBJSonAbstractObject;
begin
  o:= opt(index);
  if (o is TQBJSon) then
     Result := TQBJSon (o)
  else
     Result := nil;
end;

function TQBJSonArray.optString(index: integer): AnsiString;
begin
  Result := optString(index, '');
end;

function TQBJSonArray.optString(index: integer; defaultValue: AnsiString): AnsiString;
var
  o : TQBJSonAbstractObject;
begin
  o := opt(index);
  if (o <> nil) then begin
     Result := o.toString();
  end else begin
     Result := defaultValue;
  end;
end;

procedure TQBJSonArray.put(value: boolean);
begin
  put(_Boolean.valueOf(value));
end;

procedure TQBJSonArray.put(value: double);
begin
    put(_Double.create(value));
end;

procedure TQBJSonArray.put(value: integer);
begin
  put(_Integer.create(value));
end;

procedure TQBJSonArray.put(value: AnsiString);
begin
    put(_String.create (value));
end;

procedure TQBJSonArray.put(value: TQBJSonAbstractObject);
begin
    myArrayList.add(value);
end;

procedure TQBJSonArray.put(index: integer; value: boolean);
begin
  put(index, _Boolean.valueOf(value));
end;

procedure TQBJSonArray.put(index, value: integer);
begin
  put(index, _Integer.create(value));
end;

procedure TQBJSonArray.put(index: integer; value: double);
begin
  put(index, _Double.create(value));
end;

procedure TQBJSonArray.put(index: integer; value: AnsiString);
begin
  put (index,_String.create (value));
end;

procedure TQBJSonArray.put(index: integer; value: TQBJSonAbstractObject);
begin
    if (index < 0) then
       raise NoSuchElementException.create('TQBJSonArray[' + intToStr(index) + '] not found.')
    else
       begin
          if (value = nil) then
             raise NullPointerException.create('')
          else
             if (index < length()) then
                myArrayList[index] := value
             else
                begin
                   while (index <> length()) do
                      put(nil);
                   put(value);
                end;
       end;
end;

function TQBJSonArray.toJSon(names :TQBJSonArray): TQBJSon;
var
  jo : TQBJSon ;
  i : integer;
begin
  if ((names = nil) or (names.length() = 0) or (length() = 0)) then
     begin
        Result := nil;
        exit;
     end;
  jo:= TQBJSon.create();
  for i := 0 to names.length() do
     jo.put(names.getString(i), self.opt(i));
  Result := jo;
end;

function TQBJSonArray.toString: AnsiString;
begin
   Result := '['+join(',')+']';
end;

function TQBJSonArray.toString2(indentFactor: integer): AnsiString;
begin
  Result := toString3(indentFactor, 0);
end;

function TQBJSonArray.toString3(indentFactor, indent: integer): AnsiString;
var
  len, i, j, newindent : integer;
  sb : AnsiString;
begin
  len := length();
  if (len = 0) then
     begin
        Result := '[]';
        exit;
     end;
  sb:= '[';
  if (len = 1) then
     sb := sb + TQBJSon.valueToString(TQBJSonAbstractObject( myArrayList[0]),indentFactor, indent)
  else
     begin
        newindent := indent + indentFactor;
        sb := sb + #13#10 ;
        for i := 0 to len -1 do
           begin
              if (i > 0) then
                 sb := sb +',' + #13#10;
              for j := 0 to newindent-1 do
                 sb := sb + ' ';
              sb := sb + (TQBJSon.valueToString(TQBJSonAbstractObject(myArrayList[i]), indentFactor, newindent));
           end;
        sb := sb + #13#10;
        for i := 0 to indent-1 do
           sb := sb + ' ';
     end;
  sb:= sb + ']';
  Result := sb;
end;

function TQBJSonArray.SaveToSteam(aStream: TStream; OutFormat: integer): boolean;
var
   Str: AnsiString;
begin
   if OutFormat=1 then
      Str:=Self.ToString
   else
      begin
         if OutFormat=2 then
            Str:=Self.ToString2(3)
         else
            Str:=Self.ToString3(3,3);
      end;
   aStream.WriteBuffer(Str[1],System.Length(Str));
   result:=true;
end;

function TQBJSonArray.SaveToFile(aFilename: AnsiString; OutFormat: integer): boolean;
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   try
      Self.SaveToSteam(Stream,OutFormat);
      Stream.Position:=0;
      Stream.SaveToFile(string(aFileName));
      result:=true;
   finally
      Stream.Free;
   end;
end;

function TQBJSonArray.SaveToParcel(aQBParcel: TQBParcel; GoodsName: AnsiString; OutFormat: integer): boolean;
var
   Str: AnsiString;
begin
   if OutFormat=1 then
      Str:=Self.ToString
   else
      begin
         if OutFormat=2 then
            Str:=Self.ToString2(3)
         else
            Str:=Self.ToString3(3,3);
      end;
   aQBParcel.PutAnsiStringGoods(GoodsName,str);
   result:=true;
end;




function _NULL.Equals(const Value: TQBJSonAbstractObject): Boolean;
begin
  if (value = nil) then
     Result := true
  else
     Result := (value is _NULL) ;
end;

function _NULL.toString: AnsiString;
begin
  Result := 'null';
end;



function TQBJSonAbstractObject.Clone: TQBJSonAbstractObject;
begin
  Result:=nil;
  newNotImplmentedFeature();
end;

function TQBJSonAbstractObject.Equals(const Value: TQBJSonAbstractObject): Boolean;
begin
  Result := ((value <> nil) and (value = self));
end;

function TQBJSonAbstractObject.Hash: LongInt;
begin
  Result := integer(addr(self));
end;

function TQBJSonAbstractObject.InstanceOf(const Value: TQBJSonAbstractObject): Boolean;
begin
  Result := (value is TQBJSonAbstractObject);
end;

function TQBJSonAbstractObject.ToString: AnsiString;
begin
  Result := AnsiString(Format('%s <%p>', [ClassName, addr(Self)]));
end;

initialization
  CONST_FALSE := _Boolean.create (false);
  CONST_TRUE := _Boolean.create (true);
  CNULL := _NULL.create;

finalization
  CONST_FALSE.free;
  CONST_TRUE.Free;
  CNULL.free;
end.
