//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//

unit QBMessage;

interface

uses
  Windows, Classes,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  Sysutils, QBCommon;

type
   TMsgFieldType=(mfByte,mfWord,mfInteger,mfReal,mfBoolean,mfAnsiString,mfString,mfBinary,mfInvalid);

type
   TQBMessage=Class
      private
         fPosition: integer;
         fFieldCount: integer;
         fMsgBody: AnsiString;
         procedure SetMsgBody(aBody: AnsiString);
         function GetBodySize: integer;

      public
         property Position: integer read fPosition write fPosition;
         property FieldCount: integer read fFieldCount write fFieldCount;
         property MsgBody: AnsiString read fMsgBody write SetMsgBody;
         property MsgBodySize: integer read GetBodySize;
         constructor Create;
         destructor Destroy; override;
         procedure Clear;
         procedure PutByteField(ByteValue: Byte);
         procedure PutWordField(WordValue: Word);
         procedure PutIntField(IntValue: integer);
         procedure PutRealField(RealValue: Real);
         procedure PutBoolField(BoolValue: Boolean);
         procedure PutAnsiField(AnsiValue: AnsiString);
         procedure PutStrField(StrValue: string);
         procedure PutBinField(BinValue: AnsiString);
         function GetByteField(var ByteValue: Byte): boolean;
         function GetWordField(var WordValue: Word): boolean;
         function GetIntField(var IntValue: integer): boolean;
         function GetRealField(var RealValue: real): boolean;
         function GetBoolField(var BoolValue: boolean): boolean;
         function GetAnsiField(var AnsiValue: AnsiString): boolean;
         function GetStrField(var StrValue: string): boolean;
         function GetBinField(var BinValue: AnsiString): boolean;
         function DeleteField(Index: integer): boolean;
         function MoveToField(Index: integer): boolean;
         function GetFieldType: TMsgFieldType;
         function GetFieldSize: integer;
   End;

implementation

constructor TQBMessage.Create;
begin
   fMsgBody:='';
   fPosition:=0;
   FieldCount:=0;
end;

destructor TQBMessage.Destroy;
begin
   SetLength(fMsgBody,0);
   fPosition:=0;
   FieldCount:=0;
   inherited;
end;

procedure TQBMessage.Clear;
begin
   SetLength(fMsgBody,0);
   fPosition:=0;
   FieldCount:=0;
end;

procedure TQBMessage.PutByteField(ByteValue: byte);
var
   FieldLength: word;
begin
   FieldLength:=1;
   fPosition:=Length(fMsgBody);
   SetLength(fMsgBody,fPosition+3+1);
   fMsgBody[fPosition+1]:=AnsiChar(1);
   move(FieldLength,fMsgBody[fPosition+2],2);
   move(ByteValue,fMsgBody[fPosition+4],FieldLength);
   inc(fFieldCount);
end;

procedure TQBMessage.PutWordField(WordValue: Word);
var
   FieldLength: word;
begin
   FieldLength:=2;
   fPosition:=Length(fMsgBody);
   SetLength(fMsgBody,fPosition+3+2);
   fMsgBody[fPosition+1]:=AnsiChar(2);
   move(FieldLength,fMsgBody[fPosition+2],2);
   move(WordValue,fMsgBody[fPosition+4],FieldLength);
   inc(fFieldCount);
end;

procedure TQBMessage.PutIntField(IntValue: integer);
var
   FieldLength: word;
begin
   FieldLength:=4;
   fPosition:=Length(fMsgBody);
   SetLength(fMsgBody,fPosition+3+FieldLength);
   fMsgBody[fPosition+1]:=AnsiChar(3);
   move(FieldLength,fMsgBody[fPosition+2],2);
   move(IntValue,fMsgBody[fPosition+4],FieldLength);
   inc(fFieldCount);
end;

procedure TQBMessage.PutRealField(RealValue: Real);
var
   FieldLength: word;
begin
   FieldLength:=Sizeof(Real);
   fPosition:=Length(fMsgBody);
   SetLength(fMsgBody,fPosition+3+FieldLength);
   fMsgBody[fPosition+1]:=AnsiChar(4);
   move(FieldLength,fMsgBody[fPosition+2],2);
   move(RealValue,fMsgBody[fPosition+4],FieldLength);
   inc(fFieldCount);
end;

procedure TQBMessage.PutBoolField(BoolValue: Boolean);
var
   FieldLength: word;
begin
   FieldLength:=Sizeof(Boolean);
   fPosition:=Length(fMsgBody);
   SetLength(fMsgBody,fPosition+3+FieldLength);
   fMsgBody[fPosition+1]:=AnsiChar(5);
   move(FieldLength,fMsgBody[fPosition+2],2);
   move(BoolValue,fMsgBody[fPosition+4],FieldLength);
   inc(fFieldCount);
end;

procedure TQBMessage.PutAnsiField(AnsiValue: AnsiString);
var
   FieldLength: word;
begin
   FieldLength:=length(AnsiValue);
   fPosition:=Length(fMsgBody);
   SetLength(fMsgBody,fPosition+3+FieldLength);
   fMsgBody[fPosition+1]:=AnsiChar(6);
   move(FieldLength,fMsgBody[fPosition+2],2);
   if FieldLength>0 then
      move(AnsiValue[1],fMsgBody[fPosition+4],FieldLength);
   inc(fFieldCount);
end;

procedure TQBMessage.PutStrField(StrValue: string);
var
   tmpstr: AnsiString;
   FieldLength: word;
begin
   tmpstr:=PackString(StrValue,WinCodePage);
   FieldLength:=length(tmpstr);
   fPosition:=Length(fMsgBody);
   SetLength(fMsgBody,fPosition+3+FieldLength);
   fMsgBody[fPosition+1]:=AnsiChar(7);
   move(FieldLength,fMsgBody[fPosition+2],2);
   if FieldLength>0 then
      move(tmpstr[1],fMsgBody[fPosition+4],FieldLength);
   inc(fFieldCount);
end;

procedure TQBMessage.PutBinField(BinValue: AnsiString);
var
   FieldLength: integer;
begin
   FieldLength:=length(BinValue);
   fPosition:=Length(fMsgBody);
   SetLength(fMsgBody,fPosition+5+FieldLength);
   fMsgBody[fPosition+1]:=AnsiChar(8);
   move(FieldLength,fMsgBody[fPosition+2],4);
   if FieldLength>0 then
      move(BinValue[1],fMsgBody[fPosition+6],FieldLength);
   inc(fFieldCount);
end;

function TQBMessage.GetByteField(var ByteValue: Byte): boolean;
var
   FieldLength: word;
begin
   if fFieldCount=0 then
      begin
         ByteValue:=0;
         result:=false;
         exit;
      end;
   move(fMsgBody[fPosition+2],FieldLength,2);
   move(fMsgBody[fPosition+4],ByteValue,1);
   Result:=(fMsgBody[fPosition+1]=AnsiChar(1));
   fPosition:=fPosition+FieldLength+3;
end;

function TQBMessage.GetWordField(var WordValue: Word): boolean;
var
   FieldLength: word;
begin
   if fFieldCount=0 then
      begin
         WordValue:=0;
         result:=false;
         exit;
      end;
   move(fMsgBody[fPosition+2],FieldLength,2);
   move(fMsgBody[fPosition+4],WordValue,FieldLength);
   Result:=(fMsgBody[fPosition+1]=AnsiChar(2));
   fPosition:=fPosition+FieldLength+3;
end;

function TQBMessage.GetIntField(var IntValue: integer): boolean;
var
   FieldLength: word;
begin
   if fFieldCount=0 then
      begin
         IntValue:=0;
         result:=false;
         exit;
      end;
   move(fMsgBody[fPosition+2],FieldLength,2);
   move(fMsgBody[fPosition+4],IntValue,FieldLength);
   Result:=(fMsgBody[fPosition+1]=AnsiChar(3));
   fPosition:=fPosition+FieldLength+3;
end;

function TQBMessage.GetRealField(var RealValue: real): boolean;
var
   FieldLength: word;
begin
   if fFieldCount=0 then
      begin
         RealValue:=0.0;
         result:=false;
         exit;
      end;
   move(fMsgBody[fPosition+2],FieldLength,2);
   move(fMsgBody[fPosition+4],RealValue,FieldLength);
   Result:=(fMsgBody[fPosition+1]=AnsiChar(4));
   fPosition:=fPosition+FieldLength+3;
end;

function TQBMessage.GetBoolField(var BoolValue: boolean): boolean;
var
   FieldLength: word;
begin
   if fFieldCount=0 then
      begin
         BoolValue:=false;
         result:=false;
         exit;
      end;
   move(fMsgBody[fPosition+2],FieldLength,2);
   move(fMsgBody[fPosition+4],BoolValue,FieldLength);
   Result:=(fMsgBody[fPosition+1]=AnsiChar(5));
   fPosition:=fPosition+FieldLength+3;
end;

function TQBMessage.GetAnsiField(var AnsiValue: AnsiString): boolean;
var
   FieldLength: word;
begin
   if fFieldCount=0 then
      begin
         AnsiValue:='';
         result:=false;
         exit;
      end;
   move(fMsgBody[fPosition+2],FieldLength,2);
   SetLength(AnsiValue,FieldLength);
   if FieldLength=0 then
      AnsiValue:=''
   else
      move(fMsgBody[fPosition+4],AnsiValue[1],FieldLength);
   Result:=(fMsgBody[fPosition+1]=AnsiChar(6));
   fPosition:=fPosition+FieldLength+3;
end;

function TQBMessage.GetStrField(var StrValue: String): boolean;
var
   tmpstr: AnsiString;
   FieldLength: word;
begin
   if fFieldCount=0 then
      begin
         StrValue:='';
         result:=false;
         exit;
      end;
   move(fMsgBody[fPosition+2],FieldLength,2);
   Result:=(fMsgBody[fPosition+1]=AnsiChar(7));
   if FieldLength<12 then
      begin
         StrValue:='';
         Result:=false;
      end
   else
      begin
         SetLength(tmpstr,FieldLength);
         move(fMsgBody[fPosition+4],tmpstr[1],FieldLength);
         StrValue:=UnpackString(tmpstr);
      end;
   fPosition:=fPosition+FieldLength+3;
end;

function TQBMessage.GetBinField(var BinValue: AnsiString): boolean;
var
   FieldLength: integer;
begin
   if fFieldCount=0 then
      begin
         BinValue:='';
         result:=false;
         exit;
      end;
   move(fMsgBody[fPosition+2],FieldLength,4);
   SetLength(BinValue,FieldLength);
   if FieldLength=0 then
      BinValue:=''
   else
      move(fMsgBody[fPosition+6],BinValue[1],FieldLength);
   Result:=(fMsgBody[fPosition+1]=AnsiChar(8));
   fPosition:=fPosition+FieldLength+5;
end;

procedure TQBMessage.SetMsgBody(aBody: AnsiString);
var
   l,FieldLength: word;
   BLength: integer;
   ok: boolean;
begin
   l:=length(aBody);
   fPosition:=0;
   fMsgBody:=aBody;
   fFieldCount:=0;
   ok:=true;
   while fPosition<length(fMsgBody) do
      begin
         if (fMsgBody[fPosition+1]<AnsiChar(1)) or (fMsgBody[fPosition+1]>AnsiChar(8)) then
            begin
               ok:=false;
               break;
            end;
         if fMsgBody[fPosition+1]=AnsiChar(8) then
            begin
               move(fMsgBody[fPosition+2],BLength,4);
               fPosition:=fPosition+BLength+5;
            end
         else
            begin
               move(fMsgBody[fPosition+2],FieldLength,2);
               fPosition:=fPosition+FieldLength+3;
            end;
         inc(fFieldCount);
         if fPosition>l then
            begin
               ok:=false;
               break;
            end;
      end;
   fPosition:=0;
   if not ok then
      begin
         fMsgBody:='';
         fFieldCount:=0;
      end;
end;

function TQBMessage.GetBodySize: integer;
begin
   Result:=length(fMsgBody);
end;

function TQBMessage.MoveToField(Index: integer): boolean;
var
   j,BLength: integer;
   FieldLength: word;
begin
   if (index<0) or (index>=fFieldCount) or (fFieldCount=0) then
      begin
         Result:=false;
         exit;
      end;
   fPosition:=0;
   j:=0;
   while j<index do
      begin
         if fMsgBody[fPosition+1]=AnsiChar(8) then
            begin
               move(fMsgBody[fPosition+2],BLength,4);
               fPosition:=fPosition+BLength+5;
            end
         else
            begin
               move(fMsgBody[fPosition+2],FieldLength,2);
               fPosition:=fPosition+FieldLength+3;
            end;
         inc(j);
      end;
   result:=true;
end;

function TQBMessage.GetFieldType: TMsgFieldType;
var
   t: byte;
begin
   if fFieldCount=0 then
      begin
         result:=mfInvalid;
         exit;
      end;
   move(fMsgBody[fPosition+1],t,1);
   case t of
      1: result:=mfByte;
      2: result:=mfWord;
      3: result:=mfInteger;
      4: result:=mfReal;
      5: result:=mfBoolean;
      6: result:=mfAnsiString;
      7: result:=mfString;
      8: result:=mfBinary;
      else
         result:=mfInvalid;
   end;
end;

function TQBMessage.GetFieldSize: integer;
begin
   result:=0;
   if fFieldCount=0 then
      exit;
   if fMsgBody[fPosition+1]=AnsiChar(8) then
      move(fMsgBody[fPosition+2],result,4)
   else
      move(fMsgBody[fPosition+2],result,2);
end;

function TQBMessage.DeleteField(Index: integer): boolean;
var
   j,h: integer;
   FieldLength: integer;
   ptr1,ptr2: pointer;
begin
   if (index<0) or (index>=fFieldCount) or (fFieldCount=0) then
      begin
         Result:=false;
         exit;
      end;
   fPosition:=0;
   j:=0;
   while j<index do
      begin
         if fMsgBody[fPosition+1]=AnsiChar(8) then
            begin
               move(fMsgBody[fPosition+2],FieldLength,4);
               fPosition:=fPosition+FieldLength+5;
            end
         else
            begin
               FieldLength:=0;
               move(fMsgBody[fPosition+2],FieldLength,2);
               fPosition:=fPosition+FieldLength+3;
            end;
         inc(j);
      end;
   FieldLength:=0;
   if fMsgBody[fPosition+1]=AnsiChar(8) then
      begin
         move(fMsgBody[fPosition+2],FieldLength,4);
         h:=5;
      end
   else
      begin
         move(fMsgBody[fPosition+2],FieldLength,2);
         h:=3;
      end;
   ptr1:=@fMsgBody[fPosition+1];
   ptr2:=pointer(integer(ptr1)+FieldLength+h);
   j:=length(fMsgBody)-fPosition-FieldLength-h;
   if j>0 then
      move(ptr2^,ptr1^,j);
   setLength(fMsgBody,length(fMsgBody)-FieldLength-h);
   dec(fFieldCount);
   result:=true;
end;

end.
