//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit QBRsa;

interface

uses
   Windows, SysUtils, Classes, QBMisc,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
   DllSpread;

type
   TRsaEncode = function(SourcePtr: integer; ExpPtr: integer; ModbPtr: integer): integer; stdcall;
   TRsaDecode = function(SourcePtr: integer; ExpPtr: integer; ModbPtr: integer): integer; stdcall;

type
   TQBRsa=class
   private
      ModuleHandle: dword;
      _RsaEncode: TRsaEncode;
      _RsaDecode: TRsaDecode;
   protected

   public
      constructor Create; overload;
      destructor Destroy; override;
      function Encrypt(Source: Ansistring; ExpStr: Ansistring; ModbStr: Ansistring): Ansistring;
      function Decrypt(Source: Ansistring; ExpStr: Ansistring; ModbStr: Ansistring): Ansistring;

end;

implementation

{$R QBRsa.res}

constructor TQBRsa.Create;
var
   RStream: TResourceStream;
   tmpdir: string;
   TmpData: Ansistring;
   Stream: TMemoryStream;
   i,DataSize: integer;
   x: Pointer;
begin
   inherited;
   try
      RStream:= TResourceStream.Create(hInstance,'RsaCoding', 'Binary');
   except
      raise exception.Create('Resource file error!');
      exit;
   end;
   try
      RStream.position:=0;
      DataSize:=RStream.size;
      setlength(TmpData,DataSize);
      RStream.ReadBuffer(TmpData[1],DataSize);
   finally
      RStream.free;
   end;
   for i:=1 to DataSize do
      TmpData[i]:=Ansichar(ord(TmpData[i]) xor (i mod 128));
   TmpDir:=QBMisc.GetSystemDir;
   if copy(tmpdir,length(tmpdir),1)<>'\' then
      tmpdir:=tmpdir+'\';
   Stream:=TMemoryStream.Create;
   Stream.Write(tmpData[1],length(tmpData));
   Stream.Position:=0;
   if fileexists(tmpdir+'qbrsacore.dll') then
      deletefile(tmpdir+'qbrsacore.dll');
   if not fileexists(tmpdir+'qbrsacore.dll') then
      Stream.SaveToFile(tmpdir+'qbrsacore.dll');
   ModuleHandle:=LoadLibrary(pchar(tmpdir+'qbrsacore.dll'));
   if ModuleHandle=0 then
      begin
         raise exception.Create('Resource file error!');
         exit;
      end;
   x:=GetProcAddress(ModuleHandle,'Rsa_Encrypt');
   if x=nil then
      begin
         FreeLibrary(ModuleHandle);
         raise exception.Create('Resource file error!');
         exit;
      end;
   _RsaEncode:=TRsaEncode(x);
   x:=GetProcAddress(ModuleHandle,'Rsa_Decrypt');
   if x=nil then
      begin
         FreeLibrary(ModuleHandle);
         raise exception.Create('Resource file error!');
         exit;
      end;
   _RsaDecode:=TRsaDecode(x);
end;

destructor TQBRsa.Destroy();
begin
   FreeLibrary(ModuleHandle);
   inherited;
end;

function TQBRsa.Encrypt(Source: Ansistring; ExpStr: Ansistring; ModbStr: Ansistring): Ansistring;
var
   SourcePtr,ExpPtr,ModbPtr,TargetPtr: integer;
begin
   SourcePtr:=Str2Mem(Source);
   ExpPtr:=Str2Mem(ExpStr);
   ModbPtr:=Str2Mem(ModbStr);
   TargetPtr:=_RsaEncode(SourcePtr,ExpPtr,ModbPtr);
   Result:=Mem2Str(TargetPtr);
   FreeMemory(SourcePtr);
   FreeMemory(ExpPtr);
   FreeMemory(ModbPtr);
   FreeMemory(TargetPtr);
end;

function TQBRsa.Decrypt(Source: Ansistring; ExpStr: Ansistring; ModbStr: Ansistring): Ansistring;
var
   SourcePtr,ExpPtr,ModbPtr,TargetPtr: integer;
begin
   SourcePtr:=Str2Mem(Source);
   ExpPtr:=Str2Mem(ExpStr);
   ModbPtr:=Str2Mem(ModbStr);
   TargetPtr:=_RsaDecode(SourcePtr,ExpPtr,ModbPtr);
   Result:=Mem2Str(TargetPtr);
   FreeMemory(SourcePtr);
   FreeMemory(ExpPtr);
   FreeMemory(ModbPtr);
   FreeMemory(TargetPtr);
end;

end.
