//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//

unit QBTimeSync;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, QBCommon,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  BurroSocket, QBTimer, StdCtrls;

type
   TOnStartSyncEvent = procedure (Sender: TObject) of Object;
   TOnStopSyncEvent = procedure (Sender: TObject; Successful: boolean) of Object;

type
  TQBTimeSync = class(TComponent)
  private
    fActive: boolean;                
    fTimeServerIP: Ansistring;       
    fTimeServerPort: integer;        
    fRetryInterval: integer;         
    fSyncInterval: integer;          
    fTimeDifference: integer;        
    fSyncDate: boolean;              
    fSyncTime: boolean;              
    fQueryMask: Ansistring;              
    fDateTimeString: Ansistring;         
    fOnStartSync: TOnStartSyncEvent;
    fOnStopSync: TOnStopSyncEvent;
    SyncSuccessful: boolean;         
    TimeSocket: TBClientSocket;      
    RetryTimer: TQBTimer;            
    SyncTimer: TQBTimer;             
    procedure TimeSocketRead(Sender: TObject; Socket: TBCustomWinSocket);
    procedure TimeSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure TimeSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
    procedure RetryTimerTimer(Sender: TObject);
    procedure SyncTimerTimer(Sender: TObject);
    procedure SetActive(aActive: boolean);
    procedure SetTimeServerIP(aServerIP: Ansistring);
    procedure SetTimeServerPort(aServerPort: integer);
    procedure SetRetryInterval(aRetryInterval: integer);
    procedure SetSyncInterval(aSyncInterval: integer);
    procedure SetTimeDifference(aTimeDifference: integer);
    procedure SetSyncDate(aSyncDate: boolean);
    procedure SetSyncTime(aSyncTime: boolean);
    procedure SetQueryMask(aQueryMask: Ansistring);

  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     procedure Open;
     procedure Close;

  published
     property Active: boolean read fActive write SetActive;
     property TimeServerIP: Ansistring read fTimeServerIP write SetTimeServerIP;
     property TimeServerPort: integer read fTimeServerPort write SetTimeServerPort;
     property RetryInterval: integer read fRetryInterval write SetRetryInterval;
     property SyncInterval: integer read fSyncInterval write SetSyncInterval;
     property TimeDifference: integer read fTimeDifference write SetTimeDifference;
     property SyncDate: boolean read fSyncDate write SetSyncDate;
     property SyncTime: boolean read fSyncTime write SetSyncTime;
     property QueryMask: Ansistring read fQueryMask write SetQueryMask;
     property DateTimeString: Ansistring read fDateTimeString;
     property OnStartSync: TOnStartSyncEvent read fOnStartSync write fOnStartSync;
     property OnStopSync: TOnStopSyncEvent read fOnStopSync write fOnStopSync;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Basic', [TQBTimeSync]);
end;

procedure TQBTimeSync.TimeSocketRead(Sender: TObject; Socket: TBCustomWinSocket);
var
   tmpstr: Ansistring;
   y,m,d,h,n,s,j: word;
   DT: TDateTime;
begin
   tmpstr:=socket.ReceiveText;
   fDateTimeString:=tmpstr;

   j:=pos(AnsiString('YY'),fQueryMask);
   if j<=0 then
      exit;
   try
      y:=2000+strtoint(string(copy(tmpstr,j,2)));
   except
      y:=2000;
   end;
   if y<=2000 then
      exit;
   j:=pos(AnsiString('MM'),fQueryMask);
   if j<=0 then
      exit;
   try
      m:=strtoint(string(copy(tmpstr,j,2)));
   except
      m:=0;
   end;
   if (m<=0) or (m>12) then
      exit;
   j:=pos(AnsiString('DD'),fQueryMask);
   if j<=0 then
      exit;
   try
      d:=strtoint(string(copy(tmpstr,j,2)));
   except
      d:=0;
      exit;
   end;
   if (d<=0) or (d>31) then
      exit;
   j:=pos(AnsiString('HH'),fQueryMask);
   if j<=0 then
      exit;
   try
      h:=strtoint(string(copy(tmpstr,j,2)));
   except
      h:=0;
      exit;
   end;
   if h>23 then
      exit;
   j:=pos(AnsiString('NN'),fQueryMask);
   if j<=0 then
      exit;
   try
      n:=strtoint(string(copy(tmpstr,j,2)));
   except
      n:=0;
      exit;
   end;
   if n>59 then
      exit;
   j:=pos(AnsiString('SS'),fQueryMask);
   if j<=0 then
      exit;
   try
      s:=strtoint(string(copy(tmpstr,j,2)));
   except
      s:=0;
      exit;
   end;
   if s>59 then
      exit;
   dt:=EncodeDate(y,m,d)+EncodeTime(h,n,s,0);
   DT:=DT+fTimeDifference/24;
   if not (fSyncDate and fSyncTime) then
      begin
         DecodeDate(dt,y,m,d);
         DecodeTime(dt,h,n,s,j);
         if fSyncDate then
            DecodeTime(now,h,n,s,j)
         else
            DecodeDate(now,y,m,d);
         dt:=EncodeDate(y,m,d)+EncodeTime(h,n,s,0);
      end;
   SyncSuccessful:=true;
   setLocalDateTime(DT);
end;

procedure TQBTimeSync.TimeSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
   errorcode:=0;
   if Socket.Connected then
      Socket.Close
   else
      TimeSocketDisconnect(Sender, Socket);
end;

procedure TQBTimeSync.TimeSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
begin
   if not fActive then
      exit;
   if Assigned(fOnStopSync) then
      fOnStopSync(self,SyncSuccessful);
   if SyncSuccessful then
      begin
         SyncTimer.Interval:=fSyncInterval*1000;
         SyncTimer.Enabled:=true;
      end
   else
      begin
         RetryTimer.Interval:=fRetryInterval*1000;
         RetryTimer.Enabled:=true;
      end;
end;

procedure TQBTimeSync.RetryTimerTimer(Sender: TObject);
begin
   RetryTimer.Enabled:=false;
   if Assigned(fOnStartSync) then
      fOnStartSync(self);
   TimeSocket.Host:=fTimeServerIp;
   TimeSocket.Port:=fTimeServerPort;
   TimeSocket.Active:=true;
end;

procedure TQBTimeSync.SyncTimerTimer(Sender: TObject);
begin
   SyncTimer.Enabled:=false;
   if Assigned(fOnStartSync) then
      fOnStartSync(self);
   TimeSocket.Host:=fTimeServerIp;
   TimeSocket.Port:=fTimeServerPort;
   TimeSocket.Active:=true;
end;

procedure TQBTimeSync.SetActive(aActive: boolean);
begin
   if fActive=aActive then
      exit;
   if aActive then
      begin
         fActive:=true;
         RetryTimerTimer(nil);
      end
   else
      begin
         fActive:=false;
         if TimeSocket.Active then
            TimeSocket.Close;
      end;
end;

procedure TQBTimeSync.SetTimeServerIP(aServerIP: Ansistring);
begin
   if uppercase(fTimeServerIP)=uppercase(trim(aServerIp)) then
      exit;
   fTimeServerIP:=trim(aServerIp);
end;

procedure TQBTimeSync.SetTimeServerPort(aServerPort: integer);
begin
   if fTimeServerPort=aServerPort then
      exit;
   if (aServerPort<=0) or (aServerPort>65535) then
      begin
         MessageBox(GetActiveWindow,'Invalid Port!','Invalid',0);
         exit;
      end;
   fTimeServerPort:=aServerPort;
end;

procedure TQBTimeSync.SetRetryInterval(aRetryInterval: integer);
begin
   if fRetryInterval=aRetryInterval then
      exit;
   if (aRetryInterval<1) or (aRetryInterval>60) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-60 seconds)','Invalid',0);
         exit;
      end;
   fRetryInterval:=aRetryInterval;
end;

procedure TQBTimeSync.SetSyncInterval(aSyncInterval: integer);
begin
   if fSyncInterval=aSyncInterval then
      exit;
   if (aSyncInterval<60) or (aSyncInterval>3600) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 60-3600 seconds)','Invalid',0);
         exit;
      end;
   fSyncInterval:=aSyncInterval;
end;

procedure TQBTimeSync.SetTimeDifference(aTimeDifference: integer);
begin
   if fTimeDifference=aTimeDifference then
      exit;
   if (aTimeDifference<-23) or (aTimeDifference>23) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be -23 to +23 hours)','Invalid',0);
         exit;
      end;
   fTimeDifference:=aTimeDifference;
end;

procedure TQBTimeSync.SetSyncDate(aSyncDate: boolean);
begin
   fSyncDate:=aSyncDate;
end;

procedure TQBTimeSync.SetSyncTime(aSyncTime: boolean);
begin
   fSyncTime:=aSyncTime;
end;

procedure TQBTimeSync.SetQueryMask(aQueryMask: Ansistring);
begin
   if fQueryMask=uppercase(trim(aQueryMask)) then
      exit;
   if pos(AnsiString('YY'),uppercase(aQueryMask))<=0 then
      begin
         messagebox(GetActiveWindow,'Invalid datetime mask string! (YY missed)','Invalid',0);
         exit;
      end;
   if pos(AnsiString('MM'),uppercase(aQueryMask))<=0 then
      begin
         messagebox(GetActiveWindow,'Invalid datetime mask string! (MM missed)','Invalid',0);
         exit;
      end;
   if pos(AnsiString('DD'),uppercase(aQueryMask))<=0 then
      begin
         messagebox(GetActiveWindow,'Invalid datetime mask string! (DD missed)','Invalid',0);
         exit;
      end;
   if pos(AnsiString('HH'),uppercase(aQueryMask))<=0 then
      begin
         messagebox(GetActiveWindow,'Invalid datetime mask string! (HH missed)','Invalid',0);
         exit;
      end;
   if pos(AnsiString('NN'),uppercase(aQueryMask))<=0 then
      begin
         messagebox(GetActiveWindow,'Invalid datetime mask string! (NN missed)','Invalid',0);
         exit;
      end;
   if pos(AnsiString('SS'),uppercase(aQueryMask))<=0 then
      begin
         messagebox(GetActiveWindow,'Invalid datetime mask string! (SS missed)','Invalid',0);
         exit;
      end;
   fQueryMask:=uppercase(trim(aQueryMask));
end;

Constructor TQBTimeSync.Create(AOwner: TComponent);
begin
   inherited;
   fActive:=false;                          
   fTimeServerIP:='time.nist.gov';          
   fTimeServerPort:=13;                     
   fRetryInterval:=5;                       
   fSyncInterval:=600;                      
   fTimeDifference:=8;                      
   fSyncDate:=true;                         
   fSyncTime:=true;                         
   fQueryMask:='.......YY.MM.DD.HH.NN.SS.'; 
   fDateTimeString:='';                     
   TimeSocket:=TBClientSocket.Create(nil);
   TimeSocket.ClientType:=ctNonBlocking;
   TimeSocket.OnRead:=TimeSocketRead;
   TimeSocket.OnError:=TimeSocketError;
   TimeSocket.OnDisconnect:=TimeSocketDisconnect;
   RetryTimer:=TQBTimer.Create(nil);
   RetryTimer.Enabled:=false;
   RetryTimer.Interval:=fRetryInterval*1000;
   RetryTimer.OnTimer:=RetryTimerTimer;
   SyncTimer:=TQBTimer.Create(nil);
   SyncTimer.Enabled:=false;
   SyncTimer.Interval:=fSyncInterval*1000;
   SyncTimer.OnTimer:=SyncTimerTimer;
   SyncSuccessful:=false;
end;

Destructor TQBTimeSync.Destroy();
begin
   if fActive then
      SetActive(false);
   SyncTimer.Free;
   RetryTimer.Free;
   TimeSocket.Free;
   inherited;
end;

procedure TQBTimeSync.Open;
begin
   SetActive(true);
end;

procedure TQBTimeSync.Close;
begin
   SetActive(false);
end;

end.

