//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit QBTimer;
    
interface
    
uses
   Windows, Messages, SysUtils, Classes, Controls, Dialogs, MMSystem;
    
type
   TTimerEvent = procedure (Sender: TObject) of object;
    
   TQBTimer = class(TComponent)
   private
      
      FTimerID: DWORD;
      FActive: Boolean;
      FPeriodMax: DWORD;
      FPeriodMin: DWORD;
      FInterval: DWORD;
      FTimeCaps: TTimeCaps;
      FResolution: DWORD;
      FOnTimer: TTimerEvent;
      FIndex: Integer;
      procedure StartTimer;
      procedure TerminateTimer;
      procedure SetActive(Value: Boolean);
      procedure SetInterval(Value: DWORD);
      procedure SetResolution(Value: DWORD);

   protected
      
      procedure Timer; dynamic;
   public
      
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
   published
      
      property Enabled: Boolean read FActive write SetActive;
      property Interval: DWORD read FInterval write SetInterval;
      property Resolution: DWORD read FResolution write SetResolution;
      property OnTimer: TTimerEvent read FOnTimer write FOnTimer;
   end;
    
   procedure Register;
    
implementation

procedure   Register;
begin
   RegisterComponents('QuickBurro Basic', [TQBTimer]);
end;
    
type
   TPointerList = array of Pointer;

var
   gMMTimerList: TPointerList;
   gMMTimerListCount: DWORD;

procedure TimeProc(uID: DWORD; uMsg: DWORD; dwUser: DWORD; dw1: DWORD; dw2: DWORD);stdcall;
begin
   TQBTimer(gMMTimerList[dwUser]).Timer;
end;

constructor TQBTimer.Create(AOwner: TComponent);
begin
   inherited;
   FActive := False;
   FInterval := 1000;
   Inc(gMMTimerListCount);
   SetLength(gMMTimerList, gMMTimerListCount);
   FIndex := gMMTimerListCount - 1;
   gMMTimerList[FIndex] := Self;
   if timeGetDevCaps(@FTimeCaps, SizeOf(TimeCaps)) = TIMERR_NOERROR then
      begin
          FPeriodMax := FTimeCaps.wPeriodMax;
          FPeriodMin := FTimeCaps.wPeriodMin;
          FResolution := FPeriodMin;
      end;
end;

destructor TQBTimer.Destroy;
begin
   if FActive then
      TerminateTimer;
   inherited;
end;

procedure TQBTimer.SetActive(Value: Boolean);
begin
   if Value <> FActive then
      begin
         FActive := Value;
         if Value then
            StartTimer
         else
            TerminateTimer;
      end;
end;

procedure TQBTimer.SetInterval(Value: DWORD);
begin
   if Value <> FInterval then
      begin
          FInterval := Value;
          if FActive then
          begin
              TerminateTimer;
              StartTimer;
          end;
      end;
end;

procedure TQBTimer.SetResolution(Value: DWORD);
begin
   if Value <> FResolution then
      if Value in [FPeriodMin..FPeriodMax] then
          begin   
              TerminateTimer;   
              FResolution := Value;
              StartTimer;   
          end;   
end;

procedure TQBTimer.StartTimer;
begin
   if not (csDesigning in ComponentState) then
      begin
          timeBeginPeriod(FResolution);
          FTimerID := timeSetEvent(FInterval, FResolution, @TimeProc, FIndex, TIME_PERIODIC);
      end;
end;

procedure TQBTimer.TerminateTimer;
begin
   if FTimerID <> 0 then
      begin
          timeKillEvent(FTimerID);
          timeEndPeriod(FResolution);
      end;
end;

procedure TQBTimer.Timer;
begin
   if Assigned(FOnTimer) then
      FOnTimer(Owner);
end;

initialization

finalization
   gMMTimerList := nil;

end.
