//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit QuickUDP;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
  Windows, SysUtils, classes, WinSock,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  SyncObjs;

type
  TUDPReceiveEvent = procedure(Sender: TObject; const Buffer: Pointer; const RecvSize: Integer; const FromIP: Ansistring; const Port: Integer) of object;
  TUDPClient = class(TComponent)
  private
    FHandle: TSocket;
    FActive: Boolean;
    FRemoteIP: Ansistring;
    FRemoteHost: Ansistring;
    FRemotePort: Word;
    CS: TCriticalSection;
    procedure SetActive(const Value: Boolean);
    procedure SetRemoteIP(const Value: Ansistring);
    procedure SetRemoteHost(const Value: Ansistring);
    procedure SetRemotePort(const Value: Word);
  protected
  public
    class function ResolveHost(const psHost: Ansistring; var psIP: Ansistring): u_long; virtual;
    class function ResolveIP(const psIP: Ansistring): Ansistring; virtual;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Connect;
    procedure Disconnect;
    function SendBuf(var Buffer; BufSize: Integer): Integer;
    property Handle: TSocket read FHandle;
  published
    property Active: Boolean read FActive write SetActive default False;
    property RemoteIP: Ansistring read FRemoteIP write SetRemoteIP;
    property RemoteHost: Ansistring read FRemoteHost write SetRemoteHost;
    property RemotePort: Word read FRemotePort write SetRemotePort;
  end;

  TUDPServer = class;

  TUDPServerThread = class(TThread)
  protected
    FReceiver: TUDPServer;
    FBuffer: Pointer;
    FRecvSize: Integer;
    FPeer: Ansistring;
    FPort: Integer;
    FBufSize: Integer;
    procedure SetBufSize(const Value: Integer);
  public
    procedure Execute; override;
    procedure UDPRead;
    property BufSize: Integer read FBufSize write SetBufSize;
    property Receiver: TUDPServer read FReceiver write FReceiver;
  end;

  TUDPServer = class(TComponent)
  private
    FHandle: TSocket;
    FActive: Boolean;
    FPort: Word;
    FBufferSize: Integer;
    FMulticastIP: Ansistring;
    FOnUDPData: TUDPReceiveEvent;
    FUDPServerThread: TUDPServerThread;
    procedure SetActive(const Value: Boolean);
    procedure SetPort(const Value: Word);
    procedure SetBufferSize(const Value: Integer);
    procedure SetMulticastIP(const Value: Ansistring);
  protected
  public
    class function BindMulticast(const Socket: TSocket; const IP: Ansistring): Longint; virtual;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Connect;
    procedure Disconnect;
    procedure DoUDPRead(const Buffer: Pointer; const RecvSize: Integer; const FromIP: Ansistring; const Port: Integer); virtual;
    property Handle: TSocket read FHandle;
  published
    property Active: Boolean read FActive write SetActive default False;
    property Port: Word read FPort write SetPort;
    property BufferSize: Integer read FBufferSize write SetBufferSize default 65000;
    property OnUDPData: TUDPReceiveEvent read FOnUDPData write FOnUDPData;
    property MulticastIP: Ansistring read FMulticastIP write SetMulticastIP;
  end;

type
  EBufferUDP = Exception;

procedure Register;

resourcestring
  EUDPNOTACTIVE = 'UDP Socket not connected';
  EUDPACTIVED = 'UDP Socket already connected';
  EWSAError = 'Socket Error : %d';
  EUNABLERESOLVEHOST = 'Unable to resolve host: %s';
  EUNABLERESOLVEIP = 'Unable to resolve IP: %s';
  EZEROBYTESend = '0 bytes were sent.';
  EPACKAGETOOBIG = 'Package Size Too Big: %d';
  ENOREMOTESIDE = 'Remote Host/IP not identified!';
  ESIZEOUTOFBOUNDARY = 'Size value is out of boundary!';
  EWSAENOBUFS = 'An operation on a socket could not be performed because the system lacked sufficient buffer space or because a queue was full.';
  EWSANOTINITIALISED = 'A successful WSAStartup must occur before using this function.';
  EWSAENETDOWN = 'The network subsystem has failed.';
  EWSAEFAULT = 'optval is not in a valid part of the process address space or optlen argument is too small.';
  EWSAEINPROGRESS = 'A blocking Windows Sockets 1.1 call is in progress, or the service provider is still processing a callback function.';
  EWSAEINVAL = 'level is not valid, or the information in optval is not valid.';
  EWSAENETRESET = 'Connection has timed out when SO_KEEPALIVE is set.';
  EWSAENOPROTOOPT = 'The option is unknown or unsupported for the specified provider.';
  EWSAENOTCONN = 'Connection has been reset when SO_KEEPALIVE is set.';
  EWSAENOTSOCK = 'The descriptor is not a socket.';
  EWSAUNKNOW = 'Unknow socket error.';
implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Basic', [TUDPClient, TUDPServer]);
end;

type
  TIMR = packed record
    imr_multiaddr: Longint;
    imr_interface: Longint;
  end;

{ TUDPClient }

procedure TUDPClient.Connect;
var
  Faddr: TSockAddrIn;
begin
  CS.Enter;
  try
    if FActive then
      raise EBufferUDP.CreateRes(@EUDPACTIVED);
    if ((FRemoteHost = '') and (FRemoteIP = '')) then
      raise EBufferUDP.CreateRes(@ENOREMOTESIDE);
    if not (csDesigning in ComponentState) then
    begin
      FHandle := WinSock.Socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
      if FHandle = INVALID_SOCKET then
        raise EBufferUDP.CreateResFmt(@EWSAError, [WSAGetLastError]);
      with Faddr do
      begin
        sin_family := PF_INET;
        sin_port := WinSock.htons(FRemotePort);
        if length(FRemoteIP) > 0 then
        begin
          sin_addr.s_addr := WinSock.inet_addr(PAnsiChar(FRemoteIP));
        end;
      end;
      WinSock.Connect(FHandle, Faddr, Sizeof(Faddr));
    end;
    FActive := True;
  finally
    CS.Leave;
  end;
end;

constructor TUDPClient.Create(AOwner: TComponent);
begin
  inherited;
  CS := TCriticalSection.Create;
  FActive := False;
  FHandle := INVALID_SOCKET;
end;

destructor TUDPClient.Destroy;
begin
  if FActive then
    CloseSocket(fHandle);
  Active := False;
  CS.Free;
  inherited;
end;

procedure TUDPClient.Disconnect;
var
  OldHandle: TSocket;
begin
  CS.Enter;
  try
    if FActive then
    begin
      OldHandle := FHandle;
      FHandle := INVALID_SOCKET;
      CloseSocket(OldHandle);
    end;
  finally
    FActive := False;
    CS.Leave;
  end;
end;

class function TUDPClient.ResolveHost(const psHost: Ansistring;
  var psIP: Ansistring): u_long;
var
  pa: PAnsiChar;
  sa: TInAddr;
  aHost: PHostEnt;
begin
  psIP := psHost;
  if AnsiCompareText(psHost, AnsiString('LOCALHOST')) = 0 then
  begin
    sa.S_un_b.s_b1 := #127;
    sa.S_un_b.s_b2 := #0;
    sa.S_un_b.s_b3 := #0;
    sa.S_un_b.s_b4 := #1;
    psIP := '127.0.0.1';
    Result := sa.s_addr;
  end
  else
  begin
    Result := inet_addr(PAnsiChar(psHost));
    if Result = u_long(INADDR_NONE) then
    begin
      aHost := WinSock.GetHostByName(PAnsiChar(psHost));
      if aHost = nil then
      begin
        Result := 0;
        psIP := '';
        Exit;
      end
      else
      begin
        pa := aHost^.h_addr_list^;
        sa.S_un_b.s_b1 := pa[0];
        sa.S_un_b.s_b2 := pa[1];
        sa.S_un_b.s_b3 := pa[2];
        sa.S_un_b.s_b4 := pa[3];
        psIP := Ansistring(inet_ntoa(sa));
      end;
      Result := sa.s_addr;
    end;
  end;
end;

class function TUDPClient.ResolveIP(const psIP: Ansistring): Ansistring;
var
  i: Integer;
  P: PHostEnt;
begin
  Result := '';
  if AnsiCompareText(psIP, AnsiString('127.0.0.1')) = 0 then
  begin
    Result := 'LOCALHOST';
  end
  else
  begin
    i := WinSock.inet_addr(PAnsiChar(psIP));
    P := WinSock.GetHostByAddr(@i, 4, PF_INET);
    if P = nil then
    begin
      Result := '';
      Exit;
    end
    else
    begin
      Result := P.h_name;
    end;
  end;
end;

function TUDPClient.SendBuf(var Buffer; BufSize: Integer): Integer;
begin
  CS.Enter;
  try
    Result := 0;
    if BufSize <= 0 then
      Exit;
    if not FActive then
      raise EBufferUDP.CreateRes(@EUDPNOTACTIVE);
    Result := WinSock.send(FHandle, Buffer, BufSize, 0);
    if Result <> BufSize then
    begin
      case Result of
        0:
          raise EBufferUDP.CreateRes(@EZEROBYTESend);
        SOCKET_ERROR:
          if WSAGetLastError = WSAEMSGSIZE then
            raise EBufferUDP.CreateResFmt(@EPACKAGETOOBIG, [BufSize])
      end; 
    end;
  finally
    CS.Leave;
  end;
end;

procedure TUDPClient.SetActive(const Value: Boolean);
begin
  if FActive <> Value then
  begin
    if Value then
      Connect
    else
      Disconnect;
  end;
end;

procedure TUDPClient.SetRemoteHost(const Value: Ansistring);
var
  IsConnected: Boolean;
begin
  if FRemoteHost <> Value then
  begin
    IsConnected := Active;
    Active := False;
    FRemoteHost := Value;
    if not (csDesigning in ComponentState) then
      ResolveHost(FRemoteHost, FRemoteIP);
    Active := IsConnected;
  end;
end;

procedure TUDPClient.SetRemoteIP(const Value: Ansistring);
var
  IsConnected: Boolean;
begin
  if FRemoteIP <> Value then
  begin
    IsConnected := Active;
    Active := False;
    FRemoteIP := Value;
    if not (csDesigning in ComponentState) then
      FRemoteHost := ResolveIP(FRemoteIP);
    Active := IsConnected;
  end;
end;

procedure TUDPClient.SetRemotePort(const Value: Word);
var
  IsConnected: Boolean;
begin
  if FRemotePort <> Value then
  begin
    IsConnected := Active;
    Active := False;
    FRemotePort := Value;
    Active := IsConnected;
  end;
end;

{ TUDPServer }

class function TUDPServer.BindMulticast(const Socket: TSocket;
  const IP: Ansistring): Longint;
var
  lpMulti: TIMR;
begin
  lpMulti.imr_multiaddr := inet_addr(PAnsiChar(IP));
  lpMulti.imr_interface := 0;
  Result := SetSockOpt(Socket, IPPROTO_IP, IP_ADD_MEMBERSHIP, @lpMulti, Sizeof(lpMulti));
end;

procedure TUDPServer.Connect;
var
  m_addr: TSockAddrIn;
begin
  if FActive then
    raise EBufferUDP.CreateRes(@EUDPACTIVED);

  if csDesigning in ComponentState then
     begin
       FActive := True;
       Exit;
     end;
  FHandle := WinSock.Socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
  if FHandle = INVALID_SOCKET then
    raise EBufferUDP.CreateResFmt(@EWSAError, [WSAGetLastError]);
  with m_addr do
     begin
       sin_family := PF_INET;
       sin_port := WinSock.htons(FPort);
       sin_addr.s_addr := INADDR_ANY;
     end;
  if WinSock.bind(FHandle, m_addr, Sizeof(m_addr)) = SOCKET_ERROR then
    raise EBufferUDP.CreateResFmt(@EWSAError, [WSAGetLastError]);
  if FMulticastIP <> '' then
    if BindMulticast(FHandle, FMulticastIP) = SOCKET_ERROR then
      case WSAGetLastError of
        WSAENOBUFS: raise EBufferUDP.CreateRes(@EWSAENOBUFS);
        WSANOTINITIALISED: raise EBufferUDP.CreateRes(@EWSANOTINITIALISED);
        WSAENETDOWN: raise EBufferUDP.CreateRes(@EWSAENETDOWN);
        WSAEFAULT: raise EBufferUDP.CreateRes(@EWSAEFAULT);
        WSAEINPROGRESS: raise EBufferUDP.CreateRes(@EWSAEINPROGRESS);
        WSAEINVAL: raise EBufferUDP.CreateRes(@EWSAEINVAL);
        WSAENETRESET: raise EBufferUDP.CreateRes(@EWSAENETRESET);
        WSAENOPROTOOPT: raise EBufferUDP.CreateRes(@EWSAENOPROTOOPT);
        WSAENOTCONN: raise EBufferUDP.CreateRes(@EWSAENOTCONN);
        WSAENOTSOCK: raise EBufferUDP.CreateRes(@EWSAENOTSOCK);
      else
        raise EBufferUDP.CreateRes(@EWSAUNKNOW);
      end;
  FUDPServerThread := TUDPServerThread.Create(true);
  with FUDPServerThread do
     begin
       Receiver := Self;
       BufSize := FBufferSize;
       FreeOnTerminate := True;
       Resume;
     end;
  FActive := True;
end;

constructor TUDPServer.Create(AOwner: TComponent);
begin
  inherited;
  FHandle := INVALID_SOCKET;
  FActive := False;
  FBufferSize := 65000;
  FMulticastIP := '';
end;

destructor TUDPServer.Destroy;
begin
  if FActive then
    CloseSocket(fHandle);
  Active := False;
  inherited;
end;

procedure TUDPServer.Disconnect;
var
  OldHandle: TSocket;
begin
  if not FActive then
    Exit;
  try
    OldHandle := FHandle;
    FHandle := INVALID_SOCKET;
    CloseSocket(OldHandle);
  finally
    FActive := False;
  end;
  if FUDPServerThread <> nil then
     begin
         FUDPServerThread.Terminate;
         WaitForSingleObject(FUDPServerThread.Handle,500);
     end;
end;

procedure TUDPServer.DoUDPRead(const Buffer: Pointer; const RecvSize: Integer;
  const FromIP: Ansistring; const Port: Integer);
begin
  if Assigned(FOnUDPData) then
     FOnUDPData(Self, Buffer, RecvSize, FromIP, Port);
end;

procedure TUDPServer.SetActive(const Value: Boolean);
begin
  if FActive <> Value then
     begin
       if Value then
         Connect
       else
         Disconnect;
     end;
end;

procedure TUDPServer.SetBufferSize(const Value: Integer);
begin
  if FBufferSize <> Value then
     begin
       if ((Value >= 1024) and (Value <= 65000)) then
         FBufferSize := Value
       else
         raise EBufferUDP.CreateRes(@ESIZEOUTOFBOUNDARY);
     end;
end;

procedure TUDPServer.SetMulticastIP(const Value: Ansistring);
var
  IsConnected: Boolean;
begin
  if Value <> FMulticastIP then
     begin
       IsConnected := Active;
       Active := False;
       FMulticastIP := Value;
       Active := IsConnected;
     end;
end;

procedure TUDPServer.SetPort(const Value: Word);
var
  IsConnected: Boolean;
begin
  if FPort <> Value then
     begin
       IsConnected := Active;
       Active := False;
       FPort := Value;
       Active := IsConnected;
     end;
end;

{ TUDPServerThread }

procedure TUDPServerThread.Execute;
var
  i: Integer;
  addr_remote: TSockAddrIn;
  arSize: Integer;
begin
  GetMem(FBuffer, FBufSize);
  arSize := Sizeof(addr_remote);
  while FReceiver.Active and not Terminated do
     begin
       i := arSize;
       try
          FRecvSize := WinSock.RecvFrom(FReceiver.Handle, FBuffer^, FBufSize, 0, addr_remote, i);
          if FReceiver.Active and (FRecvSize > 0) then
             begin
                FPeer := Ansistring(inet_ntoa(addr_remote.sin_addr));
                FPort := WinSock.NToHS(addr_remote.sin_port);
                Synchronize(UDPRead);
             end;
       except
          continue;
       end;
     end;
  FreeMem(FBuffer);
end;

procedure TUDPServerThread.SetBufSize(const Value: Integer);
begin
  if FBufSize <> Value then
    FBufSize := Value;
end;

procedure TUDPServerThread.UDPRead;
begin
   FReceiver.DoUDPRead(FBuffer, FRecvSize, FPeer, FPort);
end;

var
  GWSADATA: TWSADATA;
initialization
   {$IFDEF VER130}
   WSAStartup($0101, GWSADATA);
   {$ELSE}
   WSAStartup(MAKEWORD(2,0), GWSADATA);
   {$ENDIF}
finalization
   WSACleanup;
end.

