//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.5.4.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit RemoteBinFile;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  UserConnection, QBParcel, TaskCommon, QBZlibExGz;

type
  TRemoteBinFile = class(TComponent)
  private
     fUserConn: TUserConnection;                
     fTaskTimeout: integer;                     
     fDllPassword: AnsiString;                  
     fTargetNodeId: AnsiString;                 
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetUserConn(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetDllPassword(Password: AnsiString);
     procedure SetTargetNodeId(NodeId: AnsiString);
     function BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;           
     Destructor Destroy(); override;                             
     function FileCreate(aFileName: string; var FileHandle: integer): boolean;
     function FileOpen(aFileName: string; OpenMode: cardinal; var FileHandle: integer): boolean;
     function FileSeek(aHandle: integer; Offset: integer; Origin: integer): boolean;
     function FileRead(aHandle: integer; bytes: integer; var BlockData: AnsiString): boolean;
     function FileWrite(aHandle: integer; BlockData: AnsiString): boolean;
     function FileClose(aHandle: integer): boolean;

  published
     property UserConn: TUserConnection read fUserConn write SetUserConn;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;            
     property DllPassword: AnsiString read fDllPassword write SetDllPassword;             
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;          

  end;

procedure Register;

implementation

procedure Register;
begin
   RegisterComponents('QuickBurro Standard', [TRemoteBinFile]);
end;

constructor TRemoteBinFile.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;                  
   fTargetNodeId:='';
end;

destructor TRemoteBinFile.Destroy;
begin
   inherited;
end;

procedure TRemoteBinFile.SetUserConn(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TRemoteBinFile.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TRemoteBinFile.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TRemoteBinFile.SetDllPassword(Password: AnsiString);
begin
   if fDllPassword=trim(password) then
      exit;
   fDllPassword:=password;
end;

procedure TRemoteBinFile.SetTargetNodeId(NodeId: AnsiString);      
begin
   if fTargetNodeId=NodeId then
      exit;
   fTargetNodeId:=NodeId;
end;

function TRemoteBinFile.BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream,Stream1: TMemoryStream;
   ok: boolean;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Stream1:=TMemoryStream.Create;
   try
      ok:=InParcel.SaveToStream(Stream);
      if ok then
         begin
            Stream.Position:=0;
            GZCompressStream(Stream,Stream1);
            ok:=(Stream1.size>0);
         end;
   except
      ok:=false;
   end;
   if not ok then
      begin
         Stream.Free;
         Stream1.Free;
         result:=false;
         exit;
      end;
   Stream1.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',53);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   Parcel.PutStreamGoods('InParcelStream',Stream1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('OutParcelStream') then
                        begin
                           Stream.Clear;
                           Stream1.Clear;
                           try
                              Parcel.GetStreamGoods('OutParcelStream',Stream);
                              GZDecompressStream(Stream,Stream1);
                              Stream1.Position:=0;
                              OutParcel.Clear;
                              result:=OutParcel.LoadFromStream(Stream1);
                           except
                              result:=false;
                           end;
                        end
                     else
                        Result:=true;
                  end;
            end;
         Parcel.Free;
      end;
   Stream.Free;
   Stream1.Free;
end;

function TRemoteBinFile.FileCreate(aFileName: string; var FileHandle: integer): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteBinFile');
   InParcel.PutIntegerGoods('FunctionId',1);
   InParcel.PutStringGoods('FileName',aFilename);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteBinFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         if ok then
            FileHandle:=OutParcel.GetIntegerGoods('FileHandle')
         else
            FileHandle:=-1;
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteBinFile.FileOpen(aFileName: string; OpenMode: cardinal; var FileHandle: integer): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteBinFile');
   InParcel.PutIntegerGoods('FunctionId',2);
   InParcel.PutStringGoods('FileName',aFilename);
   InParcel.PutLongWordGoods('OpenMode',OpenMode);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteBinFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         if ok then
            FileHandle:=OutParcel.GetIntegerGoods('FileHandle')
         else
            FileHandle:=-1;
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteBinFile.FileSeek(aHandle: integer; Offset: integer; Origin: integer): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteBinFile');
   InParcel.PutIntegerGoods('FunctionId',3);
   InParcel.PutIntegerGoods('FileHandle',aHandle);
   InParcel.PutIntegerGoods('Offset',Offset);
   InParcel.PutIntegerGoods('Origin',Origin);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteBinFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteBinFile.FileRead(aHandle: integer; bytes: integer; var BlockData: AnsiString): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteBinFile');
   InParcel.PutIntegerGoods('FunctionId',4);
   InParcel.PutIntegerGoods('FileHandle',aHandle);
   InParcel.PutIntegerGoods('ReadBytes',bytes);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteBinFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         if ok then
            BlockData:=OutParcel.GetBinaryGoods('FileBlock')
         else
            BlockData:='';
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteBinFile.FileWrite(aHandle: integer; BlockData: AnsiString): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteBinFile');
   InParcel.PutIntegerGoods('FunctionId',5);
   InParcel.PutIntegerGoods('FileHandle',aHandle);
   InParcel.PutBinaryGoods('FileBlock',BlockData);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteBinFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteBinFile.FileClose(aHandle: integer): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteBinFile');
   InParcel.PutIntegerGoods('FunctionId',6);
   InParcel.PutIntegerGoods('FileHandle',aHandle);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteBinFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

end.
