//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.25.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit RemoteDesktop;

interface

uses
  Windows, Messages, SysUtils, Classes, ExtCtrls,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  UserConnection, QBParcel, QBZlibExGz, TaskCommon;

type
  TRemoteDesktop = class(TComponent)
  private
     fUserConn: TUserConnection;                
     fTaskTimeout: integer;                     
     fDllPassword: AnsiString;                  
     fTargetNodeId: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetDllPassword(Password: AnsiString);
     procedure SetTargetNodeId(aNodeId: AnsiString);
     function BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;

  protected
     procedure Notification(AComponent: TComponent;Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;           
     Destructor Destroy(); override;                             
     function GetDesktopScreen(var ScreenImage: TImage): boolean;
     procedure MouseLeftClick(PosX: integer; PosY: integer; MaxX: integer; MaxY: integer);
     procedure MouseRightClick(PosX: integer; PosY: integer; MaxX: integer; MaxY: integer);
     procedure MouseDoubleClick(PosX: integer; PosY: integer; MaxX: integer; MaxY: integer);

  published
     property UserConn: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;          
     property DllPassword: AnsiString read fDllPassword write SetDllPassword;             
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;

  end;

procedure Register;

implementation

procedure Register;
begin
   RegisterComponents('QuickBurro Standard', [TRemoteDesktop]);
end;

constructor TRemoteDesktop.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;                  
end;

destructor TRemoteDesktop.Destroy;
begin
   inherited;
end;

procedure TRemoteDesktop.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TRemoteDesktop.SetTargetNodeId(aNodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(aNodeId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aNodeId;
end;

procedure TRemoteDesktop.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TRemoteDesktop.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TRemoteDesktop.SetDllPassword(Password: AnsiString);
begin
   if fDllPassword=trim(password) then
      exit;
   fDllPassword:=password;
end;

function TRemoteDesktop.BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream,Stream1: TMemoryStream;
   ok: boolean;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Stream1:=TMemoryStream.Create;
   try
      ok:=InParcel.SaveToStream(Stream);
      if ok then
         begin
            Stream.Position:=0;
            GZCompressStream(Stream,Stream1);
            ok:=(Stream1.size>0);
         end;
   except
      ok:=false;
   end;
   if not ok then
      begin
         Stream.Free;
         Stream1.Free;
         result:=false;
         exit;
      end;
   Stream1.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',53);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   Parcel.PutStreamGoods('InParcelStream',Stream1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('OutParcelStream') then
                        begin
                           Stream.Clear;
                           Stream1.Clear;
                           try
                              Parcel.GetStreamGoods('OutParcelStream',Stream);
                              GZDecompressStream(Stream,Stream1);
                              Stream1.Position:=0;
                              OutParcel.Clear;
                              result:=OutParcel.LoadFromStream(Stream1);
                           except
                              result:=false;
                           end;
                        end
                     else
                        Result:=true;
                  end;
            end;
         Parcel.Free;
      end;
   Stream.Free;
   Stream1.Free;
end;

function TRemoteDesktop.GetDesktopScreen(var ScreenImage: TImage): boolean;
var
   BmpStream: TMemoryStream;
   InParcel,OutParcel: TQBParcel;
begin
   if not assigned(fUserConn) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteDesktop');
   InParcel.PutIntegerGoods('FunctionId',1);
   OutParcel:=TQBParcel.Create;
   if not BinaryRPC('remotedesktop',fdllpassword,InParcel,OutParcel) then
      begin
         InParcel.Free;
         OutParcel.Free;
         result:=false;
         exit;
      end;
   BmpStream:=TMemoryStream.Create;
   OutParcel.GetStreamGoods('ScreenImage',BmpStream);
   bmpstream.Position:=0;
   try
      ScreenImage.Picture.Bitmap.LoadFromStream(BmpStream);
      result:=true;
   except
      result:=false;
   end;
   bmpstream.Free;
   InParcel.Free;
   OutParcel.Free;
end;

procedure TRemoteDesktop.MouseLeftClick(PosX: integer; PosY: integer; MaxX: integer; MaxY: integer);
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteDesktop');
   InParcel.PutIntegerGoods('FunctionId',2);
   InParcel.PutIntegerGoods('X',PosX);
   InParcel.PutIntegerGoods('Y',PosY);
   InParcel.PutIntegerGoods('WindowWidth',MaxX);
   InParcel.PutIntegerGoods('WindowHeight',MaxY);
   OutParcel:=TQBParcel.Create;
   BinaryRPC('remotedesktop',fdllpassword,InParcel,OutParcel);
   InParcel.Free;
   OutParcel.Free;
end;

procedure TRemoteDesktop.MouseRightClick(PosX: integer; PosY: integer; MaxX: integer; MaxY: integer);
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteDesktop');
   InParcel.PutIntegerGoods('FunctionId',3);
   InParcel.PutIntegerGoods('X',PosX);
   InParcel.PutIntegerGoods('Y',PosY);
   InParcel.PutIntegerGoods('WindowWidth',MaxX);
   InParcel.PutIntegerGoods('WindowHeight',MaxY);
   OutParcel:=TQBParcel.Create;
   BinaryRPC('remotedesktop',fdllpassword,InParcel,OutParcel);
   InParcel.Free;
   OutParcel.Free;
end;

procedure TRemoteDesktop.MouseDoubleClick(PosX: integer; PosY: integer; MaxX: integer; MaxY: integer);
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteDesktop');
   InParcel.PutIntegerGoods('FunctionId',4);
   InParcel.PutIntegerGoods('X',PosX);
   InParcel.PutIntegerGoods('Y',PosY);
   InParcel.PutIntegerGoods('WindowWidth',MaxX);
   InParcel.PutIntegerGoods('WindowHeight',MaxY);
   OutParcel:=TQBParcel.Create;
   BinaryRPC('remotedesktop',fdllpassword,InParcel,OutParcel);
   InParcel.Free;
   OutParcel.free;
end;

end.
