//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit RemoteFiles;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  DBClient, QBCommon, QBJson, QBParcel, TaskCommon, QBZlibExGz, UserConnection
{$IFDEF VER130}
 ,FileCtrl;
{$ELSE}
,MidasLib;
{$ENDIF}

const
    spQBNodeFolder=0;            // Quickburro node base folder
    spDesktop=1;                 // <desktop>
    spInternet=2;                // Internet Explorer (icon on desktop)
    spPrograms=3;                // Start Menu\Programs
    spControls=4;                // My Computer\Control Panel
    spPrinters=5;                // My Computer\Printers
    spPersonal=6;                // My Documents
    spFavorites=7;               // <user name>\Favorites
    spStartup=8;                 // Start Menu\Programs\Startup
    spRecent=9;                  // <user name>\Recent
    spSendTo=10;                 // <user name>\SendTo
    spBitBucket=11;              // <desktop>\Recycle Bin
    spStartMenu=12;              // <user name>\Start Menu
    spMyDocuments=13;            // logical "My Documents" desktop icon
    spMyMusic=14;                // "My Music" folder
    spMyVideo=15;                // "My Videos" folder
    spDesktopDirectory=16;       // <user name>\Desktop
    spDrives=17;                 // My Computer
    spNetwork=18;                // Network Neighborhood (My Network Places)
    spNethood=19;                // <user name>\nethood
    spFonts=20;                  // Windows\fonts
    spTemplates=21;              // <user name>\Templates
    spCommonStartMenu=22;        // All Users\Start Menu
    spCommonPrograms=23;         // All Users\Start Menu\Programs
    spCommonStartup=24;          // All Users\Startup
    spCommonDesktopDirectory=25; // All Users\Desktop
    spAppData=26;                // <user name>\Application Data
    spPrinthood=27;              // <user name>\PrintHood
    spLocalAppData=28;           // <user name>\Local Settings\Applicaiton Data (non roaming)
    spALTStartup=29;             // non localized startup
    spCommonALTStartup=30;       // non localized common startup
    spCommonFavorites=31;        // All Users\Favorites
    spInternetCache=32;          // <user name>\Local Settings\Temporary Internet Files
    spCookies=33;                // <user name>\Cookies
    spHistory=34;                // <user name>\Local Settings\History
    spCommonAppData=35;          // All Users\Application Data
    spWindows=36;                // GetWindowsDirectory()
    spSystem=37;                 // GetSystemDirectory()
    spProgramFiles=38;           // C:\Program Files
    spMyPictures=39;             // C:\Program Files\My Pictures
    spProfile=40;                // USERPROFILE
    spSystemX86=41;              // x86 system directory on RISC
    spProgramFilesX86=42;        // x86 C:\Program Files on RISC
    spProgramFilesCommon=43;     // C:\Program Files\Common
    spProgramFilesCommonX86=44;  // x86 Program Files\Common on RISC
    spCommonTemplates=45;        // All Users\Templates
    spCommonDocuments=46;        // All Users\Documents
    spCommonAdminTools=47;       // All Users\Start Menu\Programs\Administrative Tools
    spAdminTools=48;             // <user name>\Start Menu\Programs\Administrative Tools
    spConnections=49;            // Network and Dial-up Connections
    spCommonMusic=50;            // All Users\My Music
    spCommonPictures=51;         // All Users\My Pictures
    spCommonVideo=52;            // All Users\My Video
    spResources=53;              // Resource Direcotry
    spResourcesLocalized=54;     // Localized Resource Direcotry
    spCommonOEMLinks=55;         // Links to All Users OEM specific apps
    spCDBurnArea=56;             // USERPROFILE\Local Settings\Application Data\Microsoft\CD Burning
    spComputersNearMe=57;        // Computers Near Me (computered from Workgroup membership)

type
   TFileAttribute=Record
      FileVersion: String;
      FileAttribute: integer;
      FileSize: integer;
      FileCreateTime: TDateTime;
      FileModifyTime: TDateTime;
   end;

type
   TObjRecord=Record
      ObjName: string;
      ObjType: integer;
      ObjLevel: integer;
      Expanded: boolean;
   end;

type
  TRemoteFiles = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;                     
     fDllPassword: AnsiString;                  
     fTargetNodeId: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetDllPassword(Password: AnsiString);
     procedure SetTargetNodeId(aNodeId: AnsiString);
     function BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;           
     Destructor Destroy(); override;                             
     function GetDiskList(var DiskList: String): boolean;
     function GetDiskInfo(DiskId: String; DiskInfo: TQBParcel): boolean;
     function ListFiles(TargetPath: String; FileAttr: integer; FileMask: String; FileListCDS: TClientDataset): boolean;
     function CreateDirrectory(TargetDirectory: String): boolean;
     function RemoveDirectory(TargetDirectory: String): boolean;
     function CopyFile(SourceFileName: String; TargetFileName: String): boolean;
     function MergeFile(File1Name: String; File2Name: String; TargetFileName: String): boolean;
     function RenameFile(OldFileName: String; NewFileName: String): boolean;
     function SetFileAttr(TargetFileName: String; FileAttr: integer): boolean;
     function DeleteFile(TargetFileName: String): boolean;
     function SearchFiles(TargetPath: String; FileMask: String; FileListCDS: TClientDataset): boolean;
     function GetFileVersion(FileName: String; var FileVersion: String): boolean;
     function GetSpecialPath(PathType: integer; var SpecialPath: String): boolean;
     function GetFileAttribute(FileName: String; var FAttr: TFileAttribute): boolean;
     function SetFileModifyTime(FileName: String; FileTime: TDateTime): boolean;
     function ExpandFolder(TargetPath: String; ObjListCDS: TClientDataset): boolean;
     function RemoteFileToStream(FileName: string; Stream: TMemoryStream): boolean;
     function StreamToRemoteFile(Stream: TMemoryStream; FileName: string): boolean;
     function RemoteFileToJson(FileName: string; var aJson: TQBJson): boolean;
     function JsonToRemoteFile(aJson: TQBJson; FileName: string): boolean;
     function RemoteFileToParcel(FileName: string; aParcel: TQBParcel): boolean;
     function ParcelToRemoteFile(aParcel: TQBParcel; FileName: string): boolean;
     function FileExists(aFileName: string): boolean;
     function DirectoryExists(aDirName: string): boolean;

  published
     property UserConn: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;          
     property DllPassword: AnsiString read fDllPassword write SetDllPassword;             
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;

  end;

procedure Register;

implementation

procedure Register;
begin
   RegisterComponents('QuickBurro Standard', [TRemoteFiles]);
end;

constructor TRemoteFiles.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;                  
   fTargetNodeId:='';
   fDllPassword:='';
end;

destructor TRemoteFiles.Destroy;
begin
   inherited;
end;

procedure TRemoteFiles.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TRemoteFiles.SetTargetNodeId(aNodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(aNodeId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aNodeId;
end;

procedure TRemoteFiles.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TRemoteFiles.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TRemoteFiles.SetDllPassword(Password: AnsiString);
begin
   if fDllPassword=trim(password) then
      exit;
   fDllPassword:=password;
end;

function TRemoteFiles.BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream,Stream1: TMemoryStream;
   ok: boolean;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Stream1:=TMemoryStream.Create;
   try
      ok:=InParcel.SaveToStream(Stream);
      if ok then
         begin
            Stream.Position:=0;
            GZCompressStream(Stream,Stream1);
            ok:=(Stream1.size>0);
         end;
   except
      ok:=false;
   end;
   if not ok then
      begin
         Stream.Free;
         Stream1.Free;
         result:=false;
         exit;
      end;
   Stream1.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',53);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   Parcel.PutStreamGoods('InParcelStream',Stream1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('OutParcelStream') then
                        begin
                           Stream.Clear;
                           Stream1.Clear;
                           try
                              Parcel.GetStreamGoods('OutParcelStream',Stream);
                              GZDecompressStream(Stream,Stream1);
                              Stream1.Position:=0;
                              OutParcel.Clear;
                              result:=OutParcel.LoadFromStream(Stream1);
                           except
                              result:=false;
                           end;
                        end
                     else
                        Result:=true;
                  end;
            end;
         Parcel.Free;
      end;
   Stream.Free;
   Stream1.Free;
end;

function TRemoteFiles.GetDiskList(var DiskList: String): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',1);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if result then
      disklist:=OutParcel.GetStringGoods('DiskList')
   else
      DiskList:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.GetDiskInfo(DiskId: String; DiskInfo: TQBParcel): boolean;
var
   InParcel,OutParcel: TQBParcel;
   tmpstr: AnsiString;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',2);
   InParcel.PutStringGoods('DiskId',DiskId);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         OutParcel.SaveToString(tmpstr);
         DiskInfo.LoadFromString(TmpStr);
      end
   else
      DiskInfo.Clear;
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.ListFiles(TargetPath: String; FileAttr: integer;
                                    FileMask: String; FileListCDS: TClientDataset): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',3);
   InParcel.PutStringGoods('Path',TargetPath);
   InParcel.PutIntegerGoods('FileAttributes',FileAttr);
   InParcel.PutStringGoods('FileMask',FileMask);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if result then
      OutParcel.GetCDSGoods('FileList',FileListCds);
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.CreateDirrectory(TargetDirectory: String): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',4);
   InParcel.PutStringGoods('DirName',TargetDirectory);
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteFiles.RemoveDirectory(TargetDirectory: String): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',5);
   InParcel.PutStringGoods('DirName',TargetDirectory);
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteFiles.CopyFile(SourceFileName: String; TargetFileName: String): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',6);
   InParcel.PutStringGoods('SourceFileName',SourceFileName);
   InParcel.PutStringGoods('TargetFileName',TargetFileName);
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteFiles.MergeFile(File1Name: String; File2Name: String; TargetFileName: String): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',7);
   InParcel.PutStringGoods('FileName1',File1Name);
   InParcel.PutStringGoods('FileName2',File2Name);
   InParcel.PutStringGoods('TargetFileName',TargetFileName);
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteFiles.RenameFile(OldFileName: String; NewFileName: String): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',8);
   InParcel.PutStringGoods('OldFileName',OldFileName);
   InParcel.PutStringGoods('NewFileName',NewFileName);
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteFiles.SetFileAttr(TargetFileName: String; FileAttr: integer): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',9);
   InParcel.PutStringGoods('FileName',TargetFileName);
   InParcel.PutIntegerGoods('FileAttributes',FileAttr);
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteFiles.DeleteFile(TargetFileName: String): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',10);
   InParcel.PutStringGoods('FileName',TargetFileName);
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteFiles.SearchFiles(TargetPath: String; FileMask: String; FileListCDS: TClientDataset): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',11);
   InParcel.PutStringGoods('BeginPath',TargetPath);
   InParcel.PutStringGoods('FileMask',FileMask);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if result then
      OutParcel.GetCDSGoods('FileList',FileListCds);
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.GetFileVersion(FileName: String; var FileVersion: String): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutStringGoods('FileName',FileName);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if not result then
      fileversion:=''
   else
      fileversion:=OutParcel.GetStringGoods('FileVersion');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.GetSpecialPath(PathType: integer; var SpecialPath: String): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutIntegerGoods('PathType',PathType);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if not result then
      SpecialPath:=''
   else
      SpecialPath:=OutParcel.GetStringGoods('Path');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.GetFileAttribute(FileName: String; var FAttr: TFileAttribute): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',14);
   InParcel.PutStringGoods('FileName',FileName);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         FAttr.FileVersion:=OutParcel.GetStringGoods('FileVersion');
         FAttr.FileAttribute:=OutParcel.GetIntegerGoods('FileAttribute');
         FAttr.FileSize:=OutParcel.GetIntegerGoods('FileSize');
         FAttr.FileCreateTime:=OutParcel.GetDateTimeGoods('FileCreateTime');
         FAttr.FileModifyTime:=OutParcel.GetDateTimeGoods('FileModifyTime');
      end;
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.SetFileModifyTime(FileName: String; FileTime: TDateTime): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',15);
   InParcel.PutStringGoods('FileName',FileName);
   InParcel.PutDateTimeGoods('FileModifyTime',FileTime);
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteFiles.ExpandFolder(TargetPath: String; ObjListCDS: TClientDataset): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',16);
   InParcel.PutStringGoods('FolderName',TargetPath);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if result then
      OutParcel.GetCDSGoods('FolderObjects',ObjListCds);
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.RemoteFileToStream(FileName: string; Stream: TMemoryStream): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',17);
   InParcel.PutStringGoods('FileName',FileName);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         Stream.Clear;
         OutParcel.GetStreamGoods('FileContent',Stream);
         Stream.Position:=0;
      end;
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.StreamToRemoteFile(Stream: TMemoryStream; FileName: string): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',18);
   InParcel.PutStringGoods('FileName',FileName);
   Inparcel.PutStreamGoods('FileContent',Stream);
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteFiles.RemoteFileToJson(FileName: string; var aJson: TQBJson): boolean;
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   result:=RemoteFileToStream(FileName,Stream);
   if not result then
      begin
         stream.free;
         exit;
      end;
   Stream.Position:=0;
   if Assigned(aJson) then
      aJson.Free;
   aJson:=TQBJson.Create(Stream,Stream.Size);
   Stream.Free;
end;

function TRemoteFiles.JsonToRemoteFile(aJson: TQBJson; FileName: string): boolean;
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   try
      aJson.SaveToStream(Stream,3);
      result:=StreamToRemoteFile(Stream,FileName);
   finally
      Stream.Free;
   end;
end;

function TRemoteFiles.RemoteFileToParcel(FileName: string; aParcel: TQBParcel): boolean;
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   result:=RemoteFileToStream(FileName,Stream);
   if not result then
      begin
         stream.free;
         exit;
      end;
   Stream.Position:=0;
   aParcel.Clear;
   aParcel.LoadFromStream(Stream);
   Stream.Free;
end;

function TRemoteFiles.ParcelToRemoteFile(aParcel: TQBParcel; FileName: string): boolean;
var
   Stream: TMemoryStream;
begin
   Stream:=TMemoryStream.Create;
   try
      result:=aParcel.SaveToStream(Stream);
      if result then
         result:=StreamToRemoteFile(Stream,FileName);
   finally
      Stream.Free;
   end;
end;

function TRemoteFiles.FileExists(aFileName: string): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',19);
   InParcel.PutStringGoods('FileName',aFileName);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if result then
      result:=OutParcel.GetBooleanGoods('FileExists');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteFiles.DirectoryExists(aDirName: string): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteFiles');
   InParcel.PutIntegerGoods('FunctionId',20);
   InParcel.PutStringGoods('DirectoryName',aDirName);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC('remotefiles',fdllpassword,InParcel,OutParcel);
   if result then
      result:=OutParcel.GetBooleanGoods('DirectoryExists');
   InParcel.Free;
   OutParcel.Free;
end;

end.
