//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit RemoteIniFile;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  UserConnection, QBParcel, TaskCommon, QBZlibExGz;

type
  TRemoteIniFile = class(TComponent)
  private
     fUserConn: TUserConnection;                
     fTaskTimeout: integer;                     
     fDllPassword: AnsiString;                  
     fIniFilename: String;                      
     fTargetNodeId: AnsiString;                 
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetUserConn(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetDllPassword(Password: AnsiString);
     procedure SetFileName(aFileName: String);       
     procedure SetTargetNodeId(NodeId: AnsiString);      
     function BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;           
     Destructor Destroy(); override;                             
     function CreateIniFile: boolean;
     function ReadSections(var SectionList: TStringList): boolean;
     function ReadSection(SectionName: string; var KeyList: TStringList): boolean;
     function ReadSectionValues(SectionName: string; var ValueList: TStringList): boolean;
     function EraseSection(SectionName: string): boolean;
     function DeleteKey(SectionName: string; KeyName: string): boolean;
     function ReadString(SectionName: string; KeyName: string; DefaultValue: string; var StrValue: string): boolean;
     function ReadInteger(SectionName: string; KeyName: string; DefaultValue: integer; var IntValue: integer): boolean;
     function ReadBoolean(SectionName: string; KeyName: string; DefaultValue: Boolean; var BoolValue: Boolean): boolean;
     function WriteString(SectionName: string; KeyName: string; StrValue: string): boolean;
     function WriteInteger(SectionName: string; KeyName: string; IntValue: integer): boolean;
     function WriteBoolean(SectionName: string; KeyName: string; BoolValue: Boolean): boolean;

  published
     property UserConn: TUserConnection read fUserConn write SetUserConn;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;            
     property DllPassword: AnsiString read fDllPassword write SetDllPassword;             
     property IniFilename: String read fIniFilename write SetFilename;                         
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;          

  end;

procedure Register;

implementation

procedure Register;
begin
   RegisterComponents('QuickBurro Standard', [TRemoteIniFile]);
end;

constructor TRemoteIniFile.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;                  
   fIniFilename:='';
   fTargetNodeId:='';
end;

destructor TRemoteIniFile.Destroy;
begin
   inherited;
end;

procedure TRemoteIniFile.SetUserConn(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TRemoteIniFile.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TRemoteIniFile.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TRemoteIniFile.SetDllPassword(Password: AnsiString);
begin
   if fDllPassword=trim(password) then
      exit;
   fDllPassword:=password;
end;

procedure TRemoteIniFile.SetFilename(aFilename: String);
begin
   if StrComp(Pchar(fIniFilename),Pchar(aFilename))=0 then
      exit;
   fIniFilename:=aFileName;
end;

procedure TRemoteIniFile.SetTargetNodeId(NodeId: AnsiString);      
begin
   if fTargetNodeId=NodeId then
      exit;
   fTargetNodeId:=NodeId;
end;

function TRemoteIniFile.BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream,Stream1: TMemoryStream;
   ok: boolean;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Stream1:=TMemoryStream.Create;
   try
      ok:=InParcel.SaveToStream(Stream);
      if ok then
         begin
            Stream.Position:=0;
            GZCompressStream(Stream,Stream1);
            ok:=(Stream1.size>0);
         end;
   except
      ok:=false;
   end;
   if not ok then
      begin
         Stream.Free;
         Stream1.Free;
         result:=false;
         exit;
      end;
   Stream1.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',53);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   Parcel.PutStreamGoods('InParcelStream',Stream1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('OutParcelStream') then
                        begin
                           Stream.Clear;
                           Stream1.Clear;
                           try
                              Parcel.GetStreamGoods('OutParcelStream',Stream);
                              GZDecompressStream(Stream,Stream1);
                              Stream1.Position:=0;
                              OutParcel.Clear;
                              result:=OutParcel.LoadFromStream(Stream1);
                           except
                              result:=false;
                           end;
                        end
                     else
                        Result:=true;
                  end;
            end;
         Parcel.Free;
      end;
   Stream.Free;
   Stream1.Free;
end;

function TRemoteIniFile.CreateIniFile: boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',1);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.ReadSections(var SectionList: TStringList): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',2);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
         if result then
            begin
               SectionList:=TStringList.create;
               SectionList.text:=OutParcel.GetStringGoods('SectionList');
            end;
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.ReadSection(SectionName: string; var KeyList: TStringList): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',3);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
         if result then
            begin
               KeyList:=TStringList.create;
               KeyList.text:=OutParcel.GetStringGoods('KeyList');
            end;
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.ReadSectionValues(SectionName: string; var ValueList: TStringList): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',4);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
         if result then
            begin
               ValueList:=TStringList.create;
               ValueList.text:=OutParcel.GetStringGoods('ValueList');
            end;
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.EraseSection(SectionName: string): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',5);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.DeleteKey(SectionName: string; KeyName: string): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',6);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   InParcel.PutStringGoods('KeyName',KeyName);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.ReadString(SectionName: string; KeyName: string; DefaultValue: string; var StrValue: string): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',7);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   InParcel.PutStringGoods('KeyName',KeyName);
   InParcel.PutStringGoods('DefaultValue',DefaultValue);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
         if result then
            StrValue:=OutParcel.getStringGoods('StringValue');
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.ReadInteger(SectionName: string; KeyName: string; DefaultValue: integer; var IntValue: integer): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',8);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   InParcel.PutStringGoods('KeyName',KeyName);
   InParcel.PutIntegerGoods('DefaultValue',DefaultValue);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
         if result then
            IntValue:=OutParcel.getIntegerGoods('IntegerValue');
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.ReadBoolean(SectionName: string; KeyName: string; DefaultValue: Boolean; var BoolValue: Boolean): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',9);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   InParcel.PutStringGoods('KeyName',KeyName);
   InParcel.PutBooleanGoods('DefaultValue',DefaultValue);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
         if result then
            BoolValue:=OutParcel.getBooleanGoods('BooleanValue');
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.WriteString(SectionName: string; KeyName: string; StrValue: string): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',10);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   InParcel.PutStringGoods('KeyName',KeyName);
   InParcel.PutStringGoods('StringValue',StrValue);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.WriteInteger(SectionName: string; KeyName: string; IntValue: integer): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',11);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   InParcel.PutStringGoods('KeyName',KeyName);
   InParcel.PutIntegerGoods('IntegerValue',IntValue);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

function TRemoteIniFile.WriteBoolean(SectionName: string; KeyName: string; BoolValue: Boolean): boolean;
var
   InParcel,OutParcel: TQBParcel;
   ok: boolean;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteIniFile');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutStringGoods('IniFileName',fIniFilename);
   InParcel.PutStringGoods('SectionName',SectionName);
   InParcel.PutStringGoods('KeyName',KeyName);
   InParcel.PutBooleanGoods('BooleanValue',BoolValue);
   OutParcel:=TQBParcel.Create;
   Result:=BinaryRPC('RemoteIniFile',fdllpassword,InParcel,OutParcel);
   if result then
      begin
         ok:=OutParcel.GetBooleanGoods('ProcessResult');
         result:=(result and ok);
      end;
   InParcel.Free;
   OutParcel.free;
end;

end.
