//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.25.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit RemoteORM;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  UserConnection, QBCommon, QBParcel, QBZlibExGz, TaskCommon, DBClient
{$IFDEF VER130}
;
{$ELSE}
,MidasLib;
{$ENDIF}

type
  TRemoteORM = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;                     
     fDllPassword: AnsiString;                      
     fTargetNodeId: AnsiString;
     fTargetDatabaseId: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetDllPassword(Password: AnsiString);
     procedure SetTargetNodeId(aNodeId: AnsiString);
     procedure SetTargetDatabaseId(aDatabaseId: AnsiString);
     function BinaryRPC(DatabaseId: AnsiString; aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;

  protected
     procedure Notification(AComponent: TComponent;Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;           
     Destructor Destroy(); override;                             
     function GetSqlTableList(TargetDbTypeId: integer; TargetDBName: String; TableListCDS: TClientDataset): boolean;
     function GetSqlTableFieldList(TargetDBName: String; TargetTableName: String; FieldListCDS: TClientDataset): boolean;
     function GetEntityFullUnit(TargetDBName: String; TargetTableName: String; var ClassUintText: String): boolean;
     function GetEntitySimpUnit(TargetDBName: String; TargetTableName: String; var ClassUintText: String): boolean;
     function GetDynaEntityFullUnit(TargetDBName: String; TargetTableName: String; SQLCommand: String; var ClassUintText: String): boolean;
     function GetDynaEntitySimpUnit(TargetDBName: String; TargetTableName: String; SQLCommand: String; var ClassUintText: String): boolean;
     function GetComplexEntityFullUnit(TargetDBName: String; TargetTableName: String; MainKeyField: String; SubTableList: String; var ClassUintText: String): boolean;
     function GetComplexEntitySimpUnit(TargetDBName: String; TargetTableName: String; MainKeyField: String; SubTableList: String; var ClassUintText: String): boolean;

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;          
     property DllPassword: AnsiString read fDllPassword write SetDllPassword;             
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property TargetDatabaseId: AnsiString read fTargetDatabaseId write SetTargetDatabaseId;

  end;

procedure Register;

implementation

procedure Register;
begin
   RegisterComponents('QuickBurro Standard', [TRemoteORM]);
end;

constructor TRemoteORM.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;                  
   fTargetNodeId:='';
   fDllPassword:='';
end;

destructor TRemoteORM.Destroy;
begin
   inherited;
end;

procedure TRemoteORM.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TRemoteORM.SetTargetNodeId(aNodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(aNodeId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aNodeId;
end;

procedure TRemoteORM.SetTargetDatabaseId(aDatabaseId: AnsiString);
begin
   if StrIComp(PAnsiChar(aDatabaseId),PAnsiChar(fTargetDatabaseId))=0 then
      exit;
   fTargetDatabaseId:=aDatabaseId;
end;

procedure TRemoteORM.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TRemoteORM.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TRemoteORM.SetDllPassword(Password: AnsiString);
begin
   if fDllPassword=trim(password) then
      exit;
   fDllPassword:=password;
end;

function TRemoteORM.BinaryRPC(DatabaseId: AnsiString; aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream,Stream1: TMemoryStream;
   ok: boolean;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Stream1:=TMemoryStream.Create;
   try
      ok:=InParcel.SaveToStream(Stream);
      if ok then
         begin
            Stream.Position:=0;
            GZCompressStream(Stream,Stream1);
            ok:=(Stream1.size>0);
         end;
   except
      ok:=false;
   end;
   if not ok then
      begin
         Stream.Free;
         Stream1.Free;
         result:=false;
         exit;
      end;
   Stream1.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',53);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   Parcel.PutEncryptStringGoods('DatabaseId',DatabaseId);
   Parcel.PutStreamGoods('InParcelStream',Stream1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('OutParcelStream') then
                        begin
                           Stream.Clear;
                           Stream1.Clear;
                           try
                              Parcel.GetStreamGoods('OutParcelStream',Stream);
                              GZDecompressStream(Stream,Stream1);
                              Stream1.Position:=0;
                              OutParcel.Clear;
                              result:=OutParcel.LoadFromStream(Stream1);
                           except
                              result:=false;
                           end;
                        end
                     else
                        Result:=true;
                  end;
            end;
         Parcel.Free;
      end;
   Stream.Free;
   Stream1.Free;
end;

function TRemoteORM.GetSqlTableList(TargetDbTypeId: integer; TargetDBName: String; TableListCDS: TClientDataset): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteOrm');
   InParcel.PutIntegerGoods('FunctionId',1);
   InParcel.PutIntegerGoods('DatabaseTypeId',TargetDbTypeId);
   InParcel.PutStringGoods('DatabaseName',TargetDbName);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC(fTargetDatabaseId,'remoteorm',fdllpassword,InParcel,OutParcel);
   if result then
      OutParcel.GetCDSGoods('TableList',TableListCds);
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteORM.GetSqlTableFieldList(TargetDBName: String; TargetTableName: String; FieldListCDS: TClientDataset): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteOrm');
   InParcel.PutIntegerGoods('FunctionId',2);
   InParcel.PutStringGoods('TableName',TargetTableName);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC(fTargetDatabaseId,'remoteorm',fdllpassword,InParcel,OutParcel);
   if result then
      OutParcel.GetCDSGoods('FieldList',FieldListCds);
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteORM.GetEntityFullUnit(TargetDBName: String; TargetTableName: String; var ClassUintText: String): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteOrm');
   InParcel.PutIntegerGoods('FunctionId',3);
   InParcel.PutStringGoods('TableName',TargetTableName);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC(fTargetDatabaseId,'remoteorm',fdllpassword,InParcel,OutParcel);
   if result then
      ClassUintText:=string(OutParcel.GetAnsiStringGoods('EntityCode'))
   else
      ClassUintText:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteORM.GetDynaEntityFullUnit(TargetDBName: String; TargetTableName: String; SQLCommand: String; var ClassUintText: String): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteOrm');
   InParcel.PutIntegerGoods('FunctionId',5);
   InParcel.PutStringGoods('TableName',TargetTableName);
   InParcel.PutStringGoods('SQLCommand',SQLCommand);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC(fTargetDatabaseId,'remoteorm',fdllpassword,InParcel,OutParcel);
   if result then
      ClassUintText:=string(OutParcel.GetAnsiStringGoods('EntityCode'))
   else
      ClassUintText:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteORM.GetEntitySimpUnit(TargetDBName: String; TargetTableName: String; var ClassUintText: String): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteOrm');
   InParcel.PutIntegerGoods('FunctionId',4);
   InParcel.PutStringGoods('TableName',TargetTableName);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC(fTargetDatabaseId,'remoteorm',fdllpassword,InParcel,OutParcel);
   if result then
      ClassUintText:=string(OutParcel.GetAnsiStringGoods('EntityCode'))
   else
      ClassUintText:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteORM.GetDynaEntitySimpUnit(TargetDBName: String; TargetTableName: String; SQLCommand: String; var ClassUintText: String): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteOrm');
   InParcel.PutIntegerGoods('FunctionId',6);
   InParcel.PutStringGoods('TableName',TargetTableName);
   InParcel.PutStringGoods('SQLCommand',SQLCommand);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC(fTargetDatabaseId,'remoteorm',fdllpassword,InParcel,OutParcel);
   if result then
      ClassUintText:=string(OutParcel.GetAnsiStringGoods('EntityCode'))
   else
      ClassUintText:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteORM.GetComplexEntityFullUnit(TargetDBName: String; TargetTableName: String; MainKeyField: String; SubTableList: String; var ClassUintText: String): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteOrm');
   InParcel.PutIntegerGoods('FunctionId',7);
   InParcel.PutStringGoods('TableName',TargetTableName);
   InParcel.PutStringGoods('MainKeyField',MainKeyField);
   InParcel.PutStringGoods('SubTableList',SubTableList);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC(fTargetDatabaseId,'remoteorm',fdllpassword,InParcel,OutParcel);
   if result then
      ClassUintText:=string(OutParcel.GetAnsiStringGoods('EntityCode'))
   else
      ClassUintText:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteORM.GetComplexEntitySimpUnit(TargetDBName: String; TargetTableName: String; MainKeyField: String; SubTableList: String; var ClassUintText: String): boolean;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteOrm');
   InParcel.PutIntegerGoods('FunctionId',8);
   InParcel.PutStringGoods('TableName',TargetTableName);
   InParcel.PutStringGoods('MainKeyField',MainKeyField);
   InParcel.PutStringGoods('SubTableList',SubTableList);
   OutParcel:=TQBParcel.Create;
   result:=BinaryRPC(fTargetDatabaseId,'remoteorm',fdllpassword,InParcel,OutParcel);
   if result then
      ClassUintText:=string(OutParcel.GetAnsiStringGoods('EntityCode'))
   else
      ClassUintText:='';
   InParcel.Free;
   OutParcel.Free;
end;

end.
