//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit RemoteRegistry;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  QBParcel, Registry, UserConnection, TaskCommon, QBZlibExGz;

type
   TRegkeyinfo=record
      NumSubKeys: Integer;        
      maxsubkeylen:integer;       
      numvalues:integer;          
      maxvaluelen: integer;       
      MaxDataLen: Integer;        
      Filetime: TDateTime;        
   end;

type
  TRemoteRegistry = class(TComponent)
  private
     fUserConn: TUserConnection;                
     fTaskTimeout: integer;                     
     fDllPassword: AnsiString;                  
     fRootKey: String;                          
     fTargetNodeId: AnsiString;                 
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetUserConn(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetDllPassword(Password: AnsiString);
     procedure SetRootKey(aRootKey: String);         
     procedure SetTargetNodeId(NodeId: AnsiString);  
     function GetRootKey: LongWord;
     function BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;           
     Destructor Destroy(); override;                             
     function CreateKey(aKeyName: String): boolean;
     function DeleteKey(aKeyName: String): boolean;
     function DeleteValue(aKeyName: String; aValueName: String): boolean;
     function GetDataSize(aKeyName: String; aValueName: String): integer;
     function GetDataType(aKeyName: String; aValueName: String): integer;
     function GetKeyInfo(aKeyName: String; var aKeyInfo: TRegkeyinfo): boolean;
     function GetKeyNames(aKeyName: String; var aKeyNames: String): boolean;
     function GetValueNames(aKeyName: String; var aValueNames: String): boolean;
     function HasSubKeys(aKeyName: String; var SubKeyExists: boolean): boolean;
     function KeyExists(aKeyName: String; var KeyExists: boolean): boolean;
     function MoveKey(OldKeyName: String; NewKeyName: String; DeleteOld: boolean): boolean;
     function ReadString(aKeyName: String; aValueName: String): String;
     function ReadInteger(aKeyName: String; aValueName: String): Integer;
     function ReadFloat(aKeyName: String; aValueName: String): Double;
     function ReadCurrency(aKeyName: String; aValueName: String): Currency;
     function ReadBool(aKeyName: String; aValueName: String): Boolean;
     function ReadDate(aKeyName: String; aValueName: String): TDateTime;
     function ReadTime(aKeyName: String; aValueName: String): TDateTime;
     function ReadDateTime(aKeyName: String; aValueName: String): TDateTime;
     function ReadBinaryData(aKeyName: String; aValueName: String): AnsiString;
     function WriteString(aKeyName: String; aValueName: String; aValue: String): boolean;
     function WriteInteger(aKeyName: String; aValueName: String; aValue: Integer): boolean;
     function WriteFloat(aKeyName: String; aValueName: String; aValue: Double): boolean;
     function WriteCurrency(aKeyName: String; aValueName: String; aValue: Currency): boolean;
     function WriteBool(aKeyName: String; aValueName: String; aValue: Boolean): boolean;
     function WriteDate(aKeyName: String; aValueName: String; aValue: TDateTime): boolean;
     function WriteTime(aKeyName: String; aValueName: String; aValue: TDateTime): boolean;
     function WriteDateTime(aKeyName: String; aValueName: String; aValue: TDateTime): boolean;
     function WriteBinaryData(aKeyName: String; aValueName: String; aValue: AnsiString): boolean;
     function LoadFromFile(aFileName: String): boolean;
     function SaveToFile(aKeyName: String; aFileName: String): boolean;

  published
     property UserConn: TUserConnection read fUserConn write SetUserConn;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;            
     property DllPassword: AnsiString read fDllPassword write SetDllPassword;             
     property RootKey: String read fRootKey write SetRootKey;                         
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;          

  end;

procedure Register;

implementation

procedure Register;
begin
   RegisterComponents('QuickBurro Standard', [TRemoteRegistry]);
end;

constructor TRemoteRegistry.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;                   
   fRootKey:='HKEY_LOCAL_MACHINE';     
   fTargetNodeId:='';
end;

destructor TRemoteRegistry.Destroy;
begin
   inherited;
end;

function TRemoteRegistry.GetRootKey: LongWord;
begin
   if uppercase(trim(fRootKey))='HKEY_CLASSES_ROOT' then
      begin
         result:=HKEY_CLASSES_ROOT;
         exit;
      end;
   if uppercase(trim(fRootKey))='HKEY_CURRENT_USER' then
      begin
         Result:=HKEY_CURRENT_USER;
         exit;
      end;
   if uppercase(trim(fRootKey))='HKEY_LOCAL_MACHINE' then
      begin
         Result:=HKEY_LOCAL_MACHINE;
         exit;
      end;
   if uppercase(trim(fRootKey))='HKEY_USERS' then
      begin
         Result:=HKEY_USERS;
         exit;
      end;
   if uppercase(trim(fRootKey))='HKEY_PERFORMANCE_DATA' then
      begin
         result:=HKEY_PERFORMANCE_DATA;
         exit;
      end;
   if uppercase(trim(fRootKey))='HKEY_CURRENT_CONFIG' then
      begin
         result:=HKEY_CURRENT_CONFIG;
         exit;
      end;
   if uppercase(trim(fRootKey))='HKEY_DYN_DATA' then
      result:=HKEY_DYN_DATA
   else
      result:=0;
end;

procedure TRemoteRegistry.SetUserConn(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TRemoteRegistry.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TRemoteRegistry.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TRemoteRegistry.SetDllPassword(Password: AnsiString);
begin
   if fDllPassword=trim(password) then
      exit;
   fDllPassword:=password;
end;

procedure TRemoteRegistry.SetRootKey(aRootKey: String);      
var
   tmpstr: String;
begin
   tmpstr:=uppercase(trim(aRootKey));
   if fRootKey=tmpstr then
      exit;
   if (tmpstr<>'HKEY_CLASSES_ROOT')
      and (tmpstr<>'HKEY_CURRENT_USER')
      and (tmpstr<>'HKEY_LOCAL_MACHINE')
      and (tmpstr<>'HKEY_USERS')
      and (tmpstr<>'HKEY_PERFORMANCE_DATA')
      and (tmpstr<>'HKEY_CURRENT_CONFIG')
      and (tmpstr<>'HKEY_DYN_DATA') then
      begin
         messagebox(GetActiveWindow,'Invalid root key string! ','Invalid',0);
         exit;
      end;
   fRootKey:=tmpstr;
end;

procedure TRemoteRegistry.SetTargetNodeId(NodeId: AnsiString);      
begin
   if fTargetNodeId=NodeId then
      exit;
   fTargetNodeId:=NodeId;
end;

function TRemoteRegistry.BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream,Stream1: TMemoryStream;
   ok: boolean;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Stream1:=TMemoryStream.Create;
   try
      ok:=InParcel.SaveToStream(Stream);
      if ok then
         begin
            Stream.Position:=0;
            GZCompressStream(Stream,Stream1);
            ok:=(Stream1.size>0);
         end;
   except
      ok:=false;
   end;
   if not ok then
      begin
         Stream.Free;
         Stream1.Free;
         result:=false;
         exit;
      end;
   Stream1.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',53);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   Parcel.PutStreamGoods('InParcelStream',Stream1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('OutParcelStream') then
                        begin
                           Stream.Clear;
                           Stream1.Clear;
                           try
                              Parcel.GetStreamGoods('OutParcelStream',Stream);
                              GZDecompressStream(Stream,Stream1);
                              Stream1.Position:=0;
                              OutParcel.Clear;
                              result:=OutParcel.LoadFromStream(Stream1);
                           except
                              result:=false;
                           end;
                        end
                     else
                        Result:=true;
                  end;
            end;
         Parcel.Free;
      end;
   Stream.Free;
   Stream1.Free;
end;

function TRemoteRegistry.CreateKey(aKeyName: String): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',1);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.DeleteKey(aKeyName: String): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',2);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.DeleteValue(aKeyName: String; aValueName: String): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',3);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.GetDataSize(aKeyName: String; aValueName: String): integer;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=-1;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',4);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   if not BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      result:=-1
   else
      Result:=OutParcel.GetIntegerGoods('DataSize');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.GetDataType(aKeyName: String; aValueName: String): integer;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=0;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',5);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   if not BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      result:=0
   else
      Result:=OutParcel.GetIntegerGoods('DataType');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.GetKeyInfo(aKeyName: String; var aKeyInfo: TRegkeyinfo): boolean;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',6);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel);
   if Result then
      begin
         aKeyInfo.NumSubKeys:=OutParcel.GetIntegerGoods('NumSubKeys');
         aKeyInfo.maxsubkeylen:=OutParcel.GetIntegerGoods('MaxSubKeyLen');
         aKeyInfo.numvalues:=OutParcel.GetIntegerGoods('NumValues');
         aKeyInfo.maxvaluelen:=OutParcel.GetIntegerGoods('MaxValueLen');
         aKeyInfo.MaxDataLen:=OutParcel.GetIntegerGoods('MaxDataLen');
         aKeyInfo.Filetime:=OutParcel.GetDateTimeGoods('FileTime');
      end;
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.GetKeyNames(aKeyName: String; var aKeyNames: String): boolean;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',7);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel);
   if Result then
      aKeyNames:=OutParcel.GetStringGoods('KeyNameList')
   else
      aKeyNames:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.GetValueNames(aKeyName: String; var aValueNames: String): boolean;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',8);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel);
   if Result then
      aValueNames:=OutParcel.GetStringGoods('ValueNameList')
   else
      aValueNames:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.HasSubKeys(aKeyName: String; var SubKeyExists: boolean): boolean;
var
   InParcel, OutParcel: TQBParcel;
begin
   SubKeyExists:=false;
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',9);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel);
   if Result then
      SubKeyExists:=OutParcel.GetBooleanGoods('HasSubKeys');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.KeyExists(aKeyName: String; var KeyExists: boolean): boolean;
var
   InParcel, OutParcel: TQBParcel;
begin
   KeyExists:=false;
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',10);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel);
   if Result then
      KeyExists:=OutParcel.GetBooleanGoods('KeyExists');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.MoveKey(OldKeyName: String; NewKeyName: String; DeleteOld: boolean): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',11);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('OldKey',OldKeyName);
   InParcel.PutStringGoods('NewKey',NewKeyName);
   InParcel.PutBooleanGoods('DeleteOld',DeleteOld);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.ReadString(aKeyName: String; aValueName: String): String;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:='';
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',1);
   if BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      Result:=OutParcel.GetStringGoods('StringValue')
   else
      Result:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.ReadInteger(aKeyName: String; aValueName: String): Integer;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=0;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',2);
   if BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      Result:=OutParcel.GetIntegerGoods('IntegerValue')
   else
      Result:=0;
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.ReadFloat(aKeyName: String; aValueName: String): Double;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=0;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',3);
   if BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      Result:=OutParcel.GetDoubleGoods('FloatValue')
   else
      Result:=0;
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.ReadCurrency(aKeyName: String; aValueName: String): Currency;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=0;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',4);
   if BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      Result:=OutParcel.GetCurrencyGoods('CurrencyValue')
   else
      Result:=0;
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.ReadBool(aKeyName: String; aValueName: String): Boolean;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',5);
   if BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      Result:=OutParcel.GetBooleanGoods('BooleanValue')
   else
      Result:=false;
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.ReadDate(aKeyName: String; aValueName: String): TDateTime;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=strtodate('1900-01-01');
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',6);
   if BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      Result:=OutParcel.GetDateGoods('DateValue')
   else
      result:=strtodate('1900-01-01');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.ReadTime(aKeyName: String; aValueName: String): TDateTime;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=strtodate('1900-01-01');
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',7);
   if BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      Result:=OutParcel.GetTimeGoods('TimeValue')
   else
      result:=strtodate('1900-01-01');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.ReadDateTime(aKeyName: String; aValueName: String): TDateTime;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=strtodate('1900-01-01');
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',8);
   if BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      Result:=OutParcel.GetTimeGoods('DateTimeValue')
   else
      result:=strtodate('1900-01-01');
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.ReadBinaryData(aKeyName: String; aValueName: String): AnsiString;
var
   InParcel, OutParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:='';
         exit;
      end;
   OutParcel:=TQBParcel.Create;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',12);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',9);
   if BinaryRPC('RemoteRegistry',fdllpassword,InParcel,OutParcel) then
      Result:=OutParcel.GetBinaryGoods('BinaryValue')
   else
      result:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteRegistry.WriteString(aKeyName: String; aValueName: String; aValue: String): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',1);
   InParcel.PutStringGoods('StringValue',aValue);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.WriteInteger(aKeyName: String; aValueName: String; aValue: Integer): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',2);
   InParcel.PutIntegerGoods('IntegerValue',aValue);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.WriteFloat(aKeyName: String; aValueName: String; aValue: Double): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',3);
   InParcel.PutDoubleGoods('FloatValue',aValue);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.WriteCurrency(aKeyName: String; aValueName: String; aValue: Currency): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',4);
   InParcel.PutCurrencyGoods('CurrencyValue',aValue);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.WriteBool(aKeyName: String; aValueName: String; aValue: Boolean): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',5);
   InParcel.PutBooleanGoods('BooleanValue',aValue);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.WriteDate(aKeyName: String; aValueName: String; aValue: TDateTime): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',6);
   InParcel.PutDateGoods('DateValue',aValue);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.WriteTime(aKeyName: String; aValueName: String; aValue: TDateTime): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',7);
   InParcel.PutTimeGoods('TimeValue',aValue);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.WriteDateTime(aKeyName: String; aValueName: String; aValue: TDateTime): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',8);
   InParcel.PutDateTimeGoods('DateTimeValue',aValue);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.WriteBinaryData(aKeyName: String; aValueName: String; aValue: AnsiString): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',13);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('Name',aValueName);
   InParcel.PutIntegerGoods('ValueType',9);
   InParcel.PutBinaryGoods('BinaryValue',aValue);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.LoadFromFile(aFileName: String): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',14);
   InParcel.PutStringGoods('RegFileName',aFileName);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteRegistry.SaveToFile(aKeyName: String; aFileName: String): boolean;
var
   InParcel: TQBParcel;
begin
   if (not assigned(fUserConn)) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteRegistry');
   InParcel.PutIntegerGoods('FunctionId',15);
   InParcel.PutLongWordGoods('RootKey',GetRootKey);
   InParcel.PutStringGoods('Key',aKeyName);
   InParcel.PutStringGoods('RegFileName',aFileName);
   Result:=BinaryRPC('RemoteRegistry',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

end.
