//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit RemoteService;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
AnsiStrings,
{$ENDIF}
  UserConnection, TaskCommon, QBZlibExGz, QBCommon, QBParcel
{$IFDEF VER130}
;
{$ELSE}
,MidasLib;
{$ENDIF}

type
  TRemoteService = class(TComponent)
  private
     fUserConn: TUserConnection;                
     fTaskTimeout: integer;                     
     fDllPassword: AnsiString;                  
     fTargetNodeId: AnsiString;                 
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetUserConn(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetDllPassword(Password: AnsiString);
     procedure SetTargetNodeId(NodeId: AnsiString);
     function BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;           
     Destructor Destroy(); override;                             
     function StartWinService(SvcName: String): Boolean;
     function StopWinService(SvcName: String): Boolean;
     function InstallWinService(SvcName, SvcDesc, FileName: String): Boolean;
     function RemoveWinService(SvcName: String): Boolean;
     function GetWinServiceStatus(SvcName: String): String;
     function DllRegister(aDllFileName: String): Boolean;
     function DllUnregister(aDllFileName: String): Boolean;
     function RunRegister(RunName, aExeFileName: String): boolean;
     function RunUnregister(RunName: String): boolean;

  published
     property UserConn: TUserConnection read fUserConn write SetUserConn;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;          
     property DllPassword: AnsiString read fDllPassword write SetDllPassword;             
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;          

  end;

procedure Register;

implementation

procedure Register;
begin
   RegisterComponents('QuickBurro Standard', [TRemoteService]);
end;

constructor TRemoteService.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;                  
   fTargetNodeId:='';
end;

destructor TRemoteService.Destroy;
begin
   inherited;
end;

procedure TRemoteService.SetUserConn(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TRemoteService.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TRemoteService.SetTargetNodeId(NodeId: AnsiString);      
begin
   if fTargetNodeId=NodeId then
      exit;
   fTargetNodeId:=NodeId;
end;

procedure TRemoteService.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TRemoteService.SetDllPassword(Password: AnsiString);
begin
   if fDllPassword=trim(password) then
      exit;
   fDllPassword:=password;
end;

function TRemoteService.BinaryRPC(aPluginId: AnsiString; aPluginPassword: AnsiString; InParcel: TQBParcel; OutParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
   Stream,Stream1: TMemoryStream;
   ok: boolean;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         result:=false;
         exit;
      end;
   Stream:=TMemoryStream.Create;
   Stream1:=TMemoryStream.Create;
   try
      ok:=InParcel.SaveToStream(Stream);
      if ok then
         begin
            Stream.Position:=0;
            GZCompressStream(Stream,Stream1);
            ok:=(Stream1.size>0);
         end;
   except
      ok:=false;
   end;
   if not ok then
      begin
         Stream.Free;
         Stream1.Free;
         result:=false;
         exit;
      end;
   Stream1.Position:=0;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',53);
   Parcel.PutEncryptStringGoods('PluginId',aPluginId);
   Parcel.PutEncryptStringGoods('PluginPassword',aPluginPassword);
   Parcel.PutStreamGoods('InParcelStream',Stream1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Parcel.GoodsExists('OutParcelStream') then
                        begin
                           Stream.Clear;
                           Stream1.Clear;
                           try
                              Parcel.GetStreamGoods('OutParcelStream',Stream);
                              GZDecompressStream(Stream,Stream1);
                              Stream1.Position:=0;
                              OutParcel.Clear;
                              result:=OutParcel.LoadFromStream(Stream1);
                           except
                              result:=false;
                           end;
                        end
                     else
                        Result:=true;
                  end;
            end;
         Parcel.Free;
      end;
   Stream.Free;
   Stream1.Free;
end;

function TRemoteService.GetWinServiceStatus(SvcName: String): String;
var
   InParcel,OutParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteSvcManager');
   InParcel.PutIntegerGoods('FunctionId',1);
   InParcel.PutStringGoods('ServiceName',SvcName);
   OutParcel:=TQBParcel.Create;
   if BinaryRPC('RemoteSvcManager',fdllpassword,InParcel,OutParcel) then
      result:=OutParcel.GetStringGoods('ServiceStatus')
   else
      Result:='';
   InParcel.Free;
   OutParcel.Free;
end;

function TRemoteService.InstallWinService(SvcName, SvcDesc, FileName: String): Boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteSvcManager');
   InParcel.PutIntegerGoods('FunctionId',2);
   InParcel.PutStringGoods('ServiceName',SvcName);
   InParcel.PutStringGoods('ServiceDesc',SvcDesc);
   InParcel.PutStringGoods('ServiceFileName',FileName);
   result:=BinaryRPC('RemoteSvcManager',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteService.RemoveWinService(SvcName: String): Boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteSvcManager');
   InParcel.PutIntegerGoods('FunctionId',3);
   InParcel.PutStringGoods('ServiceName',SvcName);
   result:=BinaryRPC('RemoteSvcManager',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteService.StartWinService(SvcName: String): Boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteSvcManager');
   InParcel.PutIntegerGoods('FunctionId',4);
   InParcel.PutStringGoods('ServiceName',SvcName);
   result:=BinaryRPC('RemoteSvcManager',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteService.StopWinService(SvcName: String): Boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteSvcManager');
   InParcel.PutIntegerGoods('FunctionId',5);
   InParcel.PutStringGoods('ServiceName',SvcName);
   result:=BinaryRPC('RemoteSvcManager',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteService.DllRegister(aDllFileName: String): Boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteSvcManager');
   InParcel.PutIntegerGoods('FunctionId',6);
   InParcel.PutStringGoods('ComFileName',aDllFileName);
   result:=BinaryRPC('RemoteSvcManager',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteService.DllUnregister(aDllFileName: String): Boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteSvcManager');
   InParcel.PutIntegerGoods('FunctionId',7);
   InParcel.PutStringGoods('ComFileName',aDllFileName);
   result:=BinaryRPC('RemoteSvcManager',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteService.RunRegister(RunName, aExeFileName: String): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteSvcManager');
   InParcel.PutIntegerGoods('FunctionId',8);
   InParcel.PutStringGoods('RunAppName',RunName);
   InParcel.PutStringGoods('RunAppFileName',aExeFileName);
   result:=BinaryRPC('RemoteSvcManager',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

function TRemoteService.RunUnregister(RunName: String): boolean;
var
   InParcel: TQBParcel;
begin
   InParcel:=TQBParcel.Create;
   InParcel.PutStringGoods('ModuleType','Public');
   InParcel.PutStringGoods('ModuleName','RemoteSvcManager');
   InParcel.PutIntegerGoods('FunctionId',9);
   InParcel.PutStringGoods('RunAppName',RunName);
   result:=BinaryRPC('RemoteSvcManager',fdllpassword,InParcel,nil);
   InParcel.Free;
end;

end.
