//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit ScheduleTask;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  DbClient, UserConnection, QBParcel, QBCommon, QBZlibExGz, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TScheduleRecord=record
      ScheduleId: AnsiString;
      ScheduleName: AnsiString;
      ScheduleFilename: string;
      ScheduleParameters: AnsiString;
      SchedulePassword: AnsiString;
      ScheduleExecMode: integer;
      ScheduleReturnMode: integer;
      ScheduleStartupTime: AnsiString;
      ScheduleExecInterval: integer;
   end;
   
type
   TFetchScheduleListSuccessEvent = procedure (Sender: TObject) of object;
   TFetchScheduleListFailEvent = procedure (Sender: TObject) of Object;
   TAddScheduleSuccessEvent = procedure (Sender: TObject) of object;
   TAddScheduleFailEvent = procedure (Sender: TObject) of object;
   TUpdateScheduleSuccessEvent = procedure (Sender: TObject) of object;
   TUpdateScheduleFailEvent = procedure (Sender: TObject) of object;
   TRemoveScheduleSuccessEvent = procedure (Sender: TObject) of object;
   TRemoveScheduleFailEvent = procedure (Sender: TObject) of object;
   TExecuteScheduleSuccessEvent = procedure (Sender: TObject) of object;
   TExecuteScheduleFailEvent = procedure (Sender: TObject) of object;
   TExecuteExternalProgramSuccessEvent = procedure (Sender: TObject) of object;
   TExecuteExternalProgramFailEvent = procedure (Sender: TObject) of object;

type
  TScheduleTask = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fTargetNodeId: AnsiString;
     fOnFetchScheduleListSuccess: TFetchScheduleListSuccessEvent;
     fOnFetchScheduleListFail: TFetchScheduleListFailEvent;
     fOnAddScheduleSuccess: TAddScheduleSuccessEvent;
     fOnAddScheduleFail: TAddScheduleFailEvent;
     fOnUpdateScheduleSuccess: TUpdateScheduleSuccessEvent;
     fOnUpdateScheduleFail: TUpdateScheduleFailEvent;
     fOnRemoveScheduleSuccess: TRemoveScheduleSuccessEvent;
     fOnRemoveScheduleFail: TRemoveScheduleFailEvent;
     fOnExecuteScheduleSuccess: TExecuteScheduleSuccessEvent;
     fOnExecuteScheduleFail: TExecuteScheduleFailEvent;
     fOnExecuteExternalProgramSuccess: TExecuteExternalProgramSuccessEvent;
     fOnExecuteExternalProgramFail: TExecuteExternalProgramFailEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetTargetNodeId(aScheduleId: AnsiString);

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     function FetchScheduleList(aCds: TClientDataset): boolean;
     function AddSchedule(aSchedule: TScheduleRecord): boolean;
     function UpdateSchedule(OldScheduleId: AnsiString; OldSchedulePassword: AnsiString; aSchedule: TScheduleRecord): boolean;
     function RemoveSchedule(aScheduleId: AnsiString; aSchedulePassword: AnsiString): boolean;
     function ExecuteScheduleProgram(aScheduleId: AnsiString; aSchedulePassword: AnsiString; var ReturnValue: AnsiString): boolean;
     function ExecuteExternalProgram(aPrgFilename: String; aParameters: String; aReturnMode: integer; var ReturnValue: AnsiString): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property OnFetchScheduleListSuccess: TFetchScheduleListSuccessEvent read fOnFetchScheduleListSuccess write fOnFetchScheduleListSuccess;
     property OnFetchScheduleListFail: TFetchScheduleListFailEvent read fOnFetchScheduleListFail write fOnFetchScheduleListFail;
     property OnAddScheduleSuccess: TAddScheduleSuccessEvent read fOnAddScheduleSuccess write fOnAddScheduleSuccess;
     property OnAddScheduleFail: TAddScheduleFailEvent read fOnAddScheduleFail write fOnAddScheduleFail;
     property OnUpdateScheduleSuccess: TUpdateScheduleSuccessEvent read fOnUpdateScheduleSuccess write fOnUpdateScheduleSuccess;
     property OnUpdateScheduleFail: TUpdateScheduleFailEvent read fOnUpdateScheduleFail write fOnUpdateScheduleFail;
     property OnRemoveScheduleSuccess: TRemoveScheduleSuccessEvent read fOnRemoveScheduleSuccess write fOnRemoveScheduleSuccess;
     property OnRemoveScheduleFail: TRemoveScheduleFailEvent read fOnRemoveScheduleFail write fOnRemoveScheduleFail;
     property OnExecuteScheduleSuccess: TExecuteScheduleSuccessEvent read fOnExecuteScheduleSuccess write fOnExecuteScheduleSuccess;
     property OnExecuteScheduleFail: TExecuteScheduleFailEvent read fOnExecuteScheduleFail write fOnExecuteScheduleFail;
     property OnExecuteExternalProgramSuccess: TExecuteExternalProgramSuccessEvent read fOnExecuteExternalProgramSuccess write fOnExecuteExternalProgramSuccess;
     property OnExecuteExternalProgramFail: TExecuteExternalProgramFailEvent read fOnExecuteExternalProgramFail write fOnExecuteExternalProgramFail;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TScheduleTask]);
end;

procedure TScheduleTask.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TScheduleTask.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TScheduleTask.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TScheduleTask.SetTargetNodeId(aScheduleId: AnsiString);
begin
   if StrIComp(PAnsiChar(aScheduleId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aScheduleId;
end;

constructor TScheduleTask.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   LastErrorCode:='';
   LastErrorText:='';
end;

destructor TScheduleTask.Destroy;
begin
   fOnFetchScheduleListSuccess:=nil;
   fOnFetchScheduleListFail:=nil;
   fOnAddScheduleSuccess:=nil;
   fOnAddScheduleFail:=nil;
   fOnUpdateScheduleSuccess:=nil;
   fOnUpdateScheduleFail:=nil;
   fOnRemoveScheduleSuccess:=nil;
   fOnRemoveScheduleFail:=nil;
  inherited;
end;

function TScheduleTask.FetchScheduleList(aCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0203702';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnFetchScheduleListFail) then
            fOnFetchScheduleListFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',37);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     aCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('ScheduleList',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        aCds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if result then
                        begin
                           if Assigned(fOnFetchScheduleListSuccess) then
                              fOnFetchScheduleListSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0203705';
                           LastErrorText:='Decompress the return dataset fail.';
                           if Assigned(fOnFetchScheduleListFail) then
                              fOnFetchScheduleListFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnFetchScheduleListFail) then
                        fOnFetchScheduleListFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0203703';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnFetchScheduleListFail) then
                  fOnFetchScheduleListFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0203704';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnFetchScheduleListFail) then
            fOnFetchScheduleListFail(self);
      end;
end;

function TScheduleTask.AddSchedule(aSchedule: TScheduleRecord): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0203804';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnAddScheduleFail) then
            fOnAddScheduleFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',38);
   Parcel.PutEncryptStringGoods('ScheduleId',aSchedule.ScheduleId);
   Parcel.PutEncryptStringGoods('ScheduleName',aSchedule.ScheduleName);
   Parcel.PutStringGoods('ScheduleFilename',aSchedule.ScheduleFilename);
   Parcel.PutEncryptStringGoods('ScheduleParameters',aSchedule.ScheduleParameters);
   Parcel.PutEncryptStringGoods('SchedulePassword',aSchedule.SchedulePassword);
   Parcel.PutIntegerGoods('ScheduleExecMode',aSchedule.ScheduleExecMode);
   Parcel.PutIntegerGoods('ScheduleReturnMode',aSchedule.ScheduleReturnMode);
   Parcel.PutEncryptStringGoods('ScheduleStartupTime',aSchedule.ScheduleStartupTime);
   Parcel.PutIntegerGoods('ScheduleExecInterval',aSchedule.ScheduleExecInterval);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnAddScheduleSuccess) then
                        fOnAddScheduleSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnAddScheduleFail) then
                        fOnAddScheduleFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0203805';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnAddScheduleFail) then
                  fOnAddScheduleFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0203806';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnAddScheduleFail) then
            fOnAddScheduleFail(self);
      end;
end;

function TScheduleTask.UpdateSchedule(OldScheduleId: AnsiString; OldSchedulePassword: AnsiString; aSchedule: TScheduleRecord): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0203904';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnUpdateScheduleFail) then
            fOnUpdateScheduleFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',39);
   Parcel.PutEncryptStringGoods('OldScheduleId',OldScheduleId);
   Parcel.PutEncryptStringGoods('OldSchedulePassword',OldSchedulePassword);
   Parcel.PutEncryptStringGoods('ScheduleId',aSchedule.ScheduleId);
   Parcel.PutEncryptStringGoods('ScheduleName',aSchedule.ScheduleName);
   Parcel.PutStringGoods('ScheduleFilename',aSchedule.ScheduleFilename);
   Parcel.PutEncryptStringGoods('ScheduleParameters',aSchedule.ScheduleParameters);
   Parcel.PutEncryptStringGoods('SchedulePassword',aSchedule.SchedulePassword);
   Parcel.PutIntegerGoods('ScheduleExecMode',aSchedule.ScheduleExecMode);
   Parcel.PutIntegerGoods('ScheduleReturnMode',aSchedule.ScheduleReturnMode);
   Parcel.PutEncryptStringGoods('ScheduleStartupTime',aSchedule.ScheduleStartupTime);
   Parcel.PutIntegerGoods('ScheduleExecInterval',aSchedule.ScheduleExecInterval);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnUpdateScheduleSuccess) then
                        fOnUpdateScheduleSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnUpdateScheduleFail) then
                        fOnUpdateScheduleFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0203905';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnUpdateScheduleFail) then
                  fOnUpdateScheduleFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0203906';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnUpdateScheduleFail) then
            fOnUpdateScheduleFail(self);
      end;
end;

function TScheduleTask.RemoveSchedule(aScheduleId: AnsiString; aSchedulePassword: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204004';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnRemoveScheduleFail) then
            fOnRemoveScheduleFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',40);
   Parcel.PutEncryptStringGoods('ScheduleId',aScheduleId);
   Parcel.PutEncryptStringGoods('SchedulePassword',aSchedulePassword);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnRemoveScheduleSuccess) then
                        fOnRemoveScheduleSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnRemoveScheduleFail) then
                        fOnRemoveScheduleFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0204005';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnRemoveScheduleFail) then
                  fOnRemoveScheduleFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204006';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnRemoveScheduleFail) then
            fOnRemoveScheduleFail(self);
      end;
end;

function TScheduleTask.ExecuteScheduleProgram(aScheduleId: AnsiString; aSchedulePassword: AnsiString; var ReturnValue: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   ReturnValue:='';
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204104';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnExecuteScheduleFail) then
            fOnExecuteScheduleFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',41);
   Parcel.PutEncryptStringGoods('ScheduleId',aScheduleId);
   Parcel.PutEncryptStringGoods('SchedulePassword',aSchedulePassword);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     ReturnValue:=Parcel.GetEncryptStringGoods('ReturnValue');
                     if Assigned(fOnExecuteScheduleSuccess) then
                        fOnExecuteScheduleSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnExecuteScheduleFail) then
                        fOnExecuteScheduleFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0204105';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnExecuteScheduleFail) then
                  fOnExecuteScheduleFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204106';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnExecuteScheduleFail) then
            fOnExecuteScheduleFail(self);
      end;
end;

function TScheduleTask.ExecuteExternalProgram(aPrgFilename: String; aParameters: String; aReturnMode: integer; var ReturnValue: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   ReturnValue:='';
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0204204';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnExecuteExternalProgramFail) then
            fOnExecuteExternalProgramFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',42);
   Parcel.PutStringGoods('ProgramFilename',aPrgFilename);
   Parcel.PutStringGoods('Parameters',aParameters);
   Parcel.PutIntegerGoods('ReturnMode',aReturnMode);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     ReturnValue:=Parcel.GetEncryptStringGoods('ReturnValue');
                     if Assigned(fOnExecuteExternalProgramSuccess) then
                        fOnExecuteExternalProgramSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnExecuteExternalProgramFail) then
                        fOnExecuteExternalProgramFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0204205';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnExecuteExternalProgramFail) then
                  fOnExecuteExternalProgramFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0204206';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnExecuteExternalProgramFail) then
            fOnExecuteExternalProgramFail(self);
      end;
end;

procedure TScheduleTask.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=String(LastErrorCode);
   ErrorText:=String(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
