//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit SourceMemberHeartbeat;

interface

uses
   windows, Messages, SysUtils, Classes, Forms, QBParcel, TaskCommon;

type
   TSourceMemberHeartbeatThread=class(TThread)
   private
      MainObj: TComponent;
      HeartbeatInterval: integer;
      LastDateTime: TDateTime;
      RequestBody,ResponseBody: AnsiString;

   protected
      procedure Execute; override;

   public
      Constructor Create(ParentObj: TComponent; SuspendedOnCreate: boolean); Overload;
      Destructor Destroy; Override;

   end;

implementation

uses MsgSource;

Constructor TSourceMemberHeartbeatThread.Create(ParentObj: TComponent; SuspendedOnCreate: boolean);
begin
   Inherited Create(SuspendedOnCreate);
   freeOnTerminate:=true;
   MainObj:=ParentObj;
   HeartbeatInterval:=120;
   LastDateTime:=now;
end;

Destructor TSourceMemberHeartbeatThread.Destroy;
begin
   Inherited Destroy;
end;

procedure TSourceMemberHeartbeatThread.Execute;
var
   MainSource: TMsgSource;
   Parcel: TQBParcel;
   ok: boolean;
   t0: TDateTime;
   i: integer;
begin
   MainSource:=(MainObj as TMsgSource);
   while not terminated do
      begin
         t0:=now;
         while (not terminated) and (round((now-t0)*24*60*60)<5) do
            sleep(50);
         if not terminated then
            begin
               EnterCriticalSection(MainSource.ActiveSourceCs);
               i:=0;
               while i<MainSource.ActiveSourceCount do
                  begin
                     if terminated then
                        break;
                     if round((now-MainSource.ActiveSources[i].LastActiveTime)*24*60*60)<HeartbeatInterval then
                        begin
                           inc(i);
                           continue;
                        end;
                     Parcel:=TQBParcel.Create;
                     Parcel.EncryptKey:=MainSource.UserConnection.TransferKey;
                     Parcel.PutEncryptStringGoods('SystemFlag','Message_Request');
                     Parcel.PutInt64Goods('SessionId',MainSource.UserConnection.PublicSessionId);
                     Parcel.PutIntegerGoods('RequestId',7);
                     Parcel.PutEncryptStringGoods('SourceId',MainSource.ActiveSources[i].SourceId);
                     Parcel.PutEncryptStringGoods('UserId',MainSource.UserConnection.UserId+'@'+MainSource.UserConnection.UserNodeId);
                     Parcel.SaveToStringWithLength(RequestBody);
                     Parcel.Free;
                     ok:=RemoteMsgExecute(MainSource.UserConnection,MainSource.TargetNodeId,RequestBody,MainSource.TaskTimeout,ResponseBody);
                     if ok then
                        begin
                           Parcel:=TQBParcel.Create;
                           Parcel.EncryptKey:=MainSource.UserConnection.TransferKey;
                           Parcel.LoadFromString(ResponseBody);
                           if Parcel.GoodsCount>0 then
                              begin
                                 if not Parcel.GetBooleanGoods('ProcessResult') then
                                    begin
                                       Parcel.Free;
                                       MainSource._FireAutoQuitEvent(MainSource.ActiveSources[i].SourceId);
                                    end
                                 else
                                    begin
                                       Parcel.free;
                                       inc(i);
                                    end;
                              end
                           else
                              begin
                                 Parcel.Free;
                                 MainSource._FireAutoQuitEvent(MainSource.ActiveSources[i].SourceId);
                              end;
                        end
                     else
                        MainSource._FireAutoQuitEvent(MainSource.ActiveSources[i].SourceId);
                  end;
               LeaveCriticalSection(MainSource.ActiveSourceCs);
            end;
      end;
end;

end.
