//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.25.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit SysReporter;

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  DbClient, UserConnection, QBParcel, QBCommon, QBZlibExGz, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   THelloNodeSuccessEvent = procedure (Sender: TObject) of object;
   THelloNodeFailEvent = procedure (Sender: TObject) of Object;
   TFetchStatusSuccessEvent = procedure (Sender: TObject) of object;
   TFetchStatusFailEvent = procedure (Sender: TObject) of Object;

type
  TSysReporter = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fTargetNodeId: AnsiString;
     fOnFetchStatusSuccess: TFetchStatusSuccessEvent;
     fOnFetchStatusFail: TFetchStatusFailEvent;
     fOnHelloNodeSuccess: THelloNodeSuccessEvent;
     fOnHelloNodeFail: THelloNodeFailEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetTargetNodeId(aDatabaseId: AnsiString);

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     function FetchNodeStatus(StatusParcel: TQBParcel): boolean;
     function HelloNode: boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property OnFetchStatusSuccess: TFetchStatusSuccessEvent read fOnFetchStatusSuccess write fOnFetchStatusSuccess;
     property OnFetchStatusFail: TFetchStatusFailEvent read fOnFetchStatusFail write fOnFetchStatusFail;
     property OnHelloNodeSuccess: THelloNodeSuccessEvent read fOnHelloNodeSuccess write fOnHelloNodeSuccess;
     property OnHelloNodeFail: THelloNodeFailEvent read fOnHelloNodeFail write fOnHelloNodeFail;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TSysReporter]);
end;

procedure TSysReporter.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TSysReporter.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TSysReporter.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TSysReporter.SetTargetNodeId(aDatabaseId: AnsiString);
begin
   if StrIComp(PAnsiChar(aDatabaseId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aDatabaseId;
end;

constructor TSysReporter.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   LastErrorCode:='';
   LastErrorText:='';
end;

destructor TSysReporter.Destroy;
begin
   fOnHelloNodeSuccess:=nil;
   fOnHelloNodeFail:=nil;
   fOnFetchStatusSuccess:=nil;
   fOnFetchStatusFail:=nil;
  inherited;
end;

function TSysReporter.FetchNodeStatus(StatusParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0207901';
         LastErrorText:='System not ready to commit task.';
         if assigned(fOnFetchStatusFail) then
            fOnFetchStatusFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',79);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     StatusParcel.Clear;
                     result:=Parcel.GetParcelGoods('SystemStatus',StatusParcel);
                     if result then
                        begin
                           if assigned(fOnFetchStatusSuccess) then
                              fOnFetchStatusSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0207902';
                           LastErrorText:='Read status parcel failed.';
                           if assigned(fOnFetchStatusFail) then
                              fOnFetchStatusFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:='0207903';
                     LastErrorText:='The response message is invalid.';
                     if assigned(fOnFetchStatusFail) then
                        fOnFetchStatusFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0207904';
               LastErrorText:='The response message is invalid.';
               if assigned(fOnFetchStatusFail) then
                  fOnFetchStatusFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0207905';
         LastErrorText:='Communication error on commit task.';
         if assigned(fOnFetchStatusFail) then
            fOnFetchStatusFail(self);
      end;
end;

function TSysReporter.HelloNode: boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0208001';
         LastErrorText:='System not ready to commit task.';
         if assigned(fOnHelloNodeFail) then
            fOnHelloNodeFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',80);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if assigned(fOnHelloNodeSuccess) then
                        fOnHelloNodeSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:='0207902';
                     LastErrorText:='The response message is invalid.';
                     if assigned(fOnHelloNodeFail) then
                        fOnHelloNodeFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0207903';
               LastErrorText:='The response message is invalid.';
               if assigned(fOnHelloNodeFail) then
                  fOnHelloNodeFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0207904';
         LastErrorText:='Communication error on commit task.';
         if assigned(fOnHelloNodeFail) then
            fOnHelloNodeFail(self);
      end;
end;

procedure TSysReporter.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=String(LastErrorCode);
   ErrorText:=String(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
