//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit TaskCommon;

interface

uses
  Windows, SysUtils, Classes, Forms, BurroSocket,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  UserConnection;

  function RemoteExecute(UserConn: TUserConnection; TargetNodeId: AnsiString; RequestData: AnsiString; Timeout: integer; var ResponseData: AnsiString): boolean;
  function RemoteMsgExecute(UserConn: TUserConnection; TargetNodeId: AnsiString; RequestData: AnsiString; Timeout: integer; var ResponseData: AnsiString): boolean;
  function ConnectToServer(UserConn: TUserConnection; SocketIndex: integer; IpAddress: AnsiString; Port: integer): boolean;
  function SendRequest(UserConn: TUserConnection; SocketIndex: integer; RequestData: AnsiString; Timeout: integer): boolean;
  function ReceiveResponse(UserConn: TUserConnection; SocketIndex: integer; Timeout: integer; var ResponseData: AnsiString): boolean;

implementation

function RemoteExecute(UserConn: TUserConnection; TargetNodeId: AnsiString; RequestData: AnsiString; Timeout: integer; var ResponseData: AnsiString): boolean;
var
   ip: ansistring;
   MsgPort,TaskPort: integer;
   ok: boolean;
   SocketIndex: integer;
begin
   ok:=UserConn.LocateNode(TargetNodeId,Ip,MsgPort,TaskPort);
   if not ok then
      begin
         result:=false;
         exit;
      end;
   SocketIndex:=UserConn.AllocSocket;
   ok:=ConnectToServer(UserConn,SocketIndex,ip,TaskPort);
   if not ok  then
      begin
         TThread.Synchronize(nil,UserConn.Sockets[SocketIndex].Socket.Close);
         UserConn.FreeSocket(SocketIndex);
         result:=false;
         exit;
      end;
   ok:=SendRequest(UserConn,SocketIndex,RequestData,Timeout);
   if not ok then
      begin
         TThread.Synchronize(nil,UserConn.Sockets[SocketIndex].Socket.Close);
         UserConn.FreeSocket(SocketIndex);
         result:=false;
         exit;
      end;
   result:=ReceiveResponse(UserConn,SocketIndex,Timeout,ResponseData);
   TThread.Synchronize(nil,UserConn.Sockets[SocketIndex].Socket.Close);
   UserConn.FreeSocket(SocketIndex);
end;

function RemoteMsgExecute(UserConn: TUserConnection; TargetNodeId: AnsiString; RequestData: AnsiString; Timeout: integer; var ResponseData: AnsiString): boolean;
var
   ip: ansistring;
   MsgPort,TaskPort: integer;
   ok: boolean;
   SocketIndex: integer;
begin
   ok:=UserConn.LocateNode(TargetNodeId,Ip,MsgPort,TaskPort);
   if not ok then
      begin
         result:=false;
         exit;
      end;
   SocketIndex:=UserConn.AllocSocket;
   ok:=ConnectToServer(UserConn,SocketIndex,ip,MsgPort);       
   if not ok  then
      begin
         TThread.Synchronize(nil,UserConn.Sockets[SocketIndex].Socket.Close);
         UserConn.FreeSocket(SocketIndex);
         result:=false;
         exit;
      end;
   ok:=SendRequest(UserConn,SocketIndex,RequestData,Timeout);
   if not ok then
      begin
         TThread.Synchronize(nil,UserConn.Sockets[SocketIndex].Socket.Close);
         UserConn.FreeSocket(SocketIndex);
         result:=false;
         exit;
      end;
   result:=ReceiveResponse(UserConn,SocketIndex,Timeout,ResponseData);
   TThread.Synchronize(nil,UserConn.Sockets[SocketIndex].Socket.Close);
   UserConn.FreeSocket(SocketIndex);
end;

function ConnectToServer(UserConn: TUserConnection; SocketIndex: integer; IpAddress: AnsiString; Port: integer): boolean;
var
   t0: TDateTime;
begin
   try
      UserConn.Sockets[SocketIndex].Connecting:=true;
      UserConn.Sockets[SocketIndex].Connected:=false;
      UserConn.Sockets[SocketIndex].Socket.Host:=IpAddress;
      UserConn.Sockets[SocketIndex].Socket.Port:=Port;
      TThread.Synchronize(nil,UserConn.Sockets[SocketIndex].Socket.Open);
      result:=true;
   except
      result:=false;
   end;
   if result then
      begin
         t0:=now;
         while UserConn.Sockets[SocketIndex].Connecting and (round((now-t0)*24*60*60)<8) do
            begin
               application.processmessages;
               sleep(1);
            end;
         result:=UserConn.Sockets[SocketIndex].Connected;
      end;
end;

function SendRequest(UserConn: TUserConnection; SocketIndex: integer; RequestData: AnsiString; Timeout: integer): boolean;
var
   t0: TDateTime;
   j: integer;
begin
   result:=false;
   t0:=now;
   while UserConn.Sockets[SocketIndex].Connected and (round((now-t0)*24*60*60)<Timeout) do
      begin
         try
            j:=UserConn.sockets[SocketIndex].Socket.Socket.SendBuf(RequestData[1],length(RequestData));
            result:=(j<>-1);
            if result then
               break;
            application.ProcessMessages;
            sleep(1);
         except
            break;
         end;
      end;
end;

function ReceiveResponse(UserConn: TUserConnection; SocketIndex: integer; Timeout: integer; var ResponseData: AnsiString): boolean;
var
   tmpstr: AnsiString;
   AllLen,k,j: integer;
   t1: TDateTime;
begin
   ResponseData:='';
   AllLen:=0;
   k:=0;
   setlength(tmpstr,65536);
   result:=false;
   t1:=now;
   while (round((now-t1)*24*60*60)<Timeout) and UserConn.sockets[SocketIndex].Connected do
      begin
         try
            k:=UserConn.sockets[SocketIndex].Socket.Socket.ReceiveBuf(tmpstr[1],65536);
         except
            break;
         end;
         if k>0 then
            ResponseData:=ResponseData+copy(tmpstr,1,k)
         else
            begin
               application.ProcessMessages;
               sleep(1);
               continue;
            end;
         if AllLen<=0 then
            begin
               j:=pos(AnsiString(#13),ResponseData);
               if j<=0 then
                  begin
                     if length(ResponseData)>20 then
                        break;
                     application.ProcessMessages;
                     sleep(1);
                     continue;
                  end;
               if j=1 then
                  break;
               try
                  AllLen:=StrToInt(string(copy(ResponseData,1,j-1)));
                  if (AllLen<=0) or (AllLen>16*1024*1024) then
                     AllLen:=-1;
               except
                  AllLen:=-1;
               end;
               if AllLen<=0 then
                  Break;
               delete(ResponseData,1,j);
            end;
         if length(ResponseData)>=AllLen then
            begin
               result:=true;
               break;
            end;
         application.ProcessMessages;
         sleep(1);
      end;
end;

end.
 
