//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit UserConnection;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
   Windows, Messages, SysUtils, Classes, Forms,
   BurroSocket, QBParcel, QBTimer, QBCommon,
   {$IFDEF UNICODE}
   AnsiStrings,
   {$ENDIF}
   UserRequest, UserResponse, UserHeartbeat;

type
   TOnConnectSuccess = procedure(Sender: TObject) of Object;
   TOnConnectFail = procedure(Sender: TObject) of Object;
   TOnLoginSuccess = procedure(Sender: TObject) of Object;
   TOnLoginFail = procedure(Sender: TObject) of Object;
   TOnLogoutSuccess = procedure(Sender: TObject) of Object;
   TOnLogoutFail = procedure(Sender: TObject) of Object;
   TOnDisconnect = procedure(Sender: TObject) of Object;
   TOnUserSupplant = procedure(Sender: TObject) of Object;
   TMessageBroker = procedure(MsgParcel: TQBParcel; FromId: AnsiString) of Object;

type
   TSocketRecord=record
      Used: boolean;
      Socket: TBClientSocket;
      Connecting: boolean;
      Connected: boolean;
   end;

type
   TNodeAddressRecord=record
      NodeId: AnsiString;
      NodeName: AnsiString;
      NodeType: integer;
      NodeAddress: AnsiString;              
      MsgPort: integer;
      TaskPort: integer;
      IpAddress: AnsiString;                
      LastActiveDateTime: TDateTime;        
   end;

type
   TUserConnection=class(TComponent)
   private
      fTransferKey: AnsiString;               
      fRootNodeId: AnsiString;                
      fRootNodeAddress: AnsiString;           
      fRootNodeAddressingPort: integer;       
      fStandbyRootNodeAddress: AnsiString;    
      fStandbyRootNodeAddressingPort: integer;{󱸸ڵѰַ˿}
      fUserNodeId: AnsiString;                
      fUserNodeAddress: AnsiString;           
      fUserNodeMsgPort: integer;              
      fUserNodeTaskPort: integer;             
      fUserHeartbeatInterval: integer;        
      fUserId: AnsiString;                    
      fUserPassword: AnsiString;              
      fConnected: boolean;                    
      fUserLogin: boolean;                    
      fSessionId: int64;                      
      fPublicSessionId: int64;                
      fUserNodeIP: AnsiString;                
      fUserName: AnsiString;                  
      fUserType: integer;                     
      fOnConnectSuccess: TOnConnectSuccess;
      fOnConnectFail: TOnConnectFail;
      fOnLoginSuccess: TOnLoginSuccess;
      fOnLoginFail: TOnLoginFail;
      fOnLogoutSuccess: TOnLogoutSuccess;
      fOnLogoutFail: TOnLogoutFail;
      fOnDisconnect: TOnDisconnect;
      fOnUserSupplant: TOnUserSupplant;
      fMessageBroker: TMessageBroker;
      HeartbeatThread: THeartbeatThread;
      SendRequestThread: TSendRequestThread;
      ReceiveResponseThread: TReceiveResponseThread;
      RequestQueueSize: integer;
      RequestQueue: array of TQBParcel;
      RequestQueueFore: integer;
      RequestQueueRear: integer;
      RequestQueueEmpty: boolean;
      RequestQueueFull: boolean;
      RequestQueueCs: TRTLCriticalSection;
      ResponseQueueSize: integer;
      ResponseQueue: array of TQBParcel;
      ResponseQueueFore: integer;
      ResponseQueueRear: integer;
      ResponseQueueEmpty: boolean;
      ResponseQueueFull: boolean;
      ResponseQueueCs: TRTLCriticalSection;
      Dispatcher: TQBTimer;
      DispatcherRunning: boolean;
      LoginTimer: TQBTimer;
      LogoutTimer: TQBTimer;
      NodeCount: integer;
      Nodes: array of TNodeAddressRecord;
      NodeListCs: TRTLCriticalSection;
      Connecting: boolean;
      AddrSocket: TBClientSocket;
      AddrConnecting: boolean;
      AddrConnected: boolean;
      ResponseBody: AnsiString;
      procedure SetTransferKey(aKey: AnsiString);
      procedure SetRootNodeId(NodeId: AnsiString);
      procedure SetRootNodeAddress(NodeAddress: AnsiString);
      procedure SetRootNodeAddressingPort(AddressingPort: integer);
      procedure SetStandbyRootNodeAddress(NodeAddress: AnsiString);
      procedure SetStandbyRootNodeAddressingPort(AddressingPort: integer);
      procedure SetUserNodeId(NodeId: AnsiString);
      procedure SetUserNodeAddress(NodeAddress: AnsiString);
      procedure SetUserNodeMsgPort(MsgPort: integer);
      procedure SetUserNodeTaskPort(TaskPort: integer);
      procedure SetHeartbeatInterval(aSeconds: integer);
      procedure SetUserId(aUserId: AnsiString);
      procedure SetUserPassword(aUserPassword: AnsiString);
      procedure UserSocketConnect(Sender: TObject; Socket: TBCustomWinSocket);
      procedure UserSocketRead(Sender: TObject; Socket: TBCustomWinSocket);
      procedure UserSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
      procedure UserSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
      procedure TaskSocketConnect(Sender: TObject; Socket: TBCustomWinSocket);
      procedure TaskSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
      procedure TaskSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
      procedure AddrSocketConnect(Sender: TObject; Socket: TBCustomWinSocket);
      procedure AddrSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
      procedure AddrSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
      procedure LoginTimerTimer(Sender: TObject);
      procedure LogoutTimerTimer(Sender: TObject);
      procedure DispatcherTimer(Sender: TObject);
      procedure ResolveNodes(SourceList: AnsiString);
      function ConnectToServer(IpAddress: AnsiString; Port: integer): boolean;
      function SendRequest(RequestBody: AnsiString): boolean;
      function ReceiveResponse: boolean;

   public
      UserSocket: TBClientSocket;
      HeartbeatReply: boolean;
      SocketCs: TRTLCriticalSection;
      SocketCount: integer;
      Sockets: array of TSocketRecord;
      property StandbyRootNodeAddress: AnsiString read fStandbyRootNodeAddress write SetStandbyRootNodeAddress;
      property StandbyRootNodeAddressingPort: integer read fStandbyRootNodeAddressingPort write SetStandbyRootNodeAddressingPort;
      property Connected: boolean read fConnected;
      property UserLogin: boolean read fUserLogin;
      property SessionId: int64 read fSessionId;
      property PublicSessionId: int64 read fPublicSessionId;
      property UserNodeIP: AnsiString read fUserNodeIP;
      property UserName: AnsiString read fUserName;
      property UserType: integer read fUserType;
      property _MessageBroker: TMessageBroker write fMessageBroker;
      Constructor Create(AOwner: TComponent); override;
      Destructor Destroy(); override;
      function SendHeartbeat(): boolean;
      function QueryNodeAddress(NodeId: AnsiString; var IPAddress: AnsiString; var MsgPort: integer; var TaskPort: integer): boolean;
      function Connect(): boolean;
      function Disconnect(): boolean;
      function Login(): boolean;
      function Logout(): boolean;
      function LocateNode(TargetNodeId: AnsiString; var NodeAddress: AnsiString; var MsgPort: integer; var TaskPort: integer): boolean;
      function PushRequest(aRequest: TQBParcel): boolean;
      function PopRequest(): TQBParcel;
      function PushResponse(aResponse: TQBParcel): boolean;
      function PopResponse(): TQBParcel;
      procedure StartEventDispatcher;
      function AllocSocket: integer;
      procedure FreeSocket(aSocketIndex: integer);

   published
      property TransferKey: AnsiString read fTransferKey write SetTransferKey;
      property RootNodeId: AnsiString read fRootNodeId write SetRootNodeId;
      property RootNodeAddress: AnsiString read fRootNodeAddress write SetRootNodeAddress;
      property RootNodeAddressingPort: integer read fRootNodeAddressingPort write SetRootNodeAddressingPort;
      property UserNodeId: AnsiString read fUserNodeId write SetUserNodeId;
      property UserNodeAddress: AnsiString read fUserNodeAddress write SetUserNodeAddress;
      property UserNodeMsgPort: integer read fUserNodeMsgPort write SetUserNodeMsgPort;
      property UserNodeTaskPort: integer read fUserNodeTaskPort write SetUserNodeTaskPort;
      property UserHeartbeatInterval: integer read fUserHeartbeatInterval write SetHeartbeatInterval;
      property UserId: AnsiString read fUserId write SetUserId;
      property UserPassword: AnsiString read fUserPassword write SetUserPassword;
      property OnConnectSuccess: TOnConnectSuccess read fOnConnectSuccess write fOnConnectSuccess;
      property OnConnectFail: TOnConnectFail read fOnConnectFail write fOnConnectFail;
      property OnLoginSuccess: TOnLoginSuccess read fOnLoginSuccess write fOnLoginSuccess;
      property OnLoginFail: TOnLoginFail read fOnLoginFail write fOnLoginFail;
      property OnLogoutSuccess: TOnLogoutSuccess read fOnLogoutSuccess write fOnLogoutSuccess;
      property OnLogoutFail: TOnLogoutFail read fOnLogoutFail write fOnLogoutFail;
      property OnDisconnect: TOnDisconnect read fOnDisconnect write fOnDisconnect;
      property OnUserSupplant: TOnUserSupplant read fOnUserSupplant write fOnUserSupplant;

   end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TUserConnection]);
end;

Constructor TUserConnection.Create(AOwner: TComponent);
begin
   inherited create(aOwner);
   fTransferKey:='j76s!9l1w~8_9sa00g*+zx';
   fRootNodeId:='';
   fRootNodeAddress:='';
   fRootNodeAddressingPort:=5880;
   fStandbyRootNodeAddress:='';
   fStandbyRootNodeAddressingPort:=5880;
   fUserNodeId:='';
   fUserNodeAddress:='';
   fUserNodeMsgPort:=5882;
   fUserNodeTaskPort:=5881;
   fUserId:='';
   fUserPassword:='';
   fUserHeartbeatInterval:=150;
   fUserName:='';
   fUserType:=0;
   fConnected:=false;
   fUserLogin:=false;
   fSessionId:=0;
   fPublicSessionId:=0;
   Connecting:=false;
   fUserNodeIP:='';
   RequestQueueSize:=64;
   SetLength(RequestQueue,RequestQueueSize);
   RequestQueueFore:=0;
   RequestQueueRear:=0;
   RequestQueueEmpty:=true;
   RequestQueueFull:=false;
   InitializeCriticalSection(RequestQueueCs);
   ResponseQueueSize:=512;
   SetLength(ResponseQueue,ResponseQueueSize);
   ResponseQueueFore:=0;
   ResponseQueueRear:=0;
   ResponseQueueEmpty:=true;
   ResponseQueueFull:=false;
   InitializeCriticalSection(ResponseQueueCs);
   LoginTimer:=TQBTimer.Create(nil);
   LoginTimer.interval:=30000;                     
   LoginTimer.OnTimer:=LoginTimerTimer;
   LoginTimer.Enabled:=false;
   LogoutTimer:=TQBTimer.Create(nil);
   LogoutTimer.interval:=30000;                    
   LogoutTimer.OnTimer:=LogoutTimerTimer;
   LogoutTimer.Enabled:=false;
   Dispatcher:=TQBTimer.Create(nil);
   Dispatcher.interval:=50;
   Dispatcher.OnTimer:=DispatcherTimer;
   Dispatcher.Enabled:=false;
   DispatcherRunning:=false;
   NodeCount:=0;
   SetLength(Nodes,NodeCount);
   InitializeCriticalSection(NodeListCs);
   UserSocket:=TBClientSocket.Create(nil);
   UserSocket.OnConnect:=UserSocketConnect;
   UserSocket.OnRead:=UserSocketRead;
   UserSocket.OnDisconnect:=UserSocketDisconnect;
   UserSocket.OnError:=UserSocketError;
   UserSocket.ClientType:=ctNonBlocking;
   InitializeCriticalSection(SocketCs);
   SocketCount:=0;
   SetLength(Sockets,SocketCount);
   AddrSocket:=TBClientSocket.Create(nil);
   AddrSocket.OnConnect:=AddrSocketConnect;
   AddrSocket.OnDisconnect:=AddrSocketDisconnect;
   AddrSocket.OnError:=AddrSocketError;
   AddrSocket.ClientType:=ctNonBlocking;
   if not (csDesigning in ComponentState) then
      begin
         HeartbeatThread:=THeartbeatThread.Create(self,true);
         SendRequestThread:=TSendRequestThread.Create(self,true);
         ReceiveResponseThread:=TReceiveResponseThread.Create(self,true);
      end;
end;

Destructor TUserConnection.Destroy();
var
   i: integer;
begin
   fOnConnectSuccess:=nil;
   fOnConnectFail:=nil;
   fOnLoginSuccess:=nil;
   fOnLoginFail:=nil;
   fOnLogoutSuccess:=nil;
   fOnLogoutFail:=nil;
   fOnDisconnect:=nil;
   fOnUserSupplant:=nil;
   fMessageBroker:=nil;
   LoginTimer.Enabled:=false;
   LogoutTimer.Enabled:=false;
   LoginTimer.Free;
   LogoutTimer.Free;
   UserSocket.Free;
   Dispatcher.free;
   AddrSocket.Free;
   if not (csDesigning in ComponentState) then
      begin
         if HeartbeatThread.Suspended then
            begin
               HeartbeatThread.Terminate;
               HeartbeatThread.resume;
            end
         else
            HeartbeatThread.Terminate;
         if SendRequestThread.Suspended then
            begin
               SendRequestThread.Terminate;
               SendRequestThread.resume;
            end
         else
            SendRequestThread.Terminate;
         if ReceiveResponseThread.Suspended then
            begin
               ReceiveResponseThread.Terminate;
               ReceiveResponseThread.resume;
            end
         else
            ReceiveResponseThread.Terminate;
      end;
   for i:=0 to SocketCount-1 do
      begin
         while Sockets[i].Used do
            application.ProcessMessages;
         Sockets[i].Socket.Free;
         Sockets[i].Used:=false;
      end;
   SocketCount:=0;
   SetLength(Sockets,0);
   DeleteCriticalSection(RequestQueueCs);
   DeleteCriticalSection(ResponseQueueCs);
   DeleteCriticalSection(SocketCs);
   inherited Destroy;
end;

procedure TUserConnection.SetTransferKey(aKey: AnsiString);
begin
   if fTransferKey=akey then
      exit;
   if trim(akey)='' then
      begin
         messagebox(GetActiveWindow,'Invalid transfer key! ','Invalid',0);
         exit;
      end;
   fTransferKey:=aKey;
end;

procedure TUserConnection.SetRootNodeId(NodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(trim(NodeId)),PAnsiChar(fRootNodeId))=0 then
      exit;
   if (pos(AnsiString(' '),trim(nodeid))>0) or (pos(AnsiString('#'),nodeid)>0) then
      begin
         messagebox(GetActiveWindow,'Invalid root node id (do not includes space or #, please)! ','Invalid',0);
         exit;
      end;
   fRootNodeId:=NodeId;
end;

procedure TUserConnection.SetRootNodeAddress(NodeAddress: AnsiString);
begin
   if StrIComp(PAnsiChar(trim(NodeAddress)),PAnsiChar(fRootNodeAddress))=0 then
      exit;
   if (pos(AnsiString(' '),trim(nodeaddress))>0) or (trim(nodeaddress)='0.0.0.0') then
      begin
         messagebox(GetActiveWindow,'Invalid root node address (Please input a valid IP address or domain)! ','Invalid',0);
         exit;
      end;
   fRootNodeAddress:=NodeAddress;
   if StrIComp(PAnsiChar(trim(fUserNodeId)),PAnsiChar(fRootNodeId))=0 then
      fUserNodeAddress:=NodeAddress;
end;

procedure TUserConnection.SetRootNodeAddressingPort(AddressingPort: integer);
begin
   if fRootNodeAddressingPort=AddressingPort then
      exit;
   if (AddressingPort<1) or (AddressingPort>65535) then
      begin
         messagebox(GetActiveWindow,'Invalid server port! (1-65535 allowed)','Invalid',0);
         exit;
      end;
   fRootNodeAddressingPort:=AddressingPort;
end;

procedure TUserConnection.SetStandbyRootNodeAddress(NodeAddress: AnsiString);
begin
   if StrIComp(PAnsiChar(trim(NodeAddress)),PAnsiChar(fStandbyRootNodeAddress))=0 then
      exit;
   if (pos(AnsiString(' '),trim(nodeaddress))>0) or (trim(nodeaddress)='0.0.0.0') then
      begin
         messagebox(GetActiveWindow,'Invalid root node address (Please input a valid IP address or domain)! ','Invalid',0);
         exit;
      end;
   fStandbyRootNodeAddress:=NodeAddress;
end;

procedure TUserConnection.SetStandbyRootNodeAddressingPort(AddressingPort: integer);
begin
   if fStandbyRootNodeAddressingPort=AddressingPort then
      exit;
   if (AddressingPort<1) or (AddressingPort>65535) then
      begin
         messagebox(GetActiveWindow,'Invalid server port! (1-65535 allowed)','Invalid',0);
         exit;
      end;
   fStandbyRootNodeAddressingPort:=AddressingPort;
end;

procedure TUserConnection.SetUserNodeId(NodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(trim(NodeId)),PAnsiChar(fUserNodeId))=0 then
      exit;
   if pos(AnsiString(' '),trim(nodeid))>0 then
      begin
         messagebox(GetActiveWindow,'Invalid node id (do not includes space, please)! ','Invalid',0);
         exit;
      end;
   fUserNodeId:=NodeId;
   if StrIComp(PAnsiChar(trim(fUserNodeId)),PAnsiChar(fRootNodeId))=0 then
      fUserNodeAddress:=fRootNodeAddress
   else
      begin
         if StrIComp(PAnsiChar(trim(fUserNodeAddress)),PAnsiChar(fRootNodeAddress))=0 then
            fUserNodeAddress:='';
      end;
end;

procedure TUserConnection.SetUserNodeAddress(NodeAddress: AnsiString);
begin
   if StrIComp(PAnsiChar(trim(NodeAddress)),PAnsiChar(fUserNodeAddress))=0 then
      exit;
   if pos(AnsiString(' '),trim(nodeaddress))>0 then
      begin
         messagebox(GetActiveWindow,'Invalid node address (Please input a valid IP, domain, or ''0.0.0.0'')! ','Invalid',0);
         exit;
      end;
   fUserNodeAddress:=NodeAddress;
   if StrIComp(PAnsiChar(trim(fUserNodeId)),PAnsiChar(fRootNodeId))=0 then
      fRootNodeAddress:=NodeAddress;
end;

procedure TUserConnection.SetUserNodeMsgPort(MsgPort: integer);
begin
   if fUserNodeMsgPort=MsgPort then
      exit;
   if (MsgPort<1) or (MsgPort>65535) then
      begin
         messagebox(GetActiveWindow,'Invalid server port! (1-65535 allowed)','Invalid',0);
         exit;
      end;
   fUserNodeMsgPort:=MsgPort;
end;

procedure TUserConnection.SetUserNodeTaskPort(TaskPort: integer);
begin
   if fUserNodeTaskPort=TaskPort then
      exit;
   if (TaskPort<1) or (TaskPort>65535) then
      begin
         messagebox(GetActiveWindow,'Invalid server port! (1-65535 allowed)','Invalid',0);
         exit;
      end;
   fUserNodeTaskPort:=TaskPort;
end;

procedure TUserConnection.SetHeartbeatInterval(aSeconds: integer);
begin
   if fUserHeartbeatInterval=aSeconds then
      exit;
   if (aseconds<15) or (aseconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 15-300 seconds)','Invalid',0);
         exit;
      end;
   fUserHeartbeatInterval:=aseconds;
end;

procedure TUserConnection.SetUserId(aUserId: AnsiString);
begin
   if StrIComp(PAnsiChar(fUserId),PAnsiChar(aUserId))=0 then
      exit;
   fUserId:=aUserId;
end;

procedure TUserConnection.SetUserPassword(aUserPassword: AnsiString);
begin
   if fUserPassword=aUserPassword then
      exit;
   fUserPassword:=aUserPassword;
end;

procedure TUserConnection.UserSocketConnect(Sender: TObject; Socket: TBCustomWinSocket);
begin
   fConnected:=true;
   Connecting:=false;
end;

procedure TUserConnection.UserSocketRead(Sender: TObject; Socket: TBCustomWinSocket);
begin
   if ReceiveResponseThread.Suspended then
      ReceiveResponseThread.Resume;
end;

procedure TUserConnection.UserSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
begin
   LogoutTimer.Enabled:=false;
   if fConnected then
      begin
         if not HeartbeatThread.Suspended then
            HeartbeatThread.Suspend;
         Connecting:=false;
         fConnected:=false;
         fUserLogin:=false;
         EnterCriticalSection(NodeListCs);
         NodeCount:=0;
         SetLength(Nodes,0);
         LeaveCriticalSection(NodeListCs);
         if Assigned(fOnDisconnect) then
            fOnDisconnect(self);
      end
   else
      Connecting:=false;
end;

procedure TUserConnection.UserSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
   LogoutTimer.Enabled:=false;
   if fConnected then
      begin
         socket.Close;
         application.ProcessMessages;
      end;
   if fConnected then
      begin
         if not HeartbeatThread.Suspended then
            HeartbeatThread.Suspend;
         Connecting:=false;
         fConnected:=false;
         fUserLogin:=false;
         EnterCriticalSection(NodeListCs);
         NodeCount:=0;
         SetLength(Nodes,0);
         LeaveCriticalSection(NodeListCs);
         if Assigned(fOnDisconnect) then
            fOnDisconnect(self);
      end
   else
      Connecting:=false;
   errorcode:=0;
end;

procedure TUserConnection.LoginTimerTimer(Sender: TObject);
begin
   LoginTimer.Enabled:=false;
   fSessionId:=0;
   if Assigned(fOnLoginFail) then
      fOnLoginFail(self);
end;

procedure TUserConnection.LogoutTimerTimer(Sender: TObject);
begin
   LogoutTimer.Enabled:=false;
   if Assigned(fOnLogoutFail) then
      fOnLogoutFail(self);
end;

procedure TUserConnection.DispatcherTimer(Sender: TObject);
var
   tmpstr,FromUserId: AnsiString;
   ResponseParcel: TQBParcel;
   ResponseId: integer;
   Ok: boolean;
   oldNodeId,tmpNodeId,tmpNodeName,tmpNodeAddress: AnsiString;
   tmpNodeType,tmpNodeMsgPort,tmpNodeTaskPort,i,j: integer;
begin
   Dispatcher.Enabled:=false;
   while true do
      begin
         ResponseParcel:=PopResponse;
         if ResponseParcel=nil then
            break;
         ResponseId:=ResponseParcel.GetIntegerGoods('ResponseId');
         case ResponseId of
            1: begin
                  Ok:=ResponseParcel.GetBooleanGoods('ProcessResult');
                  if Ok then
                     begin
                        fSessionId:=ResponseParcel.GetInt64Goods('SessionId');
                        fPublicSessionId:=ResponseParcel.GetInt64Goods('PublicSessionId');
                        fUserName:=ResponseParcel.GetEncryptStringGoods('UserName');
                        fUserType:=ResponseParcel.GetIntegerGoods('UserType');
                        tmpstr:=ResponseParcel.GetBinaryGoods('NodeList');
                        ResolveNodes(tmpstr);
                        fUserLogin:=true;
                        LoginTimer.Enabled:=false;
                        if HeartbeatThread.Suspended then
                           HeartbeatThread.Resume;
                        if Assigned(fOnLoginSuccess) then
                           fOnLoginSuccess(self);
                     end
                  else
                     begin
                        LoginTimer.Enabled:=false;
                        fSessionId:=0;
                        if Assigned(fOnLoginFail) then
                           fOnLoginFail(self);
                     end;
               end;
            2: begin
                  Ok:=ResponseParcel.GetBooleanGoods('ProcessResult');
                  if ok then
                     begin
                        HeartBeatReply:=true;         
                        fPublicSessionId:=ResponseParcel.GetInt64Goods('PublicSessionId');
                        tmpstr:=ResponseParcel.GetBinaryGoods('NodeList');
                        ResolveNodes(tmpstr);
                     end;
               end;
            3: begin
                  Ok:=ResponseParcel.GetBooleanGoods('ProcessResult');
                  if Ok then
                     begin
                        if not HeartbeatThread.Suspended then
                           HeartbeatThread.Suspend;
                        fUserLogin:=false;
                        LogoutTimer.Enabled:=false;
                        if Assigned(fOnLogoutSuccess) then
                           fOnLogoutSuccess(self);
                        UserSocket.Close;
                     end
                  else
                     begin
                        LogoutTimer.Enabled:=false;
                        if Assigned(fOnLogoutFail) then
                           fOnLogoutFail(self);
                     end;
               end;
            4: begin
                  FromUserId:=ResponseParcel.GetEncryptStringGoods('FromUserId');
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,FromUserId);
               end;
            5: begin
                  FromUserId:=ResponseParcel.GetEncryptStringGoods('FromUserId');
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,FromUserId);
               end;
            6: begin
                  tmpNodeId:=ResponseParcel.GetEncryptStringGoods('NodeId');
                  tmpNodeName:=ResponseParcel.GetEncryptStringGoods('NodeName');
                  tmpNodeAddress:=ResponseParcel.GetEncryptStringGoods('NodeAddress');
                  tmpNodeType:=ResponseParcel.GetIntegerGoods('NodeType');
                  tmpNodeMsgPort:=ResponseParcel.GetIntegerGoods('NodeMsgPort');
                  tmpNodeTaskPort:=ResponseParcel.GetIntegerGoods('NodeTaskPort');
                  EnterCriticalSection(NodeListCs);
                  i:=NodeCount;
                  inc(NodeCount);
                  SetLength(Nodes,NodeCount);
                  Nodes[i].NodeId:=tmpNodeId;
                  Nodes[i].NodeName:=tmpNodeName;
                  Nodes[i].NodeType:=tmpNodeType;
                  Nodes[i].NodeAddress:=tmpNodeAddress;
                  Nodes[i].MsgPort:=tmpNodeMsgPort;
                  Nodes[i].TaskPort:=tmpNodeTaskPort;
                  Nodes[i].IpAddress:='0.0.0.0';
                  Nodes[i].LastActiveDateTime:=now-1;
                  LeaveCriticalSection(NodeListCs);
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
               end;
            7: begin
                  OldNodeId:=ResponseParcel.GetEncryptStringGoods('OldNodeId');
                  tmpNodeId:=ResponseParcel.GetEncryptStringGoods('NodeId');
                  tmpNodeName:=ResponseParcel.GetEncryptStringGoods('NodeName');
                  tmpNodeAddress:=ResponseParcel.GetEncryptStringGoods('NodeAddress');
                  tmpNodeType:=ResponseParcel.GetIntegerGoods('NodeType');
                  tmpNodeMsgPort:=ResponseParcel.GetIntegerGoods('NodeMsgPort');
                  tmpNodeTaskPort:=ResponseParcel.GetIntegerGoods('NodeTaskPort');
                  EnterCriticalSection(NodeListCs);
                  j:=-1;
                  for i:= 0 to NodeCount - 1 do
                     if StrIComp(PAnsiChar(Nodes[i].NodeId),PAnsiChar(OldNodeId))=0 then
                        begin
                           j:=i;
                           break;
                        end;
                  if j<>-1 then
                     begin
                        Nodes[j].NodeId:=tmpNodeId;
                        Nodes[j].NodeName:=tmpNodeName;
                        Nodes[j].NodeType:=tmpNodeType;
                        Nodes[j].NodeAddress:=tmpNodeAddress;
                        Nodes[j].MsgPort:=tmpNodeMsgPort;
                        Nodes[j].TaskPort:=tmpNodeTaskPort;
                     end;
                  LeaveCriticalSection(NodeListCs);
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
               end;
            8: begin
                  tmpNodeId:=ResponseParcel.GetEncryptStringGoods('NodeId');
                  EnterCriticalSection(NodeListCs);
                  j:=-1;
                  for i:= 0 to NodeCount - 1 do
                     if StrIComp(PAnsiChar(Nodes[i].NodeId),PAnsiChar(tmpNodeId))=0 then
                        begin
                           j:=i;
                           break;
                        end;
                  if j<>-1 then
                     begin
                        for i:= j to NodeCount - 2 do
                           Nodes[i]:=Nodes[i+1];
                        dec(NodeCount);
                        SetLength(Nodes,NodeCount);
                     end;
                  LeaveCriticalSection(NodeListCs);
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
               end;
            9: begin
                  EnterCriticalSection(NodeListCs);
                  i:=0;
                  while i<NodeCount do
                     begin
                        if ((Nodes[i].NodeAddress='0.0.0.0') or (Nodes[i].NodeAddress='')) and (round((now-Nodes[i].LastActiveDateTime)*24*60*60)>180) then
                           begin
                              for j:=i to NodeCount-2 do
                                 Nodes[j]:=Nodes[j+1];
                              dec(NodeCount);
                              SetLength(Nodes,NodeCount);
                           end
                        else
                           inc(i);
                     end;
                  LeaveCriticalSection(NodeListCs);
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
               end;
            10: begin
                  tmpNodeId:=ResponseParcel.GetEncryptStringGoods('OfflineNodeId');
                  EnterCriticalSection(NodeListCs);
                  for i:=0 to NodeCount-1 do
                     if StrIComp(PAnsiChar(tmpNodeId),PAnsiChar(Nodes[i].NodeId))=0 then
                        begin
                           Nodes[i].IpAddress:='0.0.0.0';
                           Nodes[i].LastActiveDateTime:=now-1;
                           break;
                        end;
                  LeaveCriticalSection(NodeListCs);
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
                end;
            11: begin
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
               end;
            12: begin
                   if not HeartbeatThread.Suspended then
                      HeartbeatThread.Suspend;
                   fUserLogin:=false;
                   if assigned(fOnUserSupplant) then
                      fOnUserSupplant(self);
                   UserSocket.Close;
                end;
            13: begin
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
                end;
            14: begin
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
                end;
            15: begin
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
                end;
            16: begin
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
                end;
            17: begin
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
                end;
            18: begin
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
                end;
            19: begin
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
                end;
            20: begin
                  if assigned(fMessageBroker) then
                     fMessageBroker(ResponseParcel,'system@'+fUserNodeId);
                end;
         end;
         ResponseParcel.Free;
      end;
   DispatcherRunning:=false;
end;

procedure TUserConnection.ResolveNodes(SourceList: AnsiString);
var
   tmpstr,line: ansistring;
   nodeid,nodename,nodeaddress: ansistring;
   nodetype,msgport,taskport,j,k,i: integer;
begin
   if pos(AnsiString(#13),SourceList)<=0 then
      exit;
   EnterCriticalSection(NodeListCs);
   tmpstr:=SourceList;
   j:=pos(AnsiString(#13),tmpstr);
   while j>0 do
      begin
         line:=copy(tmpstr,1,j-1);
         delete(tmpstr,1,j);
         k:=pos(AnsiString(#9),line);
         NodeId:=copy(Line,1,k-1);
         delete(line,1,k);
         k:=pos(AnsiString(#9),line);
         NodeName:=copy(Line,1,k-1);
         delete(line,1,k);
         k:=pos(AnsiString(#9),line);
         NodeType:=strtoint(string(copy(Line,1,k-1)));
         delete(line,1,k);
         k:=pos(AnsiString(#9),line);
         NodeAddress:=copy(Line,1,k-1);
         delete(line,1,k);
         k:=pos(AnsiString(#9),line);
         MsgPort:=strtoint(string(copy(line,1,k-1)));
         delete(line,1,k);
         TaskPort:=strtoint(string(line));
         j:=-1;
         for i := 0 to NodeCount - 1 do
            if StrIComp(PAnsiChar(NodeId),PAnsiChar(Nodes[i].NodeId))=0 then
               begin
                  j:=i;
                  break;
               end;
         if j=-1 then
            begin
               j:=NodeCount;
               inc(NodeCount);
               SetLength(nodes,NodeCount);
               Nodes[j].NodeId:=NodeId;
               Nodes[j].NodeName:=NodeName;
               Nodes[j].NodeType:=NodeType;
               Nodes[j].NodeAddress:=NodeAddress;
               Nodes[j].MsgPort:=msgport;
               Nodes[j].TaskPort:=taskport;
               Nodes[j].IpAddress:='0.0.0.0';
               Nodes[j].LastActiveDateTime:=now-1;    
            end
         else
            begin
               Nodes[j].NodeName:=NodeName;
               Nodes[j].NodeType:=NodeType;
               Nodes[j].MsgPort:=msgport;
               Nodes[j].TaskPort:=Taskport;
               if StrIComp(PAnsiChar(Nodes[j].NodeAddress),PAnsiChar(NodeAddress))<>0 then
                  begin
                     Nodes[j].NodeAddress:=NodeAddress;
                     Nodes[j].IpAddress:='0.0.0.0';
                     Nodes[j].LastActiveDateTime:=now-1;  
                  end;
            end;
         j:=pos(AnsiString(#13),tmpstr);
      end;
   LeaveCriticalSection(NodeListCs);
end;

procedure TUserConnection.StartEventDispatcher;
begin
   if not DispatcherRunning then
      begin
         DispatcherRunning:=true;
         Dispatcher.Enabled:=true;
      end;
end;

function TUserConnection.QueryNodeAddress(NodeId: AnsiString; var IPAddress: AnsiString; var MsgPort: integer; var TaskPort: integer): boolean;
var
   ip,RequestBody: AnsiString;
   port: integer;
   Parcel: TQBParcel;
   ok: boolean;
begin
   ok:=ConnectToServer(fRootNodeAddress,fRootNodeAddressingPort);
   if not ok then
      begin
         if fStandbyRootNodeAddress<>'' then
            begin
               ip:=fStandbyRootNodeAddress;
               port:=fStandbyRootNodeAddressingPort;
               fStandbyRootNodeAddress:=fRootNodeAddress;
               fStandbyRootNodeAddressingPort:=fRootNodeAddressingPort;
               fRootNodeAddress:=ip;
               fRootNodeAddressingPort:=port;
               ok:=ConnectToServer(fRootNodeAddress,fRootNodeAddressingPort);
               if not ok then
                  begin
                     result:=false;
                     exit;
                  end;
            end
         else
            begin
               result:=false;
               exit;
            end;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fTransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Addressing_Request');
   Parcel.PutIntegerGoods('TaskId',1);
   Parcel.PutEncryptStringGoods('NodeId',NodeId);
   Parcel.SaveToStringWithLength(RequestBody);
   ok:=SendRequest(RequestBody);
   if not ok then
      begin
         Parcel.Free;
         AddrSocket.Active:=false;
         result:=false;
         exit;
      end;
   ok:=ReceiveResponse;
   if not ok then
      begin
         Parcel.Free;
         AddrSocket.Active:=false;
         result:=false;
         exit;
      end;
   Parcel.Clear;
   try
      Parcel.LoadFromString(ResponseBody);
      ok:=(Parcel.GoodsCount>0) and parcel.GetBooleanGoods('ProcessResult');
   except
      ok:=false;
   end;
   if not ok then
      begin
         Parcel.Free;
         result:=false;
         exit;
      end;
   IpAddress:=Parcel.GetEncryptStringGoods('IPAddress');
   MsgPort:=Parcel.GetIntegerGoods('MessagePort');
   TaskPort:=Parcel.GetIntegerGoods('TaskPort');
   Parcel.Free;
   result:=true;
end;

function TUserConnection.ConnectToServer(IpAddress: AnsiString; Port: integer): boolean;
var
   t0: TDateTime;
begin
   AddrConnecting:=true;
   AddrConnected:=false;
   AddrSocket.Host:=IpAddress;
   AddrSocket.Port:=Port;
   try
      AddrSocket.Active:=true;
      t0:=now;
      while AddrConnecting and (round((now-t0)*24*60*60)<10) do
         begin
            application.processmessages;
            sleep(1);
         end;
      result:=AddrConnected;
      if not result then
         AddrSocket.Close;
   except
      result:=false;
   end;
end;

function TUserConnection.SendRequest(RequestBody: AnsiString): boolean;
var
   t0: TDateTime;
   j: integer;
begin
   result:=false;
   t0:=now;
   while AddrConnected and (round((now-t0)*24*60*60)<10) do
      begin
         try
            j:=AddrSocket.Socket.SendBuf(RequestBody[1],length(RequestBody));
            result:=(j<>-1);
            if result then
               break;
            application.ProcessMessages;
            sleep(1);
         except
            break;
         end;
      end;
   if not result then
      AddrSocket.Close;
end;

function TUserConnection.ReceiveResponse: boolean;
var
   tmpstr: AnsiString;
   AllLen,k,j: integer;
   t1: TDateTime;
begin
   ResponseBody:='';
   AllLen:=0;
   k:=0;
   setlength(tmpstr,65536);
   result:=false;
   t1:=now;
   while (round((now-t1)*24*60*60)<10) and AddrConnected do
      begin
         try
            k:=AddrSocket.Socket.ReceiveBuf(tmpstr[1],65536);
         except
            break;
         end;
         if k>0 then
            ResponseBody:=ResponseBody+copy(tmpstr,1,k)
         else
            begin
               application.ProcessMessages;
               sleep(1);
               continue;
            end;
         if AllLen<=0 then
            begin
               j:=pos(AnsiString(#13),ResponseBody);
               if j<=0 then
                  begin
                     if length(ResponseBody)>20 then
                        break;
                     application.ProcessMessages;
                     sleep(1);
                     continue;
                  end;
               if j=1 then
                  break;
               try
                  AllLen:=StrToInt(string(copy(ResponseBody,1,j-1)));
                  if (AllLen<=0) or (AllLen>16*1024*1024) then
                     AllLen:=-1;
               except
                  AllLen:=-1;
               end;
               if AllLen<=0 then
                  Break;
               delete(ResponseBody,1,j);
            end;
         if length(ResponseBody)>=AllLen then
            begin
               result:=true;
               break;
            end;
         application.ProcessMessages;
         sleep(1);
      end;
   AddrSocket.Close;
end;

function TUserConnection.Connect(): boolean;
var
   ok,ok1,ok2: boolean;
   j: integer;
   ip: string;
begin
   if fConnected then
      begin
         if Assigned(fOnConnectSuccess) then
            fOnConnectSuccess(self);
         result:=true;
         exit;
      end;
   ok1:=(trim(fRootNodeAddress)='') or (trim(fRootNodeAddress)='0.0.0.0');
   ok2:=(trim(fUserNodeAddress)='') or (trim(fUserNodeAddress)='0.0.0.0');
   if (trim(fRootNodeId)='') or (trim(fUserNodeId)='') or (ok1 and ok2) then
      begin
         if Assigned(fOnConnectFail) then
            fOnConnectFail(self);
         result:=false;
         exit;
      end;
   if StrIComp(PAnsiChar(fRootNodeId),PAnsiChar(fUserNodeId))=0 then
      fUserNodeAddress:=fRootNodeAddress;
   if IsValidIP(fUserNodeAddress) then
      fUserNodeIP:=fUserNodeAddress
   else
      begin
         if (trim(fUserNodeAddress)='') or (trim(fUserNodeAddress)='0.0.0.0') then
            ok:=QueryNodeAddress(fUserNodeId,fUserNodeIp,fUserNodeMsgPort,j)
         else
            begin
               ok:=GetIPFromName(string(fUserNodeAddress),IP);
               fUserNodeIp:=AnsiString(Ip);
            end;
         if not ok then
            begin
               if Assigned(fOnConnectFail) then
                  fOnConnectFail(self);
               result:=false;
               exit;
            end;
      end;
   Connecting:=true;
   try
      UserSocket.Host:=fUserNodeIP;
      UserSocket.Port:=fUserNodeMsgPort;
      UserSocket.Active:=true;
      while Connecting do
         application.ProcessMessages;
      result:=fConnected;
   except
      result:=false;
   end;
   if result then
      begin
         if Assigned(fOnConnectSuccess) then
            fOnConnectSuccess(self);
      end
   else
      begin
         if Assigned(fOnConnectFail) then
            fOnConnectFail(self);
      end;
end;

function TUserConnection.Disconnect(): boolean;
begin
   result:=true;
   if fConnected then
      UserSocket.Close;
end;

function TUserConnection.Login(): boolean;
var
   RequestParcel: TQBParcel;
begin
   if not fConnected then
      begin
         if Assigned(fOnLoginFail) then
            fOnLoginFail(self);
         result:=false;
         exit;
      end;
   RequestParcel:=TQBParcel.Create;
   RequestParcel.EncryptKey:=fTransferKey;
   RequestParcel.PutEncryptStringGoods('SystemFlag','Message_Request');
   RequestParcel.PutIntegerGoods('RequestId',1);
   RequestParcel.PutEncryptStringGoods('UserId',fUserId);
   RequestParcel.PutEncryptStringGoods('UserPassword',fUserPassword);
   if PushRequest(RequestParcel) then
      begin
         LoginTimer.Enabled:=true;
         if SendRequestThread.Suspended then
            SendRequestThread.Resume;
         while LoginTimer.Enabled do
            application.ProcessMessages;
         Result:=fUserLogin;
      end
   else
      begin
         if Assigned(fOnLoginFail) then
            fOnLoginFail(self);
         result:=false;
      end;
end;

function TUserConnection.SendHeartbeat(): boolean;
var
   RequestParcel: TQBParcel;
begin
   if not fUserLogin then
      begin
         result:=false;
         exit;
      end;
   RequestParcel:=TQBParcel.Create;
   RequestParcel.EncryptKey:=fTransferKey;
   RequestParcel.PutEncryptStringGoods('SystemFlag','Message_Request');
   RequestParcel.PutIntegerGoods('RequestId',2);
   RequestParcel.PutInt64Goods('SessionId',fSessionId);
   if PushRequest(RequestParcel) then
      begin
         if SendRequestThread.Suspended then
            SendRequestThread.Resume;
         Result:=true;
      end
   else
      result:=false;
end;

function TUserConnection.Logout(): boolean;
var
   RequestParcel: TQBParcel;
begin
   if not fUserLogin then
      begin
         if Assigned(fOnLogoutFail) then
            fOnLogoutFail(self);
         result:=false;
         exit;
      end;
   RequestParcel:=TQBParcel.Create;
   RequestParcel.EncryptKey:=fTransferKey;
   RequestParcel.PutEncryptStringGoods('SystemFlag','Message_Request');
   RequestParcel.PutIntegerGoods('RequestId',3);
   RequestParcel.PutInt64Goods('SessionId',fSessionId);
   if PushRequest(RequestParcel) then
      begin
         LogoutTimer.Enabled:=true;
         if SendRequestThread.Suspended then
            SendRequestThread.Resume;
         while LogoutTimer.Enabled do
            application.ProcessMessages;
         Result:=(not fUserLogin);
      end
   else
      begin
         if Assigned(fOnLogoutFail) then
            fOnLogoutFail(self);
         result:=false;
      end;
end;

function TUserConnection.LocateNode(TargetNodeId: AnsiString; var NodeAddress: AnsiString; var MsgPort: integer; var TaskPort: integer): boolean;
var
   i: integer;
   ok: boolean;
   ip: string;
begin
   result:=false;
   NodeAddress:='0.0.0.0';
   MsgPort:=0;
   TaskPort:=0;
   if not fUserLogin then
      exit;
   if (StrIComp(PAnsiChar(TargetNodeId),PAnsiChar(fUserNodeId))=0) then
      begin
         NodeAddress:=fUserNodeIP;
         MsgPort:=fUserNodeMsgPort;
         TaskPort:=fUserNodeTaskPort;
         result:=true;
         exit;
      end;
   if StrIComp(PAnsiChar(TargetNodeId),PAnsiChar(fRootNodeId))=0 then
      begin
          if IsValidIP(fRootNodeAddress) then
             begin
                NodeAddress:=fRootNodeAddress;
                MsgPort:=fUserNodeMsgPort;
                TaskPort:=fUserNodeTaskPort;
                result:=true;
                exit;
             end;
          ok:=GetIpFromName(string(fRootNodeAddress),ip);
          if ok then
             begin
                NodeAddress:=AnsiString(ip);
                MsgPort:=fUserNodeMsgPort;
                TaskPort:=fUserNodeTaskPort;
                result:=true;
                exit;
             end;
          result:=false;
          exit;
      end;
   EnterCriticalSection(NodeListCs);
   for i := 0 to NodeCount - 1 do
      if StrIComp(PAnsiChar(Nodes[i].NodeId),PAnsiChar(TargetNodeId))=0 then
         begin
            if IsValidIP(Nodes[i].NodeAddress) then
               begin
                  NodeAddress:=Nodes[i].NodeAddress;
                  MsgPort:=Nodes[i].MsgPort;
                  TaskPort:=Nodes[i].TaskPort;
                  result:=true;
                  break;
               end;
            if (Nodes[i].NodeAddress<>'0.0.0.0') and (Nodes[i].NodeAddress<>'') then
               begin
                  ok:=GetIpFromName(string(Nodes[i].NodeAddress),ip);
                  if ok then
                     begin
                        NodeAddress:=AnsiString(ip);
                        MsgPort:=Nodes[i].MsgPort;
                        TaskPort:=Nodes[i].TaskPort;
                        result:=true;
                        break;
                     end;
               end;
            if round((now-Nodes[i].LastActiveDateTime)*24*60*60)<180 then
               begin
                  NodeAddress:=Nodes[i].IpAddress;
                  MsgPort:=Nodes[i].MsgPort;
                  TaskPort:=Nodes[i].TaskPort;
                  result:=true;
                  break;
               end;
            ok:=QueryNodeAddress(TargetNodeId,NodeAddress,MsgPort,TaskPort);
            if ok then
               begin
                  Nodes[i].IpAddress:=NodeAddress;
                  Nodes[i].MsgPort:=MsgPort;
                  Nodes[i].TaskPort:=TaskPort;
                  Nodes[i].LastActiveDateTime:=now;
               end;
            Result:=ok;
            break;
         end;
   LeaveCriticalSection(NodeListCs);
end;

function TUserConnection.PushRequest(aRequest: TQBParcel): boolean;
begin
   EnterCriticalSection(RequestQueueCs);
   if RequestQueueFull then
      begin
         LeaveCriticalSection(RequestQueueCs);
         result:=false;
         exit;
      end;
   RequestQueue[RequestQueueRear]:=aRequest;
   RequestQueueEmpty:=false;
   inc(RequestQueueRear);
   if RequestQueueRear=RequestQueueSize then
      RequestQueueRear:=0;
   if RequestQueueRear=RequestQueueFore then
      RequestQueueFull:=true;
   LeaveCriticalSection(RequestQueueCs);
   Result:=true;
end;

function TUserConnection.PopRequest(): TQBParcel;
begin
   EnterCriticalSection(RequestQueueCs);
   if RequestQueueEmpty then
      begin
         Result:=nil;
         LeaveCriticalSection(RequestQueueCs);
         exit;
      end;
   Result:=RequestQueue[RequestQueueFore];
   RequestQueueFull:=false;
   inc(RequestQueueFore);
   if RequestQueueFore=RequestQueueSize then
      RequestQueueFore:=0;
   if RequestQueueFore=RequestQueueRear then
      RequestQueueEmpty:=true;
   LeaveCriticalSection(RequestQueueCs);
end;

function TUserConnection.PushResponse(aResponse: TQBParcel): boolean;
begin
   EnterCriticalSection(ResponseQueueCs);
   if ResponseQueueFull then
      begin
         LeaveCriticalSection(ResponseQueueCs);
         result:=false;
         exit;
      end;
   ResponseQueue[ResponseQueueRear]:=aResponse;
   ResponseQueueEmpty:=false;
   inc(ResponseQueueRear);
   if ResponseQueueRear=ResponseQueueSize then
      ResponseQueueRear:=0;
   if ResponseQueueRear=ResponseQueueFore then
      ResponseQueueFull:=true;
   LeaveCriticalSection(ResponseQueueCs);
   Result:=true;
end;

function TUserConnection.PopResponse(): TQBParcel;
begin
   EnterCriticalSection(ResponseQueueCs);
   if ResponseQueueEmpty then
      begin
         Result:=nil;
         LeaveCriticalSection(ResponseQueueCs);
         exit;
      end;
   Result:=ResponseQueue[ResponseQueueFore];
   ResponseQueueFull:=false;
   inc(ResponseQueueFore);
   if ResponseQueueFore=ResponseQueueSize then
      ResponseQueueFore:=0;
   if ResponseQueueFore=ResponseQueueRear then
      ResponseQueueEmpty:=true;
   LeaveCriticalSection(ResponseQueueCs);
end;

function TUserConnection.AllocSocket: integer;
var
   i: integer;
begin
   EnterCriticalSection(SocketCs);
   result:=-1;
   for i:=0 to SocketCount-1 do
      if (not Sockets[i].Used) and (not Sockets[i].Socket.Socket.Connected) then
         begin
            result:=i;
            sockets[i].Used:=true;
            Sockets[i].Connecting:=false;
            Sockets[i].Connected:=false;
            break;
         end;
   if result=-1 then
      begin
         i:=SocketCount;
         inc(SocketCount);
         SetLength(Sockets,SocketCount);
         Sockets[i].Used:=true;
         Sockets[i].Socket:=TBClientSocket.Create(self);
         Sockets[i].Socket.ClientType:=ctNonBlocking;
         Sockets[i].Socket.OnConnect:=TaskSocketConnect;
         Sockets[i].Socket.OnDisconnect:=TaskSocketDisconnect;
         Sockets[i].Socket.OnError:=TaskSocketError;
         Sockets[i].Connecting:=false;
         Sockets[i].Connected:=false;
         result:=i;
      end;
   LeaveCriticalSection(SocketCs);
end;

procedure TUserConnection.FreeSocket(aSocketIndex: integer);
begin
   EnterCriticalSection(SocketCs);
   Sockets[aSocketIndex].Connecting:=false;
   Sockets[aSocketIndex].Connected:=false;
   Sockets[aSocketIndex].Used:=false;
   LeaveCriticalSection(SocketCs);
   application.ProcessMessages;
end;

procedure TUserConnection.TaskSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
var
   i,j: integer;
begin
   EnterCriticalSection(SocketCs);
   j:=-1;
   for i:=0 to SocketCount-1 do
      if Sockets[i].Used and (Sockets[i].Socket.Socket=Socket) then
         begin
            j:=i;
            break;
         end;
   if j<>-1 then
      begin
         Sockets[i].Connected:=false;
         Sockets[i].Connecting:=false;
      end;
   LeaveCriticalSection(SocketCs);
   socket.Close;
   ErrorCode:=0;
end;

procedure TUserConnection.TaskSocketConnect(Sender: TObject; Socket: TBCustomWinSocket);
var
   i,j: integer;
begin
   EnterCriticalSection(SocketCs);
   j:=-1;
   for i:=0 to SocketCount-1 do
      if Sockets[i].Used and (Sockets[i].Socket.Socket=Socket) then
         begin
            j:=i;
            break;
         end;
   if j<>-1 then
      begin
         Sockets[i].Connected:=true;
         Sockets[i].Connecting:=false;
      end;
   LeaveCriticalSection(SocketCs);
end;

procedure TUserConnection.TaskSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
var
   i,j: integer;
begin
   EnterCriticalSection(SocketCs);
   j:=-1;
   for i:=0 to SocketCount-1 do
      if Sockets[i].Used and (Sockets[i].Socket.Socket=Socket) then
         begin
            j:=i;
            break;
         end;
   if j<>-1 then
      begin
         Sockets[i].Connected:=false;
         Sockets[i].Connecting:=false;
      end;
   LeaveCriticalSection(SocketCs);
end;

procedure TUserConnection.AddrSocketError(Sender: TObject; Socket: TBCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
begin
   socket.Close;
   ErrorCode:=0;
end;

procedure TUserConnection.AddrSocketConnect(Sender: TObject; Socket: TBCustomWinSocket);
begin
   AddrConnected:=true;
   AddrConnecting:=false;
end;

procedure TUserConnection.AddrSocketDisconnect(Sender: TObject; Socket: TBCustomWinSocket);
begin
   AddrConnected:=false;
   AddrConnecting:=false;
end;

end.
