//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit UserGroup;

{$HINTS OFF}
{$WARNINGS OFF}

interface

uses
  Windows, Messages, SysUtils, Classes,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  DbClient, UserConnection, QBParcel, QBCommon, QBZlibExGz, GroupMemberHeartbeat, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TActiveGroupRecord=record
       GroupId: AnsiString;
       LastActiveTime: TDateTime;
   end;

type
   TFetchGroupListSuccessEvent = procedure (Sender: TObject) of object;
   TFetchGroupListFailEvent = procedure (Sender: TObject) of Object;
   TAddGroupSuccessEvent = procedure (Sender: TObject) of object;
   TAddGroupFailEvent = procedure (Sender: TObject) of object;
   TUpdateGroupSuccessEvent = procedure (Sender: TObject) of object;
   TUpdateGroupFailEvent = procedure (Sender: TObject) of object;
   TRemoveGroupSuccessEvent = procedure (Sender: TObject) of object;
   TRemoveGroupFailEvent = procedure (Sender: TObject) of object;
   TFetchMemberListSuccessEvent = procedure (Sender: TObject) of object;
   TFetchMemberListFailEvent = procedure (Sender: TObject) of Object;
   TMemberJoinSuccessEvent = procedure (Sender: TObject) of object;
   TMemberJoinFailEvent = procedure (Sender: TObject) of object;
   TMemberQuitSuccessEvent = procedure (Sender: TObject) of object;
   TMemberQuitFailEvent = procedure (Sender: TObject) of object;
   TPostMessageSuccessEvent = procedure (Sender: TObject) of object;
   TPostMessageFailEvent = procedure (Sender: TObject) of object;
   TGroupAbnormalEvent = procedure (Sender: TObject; GroupId: AnsiString) of object;

type
  TUserGroup = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fTargetNodeId: AnsiString;
     fOnFetchGroupListSuccess: TFetchGroupListSuccessEvent;
     fOnFetchGroupListFail: TFetchGroupListFailEvent;
     fOnAddGroupSuccess: TAddGroupSuccessEvent;
     fOnAddGroupFail: TAddGroupFailEvent;
     fOnUpdateGroupSuccess: TUpdateGroupSuccessEvent;
     fOnUpdateGroupFail: TUpdateGroupFailEvent;
     fOnRemoveGroupSuccess: TRemoveGroupSuccessEvent;
     fOnRemoveGroupFail: TRemoveGroupFailEvent;
     fOnFetchMemberListSuccess: TFetchMemberListSuccessEvent;
     fOnFetchMemberListFail: TFetchMemberListFailEvent;
     fOnMemberJoinSuccess: TMemberJoinSuccessEvent;
     fOnMemberJoinFail: TMemberJoinFailEvent;
     fOnMemberQuitSuccess: TMemberQuitSuccessEvent;
     fOnMemberQuitFail: TMemberQuitFailEvent;
     fOnPostMessageSuccess: TPostMessageSuccessEvent;
     fOnPostMessageFail: TPostMessageFailEvent;
     fOnGroupAbnormal: TGroupAbnormalEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     HeartbeatThread: TGroupMemberHeartbeatThread;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetTargetNodeId(aGroupId: AnsiString);
     procedure AddActiveGroup(GroupId: AnsiString);
     procedure RemoveActiveGroup(GroupId: AnsiString);

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     ActiveGroupCount: integer;
     ActiveGroups: array of TActiveGroupRecord;
     ActiveGroupCs: TRTLCriticalSection;
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     procedure _FireAutoQuitEvent(GroupId: AnsiString);
     function FetchGroupList(aCds: TClientDataset): boolean;
     function AddGroup(aGroupId: AnsiString; aGroupName: AnsiString; aGroupDesc: AnsiString;
              aGroupType: integer; aGroupOwnerId: AnsiString; AllowedMembers: integer): boolean;
     function UpdateGroup(OldGroupId: AnsiString; aGroupId: AnsiString; aGroupName: AnsiString; aGroupDesc: AnsiString;
              aGroupType: integer; aGroupOwnerId: AnsiString; AllowedMembers: integer): boolean;
     function RemoveGroup(aGroupId: AnsiString): boolean;
     function FetchMemberList(aGroupId: AnsiString; aCds: TClientDataset): boolean;
     function JoinToGroup(aGroupId: AnsiString): boolean;
     function QuitFromGroup(aGroupId: AnsiString): boolean;
     function PostGroupMessage(aGroupId: AnsiString; aMsgParcel: TQBParcel): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property OnFetchGroupListSuccess: TFetchGroupListSuccessEvent read fOnFetchGroupListSuccess write fOnFetchGroupListSuccess;
     property OnFetchGroupListFail: TFetchGroupListFailEvent read fOnFetchGroupListFail write fOnFetchGroupListFail;
     property OnAddGroupSuccess: TAddGroupSuccessEvent read fOnAddGroupSuccess write fOnAddGroupSuccess;
     property OnAddGroupFail: TAddGroupFailEvent read fOnAddGroupFail write fOnAddGroupFail;
     property OnUpdateGroupSuccess: TUpdateGroupSuccessEvent read fOnUpdateGroupSuccess write fOnUpdateGroupSuccess;
     property OnUpdateGroupFail: TUpdateGroupFailEvent read fOnUpdateGroupFail write fOnUpdateGroupFail;
     property OnRemoveGroupSuccess: TRemoveGroupSuccessEvent read fOnRemoveGroupSuccess write fOnRemoveGroupSuccess;
     property OnRemoveGroupFail: TRemoveGroupFailEvent read fOnRemoveGroupFail write fOnRemoveGroupFail;
     property OnFetchMemberListSuccess: TFetchMemberListSuccessEvent read fOnFetchMemberListSuccess write fOnFetchMemberListSuccess;
     property OnFetchMemberListFail: TFetchMemberListFailEvent read fOnFetchMemberListFail write fOnFetchMemberListFail;
     property OnMemberJoinSuccess: TMemberJoinSuccessEvent read fOnMemberJoinSuccess write fOnMemberJoinSuccess;
     property OnMemberJoinFail: TMemberJoinFailEvent read fOnMemberJoinFail write fOnMemberJoinFail;
     property OnMemberQuitSuccess: TMemberQuitSuccessEvent read fOnMemberQuitSuccess write fOnMemberQuitSuccess;
     property OnMemberQuitFail: TMemberQuitFailEvent read fOnMemberQuitFail write fOnMemberQuitFail;
     property OnPostMessageSuccess: TPostMessageSuccessEvent read fOnPostMessageSuccess write fOnPostMessageSuccess;
     property OnPostMessageFail: TPostMessageFailEvent read fOnPostMessageFail write fOnPostMessageFail;
     property OnGroupAbnormal: TGroupAbnormalEvent read fOnGroupAbnormal write fOnGroupAbnormal;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TUserGroup]);
end;

procedure TUserGroup.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TUserGroup.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TUserGroup.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TUserGroup.SetTargetNodeId(aGroupId: AnsiString);
begin
   if StrIComp(PAnsiChar(aGroupId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aGroupId;
end;

constructor TUserGroup.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   LastErrorCode:='';
   LastErrorText:='';
   ActiveGroupCount:=0;
   SetLength(ActiveGroups,0);
   InitializeCriticalSection(ActiveGroupCs);
   if not (csDesigning in ComponentState) then
      HeartbeatThread:=TGroupMemberHeartbeatThread.Create(self,true);
end;

destructor TUserGroup.Destroy;
begin
   fOnFetchGroupListSuccess:=nil;
   fOnFetchGroupListFail:=nil;
   fOnAddGroupSuccess:=nil;
   fOnAddGroupFail:=nil;
   fOnUpdateGroupSuccess:=nil;
   fOnUpdateGroupFail:=nil;
   fOnRemoveGroupSuccess:=nil;
   fOnRemoveGroupFail:=nil;
   fOnFetchMemberListSuccess:=nil;
   fOnFetchMemberListFail:=nil;
   fOnMemberJoinSuccess:=nil;
   fOnMemberJoinFail:=nil;
   fOnMemberQuitSuccess:=nil;
   fOnMemberQuitFail:=nil;
   fOnPostMessageSuccess:=nil;
   fOnPostMessageFail:=nil;
   fOnGroupAbnormal:=nil;
   if not (csDesigning in ComponentState) then
      begin
         if HeartbeatThread.Suspended then
            begin
               HeartbeatThread.Terminate;
               HeartbeatThread.Resume;
            end
         else
            HeartbeatThread.Terminate;
      end;
   ActiveGroupCount:=0;
   SetLength(ActiveGroups,0);
   DeleteCriticalSection(ActiveGroupCs);
  inherited;
end;

procedure TUserGroup.AddActiveGroup(GroupId: AnsiString);
var
   i,j: integer;
begin
   EnterCriticalSection(ActiveGroupCs);
   j:=-1;
   for i := 0 to ActiveGroupCount - 1 do
      if StrIComp(PAnsiChar(GroupId),PAnsiChar(ActiveGroups[i].GroupId))=0 then
         begin
            j:=i;
            break;
         end;
   if j=-1 then
      begin
         j:=ActiveGroupCount;
         inc(ActiveGroupCount);
         SetLength(ActiveGroups,ActiveGroupCount);
      end;
   ActiveGroups[j].GroupId:=GroupId;
   ActiveGroups[j].LastActiveTime:=now;
   if HeartbeatThread.Suspended then
      HeartbeatThread.Resume;
   LeaveCriticalSection(ActiveGroupCs);
end;

procedure TUserGroup.RemoveActiveGroup(GroupId: AnsiString);
var
   i,j: integer;
begin
   EnterCriticalSection(ActiveGroupCs);
   j:=-1;
   for i := 0 to ActiveGroupCount - 1 do
      if StrIComp(PAnsiChar(GroupId),PAnsiChar(ActiveGroups[i].GroupId))=0 then
         begin
            j:=i;
            break;
         end;
   if j<>-1 then
      begin
         for i := j to ActiveGroupCount - 2 do
            ActiveGroups[i]:=ActiveGroups[i+1];
         dec(ActiveGroupCount);
         SetLength(ActiveGroups,ActiveGroupCount);
      end;
   if not HeartbeatThread.Suspended then
      HeartbeatThread.Suspend;
   LeaveCriticalSection(ActiveGroupCs);
end;

function TUserGroup.FetchGroupList(aCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0202402';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnFetchGroupListFail) then
            fOnFetchGroupListFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',24);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     aCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('GroupList',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        aCds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if result then
                        begin
                           if Assigned(fOnFetchGroupListSuccess) then
                              fOnFetchGroupListSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0202405';
                           LastErrorText:='Decompress the return dataset fail.';
                           if Assigned(fOnFetchGroupListFail) then
                              fOnFetchGroupListFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnFetchGroupListFail) then
                        fOnFetchGroupListFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0202403';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnFetchGroupListFail) then
                  fOnFetchGroupListFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0202404';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnFetchGroupListFail) then
            fOnFetchGroupListFail(self);
      end;
end;

function TUserGroup.AddGroup(aGroupId: AnsiString; aGroupName: AnsiString; aGroupDesc: AnsiString;
        aGroupType: integer; aGroupOwnerId: AnsiString; AllowedMembers: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0202504';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnAddGroupFail) then
            fOnAddGroupFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',25);
   Parcel.PutEncryptStringGoods('GroupId',aGroupId);
   Parcel.PutEncryptStringGoods('GroupName',aGroupName);
   Parcel.PutEncryptStringGoods('GroupDesc',aGroupDesc);
   Parcel.PutIntegerGoods('GroupType',aGroupType);
   Parcel.PutEncryptStringGoods('GroupOwnerId',aGroupOwnerId);
   Parcel.PutIntegerGoods('AllowedMembers',AllowedMembers);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnAddGroupSuccess) then
                        fOnAddGroupSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnAddGroupFail) then
                        fOnAddGroupFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0202505';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnAddGroupFail) then
                  fOnAddGroupFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0202506';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnAddGroupFail) then
            fOnAddGroupFail(self);
      end;
end;

function TUserGroup.UpdateGroup(OldGroupId: AnsiString; aGroupId: AnsiString; aGroupName: AnsiString; aGroupDesc: AnsiString;
        aGroupType: integer; aGroupOwnerId: AnsiString; AllowedMembers: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0202604';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnUpdateGroupFail) then
            fOnUpdateGroupFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',26);
   Parcel.PutEncryptStringGoods('OldGroupId',OldGroupId);
   Parcel.PutEncryptStringGoods('GroupId',aGroupId);
   Parcel.PutEncryptStringGoods('GroupName',aGroupName);
   Parcel.PutEncryptStringGoods('GroupDesc',aGroupDesc);
   Parcel.PutIntegerGoods('GroupType',aGroupType);
   Parcel.PutEncryptStringGoods('GroupOwnerId',aGroupOwnerId);
   Parcel.PutIntegerGoods('AllowedMembers',AllowedMembers);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if StrIComp(PAnsiChar(OldGroupId),PAnsiChar(aGroupId))<>0 then
                        RemoveActiveGroup(OldGroupId);
                     if Assigned(fOnUpdateGroupSuccess) then
                        fOnUpdateGroupSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnUpdateGroupFail) then
                        fOnUpdateGroupFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0202605';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnUpdateGroupFail) then
                  fOnUpdateGroupFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0202606';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnUpdateGroupFail) then
            fOnUpdateGroupFail(self);
      end;
end;

function TUserGroup.RemoveGroup(aGroupId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0202704';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnRemoveGroupFail) then
            fOnRemoveGroupFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',27);
   Parcel.PutEncryptStringGoods('GroupId',aGroupId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     RemoveActiveGroup(aGroupId);
                     if Assigned(fOnRemoveGroupSuccess) then
                        fOnRemoveGroupSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnRemoveGroupFail) then
                        fOnRemoveGroupFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0202705';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnRemoveGroupFail) then
                  fOnRemoveGroupFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0202706';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnRemoveGroupFail) then
            fOnRemoveGroupFail(self);
      end;
end;

function TUserGroup.FetchMemberList(aGroupId: AnsiString; aCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0202802';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnFetchMemberListFail) then
            fOnFetchMemberListFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',28);
   Parcel.PutEncryptStringGoods('GroupId',aGroupId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     aCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('UserList',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        aCds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if result then
                        begin
                           if Assigned(fOnFetchMemberListSuccess) then
                              fOnFetchMemberListSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0202805';
                           LastErrorText:='Decompress the return dataset fail.';
                           if Assigned(fOnFetchMemberListFail) then
                              fOnFetchMemberListFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnFetchMemberListFail) then
                        fOnFetchMemberListFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0202803';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnFetchMemberListFail) then
                  fOnFetchMemberListFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0202804';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnFetchMemberListFail) then
            fOnFetchMemberListFail(self);
      end;
end;

function TUserGroup.JoinToGroup(aGroupId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0202903';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnMemberJoinFail) then
            fOnMemberJoinFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',29);
   Parcel.PutEncryptStringGoods('GroupId',aGroupId);
   Parcel.PutEncryptStringGoods('UserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.PutEncryptStringGoods('UserName',fUserConn.UserName);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     AddActiveGroup(aGroupId);
                     if Assigned(fOnMemberJoinSuccess) then
                        fOnMemberJoinSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnMemberJoinFail) then
                        fOnMemberJoinFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0202905';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnMemberJoinFail) then
                  fOnMemberJoinFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0202904';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnMemberJoinFail) then
            fOnMemberJoinFail(self);
      end;
end;

function TUserGroup.QuitFromGroup(aGroupId: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0203003';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnMemberQuitFail) then
            fOnMemberQuitFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',30);
   Parcel.PutEncryptStringGoods('GroupId',aGroupId);
   Parcel.PutEncryptStringGoods('UserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     RemoveActiveGroup(aGroupId);
                     if Assigned(fOnMemberQuitSuccess) then
                        fOnMemberQuitSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnMemberQuitFail) then
                        fOnMemberQuitFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0203005';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnMemberQuitFail) then
                  fOnMemberQuitFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0203004';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnMemberQuitFail) then
            fOnMemberQuitFail(self);
      end;
end;

function TUserGroup.PostGroupMessage(aGroupId: AnsiString; aMsgParcel: TQBParcel): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0203205';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnPostMessageFail) then
            fOnPostMessageFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',32);
   Parcel.PutEncryptStringGoods('GroupId',aGroupId);
   Parcel.PutParcelGoods('MessageBody',aMsgParcel);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnPostMessageSuccess) then
                        fOnPostMessageSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnPostMessageFail) then
                        fOnPostMessageFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0203207';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnPostMessageFail) then
                  fOnPostMessageFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0203206';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnPostMessageFail) then
            fOnPostMessageFail(self);
      end;
   result:=true;
end;

procedure TUserGroup._FireAutoQuitEvent(GroupId: AnsiString);
var
   i,j: integer;
begin
   j:=-1;
   for i := 0 to ActiveGroupCount - 1 do
      if StrIComp(PAnsiChar(GroupId),PAnsiChar(ActiveGroups[i].GroupId))=0 then
         begin
            j:=i;
            break;
         end;
   if j<>-1 then
      begin
         for i := j to ActiveGroupCount - 2 do
            ActiveGroups[i]:=ActiveGroups[i+1];
         dec(ActiveGroupCount);
         SetLength(ActiveGroups,ActiveGroupCount);
      end;
   if Assigned(fOnGroupAbnormal) then
      fOnGroupAbnormal(self,GroupId);
end;

procedure TUserGroup.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=String(LastErrorCode);
   ErrorText:=String(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
