//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit UserHeartbeat;

interface
   uses windows, Messages, SysUtils, Classes;

type
   THeartbeatThread=class(TThread)
   private
      MainObj: TComponent;
      ClientHeartbeatInterval: integer;
      LastDateTime: TDateTime;

   protected
      procedure Execute; override;
      procedure UserDisconnect;

   public
      Constructor Create(ParentObj: TComponent; SuspendedOnCreate: boolean); Overload;
      Destructor Destroy; Override;

   end;

implementation

uses UserConnection;

Constructor THeartbeatThread.Create(ParentObj: TComponent; SuspendedOnCreate: boolean);
var
   MainConnection: TUserConnection;
begin
   Inherited Create(SuspendedOnCreate);
   freeOnTerminate:=true;
   MainObj:=ParentObj;
   MainConnection:=(MainObj as TUserConnection);
   ClientHeartbeatInterval:=MainConnection.UserHeartbeatInterval;
   LastDateTime:=now;
end;

Destructor THeartbeatThread.Destroy;
begin
   Inherited Destroy;
end;

procedure THeartbeatThread.UserDisconnect;
var
   MainConnection: TUserConnection;
begin
   MainConnection:=(MainObj as TUserConnection);
   MainConnection.Disconnect;
end;

procedure THeartbeatThread.Execute;
var
   MainConnection: TUserConnection;
   t0: TDateTime;
begin
   MainConnection:=(MainObj as TUserConnection);
   while not terminated do
      begin
         LastDateTime:=now;
         while (not terminated) and (round((now-lastdatetime)*24*60*60)<(ClientHeartbeatInterval div 2)) do
            sleep(500);
         if not terminated then
            begin
               MainConnection.HeartbeatReply:=false;
               t0:=now;
               MainConnection.SendHeartbeat;
               while (not MainConnection.HeartbeatReply) and (round((now-t0)*60*24*24)<30) do
                  sleep(500);
               if not MainConnection.HeartbeatReply then
                  synchronize(UserDisconnect);
            end;
         LastDateTime:=now;
         while (not terminated) and (round((now-lastdatetime)*24*60*60)<(ClientHeartbeatInterval div 2)) do
            sleep(500);
      end;
end;

end.
