//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit UserManage;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms,
{$IFDEF UNICODE}
  AnsiStrings,
{$ENDIF}
  DbClient, UserConnection,
  QBParcel, QBCommon, QBZlibExGz, TaskCommon
{$IFDEF VER130}
  ;
{$ELSE}
  ,MidasLib;
{$ENDIF}

type
   TFetchUserListSuccessEvent = procedure (Sender: TObject) of object;
   TFetchUserListFailEvent = procedure (Sender: TObject) of Object;
   TAddUserSuccessEvent = procedure (Sender: TObject) of object;
   TAddUserFailEvent = procedure (Sender: TObject) of object;
   TUpdateUserSuccessEvent = procedure (Sender: TObject) of object;
   TUpdateUserFailEvent = procedure (Sender: TObject) of object;
   TRemoveUserSuccessEvent = procedure (Sender: TObject) of object;
   TRemoveUserFailEvent = procedure (Sender: TObject) of object;

type
  TUserManage = class(TComponent)
  private
     fUserConn: TUserConnection;
     fTaskTimeout: integer;
     fTargetNodeId: AnsiString;
     fOnFetchUserListSuccess: TFetchUserListSuccessEvent;
     fOnFetchUserListFail: TFetchUserListFailEvent;
     fOnAddUserSuccess: TAddUserSuccessEvent;
     fOnAddUserFail: TAddUserFailEvent;
     fOnUpdateUserSuccess: TUpdateUserSuccessEvent;
     fOnUpdateUserFail: TUpdateUserFailEvent;
     fOnRemoveUserSuccess: TRemoveUserSuccessEvent;
     fOnRemoveUserFail: TRemoveUserFailEvent;
     LastErrorCode: AnsiString;
     LastErrorText: AnsiString;
     RequestBody: AnsiString;
     ResponseBody: AnsiString;
     procedure SetConnection(Conn: TUserConnection);
     procedure SetTaskTimeout(seconds: integer);
     procedure SetTargetNodeId(aNodeId: AnsiString);

  protected
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
     Constructor Create(AOwner: TComponent); override;
     Destructor Destroy(); override;
     function FetchUserList(aCds: TClientDataset): boolean;
     function AddUser(aUserId: AnsiString; aUserName: AnsiString; aUserPassword: AnsiString; aUserType: integer): boolean;
     function UpdateUser(OldUserId: AnsiString; OldUserPassword: AnsiString; aUserId: AnsiString; aUserName: AnsiString; aUserPassword: AnsiString; aUserType: integer): boolean;
     function RemoveUser(aUserId: AnsiString; aUserPassword: AnsiString): boolean;
     function ChangeUserPassword(aUserId: AnsiString; OldUserPassword: AnsiString; NewUserPassword: AnsiString): boolean;
     procedure GetLastError(var ErrorCode: String; var ErrorText: String);

  published
     property UserConnection: TUserConnection read fUserConn write SetConnection;
     property TaskTimeout: integer read fTaskTimeout write SetTaskTimeout;
     property TargetNodeId: AnsiString read fTargetNodeId write SetTargetNodeId;
     property OnFetchUserListSuccess: TFetchUserListSuccessEvent read fOnFetchUserListSuccess write fOnFetchUserListSuccess;
     property OnFetchUserListFail: TFetchUserListFailEvent read fOnFetchUserListFail write fOnFetchUserListFail;
     property OnAddUserSuccess: TAddUserSuccessEvent read fOnAddUserSuccess write fOnAddUserSuccess;
     property OnAddUserFail: TAddUserFailEvent read fOnAddUserFail write fOnAddUserFail;
     property OnUpdateUserSuccess: TUpdateUserSuccessEvent read fOnUpdateUserSuccess write fOnUpdateUserSuccess;
     property OnUpdateUserFail: TUpdateUserFailEvent read fOnUpdateUserFail write fOnUpdateUserFail;
     property OnRemoveUserSuccess: TRemoveUserSuccessEvent read fOnRemoveUserSuccess write fOnRemoveUserSuccess;
     property OnRemoveUserFail: TRemoveUserFailEvent read fOnRemoveUserFail write fOnRemoveUserFail;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickBurro Standard', [TUserManage]);
end;

procedure TUserManage.SetConnection(Conn: TUserConnection);
begin
   if assigned(fUserConn) and (fUserConn=Conn) then
      exit;
   fUserConn:=Conn;
   fUserConn.FreeNotification(self);
end;

procedure TUserManage.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = fUserConn) then
    fUserConn := nil;
end;

procedure TUserManage.SetTaskTimeout(seconds: integer);
begin
   if fTaskTimeout=Seconds then
      exit;
   if (seconds<1) or (seconds>300) then
      begin
         messagebox(GetActiveWindow,'Invalid value! (Must be 1-300 seconds)','Invalid',0);
         exit;
      end;
   fTaskTimeout:=seconds;
end;

procedure TUserManage.SetTargetNodeId(aNodeId: AnsiString);
begin
   if StrIComp(PAnsiChar(aNodeId),PAnsiChar(fTargetNodeId))=0 then
      exit;
   fTargetNodeId:=aNodeId;
end;

constructor TUserManage.Create(AOwner: TComponent);
begin
   inherited;
   fUserConn:=nil;
   fTaskTimeout:=10;
   LastErrorCode:='';
   LastErrorText:='';
end;

destructor TUserManage.Destroy;
begin
   fOnFetchUserListSuccess:=nil;
   fOnFetchUserListFail:=nil;
   fOnAddUserSuccess:=nil;
   fOnAddUserFail:=nil;
   fOnUpdateUserSuccess:=nil;
   fOnUpdateUserFail:=nil;
   fOnRemoveUserSuccess:=nil;
   fOnRemoveUserFail:=nil;
  inherited;
end;

function TUserManage.FetchUserList(aCds: TClientDataset): boolean;
var
   Parcel: TQBParcel;
   Stream1,Stream2: TMemoryStream;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200102';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnFetchUserListFail) then
            fOnFetchUserListFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',1);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     aCds.close;
                     Stream1:=TMemoryStream.Create;
                     Stream2:=TMemoryStream.Create;
                     try
                        Parcel.GetStreamGoods('UserList',Stream1);
                        GZDecompressStream(Stream1,Stream2);
                        Stream2.Position:=0;
                        aCds.LoadFromStream(Stream2);
                     except
                        result:=false;
                     end;
                     Stream1.Free;
                     Stream2.Free;
                     if result then
                        begin
                           if Assigned(fOnFetchUserListSuccess) then
                              fOnFetchUserListSuccess(self);
                        end
                     else
                        begin
                           LastErrorCode:='0200105';
                           LastErrorText:='Decompress the return dataset fail.';
                           if Assigned(fOnFetchUserListFail) then
                              fOnFetchUserListFail(self);
                        end;
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnFetchUserListFail) then
                        fOnFetchUserListFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0200103';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnFetchUserListFail) then
                  fOnFetchUserListFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200104';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnFetchUserListFail) then
            fOnFetchUserListFail(self);
      end;
end;

function TUserManage.AddUser(aUserId: AnsiString; aUserName: AnsiString; aUserPassword: AnsiString; aUserType: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200203';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnAddUserFail) then
            fOnAddUserFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',2);
   Parcel.PutEncryptStringGoods('UserId',aUserId);
   Parcel.PutEncryptStringGoods('UserName',aUserName);
   Parcel.PutEncryptStringGoods('UserPassword',aUserPassword);
   Parcel.PutIntegerGoods('UserType',aUserType);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnAddUserSuccess) then
                        fOnAddUserSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnAddUserFail) then
                        fOnAddUserFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0200204';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnAddUserFail) then
                  fOnAddUserFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200205';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnAddUserFail) then
            fOnAddUserFail(self);
      end;
end;

function TUserManage.UpdateUser(OldUserId: AnsiString; OldUserPassword: AnsiString; aUserId: AnsiString; aUserName: AnsiString; aUserPassword: AnsiString; aUserType: integer): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200303';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnUpdateUserFail) then
            fOnUpdateUserFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',3);
   Parcel.PutEncryptStringGoods('OldUserId',OldUserId);
   Parcel.PutEncryptStringGoods('OldUserPassword',OldUserPassword);
   Parcel.PutEncryptStringGoods('UserId',aUserId);
   Parcel.PutEncryptStringGoods('UserName',aUserName);
   Parcel.PutEncryptStringGoods('UserPassword',aUserPassword);
   Parcel.PutIntegerGoods('UserType',aUserType);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnUpdateUserSuccess) then
                        fOnUpdateUserSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnUpdateUserFail) then
                        fOnUpdateUserFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0200304';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnUpdateUserFail) then
                  fOnUpdateUserFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200305';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnUpdateUserFail) then
            fOnUpdateUserFail(self);
      end;
end;

function TUserManage.RemoveUser(aUserId: AnsiString; aUserPassword: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0200403';
         LastErrorText:='System not ready to commit task.';
         if Assigned(fOnRemoveUserFail) then
            fOnRemoveUserFail(self);
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',4);
   Parcel.PutEncryptStringGoods('UserId',aUserId);
   Parcel.PutEncryptStringGoods('UserPassword',aUserPassword);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if result then
                  begin
                     if Assigned(fOnRemoveUserSuccess) then
                        fOnRemoveUserSuccess(self);
                  end
               else
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                     if Assigned(fOnRemoveUserFail) then
                        fOnRemoveUserFail(self);
                  end;
            end
         else
            begin
               LastErrorCode:='0200404';
               LastErrorText:='The response message is invalid.';
               if Assigned(fOnRemoveUserFail) then
                  fOnRemoveUserFail(self);
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0200405';
         LastErrorText:='Communication error on commit task.';
         if Assigned(fOnRemoveUserFail) then
            fOnRemoveUserFail(self);
      end;
end;

function TUserManage.ChangeUserPassword(aUserId: AnsiString; OldUserPassword: AnsiString; NewUserPassword: AnsiString): boolean;
var
   Parcel: TQBParcel;
begin
   if (trim(fTargetNodeId)='') or (fUserConn=nil) or (not fUserConn.UserLogin) then
      begin
         LastErrorCode:='0208203';
         LastErrorText:='System not ready to commit task.';
         result:=false;
         exit;
      end;
   Parcel:=TQBParcel.Create;
   Parcel.EncryptKey:=fUserConn.TransferKey;
   Parcel.PutEncryptStringGoods('SystemFlag','Task_Request');
   Parcel.PutEncryptStringGoods('RequestUserId',fUserConn.UserId+'@'+fUserConn.UserNodeId);
   if StrIComp(PAnsiChar(fUserConn.UserNodeId),PAnsiChar(fTargetNodeId))=0 then
      Parcel.PutInt64Goods('SessionId',fUserConn.SessionId)
   else
      Parcel.PutInt64Goods('SessionId',fUserConn.PublicSessionId);
   Parcel.PutIntegerGoods('TaskId',82);
   Parcel.PutEncryptStringGoods('UserId',aUserId);
   Parcel.PutEncryptStringGoods('OldUserPassword',OldUserPassword);
   Parcel.PutEncryptStringGoods('NewUserPassword',NewUserPassword);
   Parcel.SaveToStringWithLength(RequestBody);
   Parcel.Free;
   result:=RemoteExecute(fUserConn,fTargetNodeId,RequestBody,fTaskTimeout,ResponseBody);
   if result then
      begin
         Parcel:=TQBParcel.Create;
         Parcel.EncryptKey:=fUserConn.TransferKey;
         Parcel.LoadFromString(ResponseBody);
         result:=(Parcel.GoodsCount>0);
         if result then
            begin
               result:=Parcel.GetBooleanGoods('ProcessResult');
               if not result then
                  begin
                     LastErrorCode:=Parcel.GetEncryptStringGoods('ErrorCode');
                     LastErrorText:=Parcel.GetEncryptStringGoods('ErrorText');
                  end;
            end
         else
            begin
               LastErrorCode:='0208204';
               LastErrorText:='The response message is invalid.';
            end;
         Parcel.Free;
      end
   else
      begin
         LastErrorCode:='0208205';
         LastErrorText:='Communication error on commit task.';
      end;
end;

procedure TUserManage.GetLastError(var ErrorCode: String; var ErrorText: String);
begin
   ErrorCode:=string(LastErrorCode);
   ErrorText:=string(LastErrorText);
   LastErrorCode:='';
   LastErrorText:='';
end;

end.
