//
//---------------------------------------------------------------
//                                                               
//                                                               
//    QuickBurro Middleware Application Development Package      
//                                                               
//    Version 4.0                                               
//    Update Date: 2011.4.18.                                    
//    Author: Jopher(W.G.Z.)                                     
//    QQ: 779545524                                              
//    Email: Jopher@189.cn  or wpy020327@163.com  
//    Homepage: http://www.jopher.net/               
//                                                               
//    Copyright(C) Jopher Software Studio. All rights reserved   
//                                                               
//                                                               
//---------------------------------------------------------------
//
unit UserResponse;

{$HINTS OFF}
{$WARNINGS OFF}

interface
   uses windows, Messages, SysUtils, Classes,
   {$IFDEF UNICODE}
   AnsiStrings,
   {$ENDIF}
   BurroSocket, QBParcel, QBCommon;

type
   TReceiveResponseThread=class(TThread)
   private
      MainObj: TComponent;
      Socket: TBClientWinSocket;
      ResponseLength: integer;
      ResponseBody: AnsiString;
      ResponseParcel: TQBParcel;
      Buffer: array [0..65535] of AnsiChar;
      RestResponse: AnsiString;

   protected
      procedure Execute; override;
      function ReceiveResponse: boolean;

   public
      Constructor Create(ParentObj: TComponent; SuspendedOnCreate: boolean); Overload;
      Destructor Destroy; Override;

   end;

implementation

uses UserConnection;

Constructor TReceiveResponseThread.Create(ParentObj: TComponent; SuspendedOnCreate: boolean);
var
   MainConnection: TUserConnection;
begin
   Inherited Create(SuspendedOnCreate);
   freeOnTerminate:=true;
   MainObj:=ParentObj;
   MainConnection:=(MainObj as TUserConnection);
end;

Destructor TReceiveResponseThread.Destroy;
begin
   Inherited Destroy;
end;

procedure TReceiveResponseThread.Execute;
var
   ok: boolean;
   MainConnection: TUserConnection;
begin
   MainConnection:=(MainObj as TUserConnection);
   while not terminated do
      begin
         Socket:=MainConnection.UserSocket.Socket;
         ResponseLength:=0;
         ResponseBody:='';
         RestResponse:='';
         while not terminated do
            begin
               ok:=ReceiveResponse;
               if not ok then
                  begin
                     socket.Close;
                     break;
                  end;
               ResponseParcel:=TQBParcel.Create;
               try
                  ResponseParcel.LoadFromString(ResponseBody);
                  ResponseParcel.EncryptKey:=MainConnection.TransferKey;    
                  ok:=(ResponseParcel.GoodsCount>0);
               except
                  ok:=false;
               end;
               if not ok then
                  begin
                     ResponseParcel.Free;
                     socket.Close;
                     break;
                  end;
               if MainConnection.PushResponse(ResponseParcel) then
                  MainConnection.StartEventDispatcher
               else
                  ResponseParcel.Free;
               if RestResponse='' then
                  break;
               ResponseLength:=0;
               ResponseBody:=RestResponse;
               RestResponse:='';
            end;
         if not terminated then
            Synchronize(self.Suspend);
      end;
end;

function TReceiveResponseThread.ReceiveResponse: boolean;
var
   tmpstr: AnsiString;
   j,bytes: integer;
begin
   result:=false;
   while (not terminated) and socket.connected do
      begin
         j:=socket.ReceiveLength;
         if j>65536 then
            j:=65536;
         if j>0 then
            begin
               try
                  bytes:=Socket.ReceiveBuf(buffer,j);
               except
                  bytes:=-1;
               end;
               if bytes<0 then
                  break;
               if bytes>0 then
                  begin
                     j:=length(ResponseBody);
                     setlength(ResponseBody,j+bytes);
                     move(buffer[0],ResponseBody[j+1],bytes);
                  end;
            end;
         if ResponseLength<=0 then
            begin
               j:=pos(AnsiChar(#13),ResponseBody);
               if j>0 then
                  begin
                     tmpstr:=copy(ResponseBody,1,j-1);
                     delete(ResponseBody,1,j);
                     if trim(tmpstr)='' then
                        ResponseLength:=-1
                     else
                        try
                           ResponseLength:=StrToInt(string(tmpstr));
                        except
                           ResponseLength:=-1;
                        end;
                     if ResponseLength<=0 then
                        exit;
                  end;
            end;
         if ResponseLength>0 then
            begin
               j:=length(ResponseBody);
               if j>=ResponseLength then
                  begin
                     if j>ResponseLength then
                        begin
                           RestResponse:=copy(ResponseBody,ResponseLength+1,j-ResponseLength);
                           Delete(ResponseBody,ResponseLength+1,j-ResponseLength);
                        end;
                     result:=true;
                     break;
                  end;
            end
         else
            begin
               if length(ResponseBody)>32 then
                  break;                              
            end;
         sleep(1);
      end;
end;

end.
